const log = require("npmlog");
const fs = require("fs");
const https = require("https");
// const path = require("path");
const { toHump, toBigHump } = require("../../utils");

const { ICON_FILE_PATH, USER_COOKIE, getConfigFile } = require("../../utils/icon/index");
const EXPORT_SIGN = "export start";
const SVG_SIGN = "SVG start";
const ICON_FONT_SIGN = "ICON start";

/**
 * 定义命令相关选项
 *
 * @param {*} args
 */
function createBuilder(args) {
    const opts = {
        [ICON_FILE_PATH]: {
            alias: "ifp",
            describe: "存放iconfont的文件路径，默认使用配置中的路径",
        },
    };
    args.options(opts);
}
function createHandler(args) {
    const packages = args.packages || {};
    let { [ICON_FILE_PATH]: iconFilePath } = packages;
    let cookie;

    if (!iconFilePath) {
        let config = getConfigFile();

        iconFilePath = config[ICON_FILE_PATH];
        cookie = config[USER_COOKIE];
    }
    if (!iconFilePath) {
        log.error(`icon 命令错误：获取${ICON_FILE_PATH}配置失败，请检查！`);
        process.exit(1);
    }
    if (!cookie) {
        log.error(`icon 命令错误：获取user-cookie配置失败，请检查！`);
        process.exit(1);
    }

    let iconfontFileData = fs.readFileSync(iconFilePath, "utf-8");
    if (iconfontFileData) {
        getIconFontInfo(cookie).then((resp) => {
            if (resp && resp.data && resp.data.icons) {
                const icons = resp.data.icons;
                if (icons) {
                    log.info(`icon 命令提示：开始自动添加新增icon组件！`);

                    icons.forEach((icon) => {
                        const fontClass = icon.font_class;
                        if (fontClass) {
                            const iconfontSVGName = createSVGName(fontClass);
                            const iconfontName = createIconName(fontClass);

                            if (!iconfontSVGName || !iconfontName) {
                                log.error(`icon 命令错误：生成icon名称或者svg名称为空，请重试！`);
                                process.exit(1);
                            }
                            const includeSVGName = iconfontFileData.includes(iconfontSVGName);
                            if (!includeSVGName) {
                                const SVGComponentCode = createSVGCOmponentCode(
                                    iconfontSVGName,
                                    icon
                                );
                                const IconComponentCode = createIconCOmponentCode(
                                    iconfontName,
                                    iconfontSVGName
                                );

                                if (!SVGComponentCode || !IconComponentCode) {
                                    log.error(
                                        `icon 命令错误：生成${iconfontName}代码失败失败，请重试！`
                                    );
                                    process.exit(1);
                                }
                                iconfontFileData = insertCode(
                                    iconfontFileData,
                                    iconfontName,
                                    SVGComponentCode,
                                    IconComponentCode,
                                    icon
                                );
                            }
                        }
                    });
                    if (iconfontFileData) {
                        fs.writeFile(iconFilePath, iconfontFileData, "utf8", (err) => {
                            if (err) {
                                log.error(
                                    `icon 命令错误：重写字体文件错误，错误信息：${err}，请检查！`
                                );
                                process.exit(1);
                            }
                        });
                    }
                }
            } else {
                log.error(`icon 命令错误：获取${ICON_FILE_PATH},user-cookie配置失败，请检查！`);
                process.exit(1);
            }
        });
    } else {
        log.error(`icon 命令错误：获取iconfont信息失败，请重试！`);
        process.exit(1);
    }
}

const iconConfig = {
    cmd: "update-icon [packages...]",
    desc: "fetch the iconfonts of udesk_webapp project from https://www.iconfont.cn/, then update icon-file-path's file .",
    builder: createBuilder,
    handler: createHandler,
};

module.exports = iconConfig;

const getIconFontInfo = (cookie) => {
    return new Promise((resolve, reject) => {
        https.get(
            "https://www.iconfont.cn/api/project/detail.json?pid=173500&t=1632621391075&ctoken=be14PhMeDkRDRuYiado4b6Ru",
            {
                headers: {
                    accept: "application/json, text/javascript, */*; q=0.01",
                    "accept-language": "zh-CN,zh;q=0.9",
                    "cache-control": "no-cache",
                    pragma: "no-cache",
                    "sec-ch-ua": '"Chromium";v="92", " Not A;Brand";v="99", "Google Chrome";v="92"',
                    "sec-ch-ua-mobile": "?0",
                    "sec-fetch-dest": "empty",
                    "sec-fetch-mode": "cors",
                    "sec-fetch-site": "same-origin",
                    "x-requested-with": "XMLHttpRequest",
                    cookie,
                },
                referrer:
                    "https://www.iconfont.cn/manage/index?spm=a313x.7781069.1998910419.13&manage_type=myprojects&projectId=173500&keyword=&project_type=&page=",
                referrerPolicy: "strict-origin-when-cross-origin",
                body: null,
                method: "GET",
                mode: "cors",
            },
            (resp) => {
                let data = [];
                resp.setEncoding("utf8");
                resp.on("data", (chunk) => data.push(chunk));
                resp.on("end", () => {
                    data = data.join("");
                    try {
                        data = JSON.parse(data);
                    } catch (e) {
                        data = {};
                    }

                    resolve(data);
                });
            }
        );
    });
};

function createSVGName(name) {
    const formatFontClass = toHump(name);
    return `${formatFontClass}SVG`;
}

/**
 * 创建Iconfont名字
 *
 * @param {*} name
 * @return {*}
 */
function createIconName(name) {
    const formatFontClass = toBigHump(name);
    return `${formatFontClass}Icon`;
}
function formatSVG(svgString) {
    return svgString.replace(/class="[a-zA-Z]+"/, "").replace(/style="[^"]+"/, "");
}
function createSVGCOmponentCode(SVGName, { show_svg }) {
    if (SVGName && show_svg) {
        return `const ${SVGName} = ()=>(${formatSVG(show_svg)})`;
    }
}

function createIconCOmponentCode(iconfontName, iconfontSVGName) {
    if (iconfontName && iconfontSVGName) {
        return `const ${iconfontName}: React.FC<ComponentProps<typeof Icon>> = React.forwardRef((props:any, ref:any) => (<Icon ref={ref} component={${iconfontSVGName}} {...props} />));
        ${iconfontName}.displayName = '${iconfontName}';`;
    }
}

function insertCode(fileData, iconfontName, SVGComponentCode, IconComponentCode, icon) {
    const newFileData = fileData
        .replace(EXPORT_SIGN, function (match) {
            return `${match}\n    ${iconfontName},// 名称为：${icon.name},font_class为：${icon.font_class}`;
        })
        .replace(SVG_SIGN, function (match) {
            return `${match}\n// 名称为：${icon.name},font_class为：${icon.font_class}\n${SVGComponentCode}`;
        })
        .replace(ICON_FONT_SIGN, function (match) {
            return `${match}\n// 名称为：${icon.name},font_class为：${icon.font_class}\n${IconComponentCode}`;
        });
    log.info(`icon 命令提示：${iconfontName}替换成功！`);
    return newFileData;
}
