'use strict';

const autoprefixer = require('autoprefixer');
const path = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ParallelUglifyPlugin = require('webpack-parallel-uglify-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const ManifestPlugin = require('webpack-manifest-plugin');
const InterpolateHtmlPlugin = require('react-dev-utils/InterpolateHtmlPlugin');
const eslintFormatter = require('react-dev-utils/eslintFormatter');
const WebpackMessagesPlugin = require('webpack-messages');
const paths = require('./paths');
const { loadEnv, getClientEnvironment } = require('./ENV');

// Assert this just to be safe.
// Development builds of React are slow and not intended for production.
if (process.env.NODE_ENV !== 'production') {
    throw new Error('Production builds must have NODE_ENV=production.');
}

loadEnv();

// Get environment variables to inject into our app.
const env = getClientEnvironment();

// This is the production configuration.
// It compiles slowly and is focused on producing a fast and minimal bundle.
// The development configuration is different and lives in a separate file.
module.exports = [
    buildProdConfig({ debug: true, enableSourceMap: true }),
    buildProdConfig({ debug: false, enableSourceMap: true }),
    buildProdConfig({ debug: false, enableSourceMap: false }),
];
module.exports.buildProdConfig = buildProdConfig;

function buildProdConfig({ debug = false, enableSourceMap = true, excludeHtml = false } = {}) {
    return {
        // Don't attempt to continue if there are any errors.
        bail: true,
        // We generate sourcemaps in production. This is slow but gives good results.
        // You can exclude the *.map files from the build during deployment.
        devtool: enableSourceMap ? 'source-map' : false,
        // In production, we only want to load the polyfills and the app code.
        entry: {
            "udesk.web.toolkits.all": [
                paths.packageIndexJs 
            ],
            "udesk": [
                paths.srcUdeskEntry
            ],
            "udesk.web.toolkits.health-check": [
                paths.packageHealthCheckEntry
            ],
            "udesk.plugins.utils.crypto.md5": [
                paths.srcPluginUtilsCryptoMd5
            ],
            "udesk.plugins.utils.crypto.sha-256": [
                paths.srcPluginUtilsCryptoSha256
            ],
            "udesk.plugins.utils.crypto.sha-224": [
                paths.srcPluginUtilsCryptoSha224
            ],
        },
        // the environment in which the bundle should run
        // changes chunk loading behavior and available modules
        target: "web",
        output: {
            // The build folder.
            path: paths.appBuild,
            // Generated JS file names (with nested folders).
            // There will be one main bundle, and one file per asynchronous chunk.
            // We don't currently advertise code splitting but Webpack supports it.
            // Support functions in v4.0
            // filename: function (file) {
            //     let moduleName = file.match(/\.(.+?)$/)[1];
            //     let dir = "javascript";
            //     if (moduleName !== "all") {
            //         dir += "/" + moduleName;
            //     }
            //     return `${dir}/[${file.name}]${debug?"":".min"}.js`;
            // },
            filename: `javascript/[name]${debug?"":".min"}${enableSourceMap?"":".nomap"}.js`,
            chunkFilename: `javascript/[name]${debug?"":".min"}${enableSourceMap?"":".nomap"}.chunk.[chunkhash:8].js`,
            // the name of the exported library
            // library: "udesk.web.toolkits", // string,
            // the type of the exported library
            libraryTarget: "umd", // universal module definition
            // Point sourcemap entries to original disk location (format as URL on Windows)
            devtoolModuleFilenameTemplate: info => path.relative(paths.appSrc, info.absoluteResourcePath).replace(/\\/g, '/'),
        },
        resolve: {
            // This allows you to set a fallback for where Webpack should look for modules.
            // We placed these paths second because we want `node_modules` to "win"
            // if there are any conflicts. This matches Node resolution mechanism.
            // https://github.com/facebookincubator/create-react-app/issues/253
            modules: ['node_modules', paths.appNodeModules].concat(
                // It is guaranteed to exist because we tweak it in `env.js`
                process.env.NODE_PATH.split(path.delimiter).filter(Boolean)
            ),
            // These are the reasonable defaults supported by the Node ecosystem.
            // We also include JSX as a common component filename extension to support
            // some tools, although we do not recommend using it, see:
            // https://github.com/facebookincubator/create-react-app/issues/290
            // `web` extension prefixes have been added for better support
            // for React Native Web.
            extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx'],
            alias: {

                // Support React Native Web
                // https://www.smashingmagazine.com/2016/08/a-glimpse-into-the-future-with-react-native-for-web/
                'react-native': 'react-native-web',
            },
            plugins: [],
        },
        module: {
            strictExportPresence: true,
            rules: [
                // TODO: Disable require.ensure as it's not a standard language feature.
                // We are waiting for https://github.com/facebookincubator/create-react-app/issues/2176.
                // { parser: { requireEnsure: false } },

                // First, run the linter.
                // It's important to do this before Babel processes the JS.
                {
                    test: /\.(js|jsx|mjs)$/,
                    enforce: 'pre',
                    use: [{
                        options: {
                            formatter: eslintFormatter,
                            eslintPath: require.resolve('eslint'),

                        },
                        loader: require.resolve('eslint-loader'),
                    }, ],
                    include: ([
                        paths.appSrc
                    ]).concat(paths.packagesSrcDirRegExps),
                },
                {
                    test: /\.js$/,
                    loader: 'unlazy-loader'
                },
                {
                    // "oneOf" will traverse all following loaders until one will
                    // match the requirements. When no loader matches it will fall
                    // back to the "file" loader at the end of the loader list.
                    oneOf: [
                        // "url" loader works just like "file" loader but it also embeds
                        // assets smaller than specified size as data URLs to avoid requests.
                        {
                            test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
                            loader: require.resolve('url-loader'),
                            options: {
                                limit: 10000,
                                name: 'static/media/[name].[hash:8].[ext]',
                            },
                        },
                        // Process JS with Babel.
                        {
                            test: /\.(js|jsx|mjs)$/,
                            include: ([
                                paths.appSrc,
                            ]).concat(paths.packagesSrcDirRegExps),
                            use: [{
                                    loader: "babel-loader",
                                    options: {
                                        // This is a feature of `babel-loader` for webpack (not Babel itself).
                                        // It enables caching results in ./node_modules/.cache/babel-loader/
                                        // directory for faster rebuilds.
                                        cacheDirectory: true,
                                        compact: !debug,
                                    }
                                },
                                'webpack-conditional-loader'
                            ],
                        },
                        // The notation here is somewhat confusing.
                        // "postcss" loader applies autoprefixer to our CSS.
                        // "css" loader resolves paths in CSS and adds assets as dependencies.
                        // "style" loader normally turns CSS into JS modules injecting <style>,
                        // but unlike in development configuration, we do something different.
                        // `ExtractTextPlugin` first applies the "postcss" and "css" loaders
                        // (second argument), then grabs the result CSS and puts it into a
                        // separate file in our build process. This way we actually ship
                        // a single CSS file in production instead of JS code injecting <style>
                        // tags. If you use code splitting, however, any async bundles will still
                        // use the "style" loader inside the async code so CSS from them won't be
                        // in the main CSS file.
                        {
                            test: /\.css$/,
                            loader: ExtractTextPlugin.extract(
                                Object.assign({
                                    fallback: {
                                        loader: require.resolve('style-loader'),
                                        options: {
                                            hmr: false,
                                        },
                                    },
                                    use: [{
                                            loader: require.resolve('css-loader'),
                                            options: {
                                                importLoaders: 1,
                                                minimize: !debug,
                                                sourceMap: enableSourceMap,
                                            },
                                        },
                                        {
                                            loader: require.resolve('postcss-loader'),
                                            options: {
                                                // Necessary for external CSS imports to work
                                                // https://github.com/facebookincubator/create-react-app/issues/2677
                                                ident: 'postcss',
                                                plugins: () => [
                                                    require('postcss-flexbugs-fixes'),
                                                    autoprefixer({
                                                        browsers: [
                                                            'ie >= 9',
                                                            'Edge >= 12',
                                                            'Firefox >= 30',
                                                            'Chrome >= 35',
                                                            'Safari >= 7',
                                                            'iOS > 7',
                                                            'Android >= 4'
                                                        ],
                                                        flexbox: 'no-2009',
                                                    }),
                                                ],
                                            },
                                        },
                                    ],
                                }, {})
                            ),
                            // Note: this won't work without `new ExtractTextPlugin()` in `plugins`.
                        },
                        // "file" loader makes sure assets end up in the `build` folder.
                        // When you `import` an asset, you get its filename.
                        // This loader doesn't use a "test" so it will catch all modules
                        // that fall through the other loaders.
                        {
                            loader: require.resolve('file-loader'),
                            // Exclude `js` files to keep "css" loader working as it injects
                            // it's runtime that would otherwise processed through "file" loader.
                            // Also exclude `html` and `json` extensions so they get processed
                            // by webpacks internal loaders.
                            exclude: [/\.js$/, /\.html$/, /\.json$/],
                            options: {
                                name: 'static/media/[name].[hash:8].[ext]',
                            },
                        },
                        // ** STOP ** Are you adding a new loader?
                        // Make sure to add the new loader(s) before the "file" loader.
                    ],
                },
            ],
        },
        plugins: [
            // Makes some environment variables available in index.html.
            // The public URL is available as %PUBLIC_URL% in index.html, e.g.:
            // <link rel="shortcut icon" href="%PUBLIC_URL%/favicon.ico">
            // In production, it will be an empty string unless you specify "homepage"
            // in `package.json`, in which case it will be the pathname of that URL.
            debug ? null :
            new InterpolateHtmlPlugin(env.raw),
            // Generates an `index.html` file with the <script> injected.
            (debug || excludeHtml) ? null :
            new HtmlWebpackPlugin({
                inject: true,
                template: paths.appHtml,
                minify: {
                    removeComments: true,
                    collapseWhitespace: true,
                    removeRedundantAttributes: true,
                    useShortDoctype: true,
                    removeEmptyAttributes: true,
                    removeStyleLinkTypeAttributes: true,
                    keepClosingSlash: true,
                    minifyJS: true,
                    minifyCSS: true,
                    minifyURLs: true,
                },
            }),
            debug ? null :
            new CleanWebpackPlugin([
                paths.appBuild + "/**/*.*"
            ], {
                root: paths.appRoot
            }),
            // Makes some environment variables available to the JS code, for example:
            // if (process.env.NODE_ENV === 'production') { ... }. See `./env.js`.
            // It is absolutely essential that NODE_ENV was set to production here.
            // Otherwise React will be compiled in the very slow development mode.
            new webpack.DefinePlugin(env.stringified),
            // Minify the code.
            debug ? null :
            new webpack.optimize.UglifyJsPlugin({
                compress: {
                    warnings: false,
                    // Disabled because of an issue with Uglify breaking seemingly valid code:
                    // https://github.com/facebookincubator/create-react-app/issues/2376
                    // Pending further investigation:
                    // https://github.com/mishoo/UglifyJS2/issues/2011
                    comparisons: false,
                },
                sourceMap: enableSourceMap,
                parallel: true,
                mangle: {
                    safari10: true,
                },
                output: {
                    comments: false,
                    // Turned on because emoji and regex is not minified properly using default
                    // https://github.com/facebookincubator/create-react-app/issues/2488
                    ascii_only: true,
                }
            }),
            // debug ? null :
            // new ParallelUglifyPlugin({
            //     sourceMap: enableSourceMap,
            //     uglifyJS: {

            //     }
            // }),
            debug ? null :
            new webpack.optimize.AggressiveSplittingPlugin({
                minSize: 500 * 1024, //Byte, split point. Default: 30720
                maxSize: 1 * 1024 * 1024, //Byte, maxsize of per file. Default: 51200
                chunkOverhead: 0, //Default: 0
                entryChunkMultiplicator: 1, //Default: 1
            }),
            debug ? null :
            new webpack.BannerPlugin({
                banner: "file:[file], name:[name], filebase:[filebase], hash:[hash], chunkhash:[chunkhash], query:[query]", // the banner as string, it will be wrapped in a comment
                raw: false, // if true, banner will not be wrapped in a comment
                entryOnly: false, // if true, the banner will only be added to the entry chunks
                // test: string | RegExp | Array,
                // include: string | RegExp | Array,
                // exclude: string | RegExp | Array,
            }),
            // Note: this won't work without ExtractTextPlugin.extract(..) in `loaders`.
            new ExtractTextPlugin({
                filename: `css/[name]${debug?"":".min"}.css`,
            }),
            (debug || excludeHtml) ? null :
            // Generate a manifest file which contains a mapping of all asset filenames
            // to their corresponding output file so that tools can pick it up without
            // having to parse `index.html`.
            new ManifestPlugin({
                fileName: 'asset-manifest.json',
            }),
            // Moment.js is an extremely popular library that bundles large locale files
            // by default due to how Webpack interprets its code. This is a practical
            // solution that requires the user to opt into importing specific locales.
            // https://github.com/jmblog/how-to-optimize-momentjs-with-webpack
            // You can remove this if you don't use Moment.js:
            // new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),

            new WebpackMessagesPlugin()
        ].filter(Boolean),
        // Don't follow/bundle these modules, but request them at runtime from the environment
        externals: [],
        performance: {
            hints: "warning", // enum
            maxAssetSize: 200 * 1000, // int (in bytes),
            maxEntrypointSize: 400 * 1000, // int (in bytes)
            assetFilter: function(assetFilename) {
                // Function predicate that provides asset filenames
                return assetFilename.endsWith('.css') || assetFilename.endsWith('.js');
            }
        },
        // limit the number of parallel processed modules
        parallelism: 1, // number
        // capture timing information
        profile: true, // boolean
        // lets you precisely control what bundle information gets displayed
        // errors-only, minimal, none, normal, verbose
        stats: "errors-only",
        // Use this option to generate a JSON file containing webpack "records"
        recordsPath: paths.webpackStatsFile,
        // enables watching
        watch: false, // boolean
        // disable/enable caching
        cache: false, // boolean
        // Some libraries import Node modules but don't use them in the browser.
        // Tell Webpack to provide empty mocks for them so importing them works.
        node: {
            console: false, // boolean | "mock"
            global: true, // boolean | "mock"
            process: true, // boolean
            __filename: "mock", // boolean | "mock"
            __dirname: "mock", // boolean | "mock"
            Buffer: false, // boolean | "mock"
            setImmediate: true, // boolean | "mock" | "empty"
            dgram: 'empty',
            fs: 'empty',
            net: 'empty',
            tls: 'empty',
            child_process: 'empty',
        },
    };
}
