"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sensitiveField = exports.customFieldValueToText = void 0;
var tslib_1 = require("tslib");
var lodash_es_1 = require("lodash-es");
var numeral_1 = tslib_1.__importDefault(require("numeral"));
/**
 * 将自定字段的值转换为文字，例如多选字段'1,2'转换为'选项1,选项2'
 * @param {CustomField} field - 自定义字段
 * @param {string|string[]} value - 值
 */
function customFieldValueToText(field, value) {
    if (!value) {
        return '';
    }
    var type = field.content_type;
    var valueString = (0, lodash_es_1.isArray)(value) ? value.join(',') : value + '';
    var valueAry = (0, lodash_es_1.isArray)(value) ? value : (value + '').split(',');
    var options = field.select_options;
    switch (type) {
        case 'droplist':
        case 'multi_droplist':
        case 'checkbox':
        case 'radio':
            return options === null || options === void 0 ? void 0 : options.filter(function (i) { return valueAry.includes("".concat(i.value)); }).map(function (i) { return i.title; }).join(',');
        case 'chained_droplist': {
            if (!valueString) {
                return '';
            }
            var resultAry_1 = [];
            (0, lodash_es_1.reduce)(valueAry, function (preValue, item) {
                var _a;
                var option = (0, lodash_es_1.find)(preValue, ['value', item]);
                if (option) {
                    resultAry_1.push(option.title);
                }
                return (_a = option === null || option === void 0 ? void 0 : option.subs) !== null && _a !== void 0 ? _a : [];
            }, options);
            return resultAry_1.join('/');
        }
        case 'calculate':
            switch (field.calculate_result_type) {
                case 'duration':
                    return (0, lodash_es_1.isNil)(valueString) ? '' : (0, numeral_1.default)(valueString).format('00:00:00');
                case 'percentage':
                    return (0, lodash_es_1.isNil)(valueString) ? '' : (0, numeral_1.default)(valueString).format('0.00%');
                case 'numeric':
                    return (0, lodash_es_1.isNil)(valueString) ? '' : Number(valueString);
                default:
                    return valueString;
            }
        default:
            return valueString;
    }
}
exports.customFieldValueToText = customFieldValueToText;
/**
 * 计算字段脱敏后的结果
 * @example
 * sensitiveField({
 *     sensitive_mark_placeholder: '*',
 *     sensitive_mark_position: 3,
 *     sensitive_mark_size: 4,
 *     sensitive_mode: 2
 * })('18811112222');
 * // 结果：188****2222
 */
var sensitiveField = function (_a) {
    var name = _a.name, _b = _a.sensitive_mark_placeholder, sensitive_mark_placeholder = _b === void 0 ? '*' : _b, _c = _a.sensitive_mark_position, sensitive_mark_position = _c === void 0 ? 0 : _c, _d = _a.sensitive_mark_size, sensitive_mark_size = _d === void 0 ? 1 : _d, _e = _a.sensitive_mode, sensitive_mode = _e === void 0 ? 0 : _e;
    switch (name // 客户系统字段:客户名称|邮箱|电话  有单独的脱敏规则
    ) {
        case 'name':
            return function (v) {
                return v ? "".concat(v.substr(0, 1)).concat(sensitive_mark_placeholder.repeat(v.length - 1)) : '';
            };
        case 'email':
            return function (v) {
                if (!v) {
                    return '';
                }
                var dPosi = v.indexOf('@'), markLen = v.substring(0, dPosi).length;
                if (dPosi < 0)
                    return v;
                return "".concat(sensitive_mark_placeholder.repeat(markLen)).concat(v.substr(dPosi));
            };
        case 'cellphone':
            return function (v) {
                return v ? "".concat(v.substr(0, 3)).concat(sensitive_mark_placeholder.repeat(4)).concat(v.substr(7)) : '';
            };
    }
    if (!sensitive_mode) {
        //字段为0不脱敏
        return function (v) { return v; };
    }
    if (sensitive_mode === 1) {
        //全脱敏
        return function (v) { return (v ? sensitive_mark_placeholder.repeat(sensitive_mark_size) : ''); };
    }
    return function (v) {
        var _a, _b;
        //部分脱敏
        if (v) {
            return "".concat((_a = v === null || v === void 0 ? void 0 : v.substr(0, sensitive_mark_position)) !== null && _a !== void 0 ? _a : '').concat(sensitive_mark_placeholder.repeat(sensitive_mark_size)).concat((_b = v === null || v === void 0 ? void 0 : v.substr(sensitive_mark_position + sensitive_mark_size)) !== null && _b !== void 0 ? _b : '');
        }
        return '';
    };
};
exports.sensitiveField = sensitiveField;
