import { __assign, __rest, __spreadArray } from "tslib";
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
import { groupBy, memoize, orderBy } from "lodash";
import qs from "qs";
import { csApiGetJSON, csApiRequest } from "../../utils/csApi";
/**
 * 缓存分类目录，key名为 type + scope 知识库分类
 */
var KNOWLEDGE_CATALOG_CACHE = {};
/**
 * 缓存目录下的数据，key名为 type + scope 知识库内容，以id为key做map处理
 */
var KNOWLEDGE_CATALOG_LIST_CACHE = {};
/**
 * 缓存分类打开的节点名， key名为 type + scope 知识库分类打开目录的key
 */
var KNOWLEDGE_CATALOG_KEYS_CACHE = {};
var fetchKnowledge = memoize(function (requestOptions) { return csApiRequest(__assign({}, requestOptions)).then(function (res) { return res.data; }); }, function (requestOptions) {
    return "".concat(requestOptions.url).concat(requestOptions.method).concat(requestOptions.method === "get"
        ? qs.stringify(requestOptions.params)
        : requestOptions.data);
});
var fetchKnowledgeCatalogs = memoize(function (params) {
    return csApiGetJSON("/cs/ts/api/knowledge_catalogs", {
        params: params,
    }).then(function (res) { return res.data; });
}, function (params) { return params.knowledge_type_name + params.scope + params.query; });
/**
 *
 * @param requestOptions
 * @param params
 * @returns 封装接口，统一知识库搜索
 */
function searchKnowledge(requestOptions, params) {
    var res = requestOptions;
    if (res.method === "post") {
        res.data = qs.stringify(params);
    }
    else {
        res.params = params;
    }
    return fetchKnowledge(requestOptions);
}
/**
 *
 * @param type
 * @returns 配置请求参数
 */
function getRequestOptions(type) {
    var dataUrl = (function () {
        switch (type) {
            case "useful_expression":
                return "/cs/ts/api/useful_expressions/search";
            case "useful_link":
                return "/cs/ts/api/useful_links/search";
            case "question":
                return "/cs/ts/api/knowledge_questions/search";
            case "attachment":
                return "/spa1/knowledge_attachments";
        }
    })();
    var method = (function () {
        switch (type) {
            case "useful_expression":
            case "useful_link":
            case "question":
                return "post";
            case "attachment":
                return "get";
        }
    })();
    var res = {
        url: dataUrl,
        method: method,
    };
    if (method === "post") {
        res.headers = { "content-type": "application/x-www-form-urlencoded" };
    }
    return res;
}
/**
 * 拍平对象数组
 *
 * @param {*} data
 * @return {*}
 */
var flattenData = function (data) {
    return data.reduce(function (result, _a) {
        var _b = _a.children, children = _b === void 0 ? [] : _b, rest = __rest(_a, ["children"]);
        return __spreadArray(__spreadArray(__spreadArray([], result, true), [__assign({}, rest)], false), flattenData(children), true);
    }, []);
};
/**
 *
 * @param knowledgeType
 * @returns 输入框search搜索结果的hooks，提供searchList方法做搜索
 */
export var useSearchList = function (knowledgeType) {
    var _a = useState(false), listLoading = _a[0], setListLoading = _a[1];
    var _b = useState([]), listData = _b[0], setListData = _b[1];
    var pagingRef = useRef({ current_page: 0, total_pages: 0 });
    var requestOptions = getRequestOptions(knowledgeType);
    var previousParams = useRef();
    var searchList = useCallback(function (params) {
        var _a, _b;
        if (((_a = previousParams.current) === null || _a === void 0 ? void 0 : _a.query) !== params.query || ((_b = previousParams.current) === null || _b === void 0 ? void 0 : _b.scope) !== params.scope) {
            setListData([]);
        }
        previousParams.current = params;
        setListLoading(true);
        searchKnowledge(requestOptions, params)
            .then(function (listRes) {
            setListData(function (v) {
                return __spreadArray(__spreadArray([], v, true), getResKey(knowledgeType, listRes), true);
            });
            pagingRef.current = listRes.meta || {};
        })
            .finally(function () { return setListLoading(false); });
    }, [requestOptions]);
    return { listLoading: listLoading, listData: listData, pagingRef: pagingRef, searchList: searchList };
};
/**
 * 获取节点的祖先节点数组
 *
 * @param CatalogDataType
 * @param list
 *
 * @returns number[]
 */
function getParentIds(node, list) {
    var result = [];
    if ((node === null || node === void 0 ? void 0 : node.parent_id) != null) {
        result.push(node.parent_id);
        var parentNode = list.find(function (_node) { return _node.id === (node === null || node === void 0 ? void 0 : node.parent_id); });
        if (parentNode) {
            result = result.concat(getParentIds(parentNode, list));
        }
    }
    return result;
}
/**
 * 获取匹配到标签的分类ID，原方法迁移
 *
 * @param {ICatalog} list
 * @param {string[]} tagList
 * @param {number} num
 * @return {*}
 */
var tagsName = function (list, tagList, num) {
    var tag;
    list.children.forEach(function (items) {
        var _a, _b;
        if (items.name === tagList[num] && ((_a = items.children) === null || _a === void 0 ? void 0 : _a.length) && tagList[num + 1]) {
            num += 1;
            var tagItem = tagsName(items, tagList, num);
            if (tagItem) {
                tag = tagItem;
            }
        }
        else if ((items.name === tagList[num] && !((_b = items.children) === null || _b === void 0 ? void 0 : _b.length)) ||
            (items.name === tagList[num] && !tagList[num + 1])) {
            tag = items;
        }
    });
    return tag;
};
/**
 * 获取展开分类信息
 * @param list
 * @returns 通过分类目录计算当前已有的数据数量，并提供末级节点id，和所有节点id（给记录节点是否开启用）
 */
function addItemCount(list, matchTagList) {
    var count = 0;
    var itemIds = [];
    var itemKeys = [];
    var loopTree = function (tree) {
        (tree || []).forEach(function (i) {
            itemKeys.push(i.id);
            if (i.children && i.children.length) {
                loopTree(i.children);
            }
            else {
                itemIds.push({
                    item: i,
                    parentsIds: getParentIds(i, list),
                });
                count += i.subitem_count || 0;
            }
        });
    };
    if (matchTagList === null || matchTagList === void 0 ? void 0 : matchTagList.length) {
        // 如果传递了匹配的标签，则去分类中匹配
        var num_1 = 0;
        var flattenList_1 = flattenData(list);
        list.forEach(function (item) {
            var _a;
            if (item.name === matchTagList[num_1] &&
                matchTagList.length > 1 &&
                ((_a = item.children) === null || _a === void 0 ? void 0 : _a.length)) {
                num_1 += 1;
                var tag = tagsName(item, matchTagList, num_1);
                if (tag) {
                    var parentIds = getParentIds(tag, flattenList_1) || [];
                    itemKeys = __spreadArray([tag === null || tag === void 0 ? void 0 : tag.id], parentIds, true);
                    itemIds.push({
                        item: tag,
                        parentsIds: parentIds,
                    });
                }
            }
            else if (item.name === matchTagList[matchTagList.length - 1]) {
                itemKeys.push(item.id);
                itemIds.push({
                    item: item,
                    parentsIds: getParentIds(item, flattenList_1),
                });
            }
        });
        // 如果没有匹配则按原来规则展开
        if (!(itemKeys === null || itemKeys === void 0 ? void 0 : itemKeys.length)) {
            loopTree(list);
        }
    }
    else {
        // 如果没有传递匹配的标签，则遍历分类查找展开点
        loopTree(list);
    }
    return { count: count, itemIds: itemIds, itemKeys: itemKeys };
}
/**
 *
 * @param type
 * @returns 区分search接口返回的数据key名
 */
function getResKey(type, listRes) {
    var key = "";
    switch (type) {
        case "useful_expression":
            key = "expressions";
            break;
        case "useful_link":
            key = "links";
            break;
        case "question":
            key = "questions";
            break;
        case "attachment":
            key = "knowledge_attachments";
            break;
    }
    if (!key || !(listRes === null || listRes === void 0 ? void 0 : listRes[key])) {
        return [];
    }
    if (type === "attachment") {
        return listRes === null || listRes === void 0 ? void 0 : listRes[key].map(function (i) { return i === null || i === void 0 ? void 0 : i.knowledge_attachment; });
    }
    return listRes === null || listRes === void 0 ? void 0 : listRes[key];
}
/**
 *
 * @param knowledgeType
 * @param scope
 * @returns 知识库分类的树形组件hooks，提供缓存和初始化接口查询，判断一次请求是否超过1000条，是的话默认只请求前面几条并默认打开。
 * 并提供树形懒加载方法
 */
export var useCatalogs = function (knowledgeType, scope, stashKey, matchTagList) {
    var _a = useState(false), catalogLoading = _a[0], setCatalogLoading = _a[1];
    var _b = useState([]), catalogData = _b[0], setCatalogData = _b[1];
    var _c = useState({}), catalogList = _c[0], setCatalogList = _c[1];
    var _d = useState([]), expandedKeys = _d[0], setExpandedKeys = _d[1];
    var requestOptions = useMemo(function () {
        return getRequestOptions(knowledgeType);
    }, [knowledgeType]);
    var expandedKeysCacheKey = useMemo(function () {
        return stashKey ? "".concat(knowledgeType, "_").concat(scope, "_").concat(stashKey) : "".concat(knowledgeType, "_").concat(scope);
    }, [knowledgeType, scope, stashKey]);
    var cacheKey = useMemo(function () {
        return "".concat(knowledgeType, "_").concat(scope);
    }, [knowledgeType, scope]);
    var listParams = useCallback(function (id) {
        var p = {
            scope: scope,
            per_page: 1000,
        };
        if (knowledgeType === "attachment") {
            p.catalog_id = "all";
        }
        else if (knowledgeType === "question") {
            p.current_works = true;
        }
        if (id) {
            if (knowledgeType === "attachment") {
                p.catalog_id = id;
            }
            else {
                p.knowledge_catalog_id = id;
            }
        }
        return p;
    }, [knowledgeType, scope]);
    var setExpandedKeysFuc = useCallback(function (keys) {
        KNOWLEDGE_CATALOG_KEYS_CACHE[expandedKeysCacheKey] = keys || [];
        setExpandedKeys(keys);
    }, [expandedKeysCacheKey]);
    useEffect(function () {
        var cacheData = KNOWLEDGE_CATALOG_CACHE[cacheKey];
        if (cacheData) {
            setCatalogData(cacheData);
            setCatalogList(KNOWLEDGE_CATALOG_LIST_CACHE[cacheKey] || {});
            // 不存在匹配标签，则直接读取缓存
            if (KNOWLEDGE_CATALOG_KEYS_CACHE[expandedKeysCacheKey] && !(matchTagList === null || matchTagList === void 0 ? void 0 : matchTagList.length)) {
                setExpandedKeysFuc(KNOWLEDGE_CATALOG_KEYS_CACHE[expandedKeysCacheKey] || []);
            }
            else {
                var itemKeys = addItemCount(cacheData, matchTagList).itemKeys;
                setExpandedKeysFuc(itemKeys);
            }
        }
        else {
            setCatalogLoading(true);
            fetchKnowledgeCatalogs({
                knowledge_type_name: knowledgeType,
                scope: scope,
            })
                .then(function (res) {
                var _a, _b;
                if (res.code === 1000) {
                    var catalogResList_1 = {};
                    var catalogs = res.catalogs || [];
                    var _c = addItemCount(catalogs, matchTagList), count = _c.count, itemIds = _c.itemIds, itemKeys_1 = _c.itemKeys;
                    if (count < 1000 && knowledgeType !== "attachment") {
                        searchKnowledge(requestOptions, listParams("")).then(function (listRes) {
                            if (listRes.code === 1000) {
                                var resultList = getResKey(knowledgeType, listRes);
                                catalogResList_1 = groupBy(orderBy(resultList, ['custom_sort']), "knowledge_catalog_id");
                                KNOWLEDGE_CATALOG_LIST_CACHE[cacheKey] = catalogResList_1;
                                setCatalogList(catalogResList_1);
                                setExpandedKeysFuc(itemKeys_1);
                            }
                        });
                    }
                    else {
                        var reqList_1 = [];
                        // let expandedIds: number[] = [];
                        var reqCount = 0;
                        for (var i = 0; i < (itemIds === null || itemIds === void 0 ? void 0 : itemIds.length); i++) {
                            if (i > 5 || reqCount > 1000) {
                                break;
                            }
                            reqCount += ((_a = itemIds[i]) === null || _a === void 0 ? void 0 : _a.item.subitem_count) || 0;
                            reqList_1.push((_b = itemIds[i]) === null || _b === void 0 ? void 0 : _b.item.id);
                            // expandedIds = reqList.concat(itemIds[i]?.parentsIds || []);
                        }
                        Promise.all(reqList_1.map(function (id) { return searchKnowledge(requestOptions, listParams(id)); })).then(function (res) {
                            res.forEach(function (listRes, index) {
                                if (listRes.code === 1000) {
                                    catalogResList_1[reqList_1[index]] = getResKey(knowledgeType, listRes);
                                }
                            });
                            KNOWLEDGE_CATALOG_LIST_CACHE[cacheKey] = catalogResList_1;
                            setCatalogList(catalogResList_1);
                            setExpandedKeysFuc(itemKeys_1);
                        });
                    }
                    KNOWLEDGE_CATALOG_CACHE[cacheKey] = catalogs;
                    setCatalogData(res.catalogs || []);
                }
            })
                .finally(function () { return setCatalogLoading(false); });
        }
    }, [
        cacheKey,
        expandedKeysCacheKey,
        knowledgeType,
        listParams,
        matchTagList,
        requestOptions,
        scope,
        setExpandedKeysFuc,
    ]);
    var loadList = function (key) {
        return searchKnowledge(requestOptions, listParams(key)).then(function (listRes) {
            if (listRes.code === 1000) {
                var result_1 = getResKey(knowledgeType, listRes);
                setCatalogList(function (origin) {
                    var _a;
                    return __assign(__assign({}, origin), (_a = {}, _a[key] = result_1, _a));
                });
            }
        });
    };
    return {
        catalogLoading: catalogLoading,
        catalogData: catalogData,
        catalogList: catalogList,
        loadList: loadList,
        expandedKeys: expandedKeys,
        setExpandedKeys: setExpandedKeysFuc,
    };
};
