import React from 'react';
import type { IconFontProps, CustomIconOptions } from '@ant-design/icons/lib/components/IconFont';
interface FactoryBaseOptions {
    /**
     * 全局设置icon的前缀。如果设置了，渲染时会自动给icon的 `type` 前面添加前缀。
     *
     * 建议使用全局前缀，这样不需要在每个图标的 `type` 前面重复添加前缀 ，可读性更好。
     * 另外，一旦需要全局修改 icon 的前缀，也容易维护
     *
     * @type {string}
     */
    iconPrefix?: string;
    /**
     * 当使用 `font-class` 模式时（即设置`cssUrl`时），可以设置默认的 `font-family` css属性
     *
     * @default "iconfont"
     *
     * @type {string}
     * @memberof FactoryBaseOptions
     */
    fontFamily?: string;
    /**
     * 同时设置 Iconfont 组件的全局默认属性
     *
     * @type {UdeskIconFontProps}
     * @memberof FactoryBaseOptions
     */
    defaultProps?: UdeskIconFontProps;
}
interface ScriptFactoryOptions extends CustomIconOptions, FactoryBaseOptions {
    /**
     * iconfont项目的Symbol在线地址。获取方式：进入iconfont项目，点击`查看在线链接`，然后左侧类型选择`Symbol`
     *
     * @type {string}
     * @memberof ScriptFactoryOptions
     */
    scriptUrl: string | string[];
}
interface CssFactoryOptions extends Omit<CustomIconOptions, 'extraCommonProps'>, FactoryBaseOptions {
    /**
     * iconfont项目的FontClass在线地址。获取方式：进入iconfont项目，点击`查看在线链接`，然后左侧类型选择`Font class`
     *
     * @type {string}
     * @memberof CssFactoryOptions
     */
    cssUrl: string | string[];
}
/**
 * 字体尺寸，兼容AntDesign的 `SizeType` 类型
 *
 * @exports
 * @enum {string}
 */
export declare enum FontSize {
    /** 超小 */
    xSmall = "x-small",
    /** 小 */
    small = "small",
    /** 正常(默认值) */
    middle = "middle",
    /** 大 */
    large = "large",
    /** 超大 */
    xLarge = "x-large",
    /** 超大(二级) */
    xxLarge = "xx-large"
}
export declare type UdeskIconFontProps = Omit<IconFontProps, 'size'> & {
    /**
     * 字体尺寸
     *
     * @type {FontSize}
     */
    size?: FontSize;
    /**
     * @deprecated 已弃用，请使用 `size` 属性
     *
     * @type {('xl' | 'lg' | 'md' | 'sm' | 'xs')}
     */
    iconSize?: 'xl' | 'lg' | 'md' | 'sm' | 'xs';
};
declare type Iconfont = React.FC<UdeskIconFontProps> & {
    /**
     * FontSize枚举
     *
     */
    FontSize: {
        readonly [key in keyof typeof FontSize]: typeof FontSize[key];
    };
};
/**
 * 这是一个工厂方法，可以创建一个Iconfont组件，然后通过 `<Iconfont type='icon-example' />` 方式渲染图标。
 *
 * @param {ScriptFactoryOptions} options 创建Iconfont组件的options，必须设置`scriptUrl`
 * @returns Iconfont组件
 */
declare function createIconfont(options: ScriptFactoryOptions): Iconfont;
/**
 * 这是一个工厂方法，可以创建一个Iconfont组件，然后通过 `<Iconfont type='icon-example' />` 方式渲染图标。
 *
 * @param {CssFactoryOptions} options 创建Iconfont组件的options，必须设置`cssUrl`
 * @returns Iconfont组件
 */
declare function createIconfont(options: CssFactoryOptions): Iconfont;
export { createIconfont };
