import React, { ReactText } from 'react';
import type { AutoCompleteProps, DataSourceItemType } from 'antd/es/auto-complete';
import type { InternalSelectProps, RefSelectProps, BaseOptionType, DefaultOptionType } from 'antd/es/select';
import { U, HttpMethods } from '../_utils/type';
export * from 'antd/es/auto-complete';
/**
 * 破坏性变更：
 *      禁用dataSource属性；
 *      输出两种格式，value和对象。value模式输出的是value，而不是label，与AutoComplete不一样。
 *
 * 规范：
 *      - 保持主方法内的代码简洁清晰。
 *      - 适当加注释。
 *      - 把子函数提取到外面。
 *          1. props一律解构使用；
 *          2. 提取的子方法通过传参方式传递依赖
 *          3. 参数名在传递过程中命名保持不变，方便拷贝参数调用
 *          4. 在子方法调整参数时，先声明，等写完子方法后，再依次修改调用代码，vscode会提示
 *          5. 如果参数类型是可空类型的话，不要使用可选参数，而是使用联合类型，这样多传、或少传能被发现。
 *              例如：string | U，如果是函数类型的话，需要用括号表达式，例如 ((value: string) => void) | U
 *          6. 严格声明类型
 *          好处：
 *              代码脉络清晰；可以看到哪些代码是每次render都重复执行的，是否存在性能问题；
 *      - 方法只要不是void，一律显示声明返回类型，编译器会帮着检查是否所有分支都有返回值；
 *      - 避免直接声明匿名函数，要使用 useMemo 或 useCallback
 *      - 避免在 useEffect 中做大量的副作用，会造成依赖过多，重复执行过于频繁。要尽量把一个大的 useEffect
 *          拆分为多个 `细粒度、单一职责` 的小的 useEffect，这样隔离性和可控性更好
 *      - 过度依赖 useEffect 来监听并执行副作用，优先考虑在各种click事件、或各种action中，一次性把副作用做完，
 *          render 主方法尽量只是用来做渲染
 *
 * 优势
 *      - 更合理的组织代码，可以按照 逻辑相关性 把关联代码尽量组织在一起
 *      - 可以更细粒度地复用某些逻辑片段，而不是做成一个组件
 *      - 简单组件代码紧凑型更好，更容易维护
 *      - 开始代码更简单
 */
/** 输出的值的类型，可能是对象，value或者label字符串 */
declare type TValue = TOption | ReactText | U;
/** 用户可以传入的Options的数据结构，默认 {label,value}，也可以通过labelField和valueField来自定义 */
declare type TOption = {
    [K: string]: any;
    label?: string;
    value?: ReactText;
    options?: TOption[];
};
declare type AutoCompleteOption = BaseOptionType | DefaultOptionType;
/** 我们重写了这几个属性的数据类型，AutoComplete里只支持字符串，但我们要变成TValue */
declare type IncompatibleFields = 'value' | 'defaultValue' | 'onSelect' | 'onDeselect' | 'onChange';
declare type OverrideProps = Pick<InternalSelectProps<TValue>, IncompatibleFields>;
/** 除了几个不兼容的字段，其余的兼容字段 */
declare type CompatibleProps = Omit<AutoCompleteProps, keyof OverrideProps>;
/** AutoComplete 组件输入/出的值类型 */
declare type OutputMode = 'value' | 'labelInValue' | 'string';
/**
 * UdeskAutoComplete 组件的Props
 *
 * @interface UdeskAutoCompleteProps
 * @extends {CompatibleProps}
 * @extends {OverrideProps}
 */
interface UdeskAutoCompleteProps extends CompatibleProps, OverrideProps {
    /** @deprecated Please use `options` instead. */
    dataSource?: DataSourceItemType[];
    /** 重写基类的onChange签名，基类只输出了 string 这种格式，不是 TValue 类型 */
    onChange?(value: TValue, option: AutoCompleteOption): void;
    /**
     * 组件输出值的类型，也适用于 `value` 和 `defaultValue` 属性
     * - 如果类型是 `labelInValue`，则会输出与 `options` 相同类型的对象。
     * 当无法获取全量 options，（即只能通过自动补全的方式搜索options时），推荐使用这种模式。
     *
     * - 如果类型是 `value`，则会输出选中 option 对象的 `value`
     * (或者使用valueField自定义字段名) 字段的值。
     * 当可以获取全量 `options` 时，推荐这种模式。
     *
     * - 如果类型是 `string`，则会把文本框中的字符串原样输出，不做任何转换。
     * 这是Ant Design AutoComplete组件的默认工作模式。当你需要的仅仅是自动补全文本的功能时，推荐使用这种模式。
     *
     * @default "value"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    outputMode?: OutputMode;
    /**
     * 是否允许输出空值（仅当 `outputMode` 是 `value` 或 `labelInValue` 模式时，才起作用）。
     *
     * 如果清空了输入框中内容，则当失去焦点时：
     * 如果设置为 `true`，则会输出的值为 `undefined`；
     * 如果设置为 `false`，则会自动恢复成上一个`有效`的值（即从 options 中匹配的值）
     *
     * @type {boolean}
     * @memberof UdeskAutoCompleteProps
     */
    allowEmpty?: boolean;
    /**
     * 自动补全下拉项的 url 或自定义函数，仅当没有设置 `options` 时起作用。
     *
     * 当用户输入了某个文本，或者输入框获取焦点时，组件会自动调用此方法（或请求此url）
     * 自动查询下拉项的数据源，请求时会把当前 `输入的文本` 作为参数传递过去，
     * 一般情况下是向后端发送一个请求，后端返回该文本 `前字匹配` 的下拉项，
     * 更多详细请求参数与返回数据结构，请参考 @interface SearchMethod 的文档。
     *
     * 支持传入 `url` 和 自定义函数。
     * - 如果传入 `url`，则会自动向此 url 发送请求，
     * 可以通过 `searchHttpMethod` 属性定义请求的 method，
     * 通过 `queryParamName` 属性定义发送请求时，查询文本的参数名
     * - 如果传入 自定义函数，请参考 @interface SearchMethod 的文档进行自定义查询，
     * 同时支持同步和异步方式返回数据。
     *
     * 不管哪种方式发送的请求，都期望返回的数据结构是一个 `options` 数组。
     * 如果返回的数据结构不满足上述要求时，可以通过 `getResponseData` 方法进行自定义转换，
     * 返回 `options` 数组。
     *
     * 如果 `search` 是一个异步的自定义方法，如果使用 `Promise.catch` 或 `Promise.then`的第二个参数
     * 处理了请求错误，必须最后再把 reason `抛出来`！否则异常会被吃掉，无法通知组件内部请求失败了！！
     * 组件内部会默认显示一个错误提示，如果不想要显示这个错误提示，请设置 `reason.hideErrorMessage = true`，
     * 可以隐藏掉。
     *
     * @default undefined
     *
     * @type {(string | SearchMethod)}
     * @memberof UdeskAutoCompleteProps
     */
    search?: string | SearchMethod;
    /**
     * 如果 `search` 属性设置为 url，则这个属性可以设置 http 请求的 method，
     * 值必须为 @enum HttpMethods 枚举的值之一。
     *
     * @default "GET"
     *
     * @type {HttpMethods}
     * @memberof UdeskAutoCompleteProps
     */
    searchHttpMethod?: HttpMethods;
    /**
     * 当获取焦点自动发请求时，是否使用空搜索。
     *
     * 设置成 `true` 的理由是，当用户点击组件时，如果自动用当前的 value 来查询，
     * 则很可能只有一个搜索结果，即下拉框只有一项。此时用户可能会误以为只有一个下拉项可选择，
     * 用户要选择其他值的话，必须先把输入框清空才能发起空搜索，看到其它值。
     *
     * 设置成 `false` 的理由是，组件明明有值，可搜索出来的下拉项与当前值一点关系都没有，
     * 让用户感觉有点迷惑，误以为组件有bug。
     *
     * 这两种情况都有道理，请根据自己的实际需要选择，默认为使用空搜索。
     *
     * @default true
     *
     * @type {boolean}
     * @memberof UdeskAutoCompleteProps
     */
    emptySearchOnFocus?: boolean;
    /**
     * 如果 `search` 属性设置为 url，则这个属性可以设置 http 请求的 `查询文本` 的参数名
     *
     * 例如，如果url为 http://example.com/search，并且用户输入了文本 `abc`，则发送请求为
     * http://example.com/search?query=abc
     *
     * @default 'keyword'
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    queryParamName?: string;
    /**
     * 在发送 `search` 请求时，可以设置 `pageNum` 参数的 url参数名
     *
     * @default "pageNum"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    pageNumParamName?: string;
    /**
     * 在发送 `search` 请求时，可以设置 `pageSize` 参数的 url参数名
     *
     * @default "pageSize"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    pageSizeParamName?: string;
    /**
     * 设置每次分页的默认条数，此属性可以控制每次自动补全时一次性返回的记录条数，以及滚动加载时每页加载的条数。
     *
     * @default 50
     *
     * @type {number}
     * @memberof UdeskAutoCompleteProps
     */
    pageSize?: number;
    /**
     * 默认会对 search 请求进行 `防抖` 处理，这个属性控制防抖的延迟时间（毫秒）。
     *
     * @default 500
     *
     * @type {number}
     * @memberof UdeskAutoCompleteProps
     */
    searchDebounce?: number;
    /**
     * 设置 `options` 数据结构的 `value` 字段名。
     * 默认情况下，`options` 期望接受 `{label,value}`这样的数据结构。但如果你的业务数据不一样的话，
     * 你可以自定义 `value` 和 `label`  字段的字段名，以支持类似 `{id,name}` 这样的数据结构。
     *
     * 如果 `outputMode` 设置为 `value` 的话，当从选中的 option 中提取 `value`时，也会使用到此属性。
     *
     * @default "value"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    valueField?: string;
    /**
     * 设置 `options` 数据结构的 `label` 字段名。
     * 默认情况下，`options` 期望接受 `{label,value}`这样的数据结构。但如果你的业务数据不一样的话，
     * 你可以自定义 `value` 和 `label`  字段的字段名，以支持类似 `{id,name}` 这样的数据结构。
     *
     * @default "label"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    labelField?: string;
    /**
     * AutoComplete 的 `options` 支持 OptionGroup，即对options进行分组。
     * 默认情况下，子options会放在 `options` 字段下，使用此属性可以改变默认字段名。
     *
     * @default "options"
     *
     * @type {string}
     * @memberof UdeskAutoCompleteProps
     */
    childOptionsField?: string;
    /**
     * 当输入 `value` 是对象时，优先获取 `label` 字段作为回显内容，否则使用 `value` 字段。
     *
     * Ant design 默认行为是用 value 来回显，但实际情况中，value字段通常是id，用label来回显更适合大多数情况
     *
     * @default true
     *
     * @type {boolean}
     * @memberof UdeskAutoCompleteProps
     */
    labelPreferred?: boolean;
    /**
     * 如果 search 请求返回的数据结构是异构的，可以通过此方法从 response 中提取 options 数据并返回。
     *
     * @summary 注意：`search` 与 `resolveLabel` 两个请求公用此方法。所以如果用来处理 `search` 方法的请求，
     * 请返回 @type {TOptions[]} ，如果用来处理 `resolveLabel` 方法的请求，请返回 @type {string}。
     *
     * @default
     * (resp) => resp?.data || resp
     *
     * @memberof UdeskAutoCompleteProps
     */
    getResponseData?: ParseResponseData;
    /**
     * 根据 `value` 获取对应的 `label` 字符串，并回显在AutoComplete组件中。
     * 支持同步或异步方法。
     *
     * @summary
     * 当我们从 AutoComplete 列表选择一个值时，通常我们是需要保存 `value` 值，但回显的时候却希望显示 `label`，
     * 只能在初始化组件时发送一个请求单独查询 `label。`
     *
     * 如果不希望有额外的请求，需要把 `outputMode` 属性设置为 `labelInValue`，然后把组件的 `value`或`defaultValue`
     * 设置为一个 {label, value} 这样的对象即可。
     *
     * @default undefined
     *
     * @type {ResolveLabelMethod}
     * @memberof UdeskAutoCompleteProps
     */
    resolveLabel?: ResolveLabelMethod;
    /**
     * 是否启动滚动加载。如果开启，则当下拉项滚动到底部时，会自动加载更多数据。
     *
     * @default true
     *
     * @type {boolean}
     * @memberof UdeskAutoCompleteProps
     */
    infiniteScrolling?: boolean;
    /**
     * 设置一个滚动到底之前的偏移高度，也就是说，在距离底部还有这个高度时，便进行预加载数据，
     * 用户的滚动体验更平滑。
     *
     * @default listItemHeight*5 或 160
     *
     * @type {number}
     * @memberof UdeskAutoCompleteProps
     */
    scrollToEndOffset: number;
    /**
     * 无限滚动加载时，设置条数的上限值，超过此条数后，自动停止滚动加载更多
     *
     * @default 2000
     *
     * @type {number}
     * @memberof UdeskAutoCompleteProps
     */
    infiniteScrollingMaxNum?: number;
}
/**
 * `AutoComplete` 组件的`api`对象，可以调用组件实例的一些公开方法
 *
 * @interface AutoCompleteApi
 */
interface AutoCompleteApi {
}
export { UdeskAutoCompleteProps, AutoCompleteApi, OutputMode, TValue, TOption, AutoCompleteOption, SearchMethod, SearchOptions, ParseResponseData, ResolveLabelMethod, };
export * from '../_utils/type';
declare const _default: React.ForwardRefExoticComponent<UdeskAutoCompleteProps> & {
    AntdAutoComplete: (<ValueType = any, OptionType extends BaseOptionType | DefaultOptionType = DefaultOptionType>(props: import("antd").AutoCompleteProps<ValueType, OptionType> & {
        children?: React.ReactNode;
    } & {
        ref?: React.Ref<RefSelectProps> | undefined;
    }) => React.ReactElement<any, string | React.JSXElementConstructor<any>>) & {
        Option: import("rc-select/lib/Option").OptionFC;
    };
} & Omit<(<ValueType = any, OptionType extends BaseOptionType | DefaultOptionType = DefaultOptionType>(props: import("antd").AutoCompleteProps<ValueType, OptionType> & {
    children?: React.ReactNode;
} & {
    ref?: React.Ref<RefSelectProps> | undefined;
}) => React.ReactElement<any, string | React.JSXElementConstructor<any>>) & {
    Option: import("rc-select/lib/Option").OptionFC;
}, keyof React.ForwardRefExoticComponent<AutoCompleteProps<any, DefaultOptionType>>>;
export default _default;
/**
 * AutoComplete组件的自定义search方法，根据输入的搜索词异步返回搜索结果。
 *
 * 注意 options 中的 `abortSignal` 属性，在发请求时要传递给 `signal` 参数，这样在 AutoComplete
 * 组件可以在调用 search 方法时主动 abort 掉前面未完成的请求。
 *
 * 如果在方法内部用 `Promise.catch` 或 `Promise.then`的第二个参数处理了请求错误，
 * 最后必须再把 reason `抛出来`！否则异常会被吃掉，无法通知组件内部请求失败了！！
 * 组件内部会默认显示一个错误提示，如果不想要显示这个错误提示，请设置 `reason.hideErrorMessage = true`，
 * 可以隐藏掉。
 *
 * @param {string} query 用户输入的搜索词
 * @param {SearchOptions} options 调用search方法时，传递一些内部的搜索选项
 * @returns {(TOption[] | Promise<TOption[]>)}
 *
 * @interface SearchMethod
 */
interface SearchMethod {
    /**
     * @param {string} query 用户输入的搜索词
     * @param {SearchOptions} options 调用search方法时，传递一些内部的搜索选项
     * @returns {(TOption[] | Promise<TOption[]>)}
     *
     */
    (query: string, options: SearchOptions): TOption[] | Promise<TOption[]>;
}
/**
 * 自定义search方法的可选参数
 *
 * @interface SearchOptions
 */
interface SearchOptions {
    /**
     * 在滚动加载时，指定要加载的页码。分页永远是从 `DefaultPage` 开始的，不为空。
     *
     * @type {number}
     * @memberof SearchOptions
     */
    pageNum: number;
    /**
     * 在实时搜索时，指定一次性返回的记录条数。在滚动加载时，也用来控制一次分页读取的记录条数。
     *
     * @type {number}
     * @memberof SearchOptions
     */
    pageSize: number;
    /**
     * 当前已经加载完的总条数
     *
     * @type {number}
     * @memberof SearchOptions
     */
    total: number;
    /**
     * Abort 当前 search 请求的对象，推荐使用 umi-request 来发送请求，
     * 会接受 signal 参数，可以把当前对象传递过去。
     *
     * 当搜索补全项时，如果上个请求还没有完成，会把上个请求取消掉。
     *
     * @type {AbortSignal}
     * @memberof SearchOptions
     */
    abortSignal: AbortSignal;
}
/**
 * 从 Response 中提取 data 数据。通常情况下，后端返回的 response 会有一个标准的数据结构，
 * 真正的数据保存在某个字段下，通常是 `data` 字段。
 *
 * @param {*} response 服务端返回的 response 对象
 * @returns {(TOption[] | string)}
 *
 * @interface ParseResponseData
 */
interface ParseResponseData {
    /**
     * @param {*} response 服务端返回的 response 对象
     * @returns {(TOption[] | string)}
     */
    (response: any): TOption[] | string;
}
/**
 * 根据 `value` 获取对应的 `label` 字符串，并回显在AutoComplete组件中。
 * 支持同步或异步方法。
 *
 * @param {TValue} value 传给 AutoComplete 的 `value`或`defaultValue`
 * @returns {(string | Promise<string>)}
 *
 * @interface ResolveLabelMethod
 */
interface ResolveLabelMethod {
    /**
     * @param {TValue} value 传给 AutoComplete 的 `value`或`defaultValue`
     * @returns {(string | Promise<string>)}
     */
    (value: TValue): string | Promise<string>;
}
