"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash_isequal_1 = tslib_1.__importDefault(require("lodash.isequal"));
const iframe_protocol_1 = require("./iframe-protocol");
const helper_1 = require("./helper");
const BUNDLER_URL = process.env.CODESANDBOX_ENV === 'development'
    ? 'http://localhost:3002'
    : `https://sandpack-0-0-66.codesandbox.io`;
class PreviewManager {
    constructor(selector, sandboxInfo, options = {}) {
        this.id = null;
        this.options = options;
        this.sandboxInfo = sandboxInfo;
        this.bundlerURL = options.bundlerURL || BUNDLER_URL;
        this.externalResources = options.externalResources || [];
        this.managerState = undefined;
        this.errors = [];
        this.status = 'initializing';
        if (typeof selector === 'string') {
            this.selector = selector;
            const element = document.querySelector(selector);
            if (!element) {
                throw new Error(`No element found for selector '${selector}'`);
            }
            this.element = element;
            this.iframe = document.createElement('iframe');
            this.initializeElement();
        }
        else {
            this.element = selector;
            this.iframe = selector;
        }
        // this.iframe.setAttribute(
        //   'sandbox',
        //   'allow-forms allow-modals allow-popups allow-presentation allow-same-origin allow-scripts',
        // );
        this.iframeProtocol = new iframe_protocol_1.IFrameProtocol(this.iframe, this.bundlerURL);
        this.unsubscribeGlobalListener = this.iframeProtocol.globalListen((mes) => {
            if (mes.type !== 'initialized' || !this.iframe.contentWindow) {
                return;
            }
            this.iframeProtocol.register();
            this.updatePreview();
        });
        this.unsubscribeChannelListener = this.iframeProtocol.channelListen((mes) => {
            switch (mes.type) {
                case 'start': {
                    this.errors = [];
                    break;
                }
                case 'status': {
                    this.status = mes.status;
                    break;
                }
                case 'action': {
                    if (mes.action === 'show-error') {
                        const { title, path, message, line, column } = mes;
                        this.errors = [...this.errors, { title, path, message, line, column }];
                    }
                    break;
                }
                case 'state': {
                    this.managerState = mes.state;
                    break;
                }
                default:
                    break;
            }
        });
    }
    updateOptions(options) {
        if (!(0, lodash_isequal_1.default)(this.options, options)) {
            this.options = options;
            this.updatePreview();
        }
    }
    updatePreview(sandboxInfo = this.sandboxInfo) {
        this.sandboxInfo = sandboxInfo;
        const files = this.getFiles();
        const modules = Object.keys(files).reduce((prev, next) => (Object.assign(Object.assign({}, prev), { [next]: {
                code: files[next].code,
                path: next,
            } })), {});
        // 将构建所需要的数据例如源代码等，传递给 iframe，
        // iframe 中加载的打包逻辑监听到 compile 类型消息后，
        // 开始基于传递过来的数据进行构建
        this.dispatch({
            type: 'compile',
            codesandbox: true,
            version: 3,
            modules,
            clearConsoleDisabled: true,
            externalResources: this.externalResources,
            hasFileResolver: false,
            disableDependencyPreprocessing: this.sandboxInfo.disableDependencyPreprocessing,
            template: this.sandboxInfo.template,
            showOpenInCodeSandbox: this.sandboxInfo.showOpenInCodeSandbox == null
                ? true
                : this.sandboxInfo.showOpenInCodeSandbox,
            showErrorScreen: true,
            showLoadingScreen: true,
            skipEval: this.options.skipEval || false,
        });
    }
    cleanup() {
        this.unsubscribeChannelListener();
        this.unsubscribeGlobalListener();
        this.iframeProtocol.cleanup();
    }
    dispatch(message) {
        this.iframeProtocol.dispatch(message);
    }
    getFiles() {
        const { sandboxInfo } = this;
        if (sandboxInfo.files['/package.json'] === undefined) {
            return (0, helper_1.createMissingPackageJSON)(sandboxInfo.files, sandboxInfo.dependencies, sandboxInfo.entry);
        }
        return this.sandboxInfo.files;
    }
    initializeElement() {
        this.iframe.style.border = '0';
        this.iframe.style.width = this.options.width || '100%';
        this.iframe.style.height = this.options.height || '100%';
        this.iframe.style.overflow = 'hidden';
        if (!this.element.parentNode) {
            // This should never happen
            throw new Error('Given element does not have a parent.');
        }
        // 用 iframe 替换 element
        this.element.parentNode.replaceChild(this.iframe, this.element);
    }
}
exports.default = PreviewManager;
