"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeJSXElement = exports.makeJSXAttributeByCode = exports.makeJSXAttributes = exports.makeJSXAttribute = exports.value2jsxChildrenValueNode = exports.value2jsxAttributeValueNode = exports.code2jsxAttributeValueNode = exports.object2node = exports.value2node = exports.expressionCode2ast = exports.code2expression = exports.code2ast = exports.isValidExpressionCode = exports.isValidCode = void 0;
const tslib_1 = require("tslib");
/**
 * code to ast
 */
const parser_1 = require("@babel/parser");
const t = tslib_1.__importStar(require("@babel/types"));
const tango_helpers_1 = require("@music163/tango-helpers");
// @see https://babeljs.io/docs/en/babel-parser#pluginss
const babelParserConfig = {
    sourceType: 'module',
    plugins: [
        'jsx',
        'doExpressions',
        'objectRestSpread',
        'decorators-legacy',
        'classProperties',
        'asyncGenerators',
        'functionBind',
        'dynamicImport',
        'optionalChaining',
    ],
};
/**
 * 检测代码是否是合法的代码
 * @param code
 * @returns true 为合法代码，false 为非法代码
 */
function isValidCode(code) {
    try {
        (0, parser_1.parse)(code, babelParserConfig);
    }
    catch (err) {
        return false;
    }
    return true;
}
exports.isValidCode = isValidCode;
/**
 * 检测代码是否是合法的表达式代码
 * 表达式是一组代码的集合，它返回一个值；每一个合法的表达式都能计算成某个值
 * 只要你输入这段代码，可以形成一个值，就算是表达式
 * @example x = 1; // 1
 * @example 1=1
 * @example 'hello'
 * @example { foo: 'bar' }
 * @param code
 * @returns
 */
function isValidExpressionCode(code) {
    try {
        (0, parser_1.parseExpression)(code, babelParserConfig);
    }
    catch (err) {
        return false;
    }
    return true;
}
exports.isValidExpressionCode = isValidExpressionCode;
/**
 * 将源代码解析为一棵完整的 ast 树 t.File
 * @param code
 * @returns
 */
function code2ast(code) {
    try {
        return (0, parser_1.parse)(code, babelParserConfig);
    }
    catch (err) {
        tango_helpers_1.logger.error('[code2ast failed!]', err);
    }
}
exports.code2ast = code2ast;
/**
 * 将代码片段解析为 ast 节点
 * @example <Button>hello</Button>
 * @example { foo: 'foo' }
 * @example [{ foo: 'bar' }]
 * @example () => {}
 * @param code 输入字符串
 * @returns
 */
function code2expression(code) {
    if (!code) {
        return;
    }
    if (code.endsWith(';')) {
        code = code.slice(0, -1);
    }
    let expNode;
    try {
        expNode = t.cloneNode((0, parser_1.parseExpression)(code, babelParserConfig), false, true);
    }
    catch (err) {
        tango_helpers_1.logger.error('invalid code', err);
        // expNode = t.identifier('undefined');
    }
    return expNode;
}
exports.code2expression = code2expression;
/**
 * 表达式代码片段转为 ast 树
 * @param code
 * @returns File
 */
function expressionCode2ast(code) {
    const node = code2expression(code);
    return t.file(t.program([t.blockStatement([t.expressionStatement(node)])]));
}
exports.expressionCode2ast = expressionCode2ast;
/**
 * 将 js 值解析为 t.Node
 * @param value
 * @returns
 */
function value2node(value) {
    let ret;
    switch (typeof value) {
        case 'string':
            if ((0, tango_helpers_1.isWrappedCode)(value)) {
                // 再检查是否是代码 {{code}}，例如 {{this.foo}}, {{1}}
                const innerCode = (0, tango_helpers_1.getCodeOfWrappedCode)(value);
                ret = code2expression(innerCode);
            }
            else {
                // 否则当成字符串处理
                ret = t.stringLiteral(value);
            }
            break;
        case 'number':
            ret = t.numericLiteral(value);
            break;
        case 'boolean':
            ret = t.booleanLiteral(value);
            break;
        case 'function':
            ret = code2expression(String(value));
            break;
        case 'object': {
            if (value === null) {
                ret = t.nullLiteral();
            }
            else if ((0, tango_helpers_1.isPlainObject)(value)) {
                ret = object2node(value);
            }
            else if (Array.isArray(value)) {
                ret = t.arrayExpression(value.map((val) => value2node(val)));
            }
            else {
                ret = t.identifier('undefined');
                tango_helpers_1.logger.error('value2node: not support value!', ret);
            }
            break;
        }
        case 'undefined':
            ret = t.identifier('undefined');
            break;
        default: {
            tango_helpers_1.logger.error(`value2node: value <${value}> transform failed!`);
            break;
        }
    }
    return ret;
}
exports.value2node = value2node;
/**
 * 将 js 普通对象解析为 t.Node
 */
function object2node(obj, getValueNode = value2node) {
    if (!(0, tango_helpers_1.isPlainObject)(obj)) {
        return value2node(obj);
    }
    return t.objectExpression(Object.keys(obj).map((key) => {
        const valNode = getValueNode(obj[key], key);
        return t.objectProperty(t.identifier(key), valNode);
    }));
}
exports.object2node = object2node;
function code2jsxAttributeValueNode(code) {
    return t.jsxExpressionContainer(code2expression(code));
}
exports.code2jsxAttributeValueNode = code2jsxAttributeValueNode;
// FIXME: 统一处理为 code2jsxAttributeValueNode
function value2jsxAttributeValueNode(value) {
    let ret;
    switch (typeof value) {
        case 'string': {
            if (value.length > 1) {
                value = value.trim();
            }
            if ((0, tango_helpers_1.isWrappedCode)(value)) {
                const innerCode = (0, tango_helpers_1.getCodeOfWrappedCode)(value);
                ret = t.jsxExpressionContainer(code2expression(innerCode));
            }
            else {
                ret = t.stringLiteral(value);
            }
            break;
        }
        default:
            ret = t.jsxExpressionContainer(value2node(value));
            break;
    }
    return ret;
}
exports.value2jsxAttributeValueNode = value2jsxAttributeValueNode;
function value2jsxChildrenValueNode(value) {
    let ret;
    switch (typeof value) {
        case 'string':
            if (isValidExpressionCode(value)) {
                const innerString = (0, tango_helpers_1.getVariableContent)(value);
                ret = t.jsxExpressionContainer(code2expression(innerString));
            }
            else {
                ret = t.jsxText(value);
            }
            break;
        case 'number':
            ret = t.jsxText(String(value));
            break;
        case 'object':
            // value 为 JSXElement[]的情况下直接return
            return value;
        default:
            break;
    }
    return ret ? [ret] : [];
}
exports.value2jsxChildrenValueNode = value2jsxChildrenValueNode;
/**
 * 给定具体的 value 值，生成 JSXAttribute
 * @param name 属性名
 * @param value 属性值代码
 * @returns
 */
function makeJSXAttribute(name, value) {
    return t.jsxAttribute(t.jsxIdentifier(name), value2jsxAttributeValueNode(value));
}
exports.makeJSXAttribute = makeJSXAttribute;
function makeJSXAttributes(props) {
    return Object.keys(props).map((key) => makeJSXAttribute(key, props[key]));
}
exports.makeJSXAttributes = makeJSXAttributes;
/**
 * 给定具体的 value 代码，生成 JSXAttribute
 * @param name 属性名
 * @param valueCode 属性值代码
 * @returns
 */
function makeJSXAttributeByCode(name, valueCode) {
    return t.jsxAttribute(t.jsxIdentifier(name), code2jsxAttributeValueNode(valueCode));
}
exports.makeJSXAttributeByCode = makeJSXAttributeByCode;
/**
 * 生成 JSXElement
 * @param name
 * @param attributes
 * @param children
 * @param selfClosing
 * @returns
 */
function makeJSXElement(name, attributes, children, selfClosing) {
    return t.jsxElement(t.jsxOpeningElement(t.jsxIdentifier(name), attributes), t.jsxClosingElement(t.jsxIdentifier(name)), children !== null && children !== void 0 ? children : [], selfClosing);
}
exports.makeJSXElement = makeJSXElement;
