"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFilepath = exports.getRelativePath = exports.getFullPath = exports.getBlockNameByFilename = exports.isPathnameMatchRoute = exports.formatCode = exports.getPrivilegeCode = exports.getFilepath = exports.getModuleNameByFilename = exports.isValidComponentName = exports.inferFileType = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const tango_helpers_1 = require("@music163/tango-helpers");
const types_1 = require("./../types");
/**
 * 推断JS模块类型
 */
function inferFileType(filename) {
    // 增加 tangoConfigJson Module
    if (/\/tango\.config\.json$/.test(filename)) {
        return types_1.FileType.TangoConfigJson;
    }
    if (/\/appJson\.json$/.test(filename)) {
        return types_1.FileType.AppJson;
    }
    if (/\/package\.json$/.test(filename)) {
        return types_1.FileType.PackageJson;
    }
    if (/\/routes\.js$/.test(filename)) {
        return types_1.FileType.RouteModule;
    }
    // 所有 pages 下的 js 文件均认为是有效的 viewModule
    if (/\/pages\/.+\.jsx?$/.test(filename)) {
        return types_1.FileType.JsxViewModule;
    }
    // 所有 pages 下的 js 文件均认为是有效的 viewModule
    if (/\/pages\/.+\.schema\.json?$/.test(filename)) {
        return types_1.FileType.JsonViewModule;
    }
    if (/\/(blocks|components)\/index\.js/.test(filename)) {
        return types_1.FileType.ComponentsEntryModule;
    }
    if (/\/services\/.+\.js$/.test(filename)) {
        return types_1.FileType.ServiceModule;
    }
    if (/service\.js$/.test(filename)) {
        return types_1.FileType.ServiceModule;
    }
    if (/\/stores\/index\.js$/.test(filename)) {
        return types_1.FileType.StoreEntryModule;
    }
    if (/\/stores\/.+\.js$/.test(filename)) {
        return types_1.FileType.StoreModule;
    }
    if (/\.jsx?$/.test(filename)) {
        return types_1.FileType.Module;
    }
    if (/\.json$/.test(filename)) {
        return types_1.FileType.Json;
    }
    if (/\.less$/.test(filename)) {
        return types_1.FileType.Less;
    }
    if (/\.scss$/.test(filename)) {
        return types_1.FileType.Scss;
    }
    return types_1.FileType.File;
}
exports.inferFileType = inferFileType;
/**
 * 判断组件名是否合法
 * @example Button -> valid
 * @example isStrict ? div -> invalid : div -> valid
 * @param name
 * @param isStrict 是否严格模式（严格模式不匹配原生标签）
 * @returns
 */
function isValidComponentName(name, isStrict = false) {
    if (!name) {
        return false;
    }
    if (!isStrict) {
        return true;
    }
    const firstChar = name.charAt(0);
    return firstChar === firstChar.toUpperCase();
}
exports.isValidComponentName = isValidComponentName;
/**
 * 从 filename 中解析获得 moduleName
 * @example /stores/user.js -> user
 * @example /services/foo-bar.js -> fooBar
 * @param filename
 */
function getModuleNameByFilename(filename) {
    const parts = filename.split('/');
    let name = parts[parts.length - 1];
    name = name.split('.')[0];
    return (0, tango_helpers_1.camelCase)(name);
}
exports.getModuleNameByFilename = getModuleNameByFilename;
/**
 * 基于路由名生成文件路径
 * @param routePath 路由地址
 * @param baseDir base dir
 * @param ext 后缀名
 * @returns
 */
function getFilepath(routePath, baseDir, ext = '') {
    if (routePath.startsWith('/')) {
        routePath = routePath.substring(1);
    }
    const filename = routePath.replaceAll('/:', '@').split('/').join('-');
    if (!baseDir.endsWith('/')) {
        baseDir = `${baseDir}/`;
    }
    return `${baseDir}${filename}${ext}`;
}
exports.getFilepath = getFilepath;
function getPrivilegeCode(appName, routePath) {
    return `${appName}@${routePath.replaceAll('/', '%')}`;
}
exports.getPrivilegeCode = getPrivilegeCode;
const prettier = window.prettier;
const prettierPlugins = window.prettierPlugins;
/**
 * 格式化代码
 * @param code original source code
 * @param parser prettier parser, see https://prettier.io/docs/en/options.html#parser
 * @returns the formatted code
 */
function formatCode(code, parser = 'babel') {
    if (prettier && prettierPlugins) {
        return prettier.format(code, { parser, plugins: prettierPlugins });
    }
    return code;
}
exports.formatCode = formatCode;
/**
 * 是否匹配路由
 * @example isPathnameMatchRoute('/user/123', '/users/:id') -> true
 */
function isPathnameMatchRoute(pathname, route) {
    if (!pathname) {
        return false;
    }
    pathname = pathname.split('?')[0];
    if (pathname === route) {
        return true;
    }
    const str = route.replaceAll(/:\w+/gi, '\\w+');
    const pt = new RegExp(`^${str}$`, 'i');
    return pt.test(pathname);
}
exports.isPathnameMatchRoute = isPathnameMatchRoute;
/**
 * 根据文件路径计算区块的名字
 * @param filename 文件路径
 * @return 返回计算后的区块名（大驼峰）
 */
function getBlockNameByFilename(filename) {
    const name = filename.split('/').slice(-2, -1)[0];
    return (0, tango_helpers_1.upperCamelCase)(name);
}
exports.getBlockNameByFilename = getBlockNameByFilename;
/**
 * 合并两个路径
 * @param root
 * @param filename
 * @returns
 */
function getFullPath(root, filename) {
    return path_1.default.join(root, filename);
}
exports.getFullPath = getFullPath;
/**
 * 计算 targetFile 在 sourceFile 中的相对引用路径
 * @param sourceFile
 * @param targetFile
 * @returns
 */
function getRelativePath(sourceFile, targetFile) {
    sourceFile = path_1.default.dirname(sourceFile);
    return path_1.default.relative(sourceFile, targetFile);
}
exports.getRelativePath = getRelativePath;
/**
 * 判断给定字符串是否是文件路径
 * @example ./pages/index.js -- yes
 * @example ../pages/index.js -- yes
 * @example ../components -- yes
 * @example /src/pages/index.js -- yes
 * @example @music163/tango-designer -- no
 * @param str
 */
function isFilepath(str) {
    return /^(\.\.?\/|\/).*(\.[a-z]+)?$/.test(str);
}
exports.isFilepath = isFilepath;
