"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TangoJsModule = exports.TangoModule = void 0;
const mobx_1 = require("mobx");
const tango_helpers_1 = require("@music163/tango-helpers");
const helpers_1 = require("../helpers");
const file_1 = require("./file");
/**
 * JS 模块实现规范
 * - ast 操纵类方法，统一返回 this，支持外层链式调用
 * - observable state 统一用 _foo 格式，并提供 getter 方法
 */
class TangoModule extends file_1.TangoFile {
    constructor(workspace, props, isSyncCode = true) {
        super(workspace, props, isSyncCode);
    }
    /**
     * 基于最新的 ast 进行同步
     * @param code 如果传入 code，则基于 code 进行同步
     * @param isFormatCode 是否格式化代码
     * @param refreshWorkspace 是否刷新 workspace
     */
    update(code, isFormatCode = true, refreshWorkspace = true) {
        this.lastModified = Date.now();
        if ((0, tango_helpers_1.isNil)(code)) {
            this._syncByAst();
        }
        else {
            this._syncByCode(code, isFormatCode);
        }
        this._analysisAst();
        this.workspace.onFilesChange([this.filename]);
        if (refreshWorkspace) {
            this.workspace.refresh([this.filename]);
        }
    }
    addImportDeclaration(source, specifiers) {
        this.ast = (0, helpers_1.addImportDeclaration)(this.ast, source, specifiers);
        return this;
    }
    updateImportDeclaration(source, specifiers) {
        this.ast = (0, helpers_1.updateImportDeclaration)(this.ast, source, specifiers);
        return this;
    }
    /**
     * 基于最新的 ast 进行源码同步
     */
    _syncByAst() {
        const code = (0, helpers_1.ast2code)(this.ast);
        this._code = code;
        this._cleanCode = code;
    }
    /**
     * 基于输入的源码进行同步
     * @param code 源码
     * @param isFormatCode 是否格式化代码
     * @returns
     */
    _syncByCode(code, isFormatCode = true) {
        if (code === this._code) {
            return;
        }
        // 提前格式化代码
        if (isFormatCode) {
            code = (0, helpers_1.formatCode)(code);
        }
        this._code = code;
        this._cleanCode = code;
        this.ast = (0, helpers_1.code2ast)(code);
    }
    _analysisAst() {
        const { imports } = (0, helpers_1.traverseFile)(this.ast);
        this.importList = imports;
    }
}
exports.TangoModule = TangoModule;
/**
 * 普通 JS 文件
 */
class TangoJsModule extends TangoModule {
    constructor(workspace, props) {
        super(workspace, props, false);
        this.update(props.code, false, false);
        (0, mobx_1.makeObservable)(this, {
            _code: mobx_1.observable,
            _cleanCode: mobx_1.observable,
            code: mobx_1.computed,
            cleanCode: mobx_1.computed,
            update: mobx_1.action,
        });
    }
}
exports.TangoJsModule = TangoJsModule;
