import { IComponentPrototype, Dict, ITangoConfigJson } from '@music163/tango-helpers';
import { TangoHistory } from './history';
import { TangoNode } from './node';
import { TangoFile, TangoJsonFile } from './file';
import { IWorkspace } from './interfaces';
import { IFileConfig, FileType, ITangoConfigPackages, IPageConfigData } from '../types';
import { SelectSource } from './select-source';
import { DragSource } from './drag-source';
import { TangoRouteModule } from './route-module';
import { TangoStoreEntryModule, TangoStoreModule } from './store-module';
import { TangoServiceModule } from './service-module';
import { TangoViewModule } from './view-module';
import { TangoComponentsEntryModule } from './component-module';
export interface IWorkspaceOptions {
    /**
     * 入口文件
     */
    entry?: string;
    /**
     * 初始化文件列表
     */
    files?: IFileConfig[];
    /**
     * 默认的激活的路由
     */
    defaultActiveRoute?: string;
    /**
     * 组件描述列表
     */
    prototypes?: Record<string, IComponentPrototype>;
    /**
     * 工作区文件变更事件
     */
    onFilesChange?: IWorkspace['onFilesChange'];
}
/**
 * 工作区
 */
export declare class Workspace extends EventTarget implements IWorkspace {
    history: TangoHistory;
    selectSource: SelectSource;
    dragSource: DragSource;
    /**
     * 工作区的文件列表
     */
    files: Map<string, TangoFile>;
    /**
     * 组件配置
     */
    componentPrototypes: Map<string, IComponentPrototype>;
    /**
     * 入口文件
     */
    entry: string;
    /**
     * 当前路由
     */
    activeRoute: string;
    /**
     * 当前选中的文件
     */
    activeFile: string;
    /**
     * 当前选中的视图文件
     */
    activeViewFile: string;
    /**
     * 路由配置模块
     */
    routeModule: TangoRouteModule;
    /**
     * 模型入口配置模块
     */
    storeEntryModule: TangoStoreEntryModule;
    storeModules: Record<string, TangoStoreModule>;
    serviceModules: Record<string, TangoServiceModule>;
    componentsEntryModule: TangoComponentsEntryModule;
    /**
     * package.json 文件
     */
    packageJson: TangoJsonFile;
    /**
     * tango.config.json 文件
     */
    tangoConfigJson: TangoJsonFile;
    /**
     * 绑定事件
     * TODO: 是否需要自己来管理 listeners，并及时进行 gc
     */
    on: (type: string, callback: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions) => void;
    /**
     * 移除事件
     */
    off: (type: string, callback: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions) => void;
    /**
     * 工作区是否就绪
     */
    private isReady;
    /**
     * 拷贝的暂存区
     */
    private copyTempNodes;
    /**
     * 是否是合法的项目
     */
    get isValid(): boolean;
    /**
     * 项目配置，返回解析后的 tango.config.json 文件
     */
    get projectConfig(): ITangoConfigJson;
    /**
     * 当前激活的视图模块
     */
    get activeViewModule(): TangoViewModule;
    /**
     * 获取页面列表
     */
    get pages(): IPageConfigData[];
    get bizComps(): string[];
    get baseComps(): string[];
    get localComps(): string[];
    constructor(options?: IWorkspaceOptions);
    getPrototype(name: string | IComponentPrototype): IComponentPrototype;
    /**
     * 设置当前路由
     * @param routePath 路由路径
     */
    setActiveRoute(routePath: string): void;
    /**
     * 设置当前选中的文件
     * @param filename
     */
    setActiveFile(filename: string, isViewFile?: boolean): void;
    /**
     * 根据当前的路由计算当前的视图模块
     */
    setActiveViewFile(routePath: string): void;
    setComponentPrototypes(prototypes: Record<string, IComponentPrototype>): void;
    /**
     * 添加一组文件到工作区，如果文件同名，后面的文件会覆盖前面的文件
     * @param files
     */
    addFiles(files?: IFileConfig[]): void;
    /**
     * 添加文件到工作区
     * @param filename 文件名
     * @param code 代码片段
     * @param fileType 模块类型
     */
    addFile(filename: string, code: string, fileType?: FileType): void;
    addServiceFile(serviceName: string, code: string): void;
    addStoreFile(storeName: string, code: string): void;
    /**
     * 添加视图文件
     * @param viewName 文件名
     * @param code 代码
     */
    addViewFile(viewName: string, code: string): void;
    updateFile(filename: string, code: string, shouldFormatCode?: boolean): void;
    /**
     * 删除工作区的文件
     * @param filename
     */
    removeFile(filename: string): void;
    /**
     * 重命名文件
     * @param oldFilename
     * @param newFilename
     */
    renameFile(oldFilename: string, newFilename: string): void;
    /**
     * 重命名文件夹
     * @param oldFoldername 旧文件夹名
     * @param newFoldername 新文件夹名
     */
    renameFolder(oldFoldername: string, newFoldername: string): void;
    /**
     * 根据文件名获取文件对象
     * @param filename
     * @returns
     */
    getFile(filename: string): TangoFile;
    /**
     * 获取文件列表
     * @returns { [filename]: fileCode }
     */
    listFiles(): {};
    /**
     * 删除视图模块
     * @param route 路由名称
     */
    removeViewModule(routePath: string): void;
    /**
     * 添加新的路由
     */
    addRoute(routeData: IPageConfigData, importFilePath: string): void;
    /**
     * 更新页面路由配置
     * @param sourceRoutePath
     * @param targetPageData
     */
    updateRoute(sourceRoutePath: string, targetPageData: IPageConfigData): void;
    /**
     * 复制视图文件
     * @param sourceRoute
     * @param targetRouteConfig
     */
    copyViewPage(sourceRoutePath: string, targetPageData: IPageConfigData): void;
    getNode(id: string, filename?: string): TangoNode;
    /**
     * 应用代码初始化完成
     */
    ready(): void;
    /**
     * 添加新的模型文件
     * @deprecated 使用 addStoreFile 代替
     */
    addStoreModule(name: string, code: string): void;
    /**
     * 删除模型文件
     * @param name
     */
    removeStoreModule(name: string): void;
    /**
     * 添加模型属性
     * @param storeName
     * @param stateName
     * @param initValue
     */
    addStoreState(storeName: string, stateName: string, initValue: string): void;
    /**
     * 删除模型属性
     * @param storeName
     * @param stateName
     */
    removeStoreState(storeName: string, stateName: string): void;
    /**
     * 根据变量路径删除状态变量
     * @param variablePath
     */
    removeStoreVariable(variablePath: string): void;
    /**
     * 根据变量路径更新状态变量的值
     * @param variablePath 变量路径
     * @param code 变量代码
     */
    updateStoreVariable(variablePath: string, code: string): void;
    /**
     * 获取服务函数的详情
     * TODO: 不要 services 前缀
     * @param serviceKey `services.list` 或 `services.sub.list`
     * @returns
     */
    getServiceFunction(serviceKey: string): {
        name: string;
        moduleName: string;
        config: any;
    };
    /**
     * 获取服务函数的列表
     * @returns 返回服务函数的列表 { [serviceKey: string]: Dict }
     */
    listServiceFunctions(): Record<string, Dict>;
    /**
     * 更新服务函数
     */
    updateServiceFunction(serviceName: string, payload: Dict, moduleName?: string): void;
    /**
     * 新增服务函数，支持批量添加
     */
    addServiceFunction(name: string, config: Dict, moduleName?: string): void;
    addServiceFunctions(configs: Dict<Dict>, modName?: string): void;
    /**
     * 删除服务函数
     * @param name
     */
    removeServiceFunction(serviceKey: string): void;
    /**
     * 更新服务的基础配置
     */
    updateServiceBaseConfig(config: Dict, moduleName?: string): void;
    addDependency(data: any): void;
    /**
     * 获取 package.json 中的依赖列表
     * @returns
     * TODO: fix this logic to merge dependencies from package.json and tango.config.json
     */
    listDependencies(): any;
    getDependency(pkgName: string): any;
    /**
     * 更新依赖，没有就添加
     * @param name
     * @param version
     */
    updateDependency(name: string, version: string, options?: {
        package?: ITangoConfigPackages;
        [x: string]: any;
    }): void;
    /**
     * 移除依赖
     * @param name
     */
    removeDependency(name: string): void;
    /**
     * 删除业务组件
     * @param name
     */
    removeBizComp(name: string): void;
    /**
     * 添加业务组件
     * @param name
     */
    addBizComp(name: string, version: string, options?: {
        package?: ITangoConfigPackages;
        [x: string]: any;
    }): void;
    /**
     * 删除选中节点
     */
    removeSelectedNode(): void;
    /**
     * 复制选中结点
     */
    copySelectedNode(): void;
    /**
     * 粘贴选中结点
     * @deprecated 考虑废弃
     * TODO: 重构该逻辑，抽离出公共的方法
     */
    pasteSelectedNode(): void;
    /**
     * 克隆选中节点，追加到当前结点的后方
     */
    cloneSelectedNode(): void;
    /**
     * 在目标节点中插入子节点
     * @param targetNodeId 目标节点dnd-id
     * @param sourceName 插入的组件名称
     * @returns
     */
    insertToNode(targetNodeId: string, sourceName: string | IComponentPrototype): void;
    /**
     * 替换目标节点
     */
    replaceNode(targetNodeId: string, sourceName: string | IComponentPrototype): void;
    /**
     * 在选中节点中插入子节点
     * @param childName 节点名
     */
    insertToSelectedNode(childName: string | IComponentPrototype): void;
    insertBeforeSelectedNode(sourceName: string | IComponentPrototype): void;
    insertAfterSelectedNode(sourceName: string | IComponentPrototype): void;
    updateSelectedNodeAttributes(attributes?: Record<string, any>, relatedImports?: string[]): void;
    /**
     * 将节点拽入视图中
     */
    dropNode(): void;
    onFilesChange(filenams: string[]): void;
    /**
     * 刷新目标文件
     * @param filenames
     */
    refresh(filenames: string[]): void;
    /**
     * 基于输入结点获得结点依赖的导入声明信息
     * @param nodes
     */
    private getImportDeclarationByNodes;
    /**
     * 根据路由路径获取文件名
     * @param routePath
     * @returns
     */
    private getFilenameByRoutePath;
    private getRealViewFilePath;
    /**
     * 文件拷贝
     * @param sourcePath
     * @param targetPath
     */
    private copyFiles;
}
