"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Workspace = void 0;
const mobx_1 = require("mobx");
const tango_helpers_1 = require("@music163/tango-helpers");
const helpers_1 = require("../helpers");
const drop_target_1 = require("./drop-target");
const history_1 = require("./history");
const module_1 = require("./module");
const file_1 = require("./file");
const types_1 = require("../types");
const select_source_1 = require("./select-source");
const drag_source_1 = require("./drag-source");
const route_module_1 = require("./route-module");
const store_module_1 = require("./store-module");
const service_module_1 = require("./service-module");
const view_module_1 = require("./view-module");
const component_module_1 = require("./component-module");
/**
 * 工作区
 */
class Workspace extends EventTarget {
    /**
     * 是否是合法的项目
     */
    get isValid() {
        return !!this.tangoConfigJson && !!this.activeViewModule;
    }
    /**
     * 项目配置，返回解析后的 tango.config.json 文件
     */
    get projectConfig() {
        var _a;
        return (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.json;
    }
    /**
     * 当前激活的视图模块
     */
    get activeViewModule() {
        if (!this.activeViewFile) {
            this.setActiveViewFile(this.activeRoute);
        }
        return this.files.get(this.activeViewFile);
    }
    /**
     * 获取页面列表
     */
    get pages() {
        var _a;
        const ret = [];
        (_a = this.routeModule) === null || _a === void 0 ? void 0 : _a.routes.forEach((item) => {
            if (item.path !== '*') {
                ret.push({
                    path: item.path,
                    name: item.component,
                });
            }
        });
        return ret;
    }
    get bizComps() {
        var _a, _b;
        const packages = (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages');
        let list = ((_b = this.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.getValue('bizDependencies')) || [];
        if (packages) {
            list = [
                ...new Set([
                    ...list,
                    ...Object.keys(packages).filter((e) => packages[e].type === 'bizDependency'),
                ]),
            ];
        }
        return list;
    }
    get baseComps() {
        var _a, _b;
        const packages = (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages');
        let list = ((_b = this.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.getValue('baseDependencies')) || [];
        if (packages) {
            list = [
                ...new Set([
                    ...list,
                    ...Object.keys(packages).filter((e) => packages[e].type === 'baseDependency'),
                ]),
            ];
        }
        return list;
    }
    get localComps() {
        var _a;
        return Object.keys(((_a = this.componentsEntryModule) === null || _a === void 0 ? void 0 : _a.exportList) || {});
    }
    constructor(options) {
        super();
        this.storeModules = {};
        this.serviceModules = {};
        /**
         * 绑定事件
         * TODO: 是否需要自己来管理 listeners，并及时进行 gc
         */
        this.on = this.addEventListener;
        /**
         * 移除事件
         */
        this.off = this.removeEventListener;
        this.history = new history_1.TangoHistory(this);
        this.selectSource = new select_source_1.SelectSource(this);
        this.dragSource = new drag_source_1.DragSource(this);
        this.componentPrototypes = new Map();
        this.entry = options === null || options === void 0 ? void 0 : options.entry;
        this.activeRoute = (options === null || options === void 0 ? void 0 : options.defaultActiveRoute) || '/';
        this.activeFile = options === null || options === void 0 ? void 0 : options.entry;
        this.activeViewFile = '';
        this.files = new Map();
        this.isReady = false;
        if (options === null || options === void 0 ? void 0 : options.onFilesChange) {
            // 使用用户提供的 onFilesChange
            this.onFilesChange = options.onFilesChange;
        }
        if (options === null || options === void 0 ? void 0 : options.files) {
            this.addFiles(options.files);
        }
        if (options === null || options === void 0 ? void 0 : options.prototypes) {
            this.setComponentPrototypes(options.prototypes);
        }
        (0, mobx_1.makeObservable)(this, {
            files: mobx_1.observable,
            activeRoute: mobx_1.observable,
            activeFile: mobx_1.observable,
            activeViewFile: mobx_1.observable,
            pages: mobx_1.computed,
            bizComps: mobx_1.computed,
            setActiveRoute: mobx_1.action,
            setActiveFile: mobx_1.action,
            addFile: mobx_1.action,
            removeFile: mobx_1.action,
        });
    }
    getPrototype(name) {
        if ((0, tango_helpers_1.isString)(name)) {
            return this.componentPrototypes.get(name);
        }
        return name;
    }
    /**
     * 设置当前路由
     * @param routePath 路由路径
     */
    setActiveRoute(routePath) {
        if (routePath === this.activeRoute) {
            return;
        }
        this.selectSource.clear();
        this.activeRoute = routePath;
        this.setActiveViewFile(routePath);
    }
    /**
     * 设置当前选中的文件
     * @param filename
     */
    setActiveFile(filename, isViewFile = false) {
        this.activeFile = filename;
        if (isViewFile) {
            this.activeViewFile = filename;
        }
    }
    /**
     * 根据当前的路由计算当前的视图模块
     */
    setActiveViewFile(routePath) {
        let filename = this.getFilenameByRoutePath(routePath);
        if (!filename) {
            // 没有找到 route 对应的文件，使用默认的 entry
            for (const [key, file] of this.files) {
                if (file.type === types_1.FileType.JsxViewModule) {
                    filename = file.filename;
                    break;
                }
            }
        }
        if (filename) {
            this.setActiveFile(filename, true);
        }
    }
    setComponentPrototypes(prototypes) {
        Object.keys(prototypes).forEach((name) => {
            this.componentPrototypes.set(name, prototypes[name]);
        });
    }
    /**
     * 添加一组文件到工作区，如果文件同名，后面的文件会覆盖前面的文件
     * @param files
     */
    addFiles(files = []) {
        files.forEach((file) => {
            this.addFile(file.filename, file.code, file.type);
        });
    }
    /**
     * 添加文件到工作区
     * @param filename 文件名
     * @param code 代码片段
     * @param fileType 模块类型
     */
    addFile(filename, code, fileType) {
        var _a;
        const moduleType = fileType || (0, helpers_1.inferFileType)(filename);
        const props = {
            filename,
            code,
            type: moduleType,
        };
        let module;
        switch (moduleType) {
            case types_1.FileType.StoreEntryModule:
                module = new store_module_1.TangoStoreEntryModule(this, props);
                this.storeEntryModule = module;
                break;
            case types_1.FileType.ComponentsEntryModule:
                module = new component_module_1.TangoComponentsEntryModule(this, props);
                this.componentsEntryModule = module;
                break;
            case types_1.FileType.RouteModule: {
                module = new route_module_1.TangoRouteModule(this, props);
                this.routeModule = module;
                // check if activeRoute exists
                const route = module.routes.find((item) => item.path === this.activeRoute);
                if (!route) {
                    this.setActiveRoute((_a = module.routes[0]) === null || _a === void 0 ? void 0 : _a.path);
                }
                break;
            }
            case types_1.FileType.JsxViewModule:
                module = new view_module_1.TangoViewModule(this, props);
                break;
            case types_1.FileType.ServiceModule:
                module = new service_module_1.TangoServiceModule(this, props);
                this.serviceModules[module.name] = module;
                break;
            case types_1.FileType.StoreModule:
                module = new store_module_1.TangoStoreModule(this, props);
                this.storeModules[module.name] = module;
                break;
            case types_1.FileType.Module:
                module = new module_1.TangoJsModule(this, props);
                break;
            case types_1.FileType.Less:
                module = new file_1.TangoLessFile(this, props);
                break;
            case types_1.FileType.PackageJson:
                module = new file_1.TangoJsonFile(this, props);
                this.packageJson = module;
                break;
            case types_1.FileType.TangoConfigJson:
                module = new file_1.TangoJsonFile(this, props);
                this.tangoConfigJson = module;
                break;
            case types_1.FileType.Json:
                module = new file_1.TangoJsonFile(this, props);
                break;
            default:
                module = new file_1.TangoFile(this, props);
        }
        this.files.set(filename, module);
    }
    addServiceFile(serviceName, code) {
        const filename = `/src/services/${serviceName}.js`;
        this.addFile(filename, code, types_1.FileType.ServiceModule);
        const indexServiceModule = this.serviceModules.index;
        indexServiceModule === null || indexServiceModule === void 0 ? void 0 : indexServiceModule.addImportDeclaration(`./${serviceName}`, []).update();
    }
    addStoreFile(storeName, code) {
        const filename = `/src/stores/${storeName}.js`;
        this.addFile(filename, code);
        if (!this.storeEntryModule) {
            this.addFile('/src/stores/index.js', '');
        }
        this.storeEntryModule.addStore(storeName).update();
    }
    /**
     * 添加视图文件
     * @param viewName 文件名
     * @param code 代码
     */
    addViewFile(viewName, code) {
        const viewRoute = viewName.startsWith('/') ? viewName : `/${viewName}`;
        const filename = `/src/pages/${viewName}.js`;
        this.addFile(filename, code);
        this.addRoute({
            name: viewName,
            path: viewRoute,
        }, filename);
    }
    updateFile(filename, code, shouldFormatCode = false) {
        const file = this.getFile(filename);
        file.update(code);
        if (shouldFormatCode && file instanceof view_module_1.TangoViewModule) {
            file.removeUnusedImportSpecifiers().update();
        }
        this.history.push({
            message: history_1.HistoryMessage.UpdateCode,
            data: {
                [filename]: code,
            },
        });
    }
    /**
     * 删除工作区的文件
     * @param filename
     */
    removeFile(filename) {
        // TODO: refactor visitFile to share this logic
        if (this.files.get(filename)) {
            // 如果是文件，直接删除
            this.files.delete(filename);
        }
        else {
            // 没有匹配到，就是一个目录，直接删除整个目录
            // FIXME: 可能存在风险，如果文件夹中的模块被复用，则会导致误删除
            Array.from(this.files.keys()).forEach((key) => {
                if (key.startsWith(`${filename}/`)) {
                    this.files.delete(key);
                }
            });
        }
    }
    /**
     * 重命名文件
     * @param oldFilename
     * @param newFilename
     */
    renameFile(oldFilename, newFilename) {
        const file = this.files.get(oldFilename);
        if (file) {
            this.removeFile(oldFilename);
            this.addFile(newFilename, file.code);
        }
    }
    /**
     * 重命名文件夹
     * @param oldFoldername 旧文件夹名
     * @param newFoldername 新文件夹名
     */
    renameFolder(oldFoldername, newFoldername) {
        Array.from(this.files.keys()).forEach((key) => {
            if (key.startsWith(`${oldFoldername}/`)) {
                const newKey = key.replace(oldFoldername, newFoldername);
                this.renameFile(key, newKey);
            }
        });
    }
    /**
     * 根据文件名获取文件对象
     * @param filename
     * @returns
     */
    getFile(filename) {
        return this.files.get(filename);
    }
    /**
     * 获取文件列表
     * @returns { [filename]: fileCode }
     */
    listFiles() {
        const ret = {};
        this.files.forEach((file) => {
            ret[file.filename] = file.cleanCode;
        });
        return ret;
    }
    /**
     * 删除视图模块
     * @param route 路由名称
     */
    removeViewModule(routePath) {
        var _a;
        // get filename first
        const filename = this.getFilenameByRoutePath(routePath);
        if (this.routeModule) {
            this.routeModule.removeRoute(routePath).update();
            this.setActiveRoute(((_a = this.routeModule.routes[0]) === null || _a === void 0 ? void 0 : _a.path) || '/');
        }
        this.removeFile(filename);
    }
    /**
     * 添加新的路由
     */
    addRoute(routeData, importFilePath) {
        var _a;
        (_a = this.routeModule) === null || _a === void 0 ? void 0 : _a.addRoute(routeData.path, importFilePath).update();
    }
    /**
     * 更新页面路由配置
     * @param sourceRoutePath
     * @param targetPageData
     */
    updateRoute(sourceRoutePath, targetPageData) {
        var _a;
        if (sourceRoutePath !== targetPageData.path) {
            (_a = this.routeModule) === null || _a === void 0 ? void 0 : _a.updateRoute(sourceRoutePath, targetPageData.path).update();
        }
    }
    /**
     * 复制视图文件
     * @param sourceRoute
     * @param targetRouteConfig
     */
    copyViewPage(sourceRoutePath, targetPageData) {
        const sourceFilePath = this.getRealViewFilePath(this.getFilenameByRoutePath(sourceRoutePath));
        const targetFilePath = (0, helpers_1.getFilepath)(targetPageData.path, '/src/pages');
        this.copyFiles(sourceFilePath, targetFilePath);
        this.addRoute(targetPageData, targetFilePath);
    }
    getNode(id, filename) {
        const file = filename ? this.getFile(filename) : this.activeViewModule;
        if (file instanceof view_module_1.TangoViewModule) {
            return file.getNode(id);
        }
    }
    /**
     * 应用代码初始化完成
     */
    ready() {
        var _a, _b;
        if (!this.isReady) {
            this.isReady = true;
            this.history.push({
                message: history_1.HistoryMessage.InitView,
                data: {
                    [(_a = this.activeViewModule) === null || _a === void 0 ? void 0 : _a.filename]: (_b = this.activeViewModule) === null || _b === void 0 ? void 0 : _b.code,
                },
            });
        }
    }
    /**
     * 添加新的模型文件
     * @deprecated 使用 addStoreFile 代替
     */
    addStoreModule(name, code) {
        this.addStoreFile(name, code);
    }
    /**
     * 删除模型文件
     * @param name
     */
    removeStoreModule(name) {
        const filename = (0, helpers_1.getFilepath)(name, '/src/stores', '.js');
        this.storeEntryModule.removeStore(name).update();
        this.removeFile(filename);
    }
    /**
     * 添加模型属性
     * @param storeName
     * @param stateName
     * @param initValue
     */
    addStoreState(storeName, stateName, initValue) {
        var _a;
        (_a = this.storeModules[storeName]) === null || _a === void 0 ? void 0 : _a.addState(stateName, initValue).update();
    }
    /**
     * 删除模型属性
     * @param storeName
     * @param stateName
     */
    removeStoreState(storeName, stateName) {
        var _a;
        (_a = this.storeModules[storeName]) === null || _a === void 0 ? void 0 : _a.removeState(stateName).update();
    }
    /**
     * 根据变量路径删除状态变量
     * @param variablePath
     */
    removeStoreVariable(variablePath) {
        const { storeName, variableName } = (0, tango_helpers_1.parseStoreVariablePath)(variablePath);
        this.removeStoreState(storeName, variableName);
    }
    /**
     * 根据变量路径更新状态变量的值
     * @param variablePath 变量路径
     * @param code 变量代码
     */
    updateStoreVariable(variablePath, code) {
        var _a;
        if ((0, tango_helpers_1.isStoreVariablePath)(variablePath)) {
            const { storeName, variableName } = (0, tango_helpers_1.parseStoreVariablePath)(variablePath);
            (_a = this.storeModules[storeName]) === null || _a === void 0 ? void 0 : _a.updateState(variableName, code).update();
        }
    }
    /**
     * 获取服务函数的详情
     * TODO: 不要 services 前缀
     * @param serviceKey `services.list` 或 `services.sub.list`
     * @returns
     */
    getServiceFunction(serviceKey) {
        var _a;
        const { name, moduleName } = (0, tango_helpers_1.parseServiceVariablePath)(serviceKey);
        if (!name) {
            return;
        }
        return {
            name,
            moduleName,
            config: (_a = this.serviceModules[moduleName]) === null || _a === void 0 ? void 0 : _a.serviceFunctions[name],
        };
    }
    /**
     * 获取服务函数的列表
     * @returns 返回服务函数的列表 { [serviceKey: string]: Dict }
     */
    listServiceFunctions() {
        const ret = {};
        Object.keys(this.serviceModules).forEach((moduleName) => {
            const module = this.serviceModules[moduleName];
            Object.keys(module.serviceFunctions).forEach((name) => {
                const serviceKey = moduleName === 'index' ? name : [moduleName, name].join('.');
                ret[serviceKey] = module.serviceFunctions[name];
            });
        });
        return ret;
    }
    /**
     * 更新服务函数
     */
    updateServiceFunction(serviceName, payload, moduleName = 'index') {
        this.serviceModules[moduleName].updateServiceFunction(serviceName, payload).update();
    }
    /**
     * 新增服务函数，支持批量添加
     */
    addServiceFunction(name, config, moduleName = 'index') {
        var _a;
        (_a = this.serviceModules[moduleName]) === null || _a === void 0 ? void 0 : _a.addServiceFunction(name, config).update();
    }
    addServiceFunctions(configs, modName = 'index') {
        var _a;
        (_a = this.serviceModules[modName]) === null || _a === void 0 ? void 0 : _a.addServiceFunctions(configs).update();
    }
    /**
     * 删除服务函数
     * @param name
     */
    removeServiceFunction(serviceKey) {
        var _a;
        const { moduleName, name } = (0, tango_helpers_1.parseServiceVariablePath)(serviceKey);
        (_a = this.serviceModules[moduleName]) === null || _a === void 0 ? void 0 : _a.deleteServiceFunction(name).update();
    }
    /**
     * 更新服务的基础配置
     */
    updateServiceBaseConfig(config, moduleName = 'index') {
        var _a;
        (_a = this.serviceModules[moduleName]) === null || _a === void 0 ? void 0 : _a.updateBaseConfig(config).update();
    }
    addDependency(data) {
        // TODO: implement it to replace addBizComp & addServiceComp
    }
    /**
     * 获取 package.json 中的依赖列表
     * @returns
     * TODO: fix this logic to merge dependencies from package.json and tango.config.json
     */
    listDependencies() {
        var _a;
        return (_a = this.packageJson) === null || _a === void 0 ? void 0 : _a.getValue('dependencies');
    }
    getDependency(pkgName) {
        var _a, _b;
        const packages = (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages');
        const dependencies = (_b = this.packageJson) === null || _b === void 0 ? void 0 : _b.getValue('dependencies'); // 兼容老版本
        const detail = Object.assign({ version: dependencies === null || dependencies === void 0 ? void 0 : dependencies[pkgName] }, ((packages === null || packages === void 0 ? void 0 : packages[pkgName]) || {}));
        return detail;
    }
    /**
     * 更新依赖，没有就添加
     * @param name
     * @param version
     */
    updateDependency(name, version, options) {
        var _a, _b;
        (_a = this.packageJson) === null || _a === void 0 ? void 0 : _a.setValue('dependencies', (deps = {}) => {
            deps[name] = version;
            return deps;
        }).update();
        (_b = this.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.setValue('packages', (packages) => {
            var _a;
            // 兼容以前的逻辑，只在拥有 package 参数时，才会更新 packages 字段
            if (!packages) {
                return undefined;
            }
            if ((options === null || options === void 0 ? void 0 : options.package) || packages[name]) {
                packages[name] = Object.assign(Object.assign({ type: (_a = packages[name]) === null || _a === void 0 ? void 0 : _a.type }, ((options === null || options === void 0 ? void 0 : options.package) || packages[name])), { version });
            }
            return packages;
        }).update();
        this.history.push({
            message: history_1.HistoryMessage.UpdateDependency,
            data: {
                [this.packageJson.filename]: this.packageJson.code,
            },
        });
    }
    /**
     * 移除依赖
     * @param name
     */
    removeDependency(name) {
        var _a, _b;
        (_a = this.packageJson) === null || _a === void 0 ? void 0 : _a.setValue('dependencies', (deps) => {
            if (deps[name]) {
                delete deps[name];
            }
            return deps;
        }).update();
        (_b = this.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.setValue('packages', (packages = {}) => {
            if (packages === null || packages === void 0 ? void 0 : packages[name]) {
                delete packages[name];
            }
            return packages;
        }).update();
        this.history.push({
            message: history_1.HistoryMessage.RemoveDependency,
            data: {
                [this.packageJson.filename]: this.packageJson.code,
            },
        });
    }
    /**
     * 删除业务组件
     * @param name
     */
    removeBizComp(name) {
        var _a;
        (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.setValue('bizDependencies', (deps) => {
            if (!deps) {
                return undefined;
            }
            return deps.filter((dep) => dep !== name);
        }).update();
        this.removeDependency(name);
    }
    /**
     * 添加业务组件
     * @param name
     */
    addBizComp(name, version, options) {
        var _a;
        const packages = this.tangoConfigJson.getValue('packages');
        this.updateDependency(name, version, Object.assign(Object.assign({}, options), (!!packages && {
            package: Object.assign(Object.assign({}, options === null || options === void 0 ? void 0 : options.package), { type: 'bizDependency' }),
        })));
        // 兼容以前的逻辑
        if (!(options === null || options === void 0 ? void 0 : options.package) && !packages) {
            // TODO: if tangoConfigJson not found, init this file
            (_a = this.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.setValue('bizDependencies', (deps = []) => {
                if (!deps.includes(name)) {
                    deps.push(name);
                }
                return deps;
            }).update();
        }
        this.tangoConfigJson &&
            this.history.push({
                message: history_1.HistoryMessage.UpdateDependency,
                data: {
                    [this.tangoConfigJson.filename]: this.tangoConfigJson.code,
                },
            });
    }
    /**
     * 删除选中节点
     */
    removeSelectedNode() {
        const file = this.selectSource.file;
        if (!file)
            return;
        // 选中的结点一定位于相同的文件中
        this.selectSource.nodes.forEach((node) => {
            file.removeNode(node.id);
        });
        file.update();
        this.selectSource.clear();
        this.history.push({
            message: history_1.HistoryMessage.RemoveNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    /**
     * 复制选中结点
     */
    copySelectedNode() {
        this.copyTempNodes = this.selectSource.nodes;
    }
    /**
     * 粘贴选中结点
     * @deprecated 考虑废弃
     * TODO: 重构该逻辑，抽离出公共的方法
     */
    pasteSelectedNode() {
        if (this.selectSource.size !== 1)
            return;
        if (!this.copyTempNodes)
            return;
        // TODO: 潜在隐患，如果跨页的话，代码里的逻辑调用也要处理
        const importDeclarations = this.getImportDeclarationByNodes(this.copyTempNodes.map((node) => node.rawNode));
        importDeclarations.forEach((importDeclaration) => {
            this.activeViewModule.updateImportSpecifiersLegacy(importDeclaration);
        });
        this.copyTempNodes.forEach((node) => {
            this.activeViewModule.insertAfter(this.selectSource.first.id, node.cloneRawNode());
        });
        this.activeViewModule.update();
        this.history.push({
            message: history_1.HistoryMessage.CloneNode,
            data: {
                [this.activeViewModule.filename]: this.activeViewModule.code,
            },
        });
    }
    /**
     * 克隆选中节点，追加到当前结点的后方
     */
    cloneSelectedNode() {
        var _a, _b;
        const file = this.selectSource.file;
        let injectedProps;
        if ((_b = (_a = this.projectConfig) === null || _a === void 0 ? void 0 : _a.designerConfig) === null || _b === void 0 ? void 0 : _b.autoGenerateComponentId) {
            // 生成新的组件 id
            injectedProps = {
                tid: file.idGenerator.generateId(this.selectSource.firstNode.component).id,
            };
        }
        file
            .insertAfter(this.selectSource.first.id, this.selectSource.firstNode.cloneRawNode(injectedProps))
            .update();
        this.history.push({
            message: history_1.HistoryMessage.CloneNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    /**
     * 在目标节点中插入子节点
     * @param targetNodeId 目标节点dnd-id
     * @param sourceName 插入的组件名称
     * @returns
     */
    insertToNode(targetNodeId, sourceName) {
        if (!targetNodeId || !sourceName) {
            return;
        }
        const sourcePrototype = this.getPrototype(sourceName);
        const newNode = (0, helpers_1.prototype2jsxElement)(sourcePrototype);
        const file = this.getNode(targetNodeId).file;
        const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(sourcePrototype, file.filename);
        file
            .insertChild(targetNodeId, newNode, 'last')
            .addImportSpecifiers(source, specifiers)
            .update();
        this.history.push({
            message: history_1.HistoryMessage.InsertNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    /**
     * 替换目标节点
     */
    replaceNode(targetNodeId, sourceName) {
        if (!targetNodeId || !sourceName) {
            return;
        }
        const sourcePrototype = this.getPrototype(sourceName);
        const newNode = (0, helpers_1.prototype2jsxElement)(sourcePrototype);
        const file = this.getNode(targetNodeId).file;
        const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(sourcePrototype, file.filename);
        file.replaceNode(targetNodeId, newNode).addImportSpecifiers(source, specifiers).update();
        this.history.push({
            message: history_1.HistoryMessage.ReplaceNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    /**
     * 在选中节点中插入子节点
     * @param childName 节点名
     */
    insertToSelectedNode(childName) {
        const insertedPrototype = this.getPrototype(childName);
        if (insertedPrototype) {
            const newNode = (0, helpers_1.prototype2jsxElement)(insertedPrototype);
            const file = this.selectSource.file;
            const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(insertedPrototype, file.filename);
            file
                .insertChild(this.selectSource.first.id, newNode, 'last')
                .addImportSpecifiers(source, specifiers)
                .update();
            this.history.push({
                message: history_1.HistoryMessage.InsertNode,
                data: {
                    [file.filename]: file.code,
                },
            });
        }
    }
    insertBeforeSelectedNode(sourceName) {
        if (!sourceName) {
            return;
        }
        const targetNodeId = this.selectSource.first.id;
        const sourcePrototype = this.getPrototype(sourceName);
        const newNode = (0, helpers_1.prototype2jsxElement)(sourcePrototype);
        const file = this.getNode(targetNodeId).file;
        const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(sourcePrototype, file.filename);
        file.insertBefore(targetNodeId, newNode).addImportSpecifiers(source, specifiers).update();
        this.history.push({
            message: history_1.HistoryMessage.InsertBeforeNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    insertAfterSelectedNode(sourceName) {
        if (!sourceName) {
            return;
        }
        const targetNodeId = this.selectSource.first.id;
        const sourcePrototype = this.getPrototype(sourceName);
        const newNode = (0, helpers_1.prototype2jsxElement)(sourcePrototype);
        const file = this.getNode(targetNodeId).file;
        const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(sourcePrototype, file.filename);
        file.insertAfter(targetNodeId, newNode).addImportSpecifiers(source, specifiers).update();
        this.history.push({
            message: history_1.HistoryMessage.InsertAfterNode,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    updateSelectedNodeAttributes(attributes = {}, relatedImports = []) {
        const file = this.selectSource.file;
        file.updateNodeAttributes(this.selectSource.first.id, attributes, relatedImports).update();
        this.history.push({
            message: history_1.HistoryMessage.UpdateAttribute,
            data: {
                [file.filename]: file.code,
            },
        });
    }
    /**
     * 将节点拽入视图中
     */
    dropNode() {
        var _a, _b, _c, _d;
        const dragSource = this.dragSource;
        const dropTarget = dragSource.dropTarget;
        if (!dragSource.prototype || !dropTarget.id) {
            // 无效的 drag source 或 drop target，提前退出
            tango_helpers_1.logger.error('invalid dragSource or dropTarget');
            return;
        }
        // TODO: 这里需要一个额外的信息，DropTarget 的最近容器节点，用于判断目标元素是否可以被置入容器中
        const dragSourcePrototype = dragSource.prototype;
        let injectedProps; // 额外注入给节点的属性
        if ((_b = (_a = this.projectConfig) === null || _a === void 0 ? void 0 : _a.designerConfig) === null || _b === void 0 ? void 0 : _b.autoGenerateComponentId) {
            injectedProps = {
                tid: this.activeViewModule.idGenerator.generateId(dragSource.prototype.name).id,
            };
        }
        let newNode;
        if (dragSource.id) {
            // 来自画布，完整的克隆该节点
            newNode = dragSource.getNode().cloneRawNode();
        }
        else {
            // 来自物料面板，创建新的初始化节点
            newNode = (0, helpers_1.prototype2jsxElement)(dragSource.prototype, injectedProps);
        }
        if (!newNode) {
            return;
        }
        const targetFile = (_c = dropTarget.node) === null || _c === void 0 ? void 0 : _c.file;
        const sourceFile = (_d = dragSource.node) === null || _d === void 0 ? void 0 : _d.file;
        // dragSourcePrototype to importDeclarations
        const { source, specifiers } = (0, helpers_1.prototype2importDeclarationData)(dragSourcePrototype, targetFile.filename);
        let isValidOperation = true;
        switch (dropTarget.method) {
            // 直接往目标节点的 children 里添加一个节点
            case drop_target_1.DropMethod.InsertChild: {
                targetFile
                    .insertChild(dropTarget.id, newNode, 'last')
                    .addImportSpecifiers(source, specifiers);
                break;
            }
            case drop_target_1.DropMethod.InsertFirstChild: {
                targetFile
                    .insertChild(dropTarget.id, newNode, 'first')
                    .addImportSpecifiers(source, specifiers);
                break;
            }
            // 往目标节点的后边插入一个节点
            case drop_target_1.DropMethod.InsertAfter: {
                targetFile.insertAfter(dropTarget.id, newNode).addImportSpecifiers(source, specifiers);
                break;
            }
            // 往目标节点的前方插入一个节点
            case drop_target_1.DropMethod.InsertBefore: {
                targetFile.insertBefore(dropTarget.id, newNode).addImportSpecifiers(source, specifiers);
                break;
            }
            // 替换目标节点
            case drop_target_1.DropMethod.ReplaceNode: {
                targetFile.replaceNode(dropTarget.id, newNode).addImportSpecifiers(source, specifiers);
                break;
            }
            default:
                isValidOperation = false;
                break;
        }
        // 如果拖拽来源有 ID，表示来自画布
        const isDraggingFromView = !!dragSource.id;
        if (isValidOperation) {
            if (isDraggingFromView) {
                sourceFile.removeNode(dragSource.id);
            }
            this.selectSource.clear();
        }
        targetFile.update();
        if (isDraggingFromView && sourceFile.filename !== targetFile.filename) {
            sourceFile.update();
        }
        dragSource.clear();
        if (isValidOperation) {
            this.history.push({
                message: history_1.HistoryMessage.DropNode,
                data: {
                    [targetFile.filename]: targetFile.code,
                },
            });
        }
    }
    onFilesChange(filenams) {
        // do nothing
    }
    /**
     * 刷新目标文件
     * @param filenames
     */
    refresh(filenames) {
        this.dispatchEvent(new CustomEvent('refresh', {
            detail: {
                filenames,
                entry: this.entry,
            },
        }));
    }
    /**
     * 基于输入结点获得结点依赖的导入声明信息
     * @param nodes
     */
    getImportDeclarationByNodes(nodes) {
        let names = nodes.reduce((prev, cur) => {
            prev = prev.concat((0, helpers_1.getJSXElementChildrenNames)(cur));
            return prev;
        }, []);
        names = (0, tango_helpers_1.uniq)(names);
        const importDeclarations = (0, helpers_1.namesToImportDeclarations)(names, this.selectSource.file.importMap);
        return importDeclarations;
    }
    /**
     * 根据路由路径获取文件名
     * @param routePath
     * @returns
     */
    getFilenameByRoutePath(routePath) {
        var _a;
        let filename;
        (_a = this.routeModule) === null || _a === void 0 ? void 0 : _a.routes.forEach((route) => {
            if ((0, helpers_1.isPathnameMatchRoute)(routePath, route.path) && route.importPath) {
                if (route.importPath.startsWith('@/')) {
                    filename = route.importPath;
                    const alias = this.tangoConfigJson.getValue('sandbox.alias') || {};
                    if (alias['@']) {
                        filename = filename.replace('@', alias['@']);
                    }
                    filename = this.getRealViewFilePath(filename);
                }
                else {
                    const absolutePath = route.importPath.replace('.', '/src');
                    filename = this.getRealViewFilePath(absolutePath);
                }
            }
        });
        return filename;
    }
    getRealViewFilePath(filePath) {
        // 如果有后缀名直接返回
        if ((0, tango_helpers_1.hasFileExtension)(filePath)) {
            return filePath;
        }
        const possiblePaths = [
            `${filePath}.js`,
            `${filePath}.jsx`,
            `${filePath}/index.js`,
            `${filePath}/index.jsx`,
        ];
        for (const filepath of possiblePaths) {
            if (this.files.has(filepath)) {
                return filepath;
            }
        }
    }
    /**
     * 文件拷贝
     * @param sourcePath
     * @param targetPath
     */
    copyFiles(sourceFilePath, targetFilePath) {
        if (this.files.has(sourceFilePath)) {
            // 来源是文件
            const file = this.files.get(sourceFilePath);
            this.addFile(`${targetFilePath}.js`, file.cleanCode, file.type);
        }
        else if (this.files.has(`${sourceFilePath}/index.js`)) {
            // 来源是目录
            Array.from(this.files.keys()).forEach((key) => {
                if (key.startsWith(`${sourceFilePath}/`)) {
                    const sourceFile = this.getFile(key);
                    this.addFile(targetFilePath + key.slice(sourceFilePath.length), sourceFile.cleanCode, sourceFile.type);
                }
            });
        }
        else {
            tango_helpers_1.logger.error('copyFiles failed, source: %s, target: %s', sourceFilePath, targetFilePath);
        }
    }
}
exports.Workspace = Workspace;
