import React from 'react';
import { IntlShape } from 'react-intl';
import { ConfigProviderProps } from 'antd/es/config-provider';
import { RequestMethod, RequestOptionsInit, RequestInterceptor, ResponseInterceptor, OnionMiddleware } from 'umi-request';
import 'moment/locale/zh-cn';
import 'moment/locale/zh-tw';
import 'moment/locale/zh-hk';
import 'moment/locale/ja';
export * from 'antd/es/config-provider';
export * from 'antd/es/config-provider/SizeContext';
/**
 * 提供Udesk-UI中的一些全局配置属性，比如设置request全局默认选项
 *
 * @interface UdeskContext
 */
interface UdeskContext {
    /**
     * 获取前是否处于 `debug` 模式，默认在开发模式下为 `true`，生产环境下为 `false`。
     * 可以在 `debug` 模式下控制台输出一些内部调试信息，或警告一些代码误用等。
     *
     * @type {boolean}
     * @memberof UdeskContext
     */
    debugMode: boolean;
    /**
     * 获取国际化对象，方便获取多语言资源，已经初始化为当前语言。
     *
     * `intl`对象的典型使用方式是: `intl.formatMessage({ id: 'key.name.with.dot' })`，
     * 可以获取指定 Key 对应的国际化文本。
     *
     * 如果希望文本里包含动态插值，需要在文本中使用 `{}` 占位符包裹字段名，
     * 例如：`"当前 ${pageNum} 页，共 ${pages} 页，总条数 {total}"`，
     * 然后在调用时第二个参数传递一个对象：`intl.formatMessage({ id: 'paginationTitle', { pageNum:1, pages: 20, total: 395 } })`
     *
     * @example
     * import { UdeskContext } from '../config-provider';
     *
     * const udeskContext = useContext(UdeskContext);
     * udeskContext.intl.formatMessage({ id: 'AutoComplete.searchError' })
     *
     * @type {IntlShape}
     * @memberof ReadonlyContext
     */
    intl: IntlShape;
    /**
     * 手动设置全局request实例，这是一个比较终极的做法，可以完全控制request的所有行为。
     *
     * 也可以通过设置 `requestOptions`来定义一些默认选项，这种方式比较常用也比较容易，
     * 以及通过 `requestInterceptors` 和 `responseInterceptors` 设置拦截器，通过 `requestMiddlewares` 属性来设置中间件。
     *
     * Request对象依赖于 `umi-request` 类库。
     *
     * @example
     * import request, { extend } from 'umi-request';
     *
     * // 方式一：使用extend方法创建一个实例
     * let requestInstance = extend({ ...options });
     *
     * // 方式二：直接使用umi中的全局request实例
     * let requestInstance = request;
     *
     * // 使用：传递给udesk-ui
     * <ConfigProvider request={request} />
     *
     * @type {RequestMethod}
     * @memberof UdeskContext
     */
    request: RequestMethod;
}
declare type UdeskConfigProviderProps = ConfigProviderProps & {
    /**
     * 手动设置全局request实例，这是一个比较终极的做法，可以完全控制request的所有行为。
     *
     * 也可以通过设置 `requestOptions`来定义一些默认选项，这种方式比较常用也比较容易，
     * 以及通过 `requestInterceptors` 和 `responseInterceptors` 设置拦截器，通过 `requestMiddlewares` 属性来设置中间件。
     *
     * Request对象依赖于 `umi-request` 类库。
     *
     */
    request?: RequestMethod;
    /**
     * 设置udesk-ui内部发送异步请求时的默认选项，比如可以给所有请求加上鉴权参数。
     *
     * 如果要设置请求拦截器的话，可以使用 `requestInterceptors` 和 `responseInterceptors` 属性，
     * 如果设置中间件的话，可以使用 `requestMiddlewares` 属性。
     *
     * 如果要完全控制请求的更多细节的话，可以使用 `request` 属性直接传递一个request对象
     *
     * @default
     * {
     *   requestType: 'json',
     *   responseType: 'json',
     *   headers: {
     *       'Content-Type': 'application/json',
     *   },
     *   credentials: 'include',
     * }
     *
     */
    requestOptions?: RequestOptionsInit;
    /**
     * 设置udesk-ui内部发送异步请求的Request拦截器。
     * 如果设置了 `request` 对象的话，此属性不起作用，你可以在创建request实例时，自己指定拦截器。
     */
    requestInterceptors?: RequestInterceptor[];
    /**
     * 设置udesk-ui内部发送异步请求的Response拦截器。
     * 如果设置了 `request` 对象的话，此属性不起作用，你可以在创建request实例时，自己指定拦截器。
     */
    responseInterceptors?: ResponseInterceptor[];
    /**
     * 设置udesk-ui内部发送异步请求的中间件处理函数。
     * 如果设置了 `request` 对象的话，此属性不起作用，你可以在创建request实例时，自己指定中间件。
     */
    requestMiddlewares?: OnionMiddleware[];
};
declare const UdeskContext: React.Context<UdeskContext>;
export { UdeskConfigProviderProps, UdeskContext };
export * from '../_utils/type';
declare const _default: React.FC<UdeskConfigProviderProps> & Omit<React.FC<import("antd/lib/config-provider").ConfigProviderProps> & {
    ConfigContext: React.Context<import("antd/lib/config-provider").ConfigConsumerProps>;
    SizeContext: React.Context<import("antd/lib/button").ButtonSize>;
    config: ({ prefixCls, iconPrefixCls, theme, }: Pick<import("antd/lib/config-provider").ConfigProviderProps, "prefixCls" | "iconPrefixCls"> & {
        theme?: import("antd/lib/config-provider/context").Theme | undefined;
    }) => void;
}, keyof React.FC<ConfigProviderProps>>;
export default _default;
