import { __awaiter } from "tslib";
import { makeObservable, observable, computed, action, toJS } from 'mobx';
import { observableGetIn, observableSetIn } from './helpers';
export class FormModel {
    get values() {
        return this._values;
    }
    set values(nextValues) {
        this._values = nextValues;
    }
    constructor(initValues, options = {}) {
        this.state = {};
        this._fieldMap = new Map();
        this._subModelMap = {};
        this._values = initValues !== null && initValues !== void 0 ? initValues : {};
        this.onChange = options === null || options === void 0 ? void 0 : options.onChange;
        makeObservable(this, {
            _values: observable,
            values: computed,
            state: observable,
            setValue: action,
        });
    }
    getValues() {
        return toJS(this.values);
    }
    getValue(name, defaultValue) {
        return observableGetIn(this.values, name, defaultValue);
    }
    setValue(name, value) {
        observableSetIn(this.values, name, value);
    }
    getField(name) {
        let field = this._fieldMap.get(name);
        if (field == null) {
            field = new Field({ parent: this, name });
            this._fieldMap.set(name, field);
        }
        return field;
    }
    getSubModel(name) {
        let subModel = this._subModelMap[name];
        if (!subModel) {
            subModel = new SubModel(this, name);
            this._subModelMap[name] = subModel;
        }
        return subModel;
    }
}
class SubModel {
    get values() {
        return this.parent.getValue(this.name);
    }
    set values(nextValues) {
        this.parent.setValue(this.name, nextValues);
    }
    constructor(parent, name) {
        this._fieldMap = new Map();
        this._subModelMap = {};
        this.parent = parent;
        this.name = name;
        makeObservable(this, {
            values: computed,
            setValue: action,
        });
    }
    getValue(name, defaultValue) {
        return observableGetIn(this.values, name, defaultValue);
    }
    setValue(name, value) {
        if (!this.values) {
            this.values = {};
        }
        observableSetIn(this.values, name, value);
    }
    getField(name) {
        let field = this._fieldMap.get(name);
        if (!field) {
            field = new Field({
                parent: this,
                name,
            });
            this._fieldMap.set(name, field);
        }
        return field;
    }
    getSubModel(name) {
        let subModel = this._subModelMap[name];
        if (!subModel) {
            subModel = new SubModel(this, name);
            this._subModelMap[name] = subModel;
        }
        return subModel;
    }
    getPath(name) {
        return [this.name, name].filter((item) => !!item).join('.');
    }
    onChange(name, values, field) {
        var _a, _b;
        (_b = (_a = this.parent).onChange) === null || _b === void 0 ? void 0 : _b.call(_a, this.getPath(name), values, field);
    }
}
export class Field {
    get value() {
        return this.parent.getValue(this.name);
    }
    set value(value) {
        this.parent.setValue(this.name, value);
    }
    get error() {
        return this.state.error;
    }
    constructor(options) {
        this.config = null;
        this.state = {};
        this.detail = {};
        this.validate = (trigger = '*') => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const shouldValidate = ['*', 'blur', 'change'].includes(trigger);
            if (!shouldValidate) {
                return;
            }
            const { validate } = this.config;
            const value = this.value;
            const actualValidate = () => __awaiter(this, void 0, void 0, function* () {
                if (validate) {
                    // FIXME: 重构 validate 的逻辑，和 antd Form 保持一致
                    return validate(value, this, trigger);
                }
                return null;
            });
            let cancelled = false;
            (_b = (_a = this.state).cancelValidation) === null || _b === void 0 ? void 0 : _b.call(_a);
            this.state.validating = true;
            this.state.cancelValidation = action(() => {
                cancelled = true;
                this.state.cancelValidation = null;
                this.state.validating = false;
            });
            return actualValidate().then(action((error) => {
                var _a, _b;
                if (cancelled) {
                    return;
                }
                this.state.cancelValidation = null;
                this.state.validating = false;
                this.state.error = error;
                // 校验错误，不触发 onChange
                if (error) {
                    return error;
                }
                (_b = (_a = this.parent) === null || _a === void 0 ? void 0 : _a.onChange) === null || _b === void 0 ? void 0 : _b.call(_a, this.name, this.value, this);
            }));
        });
        this.handleBlur = () => {
            return this.validate('blur');
        };
        this.handleChange = (nextValue, valueDetail) => {
            this.detail = valueDetail;
            this.value = nextValue;
            return this.validate('change');
        };
        this.parent = options.parent;
        this.name = options.name;
        makeObservable(this, {
            state: observable,
            value: computed,
            error: computed,
            validate: action,
            handleBlur: action,
            handleChange: action,
        });
    }
    setConfig(config) {
        this.config = config;
    }
}
