/**
 * 访问，遍历，修改 ast tree
 */
import * as t from '@babel/types';
import { TraverseOptions } from '@babel/traverse';
import { Dict } from '@music163/tango-helpers';
import type { IRouteData, IStorePropertyData, ITangoViewNodeData, IImportDeclarationPayload, InsertChildPositionType, IImportSpecifierData, IExportSpecifierData } from '../../types';
import { IdGenerator } from '../id-generator';
/**
 * 将 Node 节点构造为一棵树，并执行树的遍历操作，此操作会修改原始的节点
 * @param node
 * @param options
 * @returns 返回完整的 File AST
 */
export declare function traverseExpressionNode(node: t.Expression, options: TraverseOptions): t.File;
/**
 * 获取 jsx 节点的属性集
 * @param node
 * @returns
 */
export declare function getJSXElementAttributes(node: t.JSXElement): Record<string, any>;
/**
 * 获取 jsx 结点的子元素名字
 * @param node
 * @returns
 */
export declare function getJSXElementChildrenNames(node: t.JSXElement): string[];
/**
 * 查询所有XFormItem的字段名
 * @param ast 当前文件AST
 * @returns 所有 XFormItem 的字段名
 */
export declare function queryXFormItemFields(ast: t.File | t.JSXElement): string[];
/**
 * 根据 dnd id 判断是否是目标的 JSXElement
 * @param node
 * @param id
 */
export declare function isJSXElementById(node: t.JSXElement, jsxElementNodeId: string): boolean;
/**
 * 在目标 jsx 元素中添加子元素
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
export declare function appendChildToJSXElement(ast: t.File, targetJSXElementNodeId: string, newNode: t.JSXElement, position: InsertChildPositionType): t.File;
/**
 * 在目标 jsx 元素后添加兄弟节点
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 */
export declare function insertSiblingAfterJSXElement(ast: t.File, targetJSXElementNodeId: string, newNode: t.JSXElement): t.File;
/**
 * 在目标 jsx 元素前添加兄弟节点
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
export declare function insertSiblingBeforeJSXElement(ast: t.File, targetJSXElementNodeId: string, newNode: t.JSXElement): t.File;
/**
 * 替换目标 jsx 元素为新的元素
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
export declare function replaceJSXElement(ast: t.File, targetJSXElementNodeId: string, newNode: t.JSXElement): t.File;
/**
 * 删除目标 jsx 元素
 * @param ast
 * @param targetJSXElementNodeId
 */
export declare function removeJSXElement(ast: t.File, targetJSXElementNodeId: string): t.File;
/**
 * 替换跟结点的子元素
 * @example 例如替换 `return (<Box {...props}>empty block</Box>)` 的子元素
 * @param ast
 * @param nodes
 */
export declare function replaceRootJSXElementChildren(ast: t.File, nodes: t.JSXElement[]): t.File;
export declare function updateJSXElementAttribute(node: t.JSXElement, attrName: string, attrValue: any): t.JSXElement;
export declare function updateJSXAttributes(ast: t.File, nodeId: string, config: Record<string, any>): t.File;
/**
 * 生成新的 ImportDeclaration
 * @deprecated 使用 makeImportDeclaration2 代替
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
export declare function makeImportDeclarationLegacy(importedModule: IImportDeclarationPayload): t.ImportDeclaration;
export declare function makeImportDeclaration(source: string, specifiers: IImportSpecifierData[]): t.ImportDeclaration;
/**
 * 添加新的导入语句
 * @deprecated 使用 addImportDeclaration2 代替
 * @param ast
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
export declare function addImportDeclarationLegacy(ast: t.File, importedModule: IImportDeclarationPayload): t.File;
export declare function addImportDeclaration(ast: t.File, source: string, specifiers: IImportSpecifierData[]): t.File;
/**
 * 更新已有的导入语句
 * @deprecated 使用 updateImportDeclaration2 代替
 * @param ast
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
export declare function updateImportDeclarationLegacy(ast: t.File, importedModule: IImportDeclarationPayload): t.File;
export declare function updateImportDeclaration(ast: t.File, source: string, specifiers: IImportSpecifierData[]): t.File;
/**
 * 再已有的导入语句中添加新的导入符号
 * @param ast
 * @param source
 * @param newSpecifiers
 * @returns
 */
export declare function insertImportSpecifiers(ast: t.File, source: string, newSpecifiers: IImportSpecifierData[]): t.File;
/**
 * 新增类属性
 * @param ast
 * @param classPropertyNode
 * @returns
 */
export declare function addClassProperty(ast: t.File, classPropertyNode: t.ClassProperty): t.File;
/**
 * 更新类属性
 * @param ast
 * @param targetPropertyName
 * @param propertyValueNode
 * @returns
 */
export declare function updateClassProperty(ast: t.File, targetPropertyName: string, propertyValueNode: t.Expression): t.File;
/**
 * 在 routes.js 中添加新的路由规则
 * @param ast
 * @param pageName
 * @returns
 */
export declare function addRouteToRouteFile(ast: t.File, routePath: string, importFilePath: string): t.File;
/**
 * 更新页面路由
 * @param ast
 * @param oldRoutePath
 * @param newRoutePath
 * @returns
 */
export declare function updateRouteToRouteFile(ast: t.File, oldRoutePath: string, newRoutePath: string): t.File;
export declare function removeRouteFromRouteFile(ast: t.File, routePath: string, importPath: string): t.File;
export declare function traverseRouteFile(ast: t.File): IRouteData[];
/**
 * 在 stores/index.js 中添加新的store
 * @param ast
 * @param storeName
 * @returns
 */
export declare function addStoreToEntryFile(ast: t.File, storeName: string): t.File;
/**
 * 在 stores/index.js 中移除一行导出语句
 * @example export { default as app } from "./app";
 * @param ast
 * @param storeName
 * @returns
 */
export declare function removeStoreToEntryFile(ast: t.File, storeName: string): t.File;
export declare function traverseStoreEntryFile(ast: t.File): string[];
export declare function addStoreState(ast: t.File, stateName: string, initValue: string): t.File;
export declare function removeStoreState(ast: t.File, stateName: string): t.File;
export declare function updateStoreState(ast: t.File, stateName: string, code: string): t.File;
export declare function traverseStoreFile(ast: t.File): {
    namespace: string;
    states: IStorePropertyData[];
    actions: IStorePropertyData[];
};
export declare function deleteServiceConfigFromServiceFile(ast: t.File, serviceFunctionName: string): t.File;
/**
 * 将服务函数的配置参数转为 ast 节点
 * @param payload
 * @returns
 */
export declare function serviceConfig2Node(payload: object): t.Expression;
export declare function updateServiceConfigToServiceFile(ast: t.File, config: Dict<object>): t.File;
export declare function updateBaseConfigToServiceFile(ast: t.File, config: object): t.File;
export declare function traverseServiceFile(ast: t.File): {
    imports: Record<string, IImportSpecifierData[]>;
    services: undefined;
    baseConfig: any;
};
/**
 * 清除文件中的无效导入代码
 * @param ast
 * @returns
 */
export declare function removeUnusedImportSpecifiers(ast: t.File): t.File;
/**
 * 克隆一个全新的 JSXElement 节点
 * @param node
 * @returns
 */
export declare function cloneJSXElement(node: t.JSXElement, overrideProps?: Dict): t.JSXElement;
export declare function traverseViewFile(ast: t.File, idGenerator: IdGenerator): {
    ast: t.File;
    cleanAst: t.File;
    nodes: ITangoViewNodeData<t.JSXElement>[];
    imports: Record<string, IImportSpecifierData[]>;
    importedModules: Dict<IImportDeclarationPayload | IImportDeclarationPayload[]>;
    variables: string[];
};
export declare function traverseComponentsEntryFile(ast: t.File, baseDir?: string): {
    ast: t.File;
    exportMap: Record<string, IExportSpecifierData>;
};
/**
 * 基本的文件解析过程
 * @param ast
 */
export declare function traverseFile(ast: t.File): {
    imports: Record<string, IImportSpecifierData[]>;
};
