import axios from 'axios';
import { mergeObjects } from './helpers';
function createInstance(instanceConfig) {
    // 避免多个实例产生冲突
    const instance = axios.create(instanceConfig);
    // request 拦截器
    instance.interceptors.request.use((config) => {
        const { pathVariables } = config;
        if (pathVariables) {
            config.url = config.url.replace(/\/:(\w+)/gi, (_, key) => `/${pathVariables[key]}`);
        }
        return config;
    }, (error) => {
        return Promise.reject(error);
    });
    // response 拦截器
    instance.interceptors.response.use((response) => {
        let data = response.data;
        const { formatter } = response.config;
        if (formatter) {
            data = formatter(data);
        }
        /**
         * data 格式
         * {
         *   code: 200, // 必选，业务状态码
         *   message: 'success', // 必选，业务状态描述
         *   data: {} | [], // 必选，业务数据
         * }
         */
        const { code = 200, message } = data;
        if (code !== 200) {
            throw new Error(`${code}: ${message || 'request failed!'}, ${response.config.url}`);
        }
        if (!('data' in data)) {
            console.warn('response data should have a data field', response.config.url);
        }
        return data.data;
    }, (error) => {
        return Promise.reject(error);
    });
    // 扩展 get 方法
    instance.get = (url, params, config) => {
        return instance.request(Object.assign({ url, params }, config));
    };
    // 扩展 post 方法
    instance.postFormData = (url, data, config) => {
        return instance.post(url, data, mergeObjects(config, {
            headers: {
                'Content-Type': 'multipart/form-data',
            },
        }));
    };
    // 扩展 post 方法
    instance.postFormUrlencoded = (url, data, config) => {
        return instance.post(url, data, mergeObjects(config, { headers: { 'Content-Type': 'application/x-www-form-urlencoded' } }));
    };
    return instance;
}
export default createInstance();
