import React, { useMemo, useState } from 'react';
import { Box, Grid, Text } from 'coral-system';
import styled from 'styled-components';
import { logger, upperCamelCase, } from '@music163/tango-helpers';
import { CollapsePanel, IconFont, Search, Tabs } from '@music163/tango-ui';
import { observer, useWorkspace } from '@music163/tango-context';
import { QuestionCircleOutlined } from '@ant-design/icons';
import { Button, Empty, Spin, Popover } from 'antd';
import { getDragGhostElement } from '../helpers';
const localeMap = {
    common: '基础组件',
    atom: '原子组件',
    snippet: '组合',
    bizComp: '业务组件',
    localComp: '本地组件',
};
const emptyMenuData = {
    common: [],
};
export function useFlatMenuData(menuData) {
    const flatList = useMemo(() => {
        const set = new Set();
        Object.values(menuData).forEach((cates) => {
            cates.forEach((cate) => {
                var _a;
                (_a = cate.items) === null || _a === void 0 ? void 0 : _a.forEach((item) => {
                    set.add(item);
                });
            });
        });
        return Array.from(set);
    }, [menuData]);
    return [
        {
            title: '搜索结果',
            items: flatList,
        },
    ];
}
export const ComponentsPanel = observer(({ menuData = emptyMenuData, showBizComps = true, getBizCompName = upperCamelCase, loading = false, }) => {
    const [keyword, setKeyword] = useState('');
    const allList = useFlatMenuData(menuData);
    const workspace = useWorkspace();
    const [bizCompData, localCompData] = useMemo(() => [
        [{ title: '已安装业务组件', items: workspace.bizComps.map(getBizCompName) }],
        [{ title: '本地组件', items: workspace.localComps }],
    ], [workspace.bizComps, workspace.localComps, getBizCompName]);
    const tabs = Object.keys(menuData).map((key) => ({
        key,
        label: localeMap[key],
        children: React.createElement(MaterialList, { data: menuData[key] }),
    }));
    if (showBizComps) {
        tabs.push({
            key: 'bizComp',
            label: localeMap.bizComp,
            children: React.createElement(MaterialList, { type: "bizComp", data: bizCompData }),
        });
    }
    if (localCompData.length) {
        tabs.push({
            key: 'localComps',
            label: localeMap.localComp,
            children: React.createElement(MaterialList, { data: localCompData }),
        });
    }
    const contentNode = tabs.length === 1 ? (tabs[0].children) : (React.createElement(Tabs, { centered: true, isTabBarSticky: true, tabBarStickyOffset: 48, items: tabs }));
    return (React.createElement(Box, { className: "ComponentsView", overflowY: "auto" },
        React.createElement(Box, { px: "l", py: "m", position: "sticky", top: "0", zIndex: 1, bg: "white" },
            React.createElement(Search, { placeholder: "\u641C\u7D22\u7269\u6599", onChange: setKeyword })),
        React.createElement(Spin, { spinning: loading, tip: "\u6B63\u5728\u52A0\u8F7D\u7269\u6599\u5217\u8868..." }, !keyword ? contentNode : React.createElement(MaterialList, { data: allList, filterKeyword: keyword }))));
});
function MaterialList({ data, filterKeyword, type = 'common' }) {
    const workspace = useWorkspace();
    return (React.createElement(Box, { className: "ComponentsViewList" }, data.map((cate) => {
        let items = cate.items || [];
        if (filterKeyword) {
            items = items.filter((item) => {
                const pattern = new RegExp(filterKeyword, 'ig');
                const prototype = workspace.componentPrototypes.get(item);
                return prototype && (pattern.test(prototype.name) || pattern.test(prototype.title));
            });
        }
        const addBlock = items.length % 2 === 1;
        return (React.createElement(Box, { key: cate.title },
            React.createElement(CollapsePanel, { key: cate.title, title: cate.title, borderBottom: "solid", borderColor: "line.normal" },
                !items.length && (React.createElement(Empty, { image: Empty.PRESENTED_IMAGE_SIMPLE, description: "\u6CA1\u6709\u5339\u914D\u5230\u4EFB\u4F55\u7EC4\u4EF6" })),
                React.createElement(Grid, { columns: type === 'localComp' ? 1 : 2, spacing: "1px", bg: "background.normal", padding: "0" },
                    items.map((item) => {
                        const prototype = workspace.componentPrototypes.get(item);
                        if (!prototype) {
                            logger.log(`<${item}> prototype not found!`);
                        }
                        return prototype ? React.createElement(MaterialGrid, { key: item, data: prototype }) : null;
                    }),
                    addBlock && React.createElement(Box, { bg: "white" })))));
    })));
}
const StyledCommonGridItem = styled.div `
  position: relative;
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  padding: 8px;
  cursor: move;
  text-align: center;
  color: var(--tango-colors-text-body);
  background-color: #fff;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;

  .material-icon {
    font-size: 40px;
  }

  .info {
    position: absolute;
    right: 4px;
    top: 4px;
    cursor: pointer;
  }

  .anticon-question-circle {
    display: none;
    position: absolute;
    top: 8px;
    right: 8px;
  }

  img {
    height: 40px;
    width: 40px;
  }

  &:hover {
    box-shadow: 0 0 10px rgb(0 0 0 / 10%);

    > span {
      color: var(--tango-colors-brand);
    }

    .anticon-question-circle {
      display: inline-block;
    }
  }
`;
function MaterialGrid({ data }) {
    const workspace = useWorkspace();
    const handleDragStart = (e) => {
        e.dataTransfer.effectAllowed = 'move';
        workspace.dragSource.set({
            name: data.name,
        });
        const ghost = getDragGhostElement();
        e.dataTransfer.setDragImage(ghost, 0, 0);
    };
    const handleDragEnd = () => {
        workspace.dragSource.clear();
    };
    const icon = data.icon || 'icon-placeholder';
    return (React.createElement(StyledCommonGridItem, { draggable: true, "data-name": data.name, onDragStart: handleDragStart, onDragEnd: handleDragEnd },
        icon.startsWith('icon-') ? (React.createElement(IconFont, { className: "material-icon", type: data.icon || 'icon-placeholder' })) : (React.createElement("img", { src: icon, alt: data.name })),
        React.createElement(Text, { fontSize: "12px", lineHeight: "1.5" }, data.title),
        React.createElement(Text, { fontSize: "12px", color: "gray.50" }, data.name),
        data.docs || data.help ? (React.createElement(Popover, { placement: "right", title: data.title, content: React.createElement(CommonMaterialInfoBox, Object.assign({}, data)) },
            React.createElement(QuestionCircleOutlined, null))) : null));
}
function CommonMaterialInfoBox({ help, docs }) {
    return (React.createElement(Box, { maxWidth: 300 },
        !!help && React.createElement(Box, { mb: "m" }, help),
        !!docs && (React.createElement(Button, { size: "small", block: true, target: "_blank", href: docs }, "\u67E5\u770B\u7EC4\u4EF6\u6587\u6863"))));
}
