import React, { useEffect, useState } from 'react';
import { Dropdown, Space, Tree } from 'antd';
import { Box, css } from 'coral-system';
import { IconFont } from '@music163/tango-ui';
import { EyeOutlined, EyeInvisibleOutlined, EllipsisOutlined } from '@ant-design/icons';
import { observer, useWorkspace } from '@music163/tango-context';
import { DropMethod } from '@music163/tango-core';
import { noop, parseDndId } from '@music163/tango-helpers';
import { useSandboxQuery } from '../../context';
import { buildQueryBySlotId } from '../../helpers';
const outlineStyle = css `
  .ant-tree-title {
    color: rgb(85, 85, 85);
  }

  .ant-tree .ant-tree-treenode {
    padding: 2px;
  }

  .ant-tree-indent-unit {
    width: 12px;
  }

  .ant-tree .ant-tree-node-content-wrapper {
    white-space: nowrap;
    transition: none;
  }
  .variable-row {
    font-size: 12px;
  }
  .ant-tree-switcher-noop {
    display: none;
  }
  .material-icon {
    width: 1em;
    height: 1em;
  }
`;
const filedNames = {
    title: 'component',
    key: 'id',
    children: 'children',
};
const getNodeKeys = (data) => {
    const ids = [];
    data === null || data === void 0 ? void 0 : data.forEach((node) => {
        ids.push(node.id);
        if (node.children) {
            ids.push(...getNodeKeys(node.children));
        }
    });
    return ids;
};
const OutlineTreeNode = observer(({ node, showToggleVisibleIcon, actionItems }) => {
    const workspace = useWorkspace();
    const sandboxQuery = useSandboxQuery();
    const [visible, setVisible] = useState(true);
    const nodeLabel = (() => {
        const { codeId } = parseDndId(node.id);
        return codeId;
    })();
    const componentPrototype = workspace.componentPrototypes.get(node.component);
    const icon = (componentPrototype === null || componentPrototype === void 0 ? void 0 : componentPrototype.icon) || 'icon-placeholder';
    const iconRender = icon.startsWith('icon-') ? (React.createElement(IconFont, { className: "material-icon", type: icon })) : (React.createElement("img", { className: "material-icon", src: icon, alt: componentPrototype.name }));
    const toggleVisible = (nodeId) => {
        sandboxQuery.getElement(buildQueryBySlotId(nodeId)).style.visibility = visible
            ? 'hidden'
            : 'visible';
        setVisible(!visible);
    };
    const VisibleIcon = visible ? EyeOutlined : EyeInvisibleOutlined;
    return (React.createElement(Space, null,
        iconRender,
        " ",
        nodeLabel,
        showToggleVisibleIcon && (React.createElement(VisibleIcon, { className: "material-icon", onClick: (e) => {
                e.stopPropagation();
                toggleVisible(node.id);
            } })),
        (actionItems === null || actionItems === void 0 ? void 0 : actionItems.length) > 0 && (React.createElement(Dropdown, { menu: {
                items: actionItems.map((_) => ({
                    key: _.key,
                    label: _.item,
                })),
            }, trigger: ['click'] },
            React.createElement(EllipsisOutlined, null)))));
});
export const ComponentsTree = observer(({ showToggleVisibleIcon = true, onSelect = noop, actionItems }) => {
    var _a, _b;
    const workspace = useWorkspace();
    const sandboxQuery = useSandboxQuery();
    const [selectedKeys, setSelectedKeys] = useState(workspace.selectSource.selected.map((item) => item.id));
    const file = workspace.activeViewModule;
    const nodesTree = ((_a = file === null || file === void 0 ? void 0 : file.nodesTree) !== null && _a !== void 0 ? _a : []);
    const [expandedKeys, setExpandedKeys] = useState(getNodeKeys(nodesTree));
    useEffect(() => {
        setSelectedKeys(workspace.selectSource.selected.map((item) => item.id));
    }, [workspace.selectSource.selected]);
    useEffect(() => {
        setExpandedKeys(getNodeKeys(nodesTree));
        // nodeTree 更新后重新计算expandKeys
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [(_b = nodesTree === null || nodesTree === void 0 ? void 0 : nodesTree[0]) === null || _b === void 0 ? void 0 : _b.id]);
    if (!nodesTree.length) {
        return (React.createElement(Box, null,
            "\u6CA1\u6709\u6307\u5B9A\u6587\u4EF6\uFF0C\u6216\u6587\u4EF6",
            React.createElement("code", null, workspace.activeViewFile),
            "\u6CA1\u6709\u6709\u6548\u7684\u8282\u70B9\u6811"));
    }
    return (React.createElement(Box, { css: outlineStyle },
        React.createElement(Tree, { selectedKeys: selectedKeys, fieldNames: filedNames, treeData: nodesTree, onSelect: (keys) => {
                const slotKey = keys === null || keys === void 0 ? void 0 : keys[0];
                const data = sandboxQuery.getDraggableParentsData(buildQueryBySlotId(slotKey), true);
                if (data && data.id) {
                    workspace.selectSource.select(data);
                }
                // export selected
                onSelect(slotKey);
                setSelectedKeys(keys);
            }, blockNode: true, expandedKeys: expandedKeys, onExpand: (keys) => setExpandedKeys(keys), draggable: true, onDragStart: (data) => {
                const prototype = workspace.componentPrototypes.get(data.node.component);
                if (!prototype) {
                    return;
                }
                const { canDrag } = prototype.rules || {};
                if (canDrag && !canDrag()) {
                    return;
                }
                workspace.dragSource.set({
                    id: data.node.id,
                    name: data.node.component,
                });
            }, onDrop: (data) => {
                const dropKey = data.node.key;
                let method;
                if (data.dropToGap) {
                    // 插入节点的后面
                    method = DropMethod.InsertAfter;
                }
                else {
                    // 作为第一个子节点
                    method = DropMethod.InsertFirstChild;
                }
                workspace.dragSource.dropTarget.set({
                    id: dropKey,
                }, method);
                workspace.dropNode();
            }, titleRender: (node) => (React.createElement(OutlineTreeNode, { actionItems: actionItems, showToggleVisibleIcon: showToggleVisibleIcon, node: node })) })));
});
