declare const EVENT_MANAGER_SYMBOL: unique symbol;
export default class EventedClass<T = string> {
    #private;
    private [EVENT_MANAGER_SYMBOL]?;
    /**
     * 把任意对象改造成 EventedClass 类型，支持注册事件，并不改变原始对象的引用
     *
     * @static
     * @template TO 对象类型
     * @param {TO} target 要改造的对象
     * @returns {TTarget<TO, TE>}
     * @memberof EventedClass<T>
     */
    static injectTo<TO = unknown, TE = string>(target: TO): TTarget<TO, TE>;
    /**
     * 注册事件
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @param {(...args: any[]) => void} handler 事件回调函数
     * @returns {EventedClass<T>}
     * @memberof EventedClass<T>
     */
    on(eventName: T, handler: (...args: any[]) => void): EventedClass<T>;
    /**
     * 注册事件，较少使用
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @param {string} eventSource 设置一个事件源，类似分组的功能，在触发事件时可以选择同一事件源的分组
     * @param {(...args: any[]) => void} handler 事件回调函数
     * @returns {this}
     * @memberof EventedClass<T>
     */
    on(eventName: T, eventSource: string, handler: (...args: any[]) => void): this;
    /**
     * 移除所有事件
     *
     * @returns {this}
     * @memberof EventedClass<T>
     */
    off(): this;
    /**
     * 移除指定事件
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @returns {this}
     * @memberof EventedClass<T>
     */
    off(eventName: T): this;
    /**
     * 移除某个事件的特定回调函数
     *
     * @param {T} eventName 事件名称
     * @param {(args: any[]) => void} handler 要移除的回调函数，引用要跟注册时的保持一致
     * @returns {this}
     * @memberof EventedClass<T>
     */
    off(eventName: T, handler: (args: any[]) => void): this;
    /**
     * 移除某个事件的特定回调函数
     *
     * @param {T} eventName 事件名称
     * @param {string} eventSource 事件源名称
     * @param {(args: any[]) => void} handler 要移除的回调函数，引用要跟注册时的保持一致
     * @returns {this}
     * @memberof EventedClass<T>
     */
    off(eventName: T, eventSource: string, handler: (args: any[]) => void): this;
    /**
     * 触发事件
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @returns {this}
     * @memberof EventedClass<T>
     */
    trigger(eventName: T): this;
    /**
     * 触发事件
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @param {any[]} [args] 事件参数，数组形式
     * @returns {this}
     * @memberof EventedClass<T>
     */
    trigger(eventName: T, args?: any[]): this;
    /**
     * 触发事件
     *
     * @param {T} eventName 事件名称，支持命名空间，例如 click.mine、click.yours
     * @param {any[]} [args] 事件参数，数组形式
     * @param {string} eventSource 事件源名称
     * @returns {this}
     * @memberof EventedClass<T>
     */
    trigger(eventName: T, eventSource: string, args?: any[]): this;
}
declare type TTarget<TO, TE> = TO & EventedClass<TE>;
export {};
