import Raven from 'raven-js';
declare const defaults: {
    dsn: string;
    debug: boolean;
    releaseVersion: string;
    userIdField: string;
    userNameField: string;
    userEmailField: string;
    ignoreClientOnlyErrors: boolean;
    maxResponseTextLength: number;
    ignoreResponseCodes: never[];
};
export declare class ReactSentry {
    #private;
    static options: SentryOptions & Required<Pick<SentryOptions, keyof typeof defaults>>;
    static init(options: SentryOptions): void;
    static get initialized(): boolean;
    static ready(): void;
    static captureException: (error: Error | ErrorEvent | string, options?: Raven.RavenOptions | undefined) => void;
    static captureMessage(message: string, options?: Raven.RavenOptions): void;
    static _enableGlobalErrorCatching(): void;
    static _extractMessage(reason: any): any;
    static _getCaptureTags(error: any): Record<string, string>;
}
/** Sentry初始化options */
export declare type SentryOptions = {
    /**
     * Sentry DSN地址，必填。每个项目对应一个DSN地址。
     *
     * @type {string}
     */
    dsn: string;
    /**
     * 是否启用debug模式，会输出一些内部日志，默认为false。
     *
     * @default false
     *
     * @type {boolean}
     */
    debug?: boolean;
    /**
     * 设置当前宿主应用（或sdk）的releaseVersion，在sentry后台查看时，可以区分出来是哪个版本的应用产生的错误，默认为空。
     *
     * @type {string}
     */
    releaseVersion?: string;
    /**
     * 是否自动忽略visible=true的错误，一般这种错误都是服务端的验证，是要显示给客户看的，
     * 比如用户名、密码错误，或者非空校验错误，这种错误一般不需要上报到sentry。默认为`false`。
     *
     * @default false
     *
     * @type {boolean}
     */
    ignoreClientOnlyErrors?: boolean;
    /**
     * 自动忽略code为这些值的Response，默认值为 `[]`
     *
     * @default []
     *
     * @type {number[]}
     */
    ignoreResponseCodes?: number[];
    /**
     * 自动记录的ResponseText的最大长度，默认值为 `0`，即不限制ResponseText的长度。
     *
     * @default 0
     *
     * @type {number}
     */
    maxResponseTextLength?: number;
    /**
     * 设置一个回调函数，获取当前登录用户的对象。
     *
     */
    getUser?: () => object;
    /**
     * 从 `getUser` 返回的user对象中获取 userId， 默认为 `'id'`。
     *
     * 这是sentry的一个系统字段，如果填写了，可以在sentry后台方便查看当前登录用户是哪个人
     *
     * @default 'id'
     *
     * @type {string}
     */
    userIdField?: string;
    /**
     * 从 `getUser` 返回的user对象中获取 userName， 默认为 `'name'`。
     *
     * 这是sentry的一个系统字段，如果填写了，可以在sentry后台方便查看当前登录用户是哪个人
     *
     * @default 'name'
     *
     * @type {string}
     */
    userNameField?: string;
    /**
     * 从 `getUser` 返回的user对象中获取 userEmail， 默认为 `'email'`。
     *
     * 这是sentry的一个系统字段，如果填写了，可以在sentry后台方便查看当前登录用户是哪个人
     *
     * @default 'email'
     *
     * @type {string}
     */
    userEmailField?: string;
    /**
     * 设置一个回调函数，获取当前页面的一些额外上下文信息，
     * 当在sentry后台查看错误时，可以看到这些上下文信息，便于分析错误。
     *
     */
    getExtraContext?: () => object;
    /**
     * 设置一个回调函数，返回当前上报错误的自定义标签，便于在sentry中按照标签分类来检索错误。
     *
     */
    getTagsContext?: () => object;
    /**
     * 设置一个自定义回调函数，可以自定义决定是否上报错误。这是在错误上报前的最后一个阶段，
     * 如果返回true，则上报，如果返回false，则不上报。
     *
     */
    getShouldSendCallback?: (data: any) => boolean;
};
export default ReactSentry;
