import { Mode } from './const';
import { IRoute as RuntimeRoute } from '@umijs/renderer-react';

export type PluginConfig = PurePluginConfig;
export type PurePluginConfig = {
    /**
     * 设置主sdk的名称，在未指定sdk时使用默认名称，默认值为 `main`
     *
     * @default "main"
     *
     * @type {string}
     */
    defaultSdk?: string;

    /**
     * 设置每个sdk输出的主入口文件名。
     * 每个sdk的名称只是作为输出目录名，默认输出文件名是 umi.js，
     * 此配置可以自定义文件名
     *
     * @default "umi"
     *
     * @type {string}
     */
    outputFilename?: string;

    /**
     * Sdk的版本号，默认值为 `1.0.1`
     *
     * @default "1.0.1"
     *
     * @type {string}
     */
    version?: string;

    /**
     *
     * 在系统启动或编译时，显示当前正在编译的sdk名称
     *
     * @default true
     *
     * @type {boolean}
     */
    printSdkName?: boolean;

    /**
     * manifest.json 文件名是否带上hash
     *
     * @default false
     *
     * @type {boolean}
     */
    manifestHash?: boolean;
    /**
     * 多sdk打包模式下，output目录
     *
     * @default undefined
     *
     * @type {string}
     */
    multipleModeOutputDir?: string;
};
export type ResolvedPluginConfig = Required<PurePluginConfig>;

export interface SdkRuntime {
    getSdkOptions?(initialValue: any, context: SdkRuntimeContext): any;
}
export type SdkRuntimeContext = {
    sdkConfig: ResolvedRuntimeSdkConfig;
    pluginConfig: PluginConfig;
    pluginState: PluginState;
};

type ConfigDescriptor = {
    path: string;
    filename: string;
    isMain: boolean;
    includeRoutes?: boolean;
    routes?: RootRoutes;
    componentsMapping?: {
        name: string;
        path: string;
    }[];
};
/**
 * 三种路由规则：
 * 1. 完整的路由表
 * 2. 纯字符串数组，支持glob表达式，匹配路由自动添加所有父节点
 * 3. 混合模式，字符串自动继承父节点的路径，可以设置base
 */
export type SdkConfigFile = {
    routes: RootRoutes;
    name?: string;
    mode?: Mode;
    disableLayout?: boolean;
    autoRun?: boolean;
    uniqueId?: string;
    /**
     * 自定义sdk的入口文件，绝对路径或者相对于主项目根目录的相对路径，包含文件后缀名
     *
     * @type {string}
     */
    entry: string;
    vendorScripts?: boolean | (string | IScript)[];
    vendorStyles?: boolean | (string | ICss)[];
};
export type RuntimeSdkConfig = ConfigDescriptor &
    Required<Omit<SdkConfigFile, 'routes'>> & {
        optionsName: string;
        rendererName: string;
        startupName: string;
        optionsPath: string;
        rendererPath: string;
        entryPath: string;
        componentEntryPath?: string;
        startupPath: string;
        routes: ResolvedRoute[];
    };
type ResolvedRoute = Route & { routes?: ResolvedRoute[] };
export type ResolvedRuntimeSdkConfig = Omit<RuntimeSdkConfig, 'routes'> & {
    routes: RuntimeRoute[];
};
export interface IScript extends Partial<HTMLScriptElement> {
    content?: string;
    destroyable?: boolean;
}
export interface ICss extends Partial<HTMLLinkElement> {
    content?: string;
    destroyable?: boolean;
}
export type RootRoutes = string[] | Glob[] | (string | Glob)[] | RoutePattern[];
export type RoutePattern = Route | Glob | string;
export type Route = {
    path: string;
    component: string;
    exact?: boolean;
    routes?: RoutePattern[];
    title?: string;
    [key: string]: any;
};
export type Glob = {
    root: string;
    includeRoot: boolean;
    patterns: string[] /* (Glob | string)[] 有点太麻烦了*/;
};

export type PluginState = {
    startupSdkName: string;
    multipleMode: boolean;
    startupConfig: RuntimeSdkConfig;
    sdkConfigs: RuntimeSdkConfig[];
};
