import type { ComponentClass } from 'react';
import { SentryOptions } from './react-sentry';
import EventedClass from './evented-class';
export declare const DefaultOptions: UdeskSdkOptions;
export declare abstract class UdeskSdkClass<TO extends UdeskSdkOptions = UdeskSdkOptions, TG = any, TE = string> extends EventedClass<TE> {
    #private;
    options: TO;
    static defaultOptions: UdeskSdkOptions;
    protected static scriptUrl?: string;
    protected rootContext: RootContext<TO, TG>;
    constructor(options: TO);
    abstract renderTo(container: string | HTMLElement): Promise<RenderHookContext<TO, TG>>;
    beforeRender({ container, rootContext }: RenderHookContext<TO, TG>): void;
    update(): void;
    update(sdkOptions: TO): void;
    protected abstract routes(): IRoute[];
    destroy(): void;
    version(): string | undefined;
    protected captureException(error: Error): void;
    protected versionFileUrl(): string;
    protected globalConfigFileUrl(): string;
    protected onVersionChanged(version: string, oldVersion: string): void;
    static bindToCurrentScript<TSdk extends UdeskSdkClass = UdeskSdkClass>(sdkClass: SdkClassConstructor<TSdk>, loadingScriptElement: TScriptElement<TSdk>, sdkName?: string): void;
}
export default UdeskSdkClass;
export declare type SdkClassConstructor<T extends UdeskSdkClass = UdeskSdkClass> = {
    new (): T;
};
export interface UdeskSdkOptions {
    sdkClientPrefix?: string;
    sdkServerPrefix?: string;
    appClientPrefix?: string;
    appServerPrefix?: string;
    reloadCallback?: () => void;
    enableGlobalConfig?: boolean;
    /**
     * 全局配置文件的路径，可以是绝对路径，也可以是相对路径，路径基准是sdk主入口文件。
     *
     * @default "./global-config.json"
     *
     * @example
     * 假如sdk主文件的路径是 `http://www.udesk.cn/site/sdk/1.1.0/main.js`，
     * 1. 如果`globalConfigFilePath`的值为 `.global-config.json`，则实际路径为 `http://www.udesk.cn/site/sdk/1.1.0/global-config.json`
     * 2. 如果`globalConfigFilePath`的值为 `/global-config.json`，则实际路径为 `http://www.udesk.cn/global-config.json`
     *
     * @type {string}
     */
    globalConfigFilePath?: string;
    /**
     * Version文件的路径，可以是绝对路径，也可以是相对路径，路径基准是sdk主入口文件。
     *
     * @default "./RELEASE-VERSION"
     *
     * @example
     * 假如sdk主文件的路径是 `http://www.udesk.cn/site/sdk/1.1.0/main.js`，
     * 1. 如果`versionFilePath`的值为 `./RELEASE-VERSION`，则实际路径为 `http://www.udesk.cn/site/sdk/1.1.0/RELEASE-VERSION`
     * 2. 如果`versionFilePath`的值为 `/RELEASE-VERSION`，则实际路径为 `http://www.udesk.cn/RELEASE-VERSION`
     *
     * @type {string}
     */
    manifestPath?: string;
    enableVersionMonitoring?: boolean;
    versionMonitorInterval?: number;
    enableNewVersionTip?: boolean;
    routerBase?: string;
    landingPath?: string;
    lang?: string;
    prefixCls?: string;
    /**
     * 是否启用错误自动上报，如果启用则必须提供sentryOptions属性，默认为false。
     *
     * @type {boolean}
     * @memberof UdeskSdkOptions
     */
    enableSentry?: boolean;
    /**
     * sentry的配置，如果启用了错误自动上报，则必须提供此属性。
     *
     * @type {SentryOptions}
     * @memberof UdeskSdkOptions
     */
    sentryOptions?: SentryOptions;
}
export interface RootContext<TO extends UdeskSdkOptions = UdeskSdkOptions, TG = any> {
    sdkOptions: TO;
    globalConfig?: TG;
    onSdkOptionsChanged?: (sdkOptions?: TO) => void;
    onVersionChanged?: (version: string, oldVersion: string) => void;
}
export interface RenderHookContext<TO extends UdeskSdkOptions = UdeskSdkOptions, TG = any> {
    container: HTMLElement;
    rootContext: RootContext<TO, TG>;
}
export interface IRoute {
    path?: string;
    exact?: boolean;
    redirect?: string;
    component?: ComponentClass;
    routes?: IRoute[];
    key?: any;
    strict?: boolean;
    sensitive?: boolean;
    [k: string]: any;
}
export declare type TScriptElement<TSdk extends UdeskSdkClass = UdeskSdkClass> = HTMLScriptElement & {
    sdkClass?: SdkClassConstructor<TSdk>;
    componentSdkClass?: SdkClassConstructor<TSdk>;
    onSdkLoaded?: ({ sdkClass, }: {
        sdkClass: SdkClassConstructor<TSdk>;
        componentSdkClass: SdkClassConstructor<TSdk>;
    }) => void;
    __mfsu?: boolean;
};
