/**
 * 一种简便写法，调用`callback`回调方法之前先调用`initLicense`方法，完成后自动调用`removeLicense`方法
 *
 * @export
 * @param {() => any} callback 用户自定义回调函数，支持返回值。在此方法调用前后会自动初始化和销毁license。
 * @param {string} [encryptedData] [可选] 如果有其它渠道获取`License`加密信息的话，可以传入，优先使用用户自定义License。如果不传的话，使用默认License。
 * @param {string} [encryptKey] [可选] 也支持自定义加密密钥
 * @returns {*} 透传callback的返回值
 *
 * @example
 * 示例代码：
 *
 * import { wl } from 'yfiles/lib/utils';
 * const graphComponent = wl(() => new GraphComponent('#graphComponent'));
 */
export function wl(callback: () => any, encryptedData?: string, encryptKey?: string): any;

/**
 * 初始化`yFiles`的License数据，`yFiles`必须在设置了License之后才可以工作。
 *
 * @export
 * @param {string} [encryptedData] [可选] License的加密数据，一般不需要设置。如果不设置，会使用全局默认数据。
 * @param {string} [encryptKey] [可选] 设置加密密钥，如果不设置，则使用全局默认密钥
 *
 * @example
 * 示例代码：
 *
 * import { ll } from 'yfiles/lib/utils';
 * ll();
 */
export function ll(encryptedData?: string, encryptKey?: string): void;

/**
 * 手动移除License，防止license数据泄露。
 * 需要在每次实例化yfiles类型的时候，先调用`ll`方法初始化license，
 * 并在实例化完成后，立即调用此方法移除。
 *
 * @summary
 * 为什么需要移除License？
 * 当我们初始化license后，数据会留存在内存中，
 * 如果有人通过反向工程获取license信息的话，理论上是可以获取到的。
 * 如果用完后立即移除的话，会最大程度保持信息的安全。
 *
 * 当然，这也会让我们有点麻烦。如果不移除的话，我们只需要初始化一次，
 * 就可以在任意时间使用yfiles。如果为了更安全的话，就需要我们在每次
 * 实例化yfiles几大模块类型的时候，先初始化一下，当实例化完成后，立即
 * 调用本方法移除。
 *
 * @example
 * 示例代码：
 *
 * import { rl } from 'yfiles/lib/utils';
 * rl();
 * @export
 */
export function rl(): void;
