import { series, parallel } from 'gulp';
import {
  clean,
  copy,
  copyFlat,
  generateIcons,
  generateEntry,
  generateInline,
  generateSvgDemo
} from './tasks/creators';
import { generalConfig, remainFillConfig } from './plugins/svgo/presets';
import {
  assignAttrsAtTag,
  adjustViewBox,
  setDefaultColorAtPathTag
} from './plugins/svg2Definition/transforms';
import { twotoneStringify } from './plugins/svg2Definition/stringify';
import { readFileSync } from 'fs';
import { resolve } from 'path';
import { getIdentifier, getAllInfoName } from './utils';
import { IconDefinition } from './templates/types';
import { ExtractRegExp } from './tasks/creators/generateInline';

const iconTemplate = readFileSync(
  resolve(__dirname, './templates/icon.ts.ejs'),
  'utf8'
);

export default series(
  // 1. clean
  clean(['src', 'svg', 'inline-svg', 'inline-namespaced-svg', 'es', 'lib']),

  parallel(
    // 将UI的原图拷贝到svg下, svg名称加上 ‘$类型名’
    copyFlat({
      from: ['svg-original/fill/*/*.svg'],
      toDir: 'svg/filled',
      filename: ({ name, dirname }) => getAllInfoName({ name, dirname })
    }),
    copyFlat({
      from: ['svg-original/outlined/*/*.svg'],
      toDir: 'svg/outlined',
      filename: ({ name, dirname }) => getAllInfoName({ name, dirname })
    })
  ),


  parallel(
    // 2.1 copy helpers.ts, types.ts
    copy({
      from: ['templates/*.ts'],
      toDir: 'src'
    }),

    // 2.2 generate abstract node with the theme "filled"
    generateIcons({
      theme: 'filled',
      from: ['svg/filled/*.svg'],
      toDir: 'src/asn',
      svgoConfig: generalConfig,
      extraNodeTransformFactories: [
        assignAttrsAtTag('svg', { focusable: 'false' }),
        adjustViewBox
      ],
      stringify: JSON.stringify,
      template: iconTemplate,
      mapToInterpolate: ({ name, content }) => ({
        identifier: getIdentifier({ name, themeSuffix: 'Filled' }),
        content
      }),
      filename: ({ name }) => getIdentifier({ name, themeSuffix: 'Filled' })
    }),

    // 2.2 generate abstract node with the theme "outlined"
    generateIcons({
      theme: 'outlined',
      from: ['svg/outlined/*.svg'],
      toDir: 'src/asn',
      svgoConfig: generalConfig,
      extraNodeTransformFactories: [
        assignAttrsAtTag('svg', { focusable: 'false' }),
        adjustViewBox
      ],
      stringify: JSON.stringify,
      template: iconTemplate,
      mapToInterpolate: ({ name, content }) => ({
        identifier: getIdentifier({ name, themeSuffix: 'Outlined' }),
        content
      }),
      filename: ({ name }) => getIdentifier({ name, themeSuffix: 'Outlined' })
    }),

    // 2.3 generate abstract node with the theme "outlined"
    // 屏蔽掉twotone类型
    // generateIcons({
    //   theme: 'twotone',
    //   from: ['svg/twotone/*.svg'],
    //   toDir: 'src/asn',
    //   svgoConfig: remainFillConfig,
    //   extraNodeTransformFactories: [
    //     assignAttrsAtTag('svg', { focusable: 'false' }),
    //     adjustViewBox,
    //     setDefaultColorAtPathTag('#333')
    //   ],
    //   stringify: twotoneStringify,
    //   template: iconTemplate,
    //   mapToInterpolate: ({ name, content }) => ({
    //     identifier: getIdentifier({ name, themeSuffix: 'TwoTone' }),
    //     content
    //   }),
    //   filename: ({ name }) => getIdentifier({ name, themeSuffix: 'TwoTone' })
    // })
  ),
  parallel(
    // 3.1 generate entry file: src/index.ts
    generateEntry({
      entryName: 'index.ts',
      from: ['src/asn/*.ts'],
      toDir: 'src',
      banner: '// This index.ts file is generated automatically.\n',
      template: `export { default as <%= identifier %> } from '<%= path %>';`,
      mapToInterpolate: ({ name: identifier }) => ({
        identifier,
        path: `./asn/${identifier}`
      })
    }),

    // 3.2 generate inline SVG files
    generateInline({
      from: ['src/asn/*.ts'],
      toDir: ({ _meta }) => `inline-svg/${_meta && _meta.theme}`,
      getIconDefinitionFromSource: (content: string): IconDefinition => {
        const extract = ExtractRegExp.exec(content);
        if (extract === null || !extract[1]) {
          throw new Error('Failed to parse raw icon definition: ' + content);
        }
        return new Function(`return ${extract[1]}`)() as IconDefinition;
      }
    }),
    // 3.3 generate inline SVG files with namespace
    generateInline({
      from: ['src/asn/*.ts'],
      toDir: ({ _meta }) => `inline-namespaced-svg/${_meta && _meta.theme}`,
      getIconDefinitionFromSource: (content: string): IconDefinition => {
        const extract = ExtractRegExp.exec(content);
        if (extract === null || !extract[1]) {
          throw new Error('Failed to parse raw icon definition: ' + content);
        }
        return new Function(`return ${extract[1]}`)() as IconDefinition;
      },
      renderOptions: {
        extraSVGAttrs: { xmlns: 'http://www.w3.org/2000/svg' }
      }
    })
  )
);




//----------------------------生成图标文档--------------------------

const tep = `
<%= identifier %>:<%= content %>;

`

export const generateRd = series(
  parallel(
    // 2.2 generate abstract node with the theme "filled"
    generateSvgDemo({
      theme: 'filled',
      from: ['./svg/filled/*.svg'],
      toDir: './docs',
      svgoConfig: generalConfig,
      extraNodeTransformFactories: [
        assignAttrsAtTag('path', { fill: "#fff" }),
        assignAttrsAtTag('svg', { width: "24" })
      ],
      stringify: JSON.stringify,
      template: tep,
      mapToInterpolate: ({ name, content }) => {
        return {
          identifier: name,
          content
        }
      },
      filename: 'readme.md',
      getIconDefinitionFromSource: (content: string): IconDefinition => {
        const extract = ExtractRegExp.exec(content);
        if (extract === null || !extract[1]) {
          throw new Error('Failed to parse raw icon definition: ' + content);
        }
        return new Function(`return ${extract[1]}`)() as IconDefinition;
      },
      renderOptions: {
        extraSVGAttrs: { xmlns: 'http://www.w3.org/2000/svg' }
      }
    }),

    // 2.2 generate abstract node with the theme "outlined"
    generateSvgDemo({
      theme: 'outlined',
      from: ['./svg/outlined/*.svg'],
      toDir: './docs',
      svgoConfig: generalConfig,
      extraNodeTransformFactories: [
        assignAttrsAtTag('path', { fill: "#fff" }),
        assignAttrsAtTag('svg', { width: "24" })
      ],
      stringify: JSON.stringify,
      template: tep,
      mapToInterpolate: ({ name, content }) => {
        return {
          identifier: name,
          content
        }
      },
      filename: 'readme.md',
      getIconDefinitionFromSource: (content: string): IconDefinition => {
        const extract = ExtractRegExp.exec(content);
        if (extract === null || !extract[1]) {
          throw new Error('Failed to parse raw icon definition: ' + content);
        }
        return new Function(`return ${extract[1]}`)() as IconDefinition;
      },
      renderOptions: {
        extraSVGAttrs: { xmlns: 'http://www.w3.org/2000/svg' }
      }
    }),


  )
);
