import 'promise-polyfill/src/polyfill';
import 'whatwg-fetch';
import uuidv4 from 'uuid/v4';
import IS_DEBUG_MODE from './is-debug-mode';
import SENTRY_CONFIG from './sentry-config';

const WARNING_START = [
    "HELP!",
    "WE ARE ENCOUNTERING AN INTERNAL ERROR DURING THE APPLICATION'S STARTUP!",
    "AND THERE IS NO EXISTING WAY TO MAKE AN AUTOMATIC RECOVERY!",
    "PLEASE REPORT THIS EVENT AND THE FOLLOWING INFORMATION TO UDESK©.",
    "WE WILL FIX IT AS SOON AS POSSIBLE.",
    "-------------------------------------------------------------------------------------"
];
const WARNING_END = [
    "-------------------------------------------------------------------------------------"
];

function HealthCheck() {
    this._running = false;
    this._errorHandler = null;
    this._errorsCount = 0;
    this.start = start;
    this.stop = stop;
    this.version = "1.0.0";
}

function start() {
    this._running = true;
    let that = this;
    this._errorHandler = function (errorEvent) {
        if (!that._running) {
            return;
        }
        that._errorsCount++;
        console.error(errorEvent); // eslint-disable-line no-console
        let errorProps = "bubbles,cancelBubble,cancelable,colno,composed,defaultPrevented,eventPhase,filename,isTrusted,lineno,message,returnValue,timeStamp,type".split(",");
        let errorData = {};
        for (let i = 0; i < errorProps.length; i++) {
            errorData[errorProps[i]] = errorEvent[errorProps[i]];
        }
        if (errorEvent.error != null) {
            errorData.error = {
                message: errorEvent.error.message,
                stack: errorEvent.error.stack
            };
        } else {
            errorData.error = null;
        }

        let browserProps = "appCodeName,appName,cookieEnabled,deviceMemory,hardwareConcurrency,languages,platform,product,productSub,userAgent,vendor,vendorSub".split(",");
        let browserData = {};
        for (let i = 0; i < browserProps.length; i++) {
            browserData[browserProps[i]] = navigator[browserProps[i]];
        }
        if (navigator.connection) {
            browserData.connection = {
                downlink: navigator.connection.downlink,
                effectiveType: navigator.connection.effectiveType,
                rtt: navigator.connection.rtt
            };
        }
        if (navigator.plugins) {
            browserData.plugins = [];
            for (let i = 0; i < navigator.plugins.length; i++) {
                let plugin = navigator.plugins[i];
                browserData.plugins.push({
                    name: plugin.name,
                    filename: plugin.filename,
                    description: plugin.description
                });
            }
        }

        let device = {
            devicePixelRatio: window.devicePixelRatio,
            performance: JSON.stringify(window.performance),
            timezone: (new Date().getTimezoneOffset() / 60) + " hours"
        };
        if (window.screen) {
            device.screen = {
                availHeight: window.screen.availHeight,
                availLeft: window.screen.availLeft,
                availTop: window.screen.availTop,
                availWidth: window.screen.availWidth,
                colorDepth: window.screen.colorDepth,
                height: window.screen.height
            };
        }
        if (window.visualViewport) {
            device.visualViewport = {
                height: window.visualViewport.height,
                offsetLeft: window.visualViewport.offsetLeft,
                offsetTop: window.visualViewport.offsetTop,
                pageLeft: window.visualViewport.pageLeft,
                pageTop: window.visualViewport.pageTop,
                scale: window.visualViewport.scale,
                width: window.visualViewport.width
            };
        }

        let globalNamespaces = {};
        let globalKeys = Object.keys(window);
        for (let i = 0; i < globalKeys.length; i++) {
            let key = globalKeys[i];
            globalNamespaces[key] = undefined;
            try {
                globalNamespaces[key] = (typeof window[key]);
            } catch (err) {
                // fail silently
            }
        }

        let captureData = {
            Error: errorData,
            Browser: browserData,
            Device: device,
            "Global namespaces": globalNamespaces
        };

        if (IS_DEBUG_MODE) {
            printCatureData(captureData, that);
        } else {
            if (isDevelopmentEnvironment()) {
                printSimpleError(errorEvent, that);
            } else {
                let eventData = {
                    "project": SENTRY_CONFIG.projectId,
                    "server_name": window.location.hostname,
                    "event_id": uuidv4().replace(/-/g, ""),
                    "logger": "udesk.toolkits.health-check",
                    "platform": "javascript",
                    "request": {
                        "headers": {
                            "User-Agent": window.navigator.userAgent,
                            "Referer": window.document.referrer
                        },
                        "url": window.location.href
                    },
                    "message": getErrorMessage(errorEvent) || "[HELP] Application was failed to startup! Please fix it as soon as possible!!!",
                    "extra": captureData,
                    "tags": {
                        "environment": "production",
                        "hostname": window.location.hostname,
                        "error.from": "client",
                        "message": errorEvent.message,
                        "create.time": formatDate(new Date())
                    },
                    // "fingerprint": [],
                    // "breadcrumbs": {},
                    // "user": {},
                    // "release": "",
                };
                fetch(SENTRY_CONFIG.url, {
                    method: 'POST',
                    mode: 'cors',
                    body: JSON.stringify(eventData),
                    cache: 'default'
                }).then(function (response) {
                    if (response.ok) {
                        // return response.json();
                    } else {
                        printSimpleError(errorEvent, that);
                    }
                }).catch(function () {
                    printSimpleError(errorEvent, that);
                });
            }
        }
    };
    window.addEventListener("error", this._errorHandler);
}

function stop() {
    this._running = false;
    window.removeEventListener("error", this._errorHandler);
    this._errorHandler = null;
}

//-------------------------------------

function prettyJson(str) {
    return str.replace(/\n|\r\n/g, "<br>").replace(/ /g, "&nbsp;");
}

function formatDate(date) {
    let year = padNumber(date.getFullYear());
    let month = padNumber(date.getMonth() + 1);
    let day = padNumber(date.getDate());
    let hour = padNumber(date.getHours());
    let minute = padNumber(date.getMinutes());
    let second = padNumber(date.getSeconds());
    return `${year}-${month}-${day} ${hour}:${minute}:${second}`;
}

function padNumber(value) {
    if (value < 10) {
        return "0" + value;
    } else {
        return value + "";
    }
}

function printCatureData(captureData, context) {
    let debugContents = [
        ...WARNING_START
    ];
    let keys = Object.keys(captureData);
    for (let i = 0; i < keys.length; i++) {
        let key = keys[i];
        debugContents.push(key + ":");
        debugContents.push(prettyJson(JSON.stringify(captureData[key], null, 4)));
    }
    debugContents.push(...WARNING_END);
    showError(debugContents.join("<br>"), context);
}

function printSimpleError(errorEvent, context) {
    let debugContents = [
        ...WARNING_START,
        "Error: " + getErrorMessage(errorEvent),
        "Line: " + errorEvent.lineno,
        "Column" + errorEvent.colno,
        "Stack:" + getErrorStack(errorEvent).replace(/\n|\r\n/g, "<br>"),
        ...WARNING_END,
    ];
    showError(debugContents.join("<br>"), context);
}

function getErrorMessage(errorEvent) {
    return (errorEvent.error ? errorEvent.error.message : errorEvent.message);
}

function getErrorStack(errorEvent) {
    return (errorEvent.error ? errorEvent.error.stack : "");
}

function showError(errorMessage, context) {
    if (document.body) {
        let healthCheckContainer = document.querySelector(".health-check");
        if (healthCheckContainer == null) {
            document.body.insertAdjacentHTML("beforeend", `
            <div class="health-check">
                <div class="report-header">
                    <a class="show-report" href="javascript:void(0);">OOPS! HERE IS SOMETHING YOU MUST PAY ATTENTION! 
                    <strong class="toggle-icon">↓↓↓</strong></a>
                </div>
                <div class="report-body" style="display: none;"></div>
            </div>`);
            let showReportLink = document.querySelector(".health-check .show-report");
            if (showReportLink) {
                showReportLink.addEventListener("click", function (e) {
                    let link = e.target;
                    link.isExpanded = !link.isExpanded;

                    let reportBody = document.querySelector(".health-check .report-body");
                    let toggleIcon = document.querySelector(".health-check .toggle-icon");
                    if (link.isExpanded) {
                        if (toggleIcon) {
                            toggleIcon.innerText = "↑↑↑";
                        }
                        if (reportBody) {
                            reportBody.style.display = "";
                        }
                    } else {
                        if (toggleIcon) {
                            toggleIcon.innerText = "↓↓↓";
                        }
                        if (reportBody) {
                            reportBody.style.display = "none";
                        }
                    }
                });
            }
        }
        let reportBody = document.querySelector(".health-check .report-body");
        if (reportBody != null) {
            reportBody.insertAdjacentHTML("beforeend", `
                <br>
                Error ${context._errorsCount}:
                <br>
                ${errorMessage}`);
        }
    } else {
        document.write(errorMessage);
    }
}

function isDevelopmentEnvironment() {
    let hostname = window.location.hostname;
    for (let i = 0; i < SENTRY_CONFIG.developmentDomains.length; i++) {
        let domainRule = SENTRY_CONFIG.developmentDomains[i];
        if (typeof domainRule === "string") {
            let regExp = new RegExp("^" + domainRule.replace(/([.*+?^=!:${}()|[\]/\\])/g, "\\$&").replace("\\*", ".*") + "$", "i");
            if (regExp.test(hostname)) {
                return true;
            }
        }
    }
    return false;
}

export default HealthCheck;
