import UmiSdkClass, { plugin, UmiSdkOptions, RenderArgs } from '../umi-sdk-base';
import { PluginState, ResolvedPluginConfig, ResolvedRuntimeSdkConfig, SdkRuntime } from '../types';
import {
    GetConfigFilePluginKey,
    PluginConfigPluginKey,
    PluginStatePluginKey,
    GetDefaultSdkOptionsPluginKey,
    Mode,
} from '../const';
import * as runtime from '../sdk-runtime';
//@ts-ignore
import clientRenderFactory from '../../"%ENTRY_RENDERER_FILE%"';
//@ts-ignore
import runtimeSdkConfig from './"%ENTRY_OPTIONS_FILE%"';
import { IRoute } from '@udesk/sdk';
import { ApplyPluginsType } from '@umijs/runtime';

/**
 * 如何自定义SDKOptions？
 * 有两种方式：
 * 1. 在umi的 config 中添加 'udesk-sdk' 配置项，除了能设置 plugin 的编译时配置，还可以设置一部分SDKOptions设置，
 *    这部分 SDKOptions 会以默认值的形式传递给 SDKClass。此阶段在 plugin 初始化之前，所以只能是静态配置，不能动态变化。
 *    但可以在 'udesk-sdk' 中针对每个 sdk 设置不同的 options，以 sdk 的名称为 key。sdk 特有的配置会与外层 options 合并，且优先级更高
 *
 * 2. 添加 src/sdk-runtime.ts(x) 并导出 getSdkOptions 方法（参考示例代码），会传入 pluginConfig 和 pluginState，
 *    这些都是 plugin 在编译时的一些信息，可以根据 sdk 的不同返回不同的 SDKOptions。运行时options会直接传给SDK的构造函数，优先级比第一种高
 *    示例代码：
 *    import { UdeskSdkRuntimeContext, UmiSdkOptions } from 'umi';
 *    export function getSdkOptions({ sdkConfig, pluginConfig, pluginState }: UdeskSdkRuntimeContext): UmiSdkOptions {
 *        return {};
 *    }
 *    注意：这个文件是在运行时执行的，非编译时，所以不要使用 Node 代码，也不要导入任何 Node 模块！
 *
 * 如何手动获取当前sdk的编译时配置？
 * 示例代码：
 *    import { plugin, UdeskSdkMainConfigKey, UdeskSdkPluginConfig, UdeskSdkPluginStateKey, UdeskSdkPluginState, ApplyPluginsType } from 'umi';
 *    const pluginConfig: UdeskSdkPluginConfig = plugin.applyPlugins({
 *         key: UdeskSdkMainConfigKey,
 *         type: ApplyPluginsType.modify,
 *         initialValue: {}
 *     });
 *     const pluginState: UdeskSdkPluginState = plugin.applyPlugins({
 *         key: UdeskSdkPluginStateKey,
 *         type: ApplyPluginsType.modify,
 *         initialValue: {}
 *    });
 * 注意：上述代码必须在模块方法内使用，不能在模块的顶级作用域内调用，此时模块刚被加载，plugin系统还没初始化
 * */

/** 第一部分编译时的静态配置，通过 "%PLUGIN_CONFIG_KEY%" 写在 Plugin 的配置中 */
// @ts-ignore
const compilerDefaultSdkOptions: UmiSdkOptions = '%COMPILER_SDK_DEFAULT_OPTIONS%';
// @ts-ignore
const pluginConfig: ResolvedPluginConfig = '%PLUGIN_CONFIG%';
// @ts-ignore
const pluginState: PluginState = '%PLUGIN_STATE%';
// @ts-ignore
const entryRenderMode = '%ENTRY_RENDERER_MODE%';

// 注册成一个plugin，这样业务项目就可以调用 applyPlugin 方法获取这些值
const sdkConfig: ResolvedRuntimeSdkConfig = runtimeSdkConfig;
plugin.register({
    path:
        '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/"%OUTPUT_SDK_ENTRIES_DIR_NAME%"/"%CONFIG_FILENAME%"',
    apply: {
        [GetConfigFilePluginKey]: () => sdkConfig,
        [PluginConfigPluginKey]: () => pluginConfig,
        [PluginStatePluginKey]: () => pluginState,
    },
});

/** 第二部分的运行时配置 */
const sdkRuntime: SdkRuntime = { ...runtime };
let runtimeSdkOptions = {} as UmiSdkOptions;
Object.keys(sdkConfig).forEach(k => {
    if (k.startsWith('options.')) {
        runtimeSdkOptions[k.replace('options.', '')] = sdkConfig[k];
    }
});
if (sdkRuntime.getSdkOptions) {
    let initialValue = {
        sdkName: sdkConfig.filename,
        sdkConfig,
    };
    // 在业务项目的app.ts中导出一个 getDefaultSdkOptions({sdkName}) 方法，返回每个sdk的默认值
    let runtimeDefaultOptions = plugin.applyPlugins({
        key: GetDefaultSdkOptionsPluginKey,
        type: ApplyPluginsType.modify,
        initialValue,
    });
    runtimeSdkOptions = {
        ...sdkRuntime.getSdkOptions(
            { ...runtimeSdkOptions },
            {
                sdkConfig,
                pluginConfig,
                pluginState,
            }
        ),
        ...(runtimeDefaultOptions === initialValue ? {} : runtimeDefaultOptions),
    };
}

/**
 * UmiSDK的主入口文件，包含了SDK的所有基本功能，可以被 Manifest-Loader 加载
 * 并动态渲染一个独立的子应用。
 *
 * 除了直接使用这个类，还可以对他进行扩展，从而加入一些自定义的行为。
 * 请在 src/sdk 目录下创建与某个配置文件同名的ts文件，例如 `main.ts`，
 * 这样入口就被替换成自定义文件了。但你可能希望在默认的SDK基类基础上扩展，
 * 而不是完全自己开发。你可以在 umi 中获得sdk基类：
 * @example import { MainSDKEntryBase } from 'umi';
 *
 * 其中类型的前缀就是sdk的名称。这样你就可以获得基础sdk的所有能力了，
 * 当然如果有必要的话，你也可以重写基类的某个方法。
 *
 * @export
 * @class UmiSdkEntry
 * @extends {UmiSdkClass<TOption>}
 * @template TOption 自定义SDKOptions类型
 */
function generateUUID() {
    var d = new Date().getTime();
    var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = (d + Math.random() * 16) % 16 | 0;
        d = Math.floor(d / 16);
        return (c === 'x' ? r : (r & 0x3) | 0x8).toString(16);
    });
    return uuid;
}
export default class UmiSdkEntry<
    TOption extends UmiSdkOptions = UmiSdkOptions,
    TG = any,
    TE = string
> extends UmiSdkClass<TOption, TG, TE> {
    static defaultOptions: UmiSdkOptions = { ...compilerDefaultSdkOptions, ...runtimeSdkOptions };

    constructor(options: TOption) {
        const sdkClassUuid = generateUUID();

        super({ ...UmiSdkEntry.defaultOptions, sdkClassUuid, ...options });
    }

    protected routes() {
        // 组件形式SDK
        // @ts-ignore
        if (entryRenderMode === Mode.component) {
            return [
                {
                    path: '---',
                    component: runtime.ComponentComponent,
                },
            ];
        } else {
            return sdkConfig.routes as IRoute[];
        }
    }
    protected getClientRendererFactory() {
        return clientRenderFactory as (args: RenderArgs) => () => void;
    }
}
