import { unmountComponentAtNode } from 'react-dom';
import type { History, Plugin } from 'umi/lib/cjs';
import type { IRoute } from '@udesk/sdk';
import { ApplyPluginsType } from '@umijs/runtime';
import { GetRootContextPluginKey } from './const';
import {
    UdeskSdkClass,
    UdeskSdkOptions,
    DefaultOptions as UdeskSdkDefaultOptions,
    RenderHookContext,
} from '@udesk/sdk';
//@ts-ignore
import { history, setCreateHistoryOptions } from '"%TMP_PATH%"/core/history';
//@ts-ignore
import { plugin as pluginInstance } from '"%TMP_PATH%"/core/plugin';

export const plugin: Plugin = pluginInstance;

export interface UmiSdkOptions extends UdeskSdkOptions {
    // appendToContainer
    sdkClassUuid: string;
    isComponentSdk: boolean;

    appendRenderMode?: boolean;
}
const DefaultOptions: UmiSdkOptions = {
    ...UdeskSdkDefaultOptions,

    appendRenderMode: true,
    sdkClassUuid: '',
    isComponentSdk: false,
};

export interface RenderArgs {
    routes: IRoute[];
    rootElement?: string | HTMLElement;
    hot?: boolean;
    isComponentSdk?: boolean;
    sdkClassUuid: string;
    callback?: () => void;
}
export interface HistoryOptions {
    basename?: string;
}

export abstract class UmiSdkClass<
    TO extends UmiSdkOptions = UmiSdkOptions,
    TG = any,
    TE = string
> extends UdeskSdkClass<TO, TG, TE> {
    #containerElement?: HTMLElement;
    #isDestroyed: boolean = false;
    #firstRender: boolean = true;
    #offListeners: (() => void)[] = [];

    static defaultOptions: UmiSdkOptions = { ...DefaultOptions };

    constructor(options: TO) {
        super({ ...UmiSdkClass.defaultOptions, ...options });
    }

    public async renderTo(
        container: HTMLElement | string | undefined | null
    ): Promise<RenderHookContext<TO, TG>> {
        let domContainer: HTMLElement;
        if (container == null) {
            domContainer = document.body;
        } else if (typeof container === 'string') {
            let dom = document.getElementById(container);
            if (dom == null) {
                throw new Error(`The container #${container} does not exist!`);
            }
            domContainer = dom;
        } else {
            domContainer = container;
        }
        if (this.options.appendRenderMode) {
            let contentContainer = document.createElement('div');
            contentContainer.className = 'udesk-sdk-container';
            domContainer.appendChild(contentContainer);
            this.#containerElement = contentContainer;
        } else {
            this.#containerElement = domContainer;
        }
        this.#isDestroyed = false;

        /**
         * 在渲染umi组件树之前，设置 history 单例的 basename，以实现动态根路由的效果。
         * 由于我们已经在本插件内强行打开了 runtimeHistory 特性，所以 setCreateHistoryOptions 是可以直接使用的。
         * 有了这个方法，我们就不需要修改 plugin-history 的代码来支持动态根路由了。
         */
        let newOptions: HistoryOptions = {};
        if (this.options.routerBase != null) {
            newOptions.basename = this.options.routerBase;
        }
        (setCreateHistoryOptions as (newOpts: HistoryOptions) => void)(newOptions);

        /**
         * 这个操作相当诡异，注册了一个 RuntimePlugin，并添加了一个新的钩子，返回rootContext。
         * 目的是借助plugin实例，与umi渲染的内部React组件传递 rootContext，除此之外好像还没有更好的办法。
         * rootContext 是传递给 UdeskSdkRoot 组件，再通过 Context 把 sdkOptions 传递给更内层的组件。
         * 最初尝试过，先创建一个Context，再Context内部容器中渲染Umi组件树，但由于使用两个ReactDom.render方法创建出来的东西，
         * React不承认是一个上下文，内层组件拿不到Context中的东西！ */
        // plugin.register({
        //     path: '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/umi-sdk-base',
        //     apply: {
        //         [GetRootContextPluginKey]: () => this.rootContext,
        //     },
        // });
        const rootContextData = await plugin.applyPlugins({
            key: GetRootContextPluginKey,
            type: ApplyPluginsType.modify,
            initialValue: '',
        });
        if (!rootContextData) {
            await plugin.register({
                path: '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/umi-sdk-base',
                apply: {
                    [GetRootContextPluginKey]: { [this.options.sdkClassUuid]: this.rootContext },
                },
            });
        } else {
            await plugin.register({
                path: '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/umi-sdk-base',
                apply: {
                    [GetRootContextPluginKey]: {
                        ...(rootContextData || {}),
                        [this.options.sdkClassUuid]: this.rootContext,
                    },
                },
            });
        }

        const routes = this.routes();
        // patchRoutes会改变routes对象，只能执行一次
        if (this.#firstRender) {
            this.#firstRender = false;
            // 业务项目可以借助umi钩子修改路由，甚至动态添加路由
            (plugin as Plugin).applyPlugins({
                key: 'patchRoutes',
                type: ApplyPluginsType.event,
                args: { routes },
            });
        }

        this.beforeRender({
            container: this.#containerElement!,
            rootContext: this.rootContext,
        });

        return new Promise<RenderHookContext<TO, TG>>(resolve => {
            const renderUmiClient = this.getClientRendererFactory()({
                routes: this.routes(),
                hot: false,
                rootElement: this.#containerElement!,
                sdkClassUuid: this.options.sdkClassUuid,
                isComponentSdk: this.options?.isComponentSdk,
                callback: () => {
                    if (this.options.landingPath) {
                        this.transitionTo(this.options.landingPath);
                    }
                    resolve({
                        container: this.#containerElement!,
                        rootContext: this.rootContext,
                    });
                },
            });
            // 渲染整个umi应用
            renderUmiClient();
        }).catch(error => {
            this.captureException(error);
            return Promise.reject(error);
        });
    }
    // function ComponentSDK(props) {
    //     return <div>12</div>
    // }
    protected abstract getClientRendererFactory(): (args: RenderArgs) => () => void;

    public transitionTo(path: string, state?: any): void;
    public transitionTo(location: History.LocationDescriptor<any>): void;
    public transitionTo(path: string | History.LocationDescriptor<any>, state?: any) {
        if (typeof path === 'string') {
            history.push(path, state);
        } else {
            history.push(path);
        }
    }

    public replaceTo(path: string, state?: any): void;
    public replaceTo(location: History.LocationDescriptor<any>): void;
    public replaceTo(path: string | History.LocationDescriptor<any>, state?: any) {
        if (typeof path === 'string') {
            history.replace(path, state);
        } else {
            history.replace(path);
        }
    }

    // 暂不开放，不需要外部能关注到内部的路由变化
    // #listen(listener: History.LocationListener<History.LocationState>): () => void {
    //     let offListener = history.listen(listener);
    //     this.#offListeners.push(offListener);
    //     return offListener;
    // }

    // 暂不开放，不需要外部能关注到内部的路由变化
    // #block(
    //     prompt?: boolean | string | History.TransitionPromptHook<History.LocationState>
    // ): () => void {
    //     return history.block(prompt);
    // }

    public destroy() {
        if (this.#isDestroyed) {
            return;
        }
        plugin.unregister({
            path: '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/umi-sdk-base',
            apply: {
                [GetRootContextPluginKey]: [],
            },
        });
        super.destroy();
        const sdkClassUuid = this.options.sdkClassUuid;
        let rootContextData = plugin.applyPlugins({
            key: GetRootContextPluginKey,
            type: ApplyPluginsType.modify,
            initialValue: {},
        });
        delete rootContextData[sdkClassUuid];
        plugin.register({
            path: '"%TMP_PATH%"/"%PLUGIN_TMP_DIR_NAME%"/umi-sdk-base',
            apply: {
                [GetRootContextPluginKey]: rootContextData,
            },
        });
        this.#offListeners.forEach(off => off());
        this.#offListeners = [];
        if (this.#containerElement) {
            unmountComponentAtNode(this.#containerElement);
            if (this.#containerElement.parentNode) {
                this.#containerElement.parentNode.removeChild(this.#containerElement);
            }
        }
        this.#containerElement = undefined;
        this.rootContext = undefined;
        this.#isDestroyed = true;
    }
}

export default UmiSdkClass;
