import React, {
  forwardRef,
  ReactNode,
  useEffect,
  useContext,
  useImperativeHandle,
  useMemo,
  useState,
  useCallback,
} from 'react';
import { Button, FormBuilder, Icon, Space } from 'udesk-ui';
import type { FormProps, FormInstance, FormFieldConfig } from 'udesk-ui/es/form-builder/types/form';
import { usePrefixCls } from '../utils';
import { FilterContainer } from './components/FilterContainer';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import './style/index';

const ACTION_BUTTONS_NAME = '__actionButtons';

interface ButtonsProps {
  formInstance: FormInstance;
  open: boolean;
  onOpenStatusChange: (open: boolean) => void;
  showMore: boolean;
  onReset: () => void;
}
// transform: rotate(-180deg);
const Buttons: React.FC<ButtonsProps> = (props) => {
  const { formInstance, open, onOpenStatusChange, showMore, onReset } = props;
  const [contextLocale] = useLocaleReceiver('CommonFilter');

  return (
    <Space>
      {showMore && (
        <Button
          type="link"
          onClick={() => onOpenStatusChange(!open)}
          style={{ paddingLeft: 0, paddingRight: 0 }}
          icon={<Icon type={open ? 'UpOutlined' : 'DownOutlined'} antdIcon={true} />}
        >
          {open ? contextLocale.lang.collapse : contextLocale.lang.expand}
        </Button>
      )}

      <Button
        onClick={() => {
          formInstance?.resetFields();
          Promise.resolve().then(() => {
            onReset();
          });
        }}
      >
        {contextLocale.lang.reset}
      </Button>
      <Button type="primary" htmlType="submit">
        {contextLocale.lang.search}
      </Button>
    </Space>
  );
};

interface FilterField extends FormFieldConfig {
  isDefault?: boolean;
}

export interface CommonFilterProps {
  fields: FilterField[];
  onSearch?: (value: any, options?: any) => void;
  initialValues?: FormProps['initialValues'];
  shouldTriggerSearchOnReset?: boolean;
  prefixCls?: string;
  customComponents?: FormProps['customComponents'];
  open?: boolean;
  onOpenPropsChange?: (open: boolean) => void;
}

export interface CommonFilterInstance {
  closeFilter: () => void;
  openFilter: () => void;
  formInstance: FormInstance;
}

const CommonFilter = React.forwardRef<CommonFilterInstance, CommonFilterProps>((props, ref) => {
  const {
    prefixCls: customizePrefixCls,
    fields,
    initialValues,
    onSearch,
    shouldTriggerSearchOnReset,
    customComponents,
    open,
    onOpenPropsChange,
    ...args
  } = props;

  const [pageContainerPrefixCls, prefixCls] = usePrefixCls('common-filter', customizePrefixCls);

  const [_open, _setOpen] = useState(open);
  const [needRerender, _forceUpdate] = useState(0);
  const forceUpdate = () => {
    _forceUpdate(Math.random());
  };
  useEffect(() => {
    typeof open === 'boolean' && _setOpen(open);
  }, [open]);

  const [formInstance] = FormBuilder.useForm();

  useImperativeHandle(ref, () => {
    return {
      closeFilter: () => _setOpen(false),
      openFilter: () => _setOpen(true),
      formInstance,
    } as CommonFilterInstance;
  });

  const onReset: FormProps['onReset'] = useCallback(() => {
    shouldTriggerSearchOnReset && onSearch?.(initialValues);
  }, [onSearch, initialValues, shouldTriggerSearchOnReset]);

  const config: FormProps = useMemo(() => {
    const showMore = shouldShowMoreBtnDisplaced(fields);
    const finalFields = [
      ...fields,
      {
        type: 'Buttons',
        name: ACTION_BUTTONS_NAME,
        props: {
          formInstance,
          open: _open,
          onOpenStatusChange: _setOpen,
          showMore,
          onReset,
        },
        isDefault: true,
      },
    ];
    return {
      fields: showMore
        ? finalFields.map((i: any) => ({
            ...i,
            forceHidden:
              !_open &&
              !i.isDefault &&
              (formInstance.getFieldValue(i.name) === undefined ||
                formInstance.getFieldValue(i.name) === ''),
          }))
        : finalFields,
      layout: 'inline',
      footerRender: false,
      customComponents: Object.assign({ Buttons }, customComponents),
      initialValues: { ...initialValues, __actionButtons: true },
    };
  }, [
    fields,
    initialValues,
    shouldTriggerSearchOnReset,
    customComponents,
    _open,
    needRerender,
    onReset,
  ]);
  const onFinish: FormProps['onFinish'] = useCallback(
    (value) => {
      const ret = Object.keys(value).reduce((prev: any, key) => {
        if (key !== ACTION_BUTTONS_NAME && value[key] !== undefined) {
          prev[key] = value[key];
        }
        return prev;
      }, {});
      onSearch?.(ret);
      forceUpdate();
    },
    [onSearch],
  );
  return (
    <FilterContainer prefixCls={prefixCls} className={pageContainerPrefixCls}>
      <FormBuilder {...config} onFinish={onFinish} form={formInstance} />
    </FilterContainer>
  );
});

CommonFilter.defaultProps = {
  shouldTriggerSearchOnReset: true,
  open: false,
};

export default React.memo(CommonFilter);

function shouldShowMoreBtnDisplaced(fields: CommonFilterProps['fields']): boolean {
  return (
    Boolean(fields.length) && fields.some((i) => i.isDefault) && fields.some((i) => !i.isDefault)
  );
}
