import React, {
  forwardRef,
  ReactNode,
  useEffect,
  useContext,
  useImperativeHandle,
  useMemo,
  useState,
} from 'react';
import { PageContainer } from './components/PageContainer';
import { PageContent } from './components/PageContent';
import { PageFoot } from './components/PageFoot';
import { usePrefixCls } from '../utils';
import classNames from 'classnames';
import { PageHeader, Spin } from 'udesk-ui';
import { PageHeaderFooter } from './components/PageHeaderFooter';
import './style/index';
import { CommonPageProps } from './types';
import { PageInfo } from './context/pageInfo';

const _styles: React.CSSProperties = {
  margin: 0,
  height: '100%',
  display: 'flex',
  flexDirection: 'column',
  paddingBottom: 16,
  paddingTop: 16,
  flex: 1,
  overflow: 'auto',
};

const CommonPage: React.FC<CommonPageProps> = (props) => {
  const {
    children,
    prefixCls: customizePrefixCls,
    className,
    ghostPage,
    padding = 16,
    footBar,
    navigationTabs,
    onNavigationTabsChange,
    currentNavigationPath,
    footer,
    loading = false,
    pageBodyClassName,
    ...pageHeaderProps
  } = props;

  const [pageContainerPrefixCls, prefixCls] = usePrefixCls('common-page', customizePrefixCls);
  const [pageContentPrefixCls] = usePrefixCls('common-page-content', customizePrefixCls);
  const [pageFootPrefixCls] = usePrefixCls('common-page-foot', customizePrefixCls);

  const classes = classNames(pageContainerPrefixCls, className);
  const pageBodyClasses = classNames(pageContentPrefixCls, pageBodyClassName);

  return (
    <div style={{ display: 'flex', flexDirection: 'column', overflow: 'hidden', height: '100%' }}>
      {typeof pageHeaderProps.title !== undefined && (
        <PageHeader
          {...pageHeaderProps}
          ghost={false}
          footer={
            <PageHeaderFooter
              currentNavigationPath={currentNavigationPath}
              antdFooter={footer}
              navigationTabs={navigationTabs}
              onChange={onNavigationTabsChange}
            />
          }
        />
      )}
      <div style={_styles}>
        <PageContainer prefixCls={prefixCls} className={classes}>
          <Spin delay={200} spinning={loading}>
            <PageContent
              ghostPage={ghostPage}
              prefixCls={prefixCls}
              className={pageBodyClasses}
              padding={padding}
            >
              {children}
            </PageContent>
          </Spin>
        </PageContainer>
      </div>
      {footBar ? (
        <PageFoot prefixCls={prefixCls} className={pageFootPrefixCls}>
          {footBar}
        </PageFoot>
      ) : null}
    </div>
  );
};

const PageInfoMemo: React.FC<any> = forwardRef((props, ref) => {
  const [pageInfo, _storePageInfo] = useState({ [window.location.pathname]: {} });

  const storePageInfo = new Proxy(_storePageInfo, {
    apply(target, ctx, args) {
      window.console.log('location', window.location.pathname);
      const params = { ...pageInfo, [window.location.pathname]: typeof args[0] === 'function' ? args[0](pageInfo[window.location.pathname]) : args[0] };
      return Reflect.apply(target, ctx, [params]);
    },
  });

  useEffect(() => {
    window.console.log('pageInfo ::', pageInfo);
  }, [pageInfo]);
  const clearAllPageInfo = () => {
    storePageInfo({ [window.location.pathname]: {} });
  };

  const pageInfoValue = useMemo(() => {
    return new Proxy(
      {
        pageInfo,
        storePageInfo,
        clearAllPageInfo,
      },
      {
        get(target, key, receiver) {
          if (key === 'pageInfo') {
            return Reflect.get(target, key, receiver)[window.location.pathname];
          }
          return Reflect.get(target, key, receiver);
        },
      }
    );
  }, [pageInfo, storePageInfo]);

  useEffect(() => {
    return clearAllPageInfo;
  }, []);

  useImperativeHandle(ref, () => {
    return pageInfoValue;
  });
  return <PageInfo.Provider value={pageInfoValue}>{props.children}</PageInfo.Provider>;
});

const usePageStateMemo = (shouldClearPageStateMemo = () => false) => {
  const context = useContext(PageInfo);

  return new Proxy(context, {
    get(target, key, receiver) {
      if (key === 'pageInfo' && shouldClearPageStateMemo()) {
        Reflect.set(target, key, {}, receiver);
      }
      return Reflect.get(target, key, receiver);
    },
  });
};

type InternalCommonPageType = typeof CommonPage;

interface CommonPageType extends InternalCommonPageType {
  usePageStateMemo: typeof usePageStateMemo;
  PageInfoMemo: typeof PageInfoMemo;
}

const RefCommonPage: CommonPageType = CommonPage as CommonPageType;

RefCommonPage.usePageStateMemo = usePageStateMemo;
RefCommonPage.PageInfoMemo = PageInfoMemo;

export default RefCommonPage;
