import React, { Fragment, useCallback, useEffect, useState } from "react";
import { Checkbox, Dropdown, Input, List, Menu } from "udesk-ui";
import style from "./style.module.scss";
import ClassNames from "classnames";
import { CareDown } from "../icon/icon";
import { useTranslation } from "react-i18next";

interface IProps {
  options: Array<AnyKey>;
  value?: Array<string | number>;
  idFieldName?: string | undefined;
  nameFieldName?: string | undefined;
  trigger?: Array<any>;
  preText?: string;
  unit?: string;
  className?: string;
  onChange?: any;
  AllText?: string | undefined;
  searchPlaceholder?: string | undefined;
  showSearch?: boolean;
  getLabel?: (option: { [key: string]: any }) => string;
}

const CheckBoxSelector: React.FC<IProps> = (props) => {
  // let newProps = { ...DEFAULT_PROPS, ...props };
  const { t } = useTranslation();
  const {
    options = [],
    value = [],
    idFieldName = "id",
    nameFieldName = "name",
    trigger = ["click"],
    preText = "",
    unit = t("unit"),
    className = "",
    AllText = t("all"),
    searchPlaceholder = "",
    onChange,
    showSearch = true,
    getLabel,
  } = props;

  let [values, setValues] = useState<Array<string | number>>(value);
  let [keyword, setKeyword] = useState("");
  let [visible, setVisible] = useState(false);
  let [innerOptions, setInnerOptions] = useState<Array<any>>([]);
  let _getLabel = useCallback(
    (option) => {
      if (typeof getLabel === "function") {
        return getLabel(option);
      } else {
        return option[nameFieldName];
      }
    },
    [getLabel, nameFieldName]
  );

  useEffect(() => {
    setInnerOptions(options);
  }, [options]);
  const onCheckboxChanged = useCallback(
    (e, option) => {
      let checked = e.target.checked;
      let id = option[idFieldName];
      let newValues: Array<any> = [];
      if (checked) {
        // eslint-disable-next-line eqeqeq
        let hasValue = values.some((value: any) => value == id);
        if (!hasValue) {
          newValues = [...values, id];
        }
      } else {
        newValues = values.filter((value: any) => {
          // eslint-disable-next-line eqeqeq
          return value != id;
        });
        newValues = [...newValues];
      }
      if (typeof onChange === "function") {
        onChange(newValues);
      }
      setValues(newValues);
    },
    [idFieldName, onChange, values]
  );
  let onKeywordChanged = useCallback(
    (e) => {
      let keyword = e.target.value;
      setKeyword(e.target.value);
      if (options && keyword) {
        let optionResult = options.filter((option: any) => {
          let name = option[nameFieldName];
          return name && name.indexOf(keyword) !== -1;
        });
        setInnerOptions(optionResult);
      } else {
        setInnerOptions(options);
      }
    },
    [nameFieldName, options]
  );

  let createMenu = useCallback(
    (options) => {
      return (
        <Menu className={style["option-list"]}>
          {!!showSearch && (
            <Menu.Item key={"-1"}>
              <div className={style["search"]}>
                <Input.Search
                  className={style["search-input"]}
                  placeholder={searchPlaceholder}
                  value={keyword}
                  onChange={onKeywordChanged}
                />
              </div>
            </Menu.Item>
          )}
          <Menu.Item>
            <List
              dataSource={options}
              className={style["list"]}
              size="small"
              renderItem={(option: any) => {
                let id = option[idFieldName];
                return (
                  <List.Item className={style["list-item"]} key={id}>
                    <Checkbox
                      checked={!!isInValue(values, id)}
                      onChange={(e) => {
                        onCheckboxChanged(e, option);
                      }}
                    >
                      {_getLabel(option)}
                    </Checkbox>
                  </List.Item>
                );
              }}
            >
              {/* {
                            options.map((option: any) => {
                                let id = option[idFieldName];
                                return (
                                    <List.Item className={style["list-item"]} key={id}>
                                        <Checkbox checked={isInValue(values, id)} onChange={e => {
                                            onCheckboxChanged(e, option);
                                        }} >{option[nameFieldName]}</Checkbox>
                                    </List.Item>
                                );
                            })
                        } */}
            </List>
          </Menu.Item>
        </Menu>
      );
    },
    [
      idFieldName,
      keyword,
      onCheckboxChanged,
      onKeywordChanged,
      searchPlaceholder,
      showSearch,
      values,
      _getLabel,
    ]
  );
  let onVisibleChange = useCallback((value) => {
    setVisible(value);
  }, []);

  let selectedName = "";
  if (values?.length === 1) {
    // eslint-disable-next-line eqeqeq
    let selectedOption = options.find(
      (option) => option[idFieldName] == values[0]
    );
    selectedName = selectedOption?.[nameFieldName];
  }
  return (
    <div className={ClassNames(style["checkbox-selector"], className)}>
      <Dropdown
        overlay={createMenu(innerOptions)}
        visible={visible}
        onVisibleChange={onVisibleChange}
        trigger={trigger}
      >
        <span>
          {values?.length > 1 && (
            <Fragment>
              {preText}
              {values?.length}
              {unit}
            </Fragment>
          )}
          {values?.length === 1 && (
            <Fragment>
              {preText}
              {selectedName}
            </Fragment>
          )}
          {!values?.length && (
            <Fragment>
              {preText}
              {AllText}
            </Fragment>
          )}
          <CareDown></CareDown>
        </span>
      </Dropdown>
    </div>
  );
};

export default CheckBoxSelector;

function isInValue(values: any, id: string | number) {
  if (!values || id == null) {
    return;
  }
  if (!Array.isArray(values)) {
    values = [values];
  }
  // eslint-disable-next-line eqeqeq
  return values.find((value: any) => value == id);
}
