import React from 'react';
import PropTypes from 'prop-types';
import WidgetDropClass from "./widget-drop-class";
import ClassNames from 'classnames';
import Udesk from '../../../udesk/index';
import Locales from '../../../udesk/locales/index';
import DynamicComponent from "../../dynamic-component";
import WidgetWrapper from "./widget-wrapper";
import RGL, { WidthProvider } from "react-grid-layout";
import "react-grid-layout/css/styles.css";
import "react-resizable/css/styles.css";

const ReactGridLayout = WidthProvider(RGL);

class WidgetRoot extends WidgetDropClass {
    static defaultProps = {
        mode: "design",
        widget: null,
        allowDrop: false,
        gridCols: 12,
        enbaleWidgetDraggable: false,
        noWidgetPlacholder: "",
        gridRowHeight: 30,
        gridMargin: [10, 10],
        dropDataTransferWidgetTypeField: "widgetType",
        onSelected: null,
        addWidget: null,
        removeWidget: null,
        onWidgetMoved: null
    }

    static propTypes = {
        mode: PropTypes.string,
        widget: PropTypes.object,
        allowDrop: PropTypes.bool,
        gridCols: PropTypes.number,
        enbaleWidgetDraggable: PropTypes.bool,
        noWidgetPlacholder: PropTypes.string,
        gridRowHeight: PropTypes.number,
        gridMargin: PropTypes.array,
        dropDataTransferWidgetTypeField: PropTypes.string,
        onSelected: PropTypes.func,
        addWidget: PropTypes.func,
        removeWidget: PropTypes.func,
        onWidgetMoved: PropTypes.func
    }

    privates = {
        listeningWidthTimer: null,
        updateGridLayoutTimeout: null,
        containerElement: null,
        oldContainerWidth: null,
        reactGridLayoutInstance: null
    }

    static computes = {
        isGridDraggable: ({ props, state, privates, locales }) => {
            let {
                mode,
                enbaleWidgetDraggable
            } = props;
            return mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id && enbaleWidgetDraggable;
        },
        noWidgetPlacholder: ({ props, state, privates, locales }) => {
            let {
                noWidgetPlacholder
            } = props;
            if (noWidgetPlacholder == null || noWidgetPlacholder === "") {
                noWidgetPlacholder = Locales.current.components.customUi.grid.widgetRoot.noWidgetPlacholder;
            }
            return noWidgetPlacholder;
        },
    }

    actions = {
        onLayoutChange(layout) {
            if (layout != null && layout instanceof Array) {
                if (this.props.onWidgetMoved) {
                    this.props.onWidgetMoved(layout);
                }
            }
        }
    }

    getDragHandler() {
        return document.getElementsByClassName("udesk-react-custom-ui-grid-widget-root")[0];
    }

    isAllowableDrop() {
        return this.props.allowDrop;
    }

    _getItemLayout(layout, key) {
        return layout.find(item => item.i === key);
    }

    componentDidMount() {
        super.componentDidMount();
        let that = this;
        let listeningWidthTimer = setInterval(function () {
            let containerElement = that.privates.containerElement;
            if (containerElement != null) { 
                let containerWidth = $(containerElement).width();
                let oldContainerWidth = that.privates.oldContainerWidth;
                if (oldContainerWidth != null) { 
                    if (that.privates.oldContainerWidth !== containerWidth) { 
                        clearTimeout(that.privates.updateGridLayoutTimeout);
                        that.privates.updateGridLayoutTimeout = setTimeout(function () { 
                            let reactGridLayoutInstance = that.privates.reactGridLayoutInstance;
                            if (reactGridLayoutInstance) { 
                                reactGridLayoutInstance.onWindowResize();
                            }
                        }, 100);
                    }
                }
                that.privates.oldContainerWidth = containerWidth;
            }
        }, 500);
        this.privates.listeningWidthTimer = listeningWidthTimer;
    }
    componentWillUnmount() {
        super.componentWillUnmount();
        let listeningWidthTimer = this.privates.listeningWidthTimer;
        if (listeningWidthTimer) {
            clearInterval(listeningWidthTimer);
            clearTimeout(this.privates.updateGridLayoutTimeout);
            this.privates.oldContainerWidth = null;
            this.privates.listeningWidthTimer = null;
            this.privates.containerElement = null;
        }
    }

    render() {
        let {
            mode,
            widget,
            gridCols,
            gridRowHeight,
            addWidget,
            onSelected,
            removeWidget,
            gridMargin
        } = this.props;
        let {
            computes,
        } = this.privates;
        let layoutConfig = widget.instance.config.layoutConfig;
        return (
            <div className={ClassNames("udesk-react-custom-ui-grid-widget-root", { [`udesk-react-custom-ui-grid-widget-root-${mode}`]: true })} ref={(element) => this.privates.containerElement = element}>
                <If condition={widget}>
                    <If condition={widget.instance.widgets && widget.instance.widgets.length > 0}>
                        
                        <ReactGridLayout margin={gridMargin} isDraggable={computes.isGridDraggable} layout={layoutConfig} cols={gridCols} width={this.privates.oldContainerWidth} rowHeight={gridRowHeight} onLayoutChange={this.actions.onLayoutChange} ref={(instance) => this.privates.reactGridLayoutInstance = instance}>
                            <For of={widget.instance.widgets} each="childWidget" index="index">
                                <div key={`${childWidget.instance.id}`} data-grid={this._getItemLayout(widget.instance.config.layoutConfig, childWidget.instance.id)} className="udesk-react-custom-ui-grid-widget">
                                    <If condition={childWidget}>
                                        <WidgetWrapper widget={childWidget} mode={mode} onSelected={onSelected} removeWidget={removeWidget}>
                                            <DynamicComponent
                                                component={childWidget.settings.component}
                                                mode={mode}
                                                widget={childWidget}
                                                addWidget={addWidget}
                                                onCreated={childWidget.globalActions.onComponentCreated}
                                                onDestroyed={childWidget.globalActions.onComponentDestoryed}
                                            />
                                        </WidgetWrapper>
                                    </If>
                                </div>
                            </For>
                        </ReactGridLayout>
                    </If>
                    <If condition={computes.isGridDraggable && widget.instance.widgets && widget.instance.widgets.length === 0}>
                        <div className="no-widget-placeholder">
                            {computes.noWidgetPlacholder}
                        </div>
                    </If>
                </If>
            </div>
        );
    }
}

export default Udesk.react.udeskify(WidgetRoot);