import React, { useCallback, useMemo, useRef, useState } from "react";
import { Select, Spin } from "udesk-ui";
import apis from "../../utils/apis";
import style from "./index.module.scss";
import { Search } from "../icon/icon";
import { DownOutlined } from "@ant-design/icons";
import { useTranslation } from "react-i18next";
import useDebounce from "react-use/esm/useDebounce";

interface IOnChange {
  (value: any): void;
}

interface ISetIsScroll {
  (value: any): void;
}

interface IProps {
  value?: any;
  showArrow?: boolean;
  labelInValue?: boolean;
  filterOption?: boolean;
  showSearchModal?: boolean;
  placeholder?: string;
  onChange?: IOnChange;
  setIsScroll?: ISetIsScroll;
}

const DEFAULT_PAGE = 1;
const DEFAULT_PAGE_SIZE = 20;
const DEFAULT_TOTAL_PAGE = 0;

interface ICustomer {
  cellphone: string[];
  description: string;
  email: string;
  id: number;
  nick_name: string;
  organizations: { id: number; name: string }[];
}

const CustomerAutoComplete: React.FC<any> = (props: IProps) => {
  const { t } = useTranslation();
  let [fetching, setFetching] = useState(false);
  const [page, setPage] = useState<number>(DEFAULT_PAGE);
  const [data, setData] = useState<ICustomer[]>([]); //搜索到的数据
  const [searchValue, setSearchValue] = useState<string>("");
  const [totalPages, setTotalPages] = useState<number>(DEFAULT_TOTAL_PAGE);
  const selectRef = useRef(null);

  let {
    showArrow = true,
    labelInValue = true,
    value,
    placeholder = t("components.customerAutoComplete.selectCustomer"),
    filterOption = false,
    showSearchModal = false,
    onChange,
  } = props;

  let notFoundContent = useMemo(() => {
    return fetching && <Spin size="small" />;
  }, [fetching]);

  const fetchUser = useCallback((searchValue, page) => {
    setFetching(true);
    apis
      .postUser({
        params: {
          page: page,
          page_size: DEFAULT_PAGE_SIZE,
          prefix: searchValue,
        },
      })
      .then(
        (res: {
          code: number;
          customers: (Omit<ICustomer, "organizations"> & {
            organizations: number[];
          })[];
          meta: {
            current_page: number;
            customers_count: number;
            total_pages: number;
          };
          organizations: ICustomer["organizations"];
        }) => {
          if (res?.code === 1000) {
            setTotalPages(res?.meta?.total_pages);
            setPage(res.meta.current_page);

            const orgs = res.organizations;
            setData((v = []) => {
              const customers = res.customers.map((i) => ({
                ...i,
                organizations: i.organizations
                  .map((j) => orgs.find((l) => l.id === j))
                  .filter(Boolean) as ICustomer["organizations"],
              }));
              return [...v, ...customers];
            });
          }
          setFetching(false);
        }
      )
      .catch(() => setFetching(false));
  }, []);

  let handleChange = useCallback(
    (value: AntdSelectOptions) => {
      onChange && onChange(value);
      setFetching(false);
    },
    [onChange]
  );
  let beforeSearch = useCallback((searchValue) => {
    setSearchValue(searchValue);
    setData([]);
    setPage(DEFAULT_PAGE);
  }, []);
  useDebounce(
    () => {
      fetchUser(searchValue, 1);
    },
    500,
    [searchValue]
  );
  //处理滚动事假
  const handleScroll = useCallback(
    (e) => {
      if (
        e.currentTarget.scrollTop ===
        e.currentTarget.scrollHeight - e.currentTarget.clientHeight
      ) {
        if (page < totalPages) {
          fetchUser(searchValue, page + 1);
        }
      }
    },
    [page, totalPages, fetchUser, searchValue]
  );

  let suffixIcon = useMemo(() => {
    return (
      <>
        {showSearchModal && (
          <div className={style["search-icon"]}>
            <Search />
          </div>
        )}
        {!showSearchModal && <DownOutlined />}
      </>
    );
  }, [showSearchModal]);
  return (
    <div style={{ display: "flex" }}>
      <Select
        showSearch
        showArrow={showArrow}
        labelInValue={labelInValue}
        value={value}
        placeholder={placeholder}
        notFoundContent={notFoundContent}
        filterOption={filterOption}
        onSearch={beforeSearch}
        onChange={handleChange}
        style={{ width: "100%", minWidth: 120 }}
        getPopupContainer={(triggerNode) => triggerNode}
        suffixIcon={suffixIcon}
        onPopupScroll={handleScroll}
        allowClear={true}
        dropdownMatchSelectWidth={false}
        optionLabelProp="title"
        ref={selectRef}
        listItemHeight={54}
      >
        {data.map((d) => (
          <Select.Option
            title={`${d.nick_name}${
              d.organizations.length ? `（${d.organizations[0].name}）` : ""
            }`}
            key={d.id}
            value={d.id}
          >
            {d.nick_name}
            {d.organizations.length ? `（${d.organizations[0].name}）` : null}
            <br />
            {d.description || "\u00A0"}
          </Select.Option>
        ))}
      </Select>
      {/* 搜索客户弹窗 */}
    </div>
  );
};

export default CustomerAutoComplete;
