import React, {
  CSSProperties,
  ElementRef,
  useCallback,
  useEffect,
  useRef,
  useState,
} from "react";
import cytoscape, { EventObject, NodeSingular } from "cytoscape";
import Button from "udesk-ui/es/button";
import { getJSON } from "../../utils/ajax";
import css from "./index.module.scss";

const stylesheets = {
  default: [
    {
      selector: "node",
      style: {
        label: "data(name)",
        width: "data(width)",
        height: "data(width)",
        "font-size": 6,
        "overlay-opacity": 0,
        "border-width": 1,
        "border-style": "solid",
        "border-color": "#fff",
        opacity: 1,
      },
    },
    {
      selector: "edge",
      style: {
        "curve-style": "bezier",
        "target-arrow-shape": "triangle",
        width: 1,
        "arrow-scale": 0.2,
        "overlay-opacity": 0,
        opacity: 0.5,
      },
    },
    {
      selector: "node.hover",
      style: {
        width: "data(hoverWidth)",
        height: "data(hoverWidth)",
        opacity: 1,
      },
    },
    {
      selector: "edge.hover",
      style: {
        width: 4,
        "arrow-scale": 0.6,
        opacity: 1,
      },
    },
    {
      selector: "node.click",
      style: {
        width: "data(hoverWidth)",
        height: "data(hoverWidth)",
        opacity: 1,
      },
    },
    {
      selector: "edge.click",
      style: {
        width: 4,
        "arrow-scale": 0.6,
        opacity: 1,
      },
    },
  ],
};
const colorsMapping = {
  default: {
    nodesColors: [
      "#D6162E",
      "#294254",
      "#46A1A9",
      "#E38163",
      "#7BCEAE",
      "#61A682",
      "#D58D16",
      "#C3A29A",
      "#6E6F74",
    ],
    nodesHighLightColors: [
      "#D6162E",
      "#294254",
      "#46A1A9",
      "#E38163",
      "#7BCEAE",
      "#61A682",
      "#D58D16",
      "#C3A29A",
      "#6E6F74",
    ],
    edgesColors: [
      "#D6162E",
      "#294254",
      "#46A1A9",
      "#E38163",
      "#7BCEAE",
      "#61A682",
      "#D58D16",
      "#C3A29A",
      "#6E6F74",
    ],
    edgesHighLightColors: [
      "#D6162E",
      "#294254",
      "#46A1A9",
      "#E38163",
      "#7BCEAE",
      "#61A682",
      "#D58D16",
      "#C3A29A",
      "#6E6F74",
    ],
  },
};

// 获取颜色配置文件
function getColorMap(theme?: keyof typeof colorsMapping) {
  if (colorsMapping) {
    if (theme) {
      return colorsMapping[theme];
    }
    return colorsMapping["default"];
  }
  return null;
}

// 获取颜色
function getColor(index: number, colorList: string[]) {
  if (typeof index === "number" && colorList instanceof Array) {
    const count = index % colorList.length;
    return colorList[count];
  }
  return null;
}

// 获取图表基本配置
function getStylesheet(theme?: keyof typeof stylesheets) {
  if (stylesheets) {
    if (theme) {
      return stylesheets[theme];
    }
    return stylesheets["default"];
  }
  return null;
}

function getPointSize(params: {
  initialSize: number;
  criterion: number;
  multiple: number;
}) {
  const initialSize = params.initialSize || 1;
  const criterion = params.criterion || 1;
  const multiple = params.multiple || 1;
  let initialSizeNumber = Number(initialSize);
  let criterionNumber = Number(criterion);
  let multipleNumber = Number(multiple);
  if (Number.isNaN(initialSizeNumber) || initialSizeNumber <= 0) {
    initialSizeNumber = 1;
  }
  if (Number.isNaN(criterionNumber) || criterionNumber <= 0) {
    criterionNumber = 1;
  }
  if (Number.isNaN(multipleNumber) || multipleNumber <= 0) {
    multipleNumber = 1;
  }
  const level = Math.ceil(initialSizeNumber / criterionNumber);
  return level * multipleNumber;
}

interface Node {
  id: number;
  sys_id: number;
  name: string;
  labels: string[];
  properties: { [key: string]: any };
}

interface Relation {
  id: number;
  sys_id: number;
  name: string;
  startNodeId: number;
  endNodeId: number;
  types: string[];
  properties: { [key: string]: any };
}

function formatter(data: { nodes: Node[]; relations: Relation[] }) {
  const formattedData: any = [];
  if (data && data.nodes && data.relations) {
    // 知识点的图上大小根据知识点的连接数进行计算
    const knowledgePointWidthMap: { [key: number]: number } = {};

    if (data.relations instanceof Array && data.relations.length > 0) {
      data.relations.forEach(function (item) {
        if (item.startNodeId in knowledgePointWidthMap) {
          knowledgePointWidthMap[item.startNodeId]++;
        } else {
          knowledgePointWidthMap[item.startNodeId] = 1;
        }
      });
    }
    // 后端请求数据需要进行重构，重构所需的格式:https://js.cytoscape.org/#getting-started中的elements
    if (data.nodes instanceof Array && data.nodes.length > 0) {
      data.nodes.forEach(function (item) {
        const count = knowledgePointWidthMap[item.id];
        formattedData.push({
          data: {
            id: "knowledgeNode" + item.id,
            name: item.name,
            group: "knowledgeNode" + item.id,
            properties: item.properties,
            width: getPointSize({
              initialSize: count,
              criterion: 6,
              multiple: 8,
            }),
            hoverWidth: getPointSize({
              initialSize: count,
              criterion: 6,
              multiple: 10,
            }),
          },
        });
      });
    }
    if (data.relations instanceof Array && data.relations.length > 0) {
      data.relations.forEach(function (item) {
        formattedData.push({
          data: {
            id: "relation" + item.id,
            name: item.name,
            properties: item.properties,
            source: "knowledgeNode" + item.startNodeId,
            target: "knowledgeNode" + item.endNodeId,
            group: "relation" + item.id,
          },
        });
      });
    }
  }
  if (formattedData instanceof Array && formattedData.length > 0) {
    const colorList = getColorMap() || {
      nodesColors: [],
      nodesHighLightColors: [],
      edgesColors: [],
      edgesHighLightColors: [],
    };
    const nodesRegExp = /knowledgeNode+/;
    // const relationRegExp = /relation+/;
    let cyStylesheet: {
      selector: string;
      style: { [key: string]: any };
    }[] = [];
    let nodeColorCount = 0;
    let relationColorCount = 0;
    // const arrowColorCount = 0;
    const relationsTypes: { [key: string]: any } = {};
    formattedData.forEach(function (item) {
      // 点之间不需要同一类型
      if (nodesRegExp.test(item.data.group)) {
        const color = getColor(nodeColorCount, colorList.nodesColors);
        const highLightColor = getColor(
          nodeColorCount,
          colorList.nodesHighLightColors
        );
        if (color) {
          cyStylesheet.push(
            {
              selector: 'node[group="' + item.data.group + '"]',
              style: {
                "background-color": color ? color : null,
              },
            },
            {
              selector: 'node[group="' + item.data.group + '"].click',
              style: {
                "background-color": highLightColor ? highLightColor : null,
              },
            },
            {
              selector: 'node[group="' + item.data.group + '"].hover',
              style: {
                "background-color": highLightColor ? highLightColor : null,
              },
            }
          );
          nodeColorCount++;
        }
      } else {
        // 线之间可能有同种类型，同一类型颜色相同，所以用一个对象列出已经有颜色的类型
        let color, highLightColor;
        if (
          Object.prototype.hasOwnProperty.call(relationsTypes, item.data.group)
        ) {
          color = relationsTypes[item.data.group]["color"];
          highLightColor = relationsTypes[item.data.group]["highLightColor"];
        } else {
          color = getColor(relationColorCount, colorList.edgesColors);
          highLightColor = getColor(
            relationColorCount,
            colorList.edgesHighLightColors
          );
          relationsTypes[item.data.group] = {
            color: color,
            highLightColor: highLightColor,
          };
        }
        // <pos>-arrow-color这里pos是一个变量，箭头颜色不应该这样直接设置，因为箭头的种类很多，
        // 应该用正则替换来做，赶任务，抽空优化
        if (color) {
          cyStylesheet.push(
            {
              selector: 'edge[group="' + item.data.group + '"]',
              style: {
                "line-color": color ? color : null,
                "target-arrow-color": color,
              },
            },
            {
              selector: 'edge[group="' + item.data.group + '"].click',
              style: {
                "line-color": highLightColor ? highLightColor : null,
                "target-arrow-color": highLightColor,
              },
            },
            {
              selector: 'edge[group="' + item.data.group + '"].hover',
              style: {
                "line-color": highLightColor ? highLightColor : null,
                "target-arrow-color": highLightColor,
              },
            }
          );
          relationColorCount++;
        }
      }
    });
    const stylesheet = getStylesheet() || [];
    cyStylesheet = stylesheet.concat(cyStylesheet as any);
    return [formattedData, cyStylesheet];
  }
  return [[], []];
}

const closeStyle: CSSProperties = {
  border: "0",
  background: "transparent",
  position: "absolute",
  right: "0",
  top: "0",
  fontSize: "20px",
};
const relativeStyle: CSSProperties = {
  position: "relative",
};

const KnowledgeGraph: React.FC<{ knowledgeId: number; className?: string }> = ({
  knowledgeId,
  className,
}) => {
  const ref = useRef<ElementRef<"div">>(null);
  let hoverElement = useRef<any>(null);

  const hoverGraph = useCallback((cyEve: EventObject) => {
    const target = cyEve.target;
    const hoverEle = hoverElement.current;
    if (hoverEle) {
      if (hoverEle._private.group === "nodes") {
        hoverEle.neighborhood().removeClass("hover");
      } else if (hoverEle._private.group === "edges") {
        hoverEle.connectedNodes().removeClass("hover");
      }
      hoverEle.removeClass("hover");
    }
    if (target.length > 0) {
      // 鼠标hover高亮，给当前和相邻元素加上hover状态的class
      // 如果这些元素上有了点击状态的类名，就不要添加颜色
      if (target._private.group === "nodes") {
        const neighborhood = cyEve.target.neighborhood();
        const targetId = target._private.data.id;
        const targetNodes: any[] = [];
        neighborhood.forEach(function (item: any) {
          if (
            item._private.group === "edges" &&
            item._private.data.source === targetId
          ) {
            const node = item._private.data.target;
            if (!item.hasClass("click")) {
              item.addClass("hover");
            }
            if (!targetNodes.includes(node)) {
              targetNodes.push(node);
            }
          }
        });
        neighborhood.forEach(function (item: any) {
          if (
            item._private.group === "nodes" &&
            targetNodes.includes(item._private.data.id)
          ) {
            if (!item.hasClass("click")) {
              item.addClass("hover");
            }
          }
        });
      } else if (target._private.group === "edges") {
        target
          .connectedNodes()
          .filter((ele: NodeSingular) => !ele.hasClass("click"))
          .addClass("hover");
      }
      if (!target.hasClass("click")) {
        target.addClass("hover");
      }
      hoverElement.current = target;
    }
  }, []);

  const outOfGraph = useCallback((cyEve: EventObject) => {
    const hoverEle = hoverElement.current;
    if (hoverEle) {
      if (hoverEle._private.group === "nodes") {
        hoverEle.neighborhood().removeClass("hover");
      } else if (hoverEle._private.group === "edges") {
        hoverEle.connectedNodes().removeClass("hover");
      }
      hoverEle.removeClass("hover");
    }
  }, []);

  const clickedElementRef = useRef<any>(null);

  const clickGraph = useCallback((cyEve: EventObject) => {
    const target = cyEve.target;
    const clickedEle = clickedElementRef.current;
    if (clickedEle) {
      if (clickedEle._private.group === "nodes") {
        clickedEle.neighborhood().removeClass("click");
      } else if (clickedEle._private.group === "edges") {
        clickedEle.connectedNodes().removeClass("click");
      }
      clickedEle.removeClass("click");
    }
    // 区分一下，如果是线条节点，用connectedNodes方法
    // 如果是知识点节点，用neighborhood
    if (target.length > 0) {
      // 点击高亮，给当前点击状态和相邻元素加上点击状态的class
      if (target._private.group === "nodes") {
        const neighborhood = cyEve.target.neighborhood();
        const targetId = target._private.data.id;
        const targetNodes: any[] = [];
        neighborhood.forEach((item: any) => {
          if (
            item._private.group === "edges" &&
            item._private.data.source === targetId
          ) {
            const node = item._private.data.target;
            item.addClass("click");
            item.removeClass("hover");
            if (!targetNodes.includes(node)) {
              targetNodes.push(node);
            }
          }
        });
        neighborhood.forEach((item: any) => {
          if (
            item._private.group === "nodes" &&
            targetNodes.includes(item._private.data.id)
          ) {
            item.addClass("click");
            item.removeClass("hover");
          }
        });
      } else if (target._private.group === "edges") {
        target.connectedNodes().addClass("click").removeClass("hover");
      }
      target.addClass("click");
      target.removeClass("hover");
      clickedElementRef.current = target;
      let tooltipData = target._private.data.properties;
      if (typeof tooltipData === "object") {
        const tooltipDataKeys = Object.keys(tooltipData);
        const tooltipDataList: any[] = [];
        if (tooltipDataKeys.length > 0) {
          tooltipDataKeys.forEach(function (item) {
            tooltipDataList.push({
              key: item,
              value: tooltipData[item],
            });
          });
        }
        tooltipData = tooltipDataList;
      }
    }
  }, []);

  const [graphData, setGraphData] = useState<any>();
  useEffect(() => {
    if (!graphData && !ref.current) {
      return;
    }
    const cy = cytoscape({
      container: ref.current,
      elements: graphData[0],
      layout: { name: "cose" },
      style: graphData[1],
    });
    cy.on("vclick", clickGraph);
    cy.on("tapdragover", hoverGraph);
    cy.on("tapdragout", outOfGraph);

    return () => {
      cy.destroy();
    };
  }, [clickGraph, graphData, hoverGraph, outOfGraph]);

  const fetchData = useCallback(() => {
    getJSON("/im_robots/knowledge_diagram", {
      knowledgeId: knowledgeId,
      maxHops: 1,
    }).then((resp) => {
      const formattedData = formatter(resp);
      setGraphData(formattedData);
    });
  }, [knowledgeId]);

  const [shown, setShown] = useState(false);
  const showGraph = useCallback(() => {
    setShown(true);
    fetchData();
  }, [fetchData]);
  const onClose = useCallback(() => setShown(false), []);

  if (shown) {
    return (
      <div style={relativeStyle} className={className}>
        <div ref={ref} className={css["graphContainer"]} />
        <button className={css["closeButton"]} onClick={onClose}>
          &times;
        </button>
      </div>
    );
  } else {
    return (
      <Button onClick={showGraph} className={css["linkButton"]}>
        查看图谱
      </Button>
    );
  }
};

export default KnowledgeGraph;
