/* eslint-disable indent */
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Table from 'antd/lib/table';
import 'antd/lib/table/style/index.css';
import Form from 'antd/lib/form';
import 'antd/lib/form/style/index.css';
import Icon from 'antd/lib/icon';
import 'antd/lib/icon/style/index.css';
import Input from 'antd/lib/input';
import 'antd/lib/input/style/index.css';
import Button from 'antd/lib/button';
import 'antd/lib/button/style/index.css';
import Popover from 'antd/lib/popover';
import 'antd/lib/popover/style/index.css';
import Switch from 'antd/lib/switch';
import 'antd/lib/switch/style/index.css';
import 'antd/lib/pagination/style/index.css';
import 'antd/lib/select/style/index.css';
import Highlighter from 'react-highlight-words';
import ColumnsSetting from './components/columns-setting';
import { generateColumns, getColumnKey } from './utils/column-helper';
import { clone } from 'udesk-web-toolkits/src/udesk/utils/object';
import ReactDragListView from 'react-drag-listview';
import { Resizable } from 'react-resizable';
import './style.scss';
import locales from '../../udesk/locales';

const {
    filterTitle,
    filterConfirm,
    filterReset,
    emptyText,
    selectAll,
    selectInvert,
    sortTitle,
    requiredText,
    pleaseInput,
    columnSetting
} = locales.current.components.reactAntdTable;
const { query, reset } = locales.current.labels;

const ResizeableColumn = props => {
    const { onResize, onResizeStop, onResizeStart, width, ...restProps } = props;

    if (!width) {
        return <th {...restProps} />;
    }

    return (
        <Resizable
            width={width}
            height={0}
            onResize={onResize}
            onResizeStart={onResizeStart}
            onResizeStop={onResizeStop}
        >
            <th key='a' {...restProps} />
        </Resizable>
    );
};

const TableContext = React.createContext();

const EditableRow = ({ form, index, ...props }) => (
    <TableContext.Provider value={form}>
        <tr {...props} />
    </TableContext.Provider>
);

const EditableFormRow = Form.create()(EditableRow);

class EditableCell extends React.Component {
    state = {
        editing: false
    };

    toggleEdit = () => {
        if (this.input) {
            const editing = !this.state.editing;
            this.setState({ editing }, () => {
                if (editing) {
                    this.input.focus();
                }
            });
        }
    };

    save = e => {
        const { record, onEditData } = this.props;
        this.form.validateFields((error, values) => {
            if (error && error[e.currentTarget.id]) {
                return;
            }
            this.toggleEdit();
            // TODO handle change uncontrol component
            onEditData({ ...record, ...values });
        });
    };

    render() {
        const { editing } = this.state;
        const { editable, dataIndex, title, record, index, onEditData, ...restProps } = this.props;
        return (
            <td {...restProps}>
                <If condition={editable}>
                    <TableContext.Consumer>
                        {form => {
                            this.form = form;
                            return (
                                <Choose>
                                    <When condition={typeof record[dataIndex] === 'boolean'}>
                                        {/* TODO handle change */}
                                        <Form.Item style={{ margin: 0 }}>
                                            {form.getFieldDecorator(dataIndex, {
                                                valuePropName: 'checked',
                                                initialValue: record[dataIndex]
                                            })(<Switch onChange={this.save} />)}
                                        </Form.Item>
                                    </When>
                                    <When condition={typeof record[dataIndex] === 'string'}>
                                        {editing ? (
                                            <Form.Item style={{ margin: 0 }}>
                                                {form.getFieldDecorator(dataIndex, {
                                                    rules: [
                                                        {
                                                            required: true,
                                                            message: title + requiredText
                                                        }
                                                    ],
                                                    initialValue: record[dataIndex]
                                                })(
                                                    <Input
                                                        ref={node => (this.input = node)}
                                                        onPressEnter={this.save}
                                                        onBlur={this.save}
                                                    />
                                                )}
                                            </Form.Item>
                                        ) : (
                                                <div
                                                    className='editable-cell-value-wrap'
                                                    style={{ paddingRight: 24 }}
                                                    onClick={this.toggleEdit}
                                                >
                                                    {restProps.children}
                                                </div>
                                            )}
                                    </When>
                                </Choose>
                            );
                        }}
                    </TableContext.Consumer>
                </If>
                <If condition={!editable}>{restProps.children}</If>
            </td>
        );
    }
}

function noop() { }

const defaultPagination = {
    showSizeChanger: false,
    onShowSizeChange: noop,
    onChange: noop
};

const defaultComponents = {
    body: {
        row: EditableFormRow,
        cell: EditableCell
    }
};

class ReactAntdTable extends React.Component {
    // #region defaultProps
    static defaultProps = {
        dataSource: null,
        useFixedHeader: false,
        className: '',
        size: 'default',
        loading: false,
        bordered: false,
        // 展示树形数据时，每层缩进的宽度，以 px 为单位
        indentSize: 15,
        rowKey: 'id',
        showHeader: true,
        sortDirections: ['ascend', 'descend'],
        showCheckboxColumn: false,
        /**
         * 分页器
         * position 指定分页显示的位置 'top' | 'bottom' | 'both'
         * defaultPageSize 默认的每页条数
         * hideOnSinglePage 只有一页时是否隐藏分页器
         * pageSize 每页条数
         * pageSizeOptions 指定每页可以显示多少条
         * total 数据总数
         * onChange 页码改变的回调，参数是改变后的页码及每页条数
         * onShowSizeChange pageSize变化的回调
         */
        pagination: false,
        // whether resize columns
        resizable: false,
        // whether column config
        showColumnSetting: true,
        // whether column drag
        columnDragable: false,
        locale: {
            filterTitle,
            filterConfirm,
            filterReset,
            emptyText,
            selectAll,
            selectInvert,
            sortTitle
        },
        // title
        showTitle: false,
        title: null,
    };
    // #endregion

    // #region defaultProps
    static propTypes = {
        rowKey: PropTypes.string.isRequired,
        dataSource: PropTypes.array,
        columns: PropTypes.array,
        prefixCls: PropTypes.string,
        useFixedHeader: PropTypes.bool,
        showCheckboxColumn: PropTypes.bool,
        rowSelection: PropTypes.object,
        className: PropTypes.string,
        size: PropTypes.string,
        loading: PropTypes.oneOfType([PropTypes.bool, PropTypes.object]),
        bordered: PropTypes.bool,
        onChange: PropTypes.func,
        locale: PropTypes.object,
        dropdownPrefixCls: PropTypes.string,
        sortDirections: PropTypes.array,
        onSelectChanged: PropTypes.func,
        showTitle: PropTypes.bool,
        title: PropTypes.string,
        scroll: PropTypes.object,

        //pagination
        onShowSizeChanged: PropTypes.func,
        onPageChanged: PropTypes.func,
        onFilterDropdownVisibleChanged: PropTypes.func,
        //resize
        resizable: PropTypes.bool,
        // column setting
        showColumnSetting: PropTypes.bool,
        columnDragable: PropTypes.bool,
        //expand
        onExpand: PropTypes.func,
        //onColumnsVisibleChanged
        onColumnsVisibleChanged: PropTypes.func,

        //cell
        onRowClick: PropTypes.func,
        onDoubleClick: PropTypes.func,
        onContextMenu: PropTypes.func,
        onMouseEnter: PropTypes.func,
        onMouseLeave: PropTypes.func
    };
    // #endregion

    // #region privates
    privates = {};
    // #endregion

    // #region computes
    static computes = {};
    // #endregion

    // #region public function
    getColumnEnumSearchProps(column) {
        return {
            filtered: true,
            filters: column.filters,
            filterIcon: filtered => <Icon type='search' style={{ color: filtered ? '#1890ff' : undefined }} />,
            onFilterDropdownVisibleChange: this.actions.onFilterDropdownVisibleChanged.params(column),
            render: text => (
                <Highlighter
                    highlightStyle={{ backgroundColor: '#ffc069', padding: 0 }}
                    //TODO
                    searchWords={[]}
                    autoEscape
                    textToHighlight={text}
                />
            )
        };
    }

    getColumnTextSearchProps(column) {
        return {
            filtered: true,
            filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters }) => (
                <div
                    style={{
                        padding: 5,
                        border: '1px solid lightgrey',
                        background: 'white'
                    }}
                >
                    <Input
                        ref={node => {
                            //bind dom focus
                            this.searchInput = node;
                        }}
                        placeholder={pleaseInput}
                        value={selectedKeys[0]}
                        onChange={e => setSelectedKeys(e.target.value ? [e.target.value] : [])}
                        style={{ width: 138, marginBottom: 8, display: 'block' }}
                    />
                    <Button
                        onClick={this.actions.onCustomFilterChanged.params([getColumnKey(column)], {
                            [getColumnKey(column)]: selectedKeys
                        })}
                        icon='search'
                        size='small'
                        style={{ width: 60, marginRight: 8 }}
                    >
                        {query}
                    </Button>
                    <Button
                        onClick={this.actions.resetCustomFilter.params(column, clearFilters)}
                        size='small'
                        style={{ width: 60 }}
                    >
                        {reset}
                    </Button>
                </div>
            ),
            filterIcon: filtered => <Icon type='search' style={{ color: filtered ? '#1890ff' : undefined }} />,
            onFilterDropdownVisibleChange: visible => {
                if (visible) {
                    setTimeout(() => this.searchInput.select());
                }
                this.actions.onFilterDropdownVisibleChanged(column, visible);
            },
            render: text => (
                <Highlighter
                    highlightStyle={{ backgroundColor: '#ffc069', padding: 0 }}
                    //TODO
                    searchWords={this.searchInput ? [this.searchInput.value] : []}
                    autoEscape
                    textToHighlight={text.toString()}
                />
            )
        };
    }

    buildColumns(columns) {
        let { filterColumns, localColumns } = generateColumns(columns, this.privates.localColumns);
        filterColumns = filterColumns.map((column, index) => {
            if (this.props.resizable) {
                column.onHeaderCell = column => ({
                    width: column.width,
                    onResize: this.actions.resizeColumn.params(index),
                    onResizeStart: this.actions.onResizeStart,
                    onResizeStop: this.actions.onResizeStop
                });
            }
            column.onCell = record => ({
                record,
                editable: column.editable,
                dataIndex: column.dataIndex,
                title: column.title,
                onEditData: this.actions.onEditData
            });
            // searchMode in (text|list|none|custom)
            switch (column.searchMode) {
                case 'text':
                    return Object.assign({}, column, this.getColumnTextSearchProps(column));
                case 'enum':
                    return Object.assign({}, column, this.getColumnEnumSearchProps(column));
                case 'none':
                /* falls through */
                default:
                    break;
            }
            return column;
        });
        return { filterColumns, localColumns };
    }

    // #endregion

    // #region life cycle
    constructor(props) {
        super(props);
        this.tableRef = React.createRef();
    }
    init() { }
    componentDidMount() { }
    parseProps({ props, prevProps, state, privates, isInitial }) {
        return {
            pagination:
                props.pagination &&
                Object.assign(
                    {},
                    defaultPagination,
                    {
                        onShowSizeChange: this.actions.onShowSizeChanged,
                        onChange: this.actions.onPageChanged,
                        locale: props.paginationLocal
                    },
                    props.pagination
                ),
            filters: Object.assign({}, props.filters),
            sorter: Object.assign({}, props.sorter),
            selectedRowKeys: [],
            ...this.buildColumns(props.columns)
        };
    }
    componentWillUnmount() {
        if (this.isDestroyed) {
            return;
        }
    }
    // #endregion

    // #region actions
    actions = {
        /**
         * 改变每页显示条目事件
         * @param  {nubmer} current 当前页
         * @param  {number} pageSize 改变每页显示条目数
         */
        onShowSizeChanged(current, pageSize) {
            this.actions.update();
            this.trigger('onShowSizeChanged', current, pageSize);
        },
        /**
         * 翻页事件
         * @param  {number} pageNumber 翻到页数
         */
        onPageChanged(pageNumber) {
            this.actions.update();
            this.trigger('onPageChanged', pageNumber);
        },

        onFilterDropdownVisibleChanged(column, visible) {
            this.actions.update();
            this.trigger('onFilterDropdownVisibleChanged', column, visible);
        },

        resetCustomFilter(column, clearFilters) {
            clearFilters();
            this.actions.update();
            this.actions.onCustomFilterChanged([getColumnKey(column)], []);
        },

        resizeColumn(index, e, { size }) {
            this.privates.filterColumns = this.privates.filterColumns.map((column, i) => {
                if (i === index) {
                    column.width = size.width;
                }
                return column;
            });
            this.actions.update();
        },

        onResizeStart(e, data) {
            //resolve drag column and resize column conflict
            e.stopPropagation();
            e.preventDefault();
        },

        onResizeStop(e, data) {
            //handle resize
        },

        onCustomFilterChanged(key, nextFilter) {
            if (!this.tableRef || !this.tableRef.current) {
                return;
            }
            const { pagination, filters, sorter } = this.privates;
            Object.assign(filters, nextFilter);
            // remove not in the current columns's filters
            const currentColumnKeys = this.props.columns.map(c => getColumnKey(c));
            Object.keys(filters).forEach(columnKey => {
                if (currentColumnKeys.indexOf(columnKey) < 0) {
                    delete filters[columnKey];
                }
            });
            this.tableRef.current.getFilteredValueColumns().forEach(col => {
                const columnKey = getColumnKey(col);
                if (columnKey) {
                    delete filters[columnKey];
                }
            });

            if (pagination) {
                pagination.current = 1;
            }

            this.actions.onChanged(pagination, filters, sorter);
        },

        onChanged(pagination, filters, sorter) {
            Object.assign(this.privates.pagination, pagination);
            Object.assign(this.privates.filters, filters);
            Object.assign(this.privates.sorter, sorter);
            this.trigger('onChanged', this.privates.pagination, this.privates.filters, this.privates.sorter);
        },

        onColumnsVisibleChanged(updateLocaleColumns) {
            const { filterColumns, localColumns } = generateColumns(updateLocaleColumns, this.privates.localColumns);
            this.privates.filterColumns = filterColumns;
            this.privates.localColumns = localColumns;
            this.actions.update();
            this.trigger('onColumnsVisibleChanged', updateLocaleColumns);
        },
        onSelectChanged(selectedRowKeys, selectedRows) {
            // window.event.stopPropagation();
            // window.event.stopImmediatePropagation();
            // window.event.preventDefault();
            this.privates.selectedRowKeys = selectedRowKeys;
            this.actions.update();
            this.trigger('onSelectChanged', selectedRowKeys, selectedRows);
            // e.stopPropagation();
            // e.preventDefault();
        },
        onEditData(record, values) {
            this.trigger('onEditData', record, values);
        }
    };
    // #endregion

    // #region render
    render() {
        const {
            columns,
            onRow,
            pagination,
            onChanged,
            showTitle,
            title,
            className,
            onRowClick,
            locale,
            ...otherProps
        } = this.props;
        let rowSelection = null;
        if (this.props.showCheckboxColumn) {
            rowSelection = {
                selectedRowKeys: this.privates.selectedRowKeys,
                onChange: this.actions.onSelectChanged
            };
        }
        const titleContent = currentPageData => {
            return title;
        };

        const table = (
            <Table
                components={
                    this.props.columnDragable
                        ? Object.assign({}, defaultComponents, {
                            header: {
                                cell: ResizeableColumn
                            }
                        })
                        : defaultComponents
                }
                locale={locale}
                title={showTitle ? titleContent : null}
                ref={this.tableRef}
                columns={this.privates.filterColumns}
                scroll={this.props.scroll}
                onRow={record => {
                    return {
                        onClick: event => {
                            this.trigger('onRowClick', record, event);
                        },
                        onDoubleClick: event => {
                            this.trigger('onDoubleClick', record, event);
                        },
                        onContextMenu: event => {
                            this.trigger('onContextMenu', record, event);
                        },
                        onMouseEnter: event => {
                            this.trigger('onMouseEnter', record, event);
                        },
                        onMouseLeave: event => {
                            this.trigger('onMouseLeave', record, event);
                        }
                    };
                }}
                pagination={this.privates.pagination}
                onChange={(pagination, filters, sorter) => this.actions.onChanged(pagination, filters, sorter)}
                rowSelection={rowSelection}
                {...otherProps}
            />
        );

        return (
            <div className={ClassNames('react-antd-table', className)}>
                <If condition={this.props.showColumnSetting}>
                    <Popover
                        content={
                            <ColumnsSetting
                                localColumns={this.privates.localColumns}
                                onColumnsVisibleChanged={this.actions.onColumnsVisibleChanged}
                            />
                        }
                        placement='bottomRight'
                        title={columnSetting}
                        trigger='click'
                    >
                        <Icon type='setting' />
                    </Popover>
                </If>

                <If condition={this.props.columnDragable}>
                    <ReactDragListView.DragColumn
                        {...{
                            onDragEnd: (fromIndex, toIndex) => {
                                const newFilterColumns = clone(this.privates.filterColumns);
                                newFilterColumns.splice(
                                    this.props.showCheckboxColumn ? toIndex - 1 : toIndex,
                                    0,
                                    newFilterColumns.splice(
                                        this.props.showCheckboxColumn ? fromIndex - 1 : fromIndex,
                                        1
                                    )[0]
                                );
                                this.privates.filterColumns = newFilterColumns;
                                this.actions.update();
                            },
                            nodeSelector: 'th'
                        }}
                    >
                        {table}
                    </ReactDragListView.DragColumn>
                </If>
                <If condition={!this.props.columnDragable}>{table}</If>
            </div>
        );
    }
    // #endregion
}

export default ReactAntdTable;
