import React from 'react';
import PropTypes from 'prop-types';
import Form from 'antd/lib/form';
import 'antd/lib/form/style/index.css';
import Tooltip from 'antd/lib/tooltip';
import 'antd/lib/tooltip/style/index.css';
import Icon from 'antd/lib/icon';
import 'antd/lib/icon/style/index.css';
import Input from 'antd/lib/input';
import 'antd/lib/input/style/index.css';
import Udesk from '../../udesk';
import 'antd/es/grid/style/index.css';

import formUtils from './utils';

const propsCreator = (node, label = '', id, disabledFields) => {
    let nodeProps = node.props,
        placeholder = nodeProps.placeholder || formUtils.initPlaceholder(node, label);
    if (
        typeof disabledFields !== 'undefined' &&
        typeof id === 'string' &&
        (disabledFields === 'all' || (Array.isArray(disabledFields) && disabledFields.includes(id)))
    ) {
        nodeProps = { ...nodeProps, placeholder: '', disabled: true };
    }
    nodeProps = Object.assign({}, { placeholder }, nodeProps);
    // 返回格式化后的 node
    return { ...node, props: nodeProps };
};

Form.displayName = 'ReactForm';

const removeUndefined = obj => {
    Object.keys(obj).forEach(key => {
        if (obj[key] === undefined) delete obj[key];
    });
    return obj;
};

const isNone = obj => {
    return obj === null || obj === undefined;
};

//特殊处理 submit
const removeUselessValue = obj => {
    Object.keys(obj).forEach(key => {
        if (key === 'submit') {
            delete obj[key];
        }
        if (isNone(obj[key])) {
            delete obj[key];
        }
        //trim string result
        if (typeof obj[key] === 'string') {
            obj[key] = obj[key].trim();
        }
    });
    return obj;
};

const createLabel = (label, tooltip) => {
    if (isNone(tooltip)) {
        return label;
    } else {
        return (
            <span>
                {label}&nbsp;
                <Tooltip title={tooltip}>
                    <Icon type="question-circle-o" />
                </Tooltip>
            </span>
        );
    }
};

const getMetadata = formMetadata => {
    return Array.isArray(formMetadata) ? formMetadata : [formMetadata];
};

// Form.render
Form.buildFormRender = (form, formMetadata, data, disabledFields, formColon, formLayout) => {
    Form.render = nodes => {
        let formMetadataNodes = buildFormMetadata(formMetadata);
        let mergeNodes = [].concat(formMetadataNodes, nodes).reduce((result, current) => {
            if (!isNone(current)) {
                result.push(current);
            }
            return result;
        }, []);
        return buildFormNode(form, mergeNodes, data, disabledFields, formColon, formLayout);
    };
};

const buildFormMetadata = formMetadata => {
    if (isNone(formMetadata) || typeof formMetadata !== 'object') {
        return null;
    }
    return getMetadata(formMetadata).map(element => {
        const { webget, children, ...widgetProps } = element;
        element.webget = element.webget || Input;
        return <element.webget {...widgetProps}>{children || null}</element.webget>;
    });
};

const buildFormNode = (form, nodes, data, disabledFields, formColon, formLayout) => {
    if (nodes === null || typeof nodes !== 'object') {
        return nodes;
    }

    return React.Children.map(nodes, node => {
        if (node === null || typeof node !== 'object' || !node.props) {
            return node;
        }
        const { label, tooltip, id, ...nodeProps } = node.props;

        const {
            // UI props
            className = '',
            style,

            // Extra options
            hide,

            // Item props
            colon: uniqueColon,
            extra,
            hasFeedback,
            help,
            // label,
            labelCol,
            required,
            validateStatus,
            wrapperCol,

            // Two sides
            before,
            after,

            formItemLayout,
            // Other props
            ...otherProps
        } = nodeProps;

        const itemProps = removeUndefined({
            // colon,
            extra,
            hasFeedback,
            help,
            // label,
            labelCol,
            required,
            validateStatus,
            wrapperCol
        });

        let currentFormItemLayout = null;

        if (formLayout === 'horizontal') {
            currentFormItemLayout =
                formItemLayout ||
                (id === 'submit'
                    ? { wrapperCol: { span: 2, offset: 10 } }
                    : {
                        labelCol: {
                            xs: { span: 24 },
                            sm: { span: 4 }
                        },
                        wrapperCol: {
                            xs: { span: 24 },
                            sm: { span: 16 }
                        }
                    });
        } else {
            currentFormItemLayout =
                formItemLayout || (id === 'submit' ? { wrapperCol: { span: 2, offset: 10 } } : formLayout);
        }

        //TODO: 递归判断是否有内嵌表单
        if (id === undefined) {
            if (label === undefined) {
                const newChildren = Form.render(nodeProps.children);
                return propsCreator({ ...node, props: { ...nodeProps, children: newChildren } });
            }

            const itemColon = uniqueColon !== undefined ? uniqueColon : formColon;

            return (
                <Form.Item
                    {...currentFormItemLayout}
                    className={className}
                    style={style}
                    colon={itemColon}
                    label={createLabel(label, tooltip)}
                    {...itemProps}>
                    {before && Form.render(before)}
                    {propsCreator({ ...node, props: otherProps }, label)}
                    {after && Form.render(after)}
                </Form.Item>
            );
        } else {
            const itemStyle = hide === true ? { display: 'none' } : style;
            const itemColon = uniqueColon !== undefined ? uniqueColon : formColon;

            const {
                // Form options
                getValueFromEvent,
                initialValue,
                normalize,
                preserve,
                rules,
                trigger,
                validateFirst,
                validateTrigger,
                valuePropName,
                // rc-form options
                hidden,
                getValueProps,
                validate,

                ...fieldProps
            } = otherProps;

            const options = removeUndefined({
                getValueFromEvent,
                // initialValue,
                normalize,
                preserve,
                // rules,
                trigger,
                validateFirst,
                // validateTrigger,
                valuePropName,

                // hidden,
                getValueProps,
                validate
            });

            const isNestedField = fieldProps.form !== undefined; // 是否嵌套表单域（`a` & `a.b`）
            const isValidRules = Array.isArray(rules);

            return (
                <Form.Item
                    {...currentFormItemLayout}
                    className={`${className} ${id}`}
                    style={itemStyle}
                    colon={itemColon}
                    label={createLabel(label, tooltip)}
                    {...itemProps}>
                    {before}
                    {form.getFieldDecorator(isNestedField ? `${id}.nested` : id, {
                        initialValue: initialValue !== undefined ? initialValue : Udesk.utils.object.get(data, id),
                        rules: isValidRules && formUtils.initRules(label, id, rules),
                        trigger: trigger || 'onChange',
                        validateTrigger:
                            validateTrigger || (isValidRules && rules.includes('phone')) ? 'onBlur' : 'onChange',
                        hidden: hidden || isNestedField,
                        ...options
                    })(propsCreator({ ...node, props: fieldProps }, label, id, disabledFields))}
                    {after}
                </Form.Item>
            );
        }
    });
};

class ReactForm extends Form {
    // #region defaultProps
    static defaultProps = {
        data: {},
        colon: false,
        formLayout: 'horizontal',
        autocomplete: 'off'
    };
    // #endregion

    // #region propTypes
    static propTypes = {
        //layout
        formLayout: PropTypes.object,
        // fill form data
        // e.g. { [id]: value, ... }
        data: PropTypes.object,
        // disable form，全部禁用传 "all"，部分禁用传 id 组成的数组
        disabledFields: PropTypes.array,
        // colon
        colon: PropTypes.bool,
        // submit
        onSubmit: PropTypes.func,
        // autocomplete
        autocomplete: PropTypes.string,
        //数据驱动属性
        //[{
        // label: 字段显示名称
        // id: 字段名称,
        // rules: 规则属性 如['required', 'string', 'max=10'],
        // webget: 组件类型 如 Input
        // }]
        formMetadata: PropTypes.arrayOf(PropTypes.object)
    };
    // #endregion

    // #region public function
    onSubmit = e => {
        e.preventDefault();
        this.props.form.validateFields((err, values) => {
            if (!err) {
                const { onSubmit } = this.props;
                onSubmit(removeUselessValue(values));
            }
        });
    };
    // #endregion

    // #region life cycle
    /* eslint-disable no-useless-constructor */
    constructor(props) {
        super(props);
    }
    /* eslint-enable no-uesless-constructor */
    init() {}
    componentDidMount() {}
    componentWillUnmount() {}
    // #endregion

    // #region actions
    actions = {};
    // #endregion

    // #region render
    render() {
        const {
            className = '',
            form,
            formMetadata,
            data,
            disabledFields,
            formLayout,
            colon: formColon,
            onSubmit,
            children,
            ...otherProps
        } = this.props;

        Form.buildFormRender(form, formMetadata, data, disabledFields, formColon, formLayout);

        return (
            <Form className={`react-form ${className}`} onSubmit={this.onSubmit.bind(this)} {...otherProps}>
                {Form.render && Form.render(children)}
            </Form>
        );
    }
    // #endregion
}

export default Form.create({
    // 表单项变化时调用
    onValuesChange({ onValuesChange, ...restProps }, changedValues, allValues) {
        onValuesChange && onValuesChange(restProps, changedValues, allValues);
    }
})(ReactForm);
