import React from "react";
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import ReactTool from '../../react-field-tool';
import {
    BubbleLoader
} from 'react-css-loaders';
import ReactPager from '../../react-pager';
import ReactTable from '../../react-table';
import common from '../../../udesk';
import Locales from '../../../udesk/locales';

const REACT_TABLE_MAX_HEIGHT = 300;
const REACT_TABLE_BOTTOM_HEIGHT = 50;
const DEFAULT_FUNCTION = function () { };
const DEFAULT_PAGING = {
    pageNum: common.config.paging.defaultPageNumber,
    pageSize: common.config.paging.defaultPageSize
};

class ReportTable extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            tableColumns: [],
            stackedHeaderGroups: [],
        };
        this.reactTableMaxHeight = REACT_TABLE_MAX_HEIGHT;
    }

    componentDidMount() {
        setReactTableMaxHeight(this);
        buildTableColumnsData.call(this, this.props);
    }
    componentDidUpdate(prevProps, prevState, snapshot) {
        let {
            reportData,
            orderTypeEnum,
            orderField,
            selectedFields
        } = this.props;
        let {
            reportData: oldReportData,
            orderTypeEnum: oldOrderTypeEnum,
            orderField: oldOrderField,
            selectedFields: oldSelectedFields
        } = prevProps;

        let isRefreshTabaleColumns = (reportData !== oldReportData ||
            orderTypeEnum !== oldOrderTypeEnum ||
            orderField !== oldOrderField ||
            selectedFields !== oldSelectedFields);

        if (isRefreshTabaleColumns) {
            buildTableColumnsData.call(this, this.props);
            setReactTableMaxHeight(this);
        }
    }
    componentWillUnmount() {
        this.columnContainerInstance = null;
        this.reactTableInstance = null;
    }

    render() {
        let {
            enableTableTitle,
            tableTitleText,
            enableTableToolTip,
            tableToolTipText,
            isLoading,
            resportPaging,
            sortable,
            languageCode,
            noReportDataText,
            tableRowClickable,
            freezeHeader,
            horizontalScrollbar,
            verticalScrollbar,
            rowNavigationMode,
            loadingColor,
            loadingSize,
            loadingDuration,
            getCellNavigationLink,
            reportData,
            classNames,
            enableReactPager,
            freezeColumn,
            freezeColumnNumber,
            rowContentClick,
            pageChanged,
            onSorted,
            noSortingIconClass,
            sortAscIconClass,
            sortDescIconClass,
            enableChooseColumns,
            chooseColumnsIconClass,
            rowClick,
        } = this.props;
        if (reportData == null || typeof reportData !== "object") {
            reportData = {};
        }
        if (resportPaging == null || typeof resportPaging !== "object") {
            resportPaging = DEFAULT_PAGING;
        }
        if (noReportDataText == null || noReportDataText === "") {
            noReportDataText = Locales.get("components.reportTable.noReportData", languageCode);
        }
        if (reportData.rows == null) {
            reportData.rows = [];
        }
        if (!(reportData.rows instanceof Array)) {
            reportData.rows = [reportData.rows];
        }
        let {
            tableColumns,
            stackedHeaderGroups,
        } = this.state;

        return (
            <div className={ClassNames("component-report-table", { [`${classNames}`]: true })}>
                <If condition={enableTableTitle}>
                    <div className="report-table-header">
                        <p className="report-table-header-title">
                            {tableTitleText}
                        </p>
                        <If condition={enableTableToolTip}>
                            <ReactTool tooltipData={tableToolTipText} />
                        </If>
                    </div>
                </If>
                <div className="report-table-content" ref={(reactTable) => this.reactTableInstance = reactTable}>
                    <Choose>
                        <When condition={isLoading}>
                            <BubbleLoader color={loadingColor} duration={loadingDuration} size={loadingSize} />
                        </When>
                        <When condition={!isLoading}>
                            <Choose>
                                <When condition={(tableColumns && tableColumns.length > 0 && reportData.rows.length > 0)}>
                                    <ReactTable columns={tableColumns}
                                        items={reportData.rows}
                                        freezeHeader={freezeHeader}
                                        horizontalScrollbar={horizontalScrollbar}
                                        maxHeight={this.reactTableMaxHeight}
                                        verticalScrollbar={verticalScrollbar}
                                        sortable={sortable}
                                        tableRowClickable={tableRowClickable}
                                        rowNavigationMode={rowNavigationMode}
                                        rowContentClick={rowContentClick}
                                        getCellNavigationLink={getCellNavigationLink}
                                        onSorted={onSorted}
                                        freezeColumn={freezeColumn}
                                        freezeColumnNumber={freezeColumnNumber}
                                        stackedHeaderGroups={stackedHeaderGroups}
                                        noSortingIconClass={noSortingIconClass}
                                        sortAscIconClass={sortAscIconClass}
                                        sortDescIconClass={sortDescIconClass}
                                        enableChooseColumns={enableChooseColumns}
                                        chooseColumnsIconClass={chooseColumnsIconClass}
                                        rowClick={rowClick} />
                                    <If condition={enableReactPager}>
                                        <ReactPager pageSize={resportPaging.pageSize} total={resportPaging.total} pageNum={resportPaging.pageNum} pageChanged={pageChanged} />
                                    </If>
                                </When>
                                <When condition={(reportData.rows.length === 0)}>
                                    <div className="not-has-report-data">
                                        {noReportDataText}
                                    </div>
                                </When>
                            </Choose>
                        </When>
                    </Choose>
                </div>
            </div>
        );
    }
}

function getStackedHeaderGroups(stackedHeaderGroups, fieldItem) {
    let group = fieldItem.group;
    if (group != null) {
        let targetHeaderGroupsItem = stackedHeaderGroups.find(groupItem => groupItem.text === group);
        if (targetHeaderGroupsItem != null) {
            targetHeaderGroupsItem.columns.push(fieldItem.fieldKey);
        } else {
            let stackedHeaderGroupsItem = {
                text: group,
                stackedHeaderClass: "stacked-header-cell-item",
                columns: [fieldItem.fieldKey],
                groups: []
            };
            stackedHeaderGroups.push(stackedHeaderGroupsItem);
        }
    }
    return stackedHeaderGroups;
}
//转化表格列数据
function buildTableColumnsData(props) {
    let {
        reportData,
        orderTypeEnum,
        orderField,
        selectedFields,
        hasStackedHeaderGroups
    } = props;

    let allColumns = [];
    let stackedHeaderGroups = [];

    if (reportData) {
        let currentOrderField = undefined;
        let orderTypeEnums = undefined;
        if (orderField && orderTypeEnum) {
            currentOrderField = orderField;
            orderTypeEnums = orderTypeEnum;
        }

        if (reportData.header && reportData.header.dimensions && reportData.header.dimensions.length > 0) {
            let dimensions = reportData.header.dimensions;
            dimensions.forEach(dimensionItem => {
                let visible = true;
                let sortDirection = null;
                if (selectedFields && selectedFields.length > 0) {
                    visible = selectedFields.some(fieldKey => dimensionItem.fieldKey === fieldKey);
                }
                if (orderTypeEnums && currentOrderField && currentOrderField === dimensionItem.fieldKey) {
                    sortDirection = orderTypeEnums;
                }

                let hasIsShowPropery = Object.prototype.hasOwnProperty.call(dimensionItem,"isShow") && ("isShow" in dimensionItem);
                //isShow 为后端进行隐藏字段
                if (hasIsShowPropery) {
                    visible = dimensionItem.isShow;
                }

                let dimensionColumns = {
                    name: `${dimensionItem.fieldKey}`,
                    caption: dimensionItem.fieldTitle,
                    visible,
                    sortable: false,
                    sortDirection,
                    get: function (item, column) {
                        let fieldKey = column.name;
                        let fieldKeyOriginal = `${fieldKey}_original`;

                        let itemValue = 0;
                        if (item != null) {
                            if (item[fieldKey] != null) {
                                itemValue = item[fieldKey];
                            } else if (item[fieldKeyOriginal] != null) {
                                itemValue = item[fieldKeyOriginal];
                            }
                        }
                        if (itemValue == null) {
                            itemValue = 0;
                        }
                        return itemValue;
                    }
                };
                if (hasStackedHeaderGroups) {
                    stackedHeaderGroups = getStackedHeaderGroups(stackedHeaderGroups, dimensionItem);
                }
                allColumns.push(dimensionColumns);
            });
        }

        if (reportData.header && reportData.header.indicators && reportData.header.indicators.length > 0) {
            let indicators = reportData.header.indicators;
            indicators.forEach(item => {
                let visible = true;
                let sortDirection = null;

                if (selectedFields && selectedFields.length > 0) {
                    visible = selectedFields.some(fieldKey => item.fieldKey === fieldKey);
                }
                if (orderTypeEnums && currentOrderField && currentOrderField === item.fieldKey) {
                    sortDirection = orderTypeEnums;
                }
                let indicatorColumns = {
                    name: `${item.fieldKey}`,
                    caption: item.fieldTitle,
                    visible,
                    sortable: true,
                    sortDirection,
                    get: function (item, column) {
                        let fieldKey = column.name;
                        let fieldKeyOriginal = `${fieldKey}_original`;

                        let itemValue = 0;
                        if (item != null) {
                            if (item[fieldKey] != null) {
                                itemValue = item[fieldKey];
                            } else if (item[fieldKeyOriginal] != null) {
                                itemValue = item[fieldKeyOriginal];
                            }
                        }
                        if (itemValue == null) {
                            itemValue = 0;
                        }
                        return itemValue;
                    },
                    hasTemplate: false,
                    getYieldContent: function (name, item, index) {
                        return "";
                    }
                };
                if (hasStackedHeaderGroups) {
                    stackedHeaderGroups = getStackedHeaderGroups(stackedHeaderGroups, item);
                }
                allColumns.push(indicatorColumns);
            });
        }
    }

    this.setState({
        tableColumns: allColumns,
        stackedHeaderGroups,
    });
}


function setReactTableMaxHeight(that) {
    if (that.reactTableInstance) {
        let reactTableBox = that.reactTableInstance.getBoundingClientRect(); //获取其位置信息
        let top = document.documentElement.clientTop; //处理兼容IE 2px 问题
        let realTop = reactTableBox.top - top;

        let maxHeight = document.documentElement.clientHeight - (realTop + REACT_TABLE_BOTTOM_HEIGHT);
        if (maxHeight > REACT_TABLE_MAX_HEIGHT) {
            that.reactTableMaxHeight = maxHeight;
        }
    }
}

ReportTable.propTypes = {
    classNames: PropTypes.string,
    languageCode: PropTypes.string,
    enableTableTitle: PropTypes.bool,
    tableTitleText: PropTypes.string,
    enableTableToolTip: PropTypes.bool,
    loadingColor: PropTypes.string,
    loadingSize: PropTypes.number,
    loadingDuration: PropTypes.number,
    noReportDataText: PropTypes.string,
    tableRowClickable: PropTypes.bool,

    //table config
    freezeHeader: PropTypes.bool,
    horizontalScrollbar: PropTypes.bool,
    verticalScrollbar: PropTypes.bool,
    sortable: PropTypes.bool,
    rowNavigationMode: PropTypes.string,
    enableReactPager: PropTypes.bool,

    //method
    saveSelectedColumn: PropTypes.func,
    onSorted: PropTypes.func,
    pageChanged: PropTypes.func,
    noSortingIconClass: PropTypes.string,
    sortAscIconClass: PropTypes.string,
    sortDescIconClass: PropTypes.string,
    enableChooseColumns: PropTypes.bool,
    chooseColumnsIconClass: PropTypes.string,
};

ReportTable.defaultProps = {
    classNames: "",
    languageCode: "",
    reportData: {},
    resportPaging: DEFAULT_PAGING,
    enableTableTitle: false,
    tableTitleText: "",
    enableTableToolTip: false,
    tableToolTipText: null,
    isLoading: false,
    loadingColor: "#BEC2C1",
    loadingSize: 12,
    loadingDuration: 1.4,
    noReportDataText: "",

    //table config
    freezeHeader: true,
    horizontalScrollbar: false,
    verticalScrollbar: true,
    sortable: false,
    rowNavigationMode: "click",//click,link
    tableRowClickable: false,
    orderTypeEnum: common.enums.sortDirection.none,
    orderField: null,
    selectedFields: [],
    enableReactPager: true,
    freezeColumn: false,
    freezeColumnNumber: 1,
    hasStackedHeaderGroups: false,
    sortAscIconClass: "udesk-react-iconfont icon-udesk-react-sort-amount-asc",
    sortDescIconClass: "udesk-react-iconfont icon-udesk-react-sort-amount-desc",
    noSortingIconClass: "udesk-react-iconfont icon-udesk-react-sort",
    enableChooseColumns: false,
    chooseColumnsIconClass: "udesk-react-iconfont icon-udesk-react-gear",

    //method
    saveSelectedColumn: DEFAULT_FUNCTION,
    rowContentClick: null,
    onSorted: DEFAULT_FUNCTION,
    pageChanged: DEFAULT_FUNCTION,
    rowClick: null,
};

export default ReportTable;