import React, { useCallback, useContext, useMemo } from "react";
import { Button, Popover, Tree } from "udesk-ui";
import { TreeProps } from "udesk-ui/es/tree";
import LinkOutlined from "@ant-design/icons/LinkOutlined";
import { Spin } from "udesk-ui/es";
import ScopeSwitch from "../scope-switch";
import { useTranslation } from "react-i18next";
import {
  hit,
  RepositoryWidgetContext,
  useCatalogMap,
  useExpandedKeys,
  useRequestData,
  useSafeScope,
} from "../utilities";
import { TFunction } from "i18next";
import css from "../style.module.scss";
import { canPreviewByFileName, getPreviewUrl } from "../../../utils";
import { IAttachment } from "../types";
import SearchInput from "../search-input";
interface Props {
  query: string;
  onSearch: (query: string) => void;
}

interface Catalog {
  id: number;
  name: string;
  parent_id: number;
  sort: number;
  subitem_count: number;
  children: Catalog[];
}

const TreeNodeTitle: React.FC<{
  data: IAttachment["knowledge_attachment"];
}> = ({ data }) => {
  const { t } = useTranslation();
  const { onSend, previewUrl } = useContext(RepositoryWidgetContext);
  const onClick = useCallback(() => {
    onSend?.("attachment", data);
    hit(data.id, "KnowledgeAttachment");
  }, [data, onSend]);

  const content = useMemo(
    () => [
      <LinkOutlined key="0" />,
      <div key={"1"} style={{ flex: "1 1" }}>
        <a href={data.file_url} rel="noopener noreferrer" target="_blank">
          {data.file_name}
        </a>
      </div>,
      <div key={"2"} className={css.nodeOperations}>
        {canPreviewByFileName(data.file_name) ? (
          <Button
            type="link"
            size="small"
            target="_blank"
            href={getPreviewUrl(data.file_url, { previewUrl })}
          >
            {t("label.preview")}
          </Button>
        ) : null}
        <Button type="link" size="small" onClick={onClick}>
          {t("label.send")}
        </Button>
      </div>,
    ],
    [data.file_name, data.file_url, onClick, previewUrl, t]
  );
  if (
    data.file_type.startsWith("image") ||
    /png|jpg|jpeg|gif/.test(data.file_type)
  ) {
    return (
      <Popover
        autoAdjustOverflow={false}
        content={
          <img
            src={data.file_url}
            alt={data.file_name}
            style={{ width: "100%" }}
          />
        }
        placement="left"
        overlayStyle={{ maxWidth: "300px", maxHeight: "300px" }}
      >
        <div style={{ display: "flex", flex: "1 0", alignItems: "center" }}>
          {content}
        </div>
      </Popover>
    );
  } else {
    return <>{content}</>;
  }
};

function buildTreeData(
  catalogs: Catalog[],
  catalogMap: { [key: string]: IAttachment[] },
  t: TFunction,
  parentKey?: string
): TreeProps["treeData"] {
  return catalogs.map((i) => {
    const key = parentKey ? `${parentKey}-${i.id}` : i.id + "";
    return {
      key: key,
      title: i.name,
      selectable: false,
      className: css["treeNode"],
      children: (() => {
        if (i.children) {
          return buildTreeData(i.children, catalogMap, t, key);
        } else {
          return (catalogMap[i.id] || []).map((j) => {
            return {
              key: `${key}-content${j.knowledge_attachment.id}`,
              className: css["treeNode"],
              title: <TreeNodeTitle data={j.knowledge_attachment} />,
            };
          });
        }
      })(),
    };
  });
}

const Attachment: React.FC<Props> = ({ query, onSearch }) => {
  const { t } = useTranslation();
  const { expandedKeys, onExpand, activeScope, onScopeChange } = useContext(
    RepositoryWidgetContext
  );
  const _scope = useSafeScope(activeScope);
  const params = useMemo(() => {
    return {
      scope: _scope,
      per_page: 1000,
      catalog_id: "all",
      query,
    };
  }, [_scope, query]);
  const { loading, catalogs, data } = useRequestData("attachment", params);
  const catalogMap = useCatalogMap(data?.knowledge_attachments ?? []);
  const _expandedKeys = useExpandedKeys(catalogMap);

  const treeData = useMemo(() => {
    if (!data || !data.knowledge_attachments) {
      return [];
    }
    if (query) {
      return data.knowledge_attachments.map(
        ({ knowledge_attachment }: IAttachment) => {
          return {
            key: knowledge_attachment.id,
            className: css["treeNode"],
            title: <TreeNodeTitle data={knowledge_attachment} />,
          };
        }
      );
    } else {
      return buildTreeData(catalogs, catalogMap, t);
    }
  }, [catalogMap, catalogs, data, query, t]);
  const mainContent = useMemo(() => {
    if (loading) {
      return <Spin />;
    } else if (treeData && treeData.length) {
      return (
        <Tree
          blockNode
          selectable={false}
          expandedKeys={_expandedKeys}
          onExpand={onExpand}
          showIcon
          treeData={treeData}
        />
      );
    } else {
      return (
        <p>{t("page.im.km.no_result", { name: "$t(label.useful_link)" })}</p>
      );
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [expandedKeys, loading, onExpand, t, treeData]);
  return (
    <>
      <ScopeSwitch value={_scope} onChange={onScopeChange} />
      <SearchInput onSearch={onSearch} defaultValue={query} />
      {mainContent}
    </>
  );
};

export default Attachment;
