import React, {
  ContextType,
  useCallback,
  useEffect,
  useMemo,
  useRef,
  useState,
} from "react";
import css from "./style.module.scss";
import Tabs from "udesk-ui/es/tabs";
import { useTranslation } from "react-i18next";
import { getJSON } from "../../utils";
import Km from "./km";
import UsefulExpressionTabPane from "./useful_expression";
import UsefulLinkTabPane from "./useful_link";
import QuestionTabPane from "./question";
import AttachmentTabPane from "./attachment";
import {
  RepositoryWidgetContext,
  useSelectionCache,
  useTabSetting,
} from "./utilities";
import { KmSetting, RepositoryType, Scope } from "./types";
import Spin from "udesk-ui/es/spin";
import ErrorBoundary from "../simple-error-boundary";
import {csApiGetJSON} from "../../utils/csApi";

const TabPane = Tabs.TabPane;

interface Props {
  companyId: number; //当前用户的公司 id
  email: string; //当前用户的 email
  subdomain: string; //当前用户的公司 subdomain
  onSpaceChange?: (space: number) => void;
  onCatalogChange?: (catalog: number) => void;
  expandKeys?: string[];
  scrollTop?: number;
  onUse?: ContextType<typeof RepositoryWidgetContext>["onUse"];
  onSend?: ContextType<typeof RepositoryWidgetContext>["onSend"];
  allowSendQuestion?: boolean;
  showRobotQuestionTab?: boolean;
  kmApiBase: string;
  selectionStateKey?: string;
  showPublicScope?: boolean;
  previewUrl: string; //预览服务器的地址，用于生成最终的预览链接，必传
  customer?: Customer;
  ticket?: ITicket;
  agent?: IAgent;
  ticketCustomFields?: CustomField[];
  customerCustomFields?: CustomField[];
}

const tabComponentMap = {
  useful_expression: UsefulExpressionTabPane,
  useful_link: UsefulLinkTabPane,
  question: QuestionTabPane,
  attachment: AttachmentTabPane,
};

const RepositoryWidget: React.FC<Props> = ({
  companyId,
  email,
  subdomain = "useful_expression",
  onSpaceChange,
  onCatalogChange,
  onUse,
  allowSendQuestion,
  showRobotQuestionTab,
  onSend,
  kmApiBase,
  selectionStateKey,
  showPublicScope,
  previewUrl,
  customer,
  ticket,
  agent,
  ticketCustomFields,
  customerCustomFields,
}) => {
  const [kmSettings, setKmSettings] = useState<KmSetting>();
  const { t } = useTranslation();
  const {
    saveTab,
    tabSetting,
    loading,
    saveSpace,
    saveScope,
    saveCatalog,
  } = useTabSetting();
  const {
    activeTab,
    setActiveTab,
    query,
    setQuery,
    activeScope,
    setActiveScope,
    expandedKeys,
    setExpandedKeys,
    // scrollTop,
    // setScrollTop,
  } = useSelectionCache(
    selectionStateKey,
    useMemo(
      () => ({
        defaultTab: tabSetting?.activeTab,
        defaultScope: tabSetting?.activeScope,
      }),
      [tabSetting?.activeScope, tabSetting?.activeTab]
    )
  );

  useEffect(() => {
    csApiGetJSON("/cs/ts/api/knowledge_manager_settings").then(res=>res.data).then((res) => {
      setKmSettings(res.knowledge_manager_setting);
    });
  }, []);

  const onTabChange = useCallback(
    (tabKey) => {
      saveTab(tabKey);
      setActiveTab(tabKey);
    },
    [saveTab, setActiveTab]
  );

  const tabs: Array<RepositoryType> = useMemo(
    () => ["useful_expression", "useful_link", "question", "attachment"],
    []
  );
  const onScopeChange = useCallback(
    (_scope) => {
      saveScope(_scope);
      setActiveScope(_scope);
    },
    [saveScope, setActiveScope]
  );

  const contextValue = useMemo(
    () => ({
      allowSendQuestion,
      onUse,
      showRobotQuestionTab,
      onSend,
      kmApiBase,
      onExpand: setExpandedKeys,
      expandedKeys,
      activeScope,
      onScopeChange,
      activeCatalog: tabSetting?.activeCatalog,
      onCatalogChange: saveCatalog,
      activeSpace: tabSetting?.activeSpace,
      onSpaceChange: saveSpace,
      showPublicScope,
      previewUrl,
      customer,
      ticket,
      agent,
      ticketCustomFields,
      customerCustomFields,
    }),
    [
      activeScope,
      allowSendQuestion,
      expandedKeys,
      kmApiBase,
      setExpandedKeys,
      onScopeChange,
      onSend,
      onUse,
      saveCatalog,
      saveSpace,
      showPublicScope,
      showRobotQuestionTab,
      tabSetting?.activeCatalog,
      tabSetting?.activeSpace,
      previewUrl,
      customer,
      ticket,
      agent,
      ticketCustomFields,
      customerCustomFields,
    ]
  );

  const ref = useRef(null);

  if (loading || !kmSettings) {
    return <Spin />;
  }
  return (
    <ErrorBoundary>
      <RepositoryWidgetContext.Provider value={contextValue}>
        <div ref={ref} className={css.container}>
          {/*TODO 等以后后端支持全局搜索后放开*/}
          {/*<div>*/}
          {/*    <Input.Search value={query} onSearch={v => setQuery(v)}/>*/}
          {/*</div>*/}
          <div className={css.kmContent}>
            <Tabs
              activeKey={activeTab}
              onChange={onTabChange}
              size="small"
              animated={false}
              tabBarGutter={0}
              destroyInactiveTabPane={true}
            >
              {tabs.map((i) => {
                const TabComponent = tabComponentMap[i];
                return (
                  <TabPane tab={t(`page.im.km.${i}.title`)} key={i}>
                    <ErrorBoundary>
                      <TabComponent query={query} onSearch={setQuery} />
                    </ErrorBoundary>
                  </TabPane>
                );
              })}
              {kmSettings?.name && (
                <TabPane tab={kmSettings.name} key="km">
                  <ErrorBoundary>
                    <Km
                      {...kmSettings}
                      companyId={companyId}
                      email={email}
                      subdomain={subdomain}
                      query={query}
                      onSearch={setQuery}
                    />
                  </ErrorBoundary>
                </TabPane>
              )}
            </Tabs>
          </div>
        </div>
      </RepositoryWidgetContext.Provider>
    </ErrorBoundary>
  );
};

export default RepositoryWidget;
