import React, { useCallback, useContext, useEffect, useState } from "react";
import { Button, List, Popover, Spin } from "udesk-ui";
import { useTranslation } from "react-i18next";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import axios from "axios";
import DOMPurify from "dompurify";
import { articleToCommonQuestion, RepositoryWidgetContext } from "../utilities";
import { IArticle } from "../types";

interface Params {
  companyId: number;
  subdomain: string;
  email: string;
  limitSpaceIds: string;
  enablePrivate: boolean;
  passport: string;
}

type Props = Params & { data: IArticle };

function getArticle(
  kmBase: string,
  params: Params & { id: Props["data"]["id"] }
) {
  if (!kmBase) {
    return Promise.reject("km host is empty");
  }
  return axios
    .get(`${kmBase}/articles/${params.id}`, {
      params: {
        companyId: params.companyId,
        subdomain: params.subdomain,
        email: params.email,
        limitSpaceIds: params.limitSpaceIds,
        enablePrivate: params.enablePrivate,
        passport: params.passport,
      },
    })
    .then((res) => {
      if (res.data.code === 200) {
        return res.data.data;
      }
      throw new Error(res.data);
    });
}

const useArticle = (
  host: string,
  {
    companyId,
    subdomain,
    email,
    limitSpaceIds,
    enablePrivate,
    passport,
    articleId,
  }: Pick<
    Props,
    | "companyId"
    | "subdomain"
    | "email"
    | "limitSpaceIds"
    | "enablePrivate"
    | "passport"
  > & { articleId: Props["data"]["id"] }
) => {
  const [loading, setLoading] = useState(false);
  const [article, setArticle] = useState<any>();

  useEffect(() => {
    setLoading(true);
    getArticle(host, {
      companyId,
      subdomain,
      email,
      limitSpaceIds,
      enablePrivate,
      passport,
      id: articleId,
    })
      .then((res) => {
        setArticle(res);
      })
      .finally(() => setLoading(false));
  }, [
    companyId,
    subdomain,
    email,
    limitSpaceIds,
    enablePrivate,
    passport,
    articleId,
    host,
  ]);

  return [article, loading];
};
const PopoverContent: React.FC<Props> = ({
  data,
  companyId,
  subdomain,
  email,
  limitSpaceIds,
  enablePrivate,
  passport,
}) => {
  const { kmApiBase } = useContext(RepositoryWidgetContext);
  const [article, loading] = useArticle(kmApiBase || "", {
    companyId,
    subdomain,
    email,
    limitSpaceIds,
    enablePrivate,
    passport,
    articleId: data.id,
  });
  if (loading) {
    return <Spin />;
  }
  if (article) {
    return (
      <>
        <h4>{article.title}</h4>
        <div
          dangerouslySetInnerHTML={{
            __html: DOMPurify.sanitize(article.contentBody),
          }}
        />
      </>
    );
  }
  return null;
};

const ArticleItem: React.FC<Props> = ({ data, ...props }) => {
  const { t } = useTranslation();
  const [loading, setLoading] = useState(false);
  const { kmApiBase, onUse } = useContext(RepositoryWidgetContext);

  const onUseClick = useCallback(() => {
    if (loading) {
      return;
    }
    setLoading(true);
    getArticle(kmApiBase || "", {
      companyId: props.companyId,
      subdomain: props.subdomain,
      email: props.email,
      limitSpaceIds: props.limitSpaceIds,
      enablePrivate: props.enablePrivate,
      passport: props.passport,
      id: data.id,
    })
      .then((res) => onUse?.("km", articleToCommonQuestion(res)))
      .finally(() => setLoading(false));
  }, [
    data.id,
    kmApiBase,
    loading,
    onUse,
    props.companyId,
    props.email,
    props.enablePrivate,
    props.limitSpaceIds,
    props.passport,
    props.subdomain,
  ]);
  return (
    <List.Item
      actions={[
        <Button type="link" onClick={onUseClick}>
          {t("label.use")}
        </Button>,
      ]}
    >
      <Popover
        content={<PopoverContent data={data} {...props} />}
        placement="left"
      >
        <div>
          <FileTextTwoTone />
          {data.title}
        </div>
      </Popover>
    </List.Item>
  );
};

export default ArticleItem;
