import React, { useCallback, useContext, useEffect, useState } from "react";
import axios from "axios";
import css from "./style.module.scss";
import InfiniteScroll from "react-infinite-scroller";
import List from "udesk-ui/es/list";
import { Spin } from "udesk-ui";
import ArticleItem from "./article-item";
import { RepositoryWidgetContext } from "../utilities";
import { useSafeSpaceId } from "./km-util";
import message from "udesk-ui/es/message";

interface Props {
  companyId: number;
  subdomain: string;
  email: string;
  limitSpaceIds: string;
  enablePrivate: boolean;
  passport: string;
  spaceId: number | string;
  catalogId: number | string;
  query: string;
}

const ArticleList: React.FC<Props> = ({
  companyId,
  subdomain,
  email,
  limitSpaceIds,
  enablePrivate,
  passport,
  spaceId,
  catalogId,
  query,
  ...props
}) => {
  const [data, setData] = useState([]);
  const [page, setPage] = useState(1);
  const [paging, setPaging] = useState({ pageNum: 1, pageSize: 20, total: 0 });
  const [loading, setLoading] = useState(false);
  const { kmApiBase } = useContext(RepositoryWidgetContext);
  const safeSpaceId = useSafeSpaceId(spaceId);

  useEffect(() => {
    if (!passport || !spaceId) {
      return;
    }

    const params: { [key: string]: any } = {
      companyId,
      subdomain,
      email,
      limitSpaceIds,
      enablePrivate,
      passport,
      pageNum: page,
      query,
    };

    if (query) {
      params.spaceIds = spaceId;
      axios.post(`${kmApiBase}/spaces`, params).then((res) => {
        if (res.data.code !== 200) {
          message.error(res.data.message);
          return;
        }
        setData(res.data.data);
        setPaging(res.data.paging);
      });
    } else {
      params.spaceId = safeSpaceId;

      if (catalogId !== -1) {
        params.catalogId = catalogId;
      }
      axios.get(`${kmApiBase}/articles/list`, { params }).then((res) => {
        if (res.data.code === 200) {
          setData(res.data.data);
          setPaging(res.data.paging);
        } else {
          message.error(res.data.message);
        }
      });
    }
  }, [
    catalogId,
    passport,
    spaceId,
    companyId,
    subdomain,
    email,
    limitSpaceIds,
    enablePrivate,
    page,
    query,
    kmApiBase,
    safeSpaceId,
  ]);

  const handleInfiniteOnLoad = useCallback(() => {
    if (loading) {
      return;
    }
    setLoading(true);
    setPage((prevState) => prevState + 1);
  }, [loading]);

  return (
    <div className={css["articleList"]}>
      <InfiniteScroll
        initialLoad={false}
        pageStart={0}
        loadMore={handleInfiniteOnLoad}
        hasMore={paging.pageNum * paging.pageSize < paging.total}
        useWindow={false}
      >
        <List
          dataSource={data}
          size="small"
          renderItem={(item) => (
            <ArticleItem
              data={item}
              companyId={companyId}
              email={email}
              enablePrivate={enablePrivate}
              limitSpaceIds={limitSpaceIds}
              passport={passport}
              subdomain={subdomain}
            />
          )}
        >
          {loading && (
            <div className={css.loadingContainer}>
              <Spin />
            </div>
          )}
        </List>
      </InfiniteScroll>
    </div>
  );
};

export default ArticleList;
