import React, {
  ReactText,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import axios from "axios";
import TreeSelect, { TreeSelectProps } from "udesk-ui/es/tree-select";
import { Category } from "../question/robot-question/types";
import { useTranslation } from "react-i18next";
import { useSafeSpaceId } from "./km-util";
import { RepositoryWidgetContext } from "../utilities";

type Props = TreeSelectProps<number | string> & {
  setDefaultValue?: (value: number) => void;
  companyId: number;
  subdomain: string;
  email: string;
  limitSpaceIds: string;
  enablePrivate: boolean;
  passport: string;
  spaceId: number | string;
};

export interface CategoriesResponse {
  code: number;
  message: string;
  visible: boolean;
  paging: Paging;
  data: Category[];
}

export interface Paging {
  pageNum: number;
  pageSize: number;
  total: number;
}

interface ChildrenMap {
  [key: number]: Category[];
}

function buildTreeNodes(nodes: Category[], childrenMap: ChildrenMap) {
  if (!nodes) {
    return [];
  }
  return nodes.map((i: Category) => {
    return (
      <TreeSelect.TreeNode value={i.id} key={i.id + ""} title={i.name}>
        {buildTreeNodes(childrenMap[i.id], childrenMap)}
      </TreeSelect.TreeNode>
    );
  });
}

const CatalogSelect: React.FC<Props> = ({
  setDefaultValue,
  companyId,
  spaceId,
  subdomain,
  email,
  limitSpaceIds,
  enablePrivate,
  passport,
  ...props
}) => {
  const { kmApiBase } = useContext(RepositoryWidgetContext);
  const [childrenMap, setChildrenMap] = useState({});
  const [rootNodes, setRootNodes] = useState<Category[]>([]);
  const [treeExpandedKeys, setTreeExpandedKeys] = useState<ReactText[]>([]);
  const { t } = useTranslation();
  const safeSpaceId = useSafeSpaceId(spaceId);

  useEffect(() => {
    axios
      .get<CategoriesResponse>(`${kmApiBase}/spaces/${safeSpaceId}/catalogs`, {
        baseURL: "",
        params: {
          companyId: companyId,
          subdomain: subdomain,
          email: email,
          limitSpaceIds: limitSpaceIds,
          enablePrivate: enablePrivate,
          passport: passport,
        },
      })
      .then((res) => {
        const _result: ChildrenMap = {};
        const _topNodes: Category[] = [];
        res.data?.data?.forEach((i) => {
          if (i.parentId === 0) {
            _topNodes.push(i);
          } else {
            if (!_result[i.parentId]) {
              _result[i.parentId] = [];
            }
            _result[i.parentId].push(i);
          }
        });
        const _topNode = _topNodes[0];
        if (!_topNode) {
          return;
        }
        setRootNodes([
          {
            id: -1,
            name: t("page.im.km.km.all_article"),
            parentId: _topNode.id,
          },
          ..._result[_topNode.id],
        ]);
        setDefaultValue?.(-1);
        setChildrenMap(_result);
      });
  }, [
    companyId,
    email,
    enablePrivate,
    limitSpaceIds,
    passport,
    setDefaultValue,
    safeSpaceId,
    subdomain,
    t,
    kmApiBase,
  ]);

  const treeNodes = useMemo(() => buildTreeNodes(rootNodes, childrenMap), [
    childrenMap,
    rootNodes,
  ]);
  useEffect(() => {
    setTreeExpandedKeys(Object.keys(childrenMap));
  }, [childrenMap]);

  if (!rootNodes.length) {
    return null;
  }
  return (
    <TreeSelect
      treeExpandedKeys={treeExpandedKeys}
      onTreeExpand={setTreeExpandedKeys}
      dropdownMatchSelectWidth={false}
      bordered={false}
      {...props}
    >
      {treeNodes}
    </TreeSelect>
  );
};

export default CatalogSelect;
