import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import { getJSON } from "../../../utils";
import Spin from "udesk-ui/es/spin";
import Space from "udesk-ui/es/space";
import { Select } from "udesk-ui";
import axios from "axios";
import CatalogSelect from "./catalog-select";
import css from "./style.module.scss";
import ArticleList from "./article-list";
import { KmSetting } from "../types";
import { RepositoryWidgetContext } from "../utilities";
import message from "udesk-ui/es/message";
import { useTranslation } from "react-i18next";
import SearchInput from "../search-input";

interface Space {
  name: string;
  companyId: number;
  description: string;
  status?: number;
  type: number;
  userGroupList: any[];
  userList: any[];
  id: number;
  createUserId: number;
  createTime: string;
  updateUserId: number;
  updateTime: string;
  deleteFlag: number;
}

type Props = KmSetting & {
  companyId: number;
  subdomain: string;
  email: string;
  query: string;
  onSearch: (query: string) => void;
};

const Option = Select.Option;

const Km: React.FC<Props> = ({
  private_space,
  sync,
  companyId,
  subdomain,
  email,
  query,
  onSearch,
}) => {
  const [token, setToken] = useState("");
  const [spacesResponse, setSpacesResponse] = useState<any>();
  const {
    activeCatalog,
    onCatalogChange,
    activeSpace,
    onSpaceChange,
    kmApiBase,
  } = useContext(RepositoryWidgetContext);
  const { t } = useTranslation();

  const publicParams = useMemo(
    () => ({
      companyId,
      subdomain,
      email,
      limitSpaceIds: private_space.join(","),
      enablePrivate: sync,
      passport: token,
    }),
    [companyId, email, private_space, subdomain, sync, token]
  );

  useEffect(() => {
    getJSON(
      `/knowledge_manager_settings/token?scope=token&_=${new Date().getTime()}`
    ).then((res) => {
      setToken(res.token);
    });
  }, []);

  const getSpaces = useCallback(() => {
    if (!publicParams.passport) {
      return Promise.reject();
    }

    return axios
      .get(`${kmApiBase}/spaces`, {
        params: {
          ...publicParams,
          pageSize: 100,
        },
      })
      .then((res) => {
        if (res.data.code === 200) {
          return res;
        }
        message.error(res.data.message);
        return null;
      })
      .catch((err) => {
        message.error(err?.response?.data?.message || "请求失败");
        return null;
      });
  }, [kmApiBase, publicParams]);

  useEffect(() => {
    getSpaces().then((res) => {
      if (res?.data?.code === 200) {
        setSpacesResponse(res?.data);
      }
    });
  }, [getSpaces]);

  const allSpaces = useMemo(
    () =>
      spacesResponse && spacesResponse.data.map((i: Space) => i.id).join(","),
    [spacesResponse]
  );
  useEffect(() => {
    if (query) {
      onSpaceChange?.(allSpaces);
    } else {
      if (spacesResponse?.data?.length) {
        onSpaceChange?.(spacesResponse.data[0].id);
      }
    }
  }, [allSpaces, onSpaceChange, query, spacesResponse?.data]);
  const safeSpace = useMemo(() => {
    if (
      !query &&
      activeSpace &&
      typeof activeSpace === "string" &&
      activeSpace.indexOf(",") > -1
    ) {
      return activeSpace.substring(0, activeSpace.indexOf(","));
    }
    return activeSpace;
  }, [activeSpace, query]);

  if (!token) {
    return <Spin />;
  }

  if (!spacesResponse?.data?.length) {
    return null;
  }

  return (
    <>
      <SearchInput onSearch={onSearch} defaultValue={query} />
      <Space className={css.spaceCatalogSelectWrapper}>
        <span>{t("label.space")}:</span>
        <Select
          value={safeSpace}
          onChange={onSpaceChange}
          bordered={false}
          dropdownMatchSelectWidth={false}
        >
          {query ? (
            <Option value={allSpaces}>{t("page.im.km.km.all_space")}</Option>
          ) : null}
          {spacesResponse.data.map((i: Space) => (
            <Option key={i.id} value={i.id + ""}>
              {i.name}
            </Option>
          ))}
        </Select>
        {safeSpace && !query ? (
          <CatalogSelect
            {...publicParams}
            spaceId={safeSpace}
            value={activeCatalog}
            onChange={onCatalogChange}
            setDefaultValue={onCatalogChange}
          />
        ) : null}
      </Space>
      {safeSpace && activeCatalog ? (
        <ArticleList
          {...publicParams}
          spaceId={safeSpace}
          catalogId={activeCatalog}
          query={query}
        />
      ) : null}
    </>
  );
};

export default Km;
