import {TreeSelect} from "udesk-ui/es";
import React, {ReactNode, useCallback, useEffect, useMemo, useState} from "react";
import {useTranslation} from "react-i18next";
import {useRequestData} from "../../utilities";
import QuestionList from "./question-list";
import {ICatalog, IQuestion, Scope} from "../../types";

const {TreeNode} = TreeSelect;

function buildTreeNodes(catalogs: ICatalog[]): ReactNode {
    return catalogs.map(i => {
        return <TreeNode value={i.id} key={i.id + ''} title={i.name}>
            {buildTreeNodes(i.children || [])}
        </TreeNode>
    });
}

interface Props {
    scope?: Scope
    query: string
}

interface Meta {
    current_page: number;
    total_pages: number;
}

const CommonQuestion: React.FC<Props> = ({scope, query}) => {
    const {t} = useTranslation();
    const [selectedCatalog, setSelectedCatalog] = useState<number>(-1);
    const [page, setPage] = useState<number>(1);
    const [paging, setPaging] = useState<Meta>({current_page: 0, total_pages: 0});
    const [questions, setQuestions] = useState<IQuestion[]>([]);

    useEffect(() => {
        setPage(1);
        setSelectedCatalog(-1);
    }, [scope]);

    const params = useMemo(() => ({
        knowledge_catalog_id: (selectedCatalog === -1) ? undefined : selectedCatalog,
        scope: scope,
        per_page: 20,
        page: page,
        current_works: true,
        query: query
    }), [page, query, scope, selectedCatalog]);

    const {loading, catalogs, data} = useRequestData("question", params);
    const treeNodes = useMemo(() => buildTreeNodes(catalogs), [catalogs]);

    useEffect(() => {
        setQuestions([]);
    }, [params.knowledge_catalog_id, params.scope, params.query]);

    useEffect(() => {
        if (data?.questions) {
            setQuestions(prevState => [...prevState, ...data.questions]);
            setPaging(data.meta);
        }
    }, [data]);

    const nextPage = useCallback(() => {
        setPage(prevState => {
            if (paging.total_pages >= prevState) {
                return prevState + 1;
            }
            return prevState;
        });
    }, [paging.total_pages]);

    return <>
        <TreeSelect dropdownStyle={{maxHeight: 400, overflow: 'auto'}}
                    style={{alignSelf: 'flex-start' as const}}
                    treeDefaultExpandAll
                    bordered={false}
                    value={selectedCatalog}
                    dropdownMatchSelectWidth={false}
                    onChange={setSelectedCatalog}>
            <TreeNode value={-1} key={'-1'} title={t('page.im.km.question.all')}/>
            <TreeNode value={0} key={'0'} title={t('page.im.km.question.no_catalog')}/>
            {treeNodes}
        </TreeSelect>
        {
            questions.length ?
                <QuestionList data={questions} loading={loading} onScrollToBottom={nextPage}/> :
                <p>{t('page.im.km.no_result', {name: '$t(label.question)'})}</p>
        }
    </>
}

export default CommonQuestion;
