import Viewer from "react-viewer";
import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useRef,
  useState,
} from "react";
import css from "../../style.module.scss";
import Button from "udesk-ui/es/button";
import { useTranslation } from "react-i18next";
import { ImageDecorator } from "react-viewer/lib/ViewerProps";
import classNames from "classnames";
import createDOMPurify from "dompurify";
import useEvent from "react-use/esm/useEvent";
import { Spin } from "udesk-ui";
import Highlight from "../../hightlight";
import { RepositoryWidgetContext, makeQuestionData } from "../../utilities";
import { IQuestion } from "../../types";
import { RootState } from "../../../../store/root-reducer";
import { useSelector } from "../../../../store/store-hook";
const DOMPurify = createDOMPurify(window);
const QuestionItem: React.FC<{
  data: IQuestion;
  getImages: (images: ImageDecorator[]) => void;
}> = ({ data, getImages }) => {
  const initData = useSelector((state: RootState) => state.initData);
  const { chat } = initData;
  const { t } = useTranslation();
  const { allowSendQuestion, onUse, onSend } = useContext(
    RepositoryWidgetContext
  );
  const [showExpandButton, setShowExpandButton] = useState<boolean>(false);
  const [isExpand, setIsExpand] = useState(true);
  const contentStyle = useMemo(() => {
    if (isExpand) {
      return {};
    } else {
      return { height: "22px", overflowY: "hidden" as const };
    }
  }, [isExpand]);

  const contentRef = useRef<HTMLParagraphElement>(null);
  useEffect(() => {
    if (contentRef.current) {
      const imageElements = contentRef.current.querySelectorAll("img");
      const images: ImageDecorator[] = [];
      imageElements.forEach((i) => images.push({ src: i.src }));
      getImages(images);
      if (contentRef.current.clientHeight > 40 || imageElements.length > 0) {
        setShowExpandButton(true);
        setIsExpand(false);
      }
    }
  }, [getImages]);
  const toggleExpand = useCallback(() => {
    setIsExpand(!isExpand);
  }, [isExpand]);
  const _onUse = useCallback(() => {
    onUse?.("question", data);
  }, [data, onUse]);
  const onSendClick = useCallback(() => {
    let url =
      window.location.protocol + "//" + window.location.host + "/question/";
    let _data = makeQuestionData(
      {
        content: data.content,
        content_type: data.content_type,
        title: data.title,
        id: parseInt(data.id),
      },
      chat.platform,
      "",
      url
    );
    data.url = _data.url;
    onSend?.("question", data);
  }, [chat, data, onSend]);

  return (
    <div className={css["question-item"]}>
      <div className={classNames(css["align-justify"])}>
        <Highlight
          className={css["word-break"]}
          style={{ color: "#177BBB" }}
          content={data.title}
        />

        <div style={{ flexShrink: 0 }}>
          {showExpandButton && (
            <Button type="link" size="small" onClick={toggleExpand}>
              {t(isExpand ? "label.collapse" : "label.expand")}
            </Button>
          )}
          {allowSendQuestion ? (
            <Button type="link" size="small" onClick={onSendClick}>
              {t("label.send")}
            </Button>
          ) : null}
          <Button type="link" size="small" onClick={_onUse}>
            {t("label.use")}
          </Button>
        </div>
      </div>
      <div
        ref={contentRef}
        style={contentStyle}
        className={classNames(css.highlight, css["word-break"])}
        dangerouslySetInnerHTML={{
          __html: DOMPurify.sanitize(
            data.content_type === "text" ? data.content_text : data.content
          ),
        }}
      />
    </div>
  );
};

interface QuestionListProps {
  data: IQuestion[];
  onScrollToBottom: () => void;
  loading: boolean;
}

const QuestionList: React.FC<QuestionListProps> = ({
  data,
  onScrollToBottom,
  loading,
}) => {
  const [imageViewerVisible, setImageViewerVisible] = useState(false);
  const [images, setImages] = useState<ImageDecorator[]>([]);
  const [activeIndex, setActiveIndex] = useState(0);
  const dataWrapperRef = useRef<HTMLDivElement>(null);
  const onClick = useCallback(
    (e) => {
      if (e.target.tagName === "IMG") {
        setActiveIndex(images.findIndex((i) => i.src === e.target.src));
        setImageViewerVisible(true);
      }
    },
    [images]
  );
  useEvent(
    "click",
    onClick,
    document.getElementById("react-question-common-list")
  );

  useEffect(() => {
    if (dataWrapperRef.current) {
      const scrollHandle: EventListener = (e) => {
        const element = e.currentTarget as HTMLDivElement;
        if (!element) {
          return;
        }
        if (element.scrollHeight - element.scrollTop === element.clientHeight) {
          onScrollToBottom();
        }
      };
      dataWrapperRef.current.addEventListener("scroll", scrollHandle);
      return () =>
        dataWrapperRef?.current?.removeEventListener("scroll", scrollHandle);
    }
  }, [onScrollToBottom]);

  const getImages = useCallback(
    (i) => setImages((prevState) => [...prevState, ...i]),
    []
  );
  return (
    <>
      <div
        ref={dataWrapperRef}
        className={css["question-list"]}
        id="react-question-common-list"
      >
        {data.map((i: IQuestion) => (
          <QuestionItem data={i} key={i.id} getImages={getImages} />
        ))}
        {loading ? <Spin /> : null}
      </div>
      <Viewer
        activeIndex={activeIndex < 0 ? 0 : activeIndex}
        visible={imageViewerVisible}
        onClose={() => setImageViewerVisible(false)}
        images={images}
      />
    </>
  );
};

export default QuestionList;
