import React, {
  ReactText,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import axios from "axios";
import TreeSelect, { TreeSelectProps } from "udesk-ui/es/tree-select";
import { Category, RobotSettings } from "./types";
import { RepositoryWidgetContext } from "../../../repository-widget/utilities";

type Props = TreeSelectProps<number> & {
  settings: RobotSettings;
  robotId: number;
  setDefaultValue: (value: number) => void;
};

export interface CategoriesResponse {
  code: number;
  message: string;
  visible: boolean;
  paging: Paging;
  data: Category[];
}

export interface Paging {
  pageNum: number;
  pageSize: number;
  total: number;
}

interface ChildrenMap {
  [key: number]: Category[];
}

function buildTreeNodes(nodes: Category[], childrenMap: ChildrenMap) {
  if (!nodes) {
    return [];
  }
  return nodes.map((i: Category) => {
    return (
      <TreeSelect.TreeNode value={i.id} key={i.id + ""} title={i.name}>
        {buildTreeNodes(childrenMap[i.id], childrenMap)}
      </TreeSelect.TreeNode>
    );
  });
}

const CategorySelect: React.FC<Props> = ({
  settings,
  robotId,
  setDefaultValue,
  ...props
}) => {
  const { kmApiBase } = useContext(RepositoryWidgetContext);
  const [childrenMap, setChildrenMap] = useState({});
  const [rootNodes, setRootNodes] = useState<Category[]>([]);
  const [treeExpandedKeys, setTreeExpandedKeys] = useState<ReactText[]>([]);

  useEffect(() => {
    axios
      .get<CategoriesResponse>(`${kmApiBase}/v1/categories`, {
        baseURL: "",
        params: { robotId, ...settings.auth },
      })
      .then((res) => {
        const _result: ChildrenMap = {};
        const _rootNodes: Category[] = [];
        res.data.data.forEach((i) => {
          if (i.parentId === 0) {
            _rootNodes.push(i);
          } else {
            if (!_result[i.parentId]) {
              _result[i.parentId] = [];
            }
            _result[i.parentId].push(i);
          }
        });
        setDefaultValue(_rootNodes[0]?.id);
        setRootNodes(_rootNodes);
        setChildrenMap(_result);
      });
  }, [robotId, setDefaultValue, settings.auth]);

  const treeNodes = useMemo(() => buildTreeNodes(rootNodes, childrenMap), [
    childrenMap,
    rootNodes,
  ]);
  useEffect(() => {
    setTreeExpandedKeys(Object.keys(childrenMap));
  }, [childrenMap]);

  if (!rootNodes.length) {
    return null;
  }
  return (
    <TreeSelect
      treeExpandedKeys={treeExpandedKeys}
      onTreeExpand={setTreeExpandedKeys}
      dropdownMatchSelectWidth={false}
      treeDefaultExpandAll={true}
      bordered={false}
      {...props}
    >
      {treeNodes}
    </TreeSelect>
  );
};

export default CategorySelect;
