import React, { useCallback, useContext, useEffect, useState } from "react";
import { RobotQuestion, RobotSettings } from "../types";
import RobotQuestionItem from "./robot-question-item";
import axios from "axios";
import css from "../style.module.scss";
import List from "udesk-ui/es/list";
import InfiniteScroll from "react-infinite-scroller";
import { Spin } from "udesk-ui";
import { RepositoryWidgetContext } from "../../../utilities";

interface Props {
  categoryId: number;
  robotId: number;
  auth: RobotSettings["auth"];
  searchWord: string;
}

const RobotQuestionList: React.FC<Props> = ({
  categoryId,
  robotId,
  auth,
  searchWord,
}) => {
  const [data, setData] = useState<RobotQuestion[]>([]);
  const [page, setPage] = useState(1);
  const [paging, setPaging] = useState({ pageNum: 1, pageSize: 20, total: 0 });
  const [loading, setLoading] = useState(false);
  const { kmApiBase } = useContext(RepositoryWidgetContext);

  useEffect(() => {
    setData([]);
    setPage(1);
  }, [categoryId, robotId, searchWord]);

  useEffect(() => {
    setLoading(true);

    axios
      .post(
        `${kmApiBase}/v1/questions/search?sign=${auth.sign}&email=${auth.email}&timestamp=${auth.timestamp}&appID=${auth.appID}`,
        {
          categoryId,
          robotId,
          pageNum: page,
          pageSize: 20,
          searchWord: searchWord,
        }
      )
      .then((res) => {
        setPaging(res.data.paging);
        setData((prevState) => [...prevState, ...res.data.data]);
      })
      .finally(() => setLoading(false));
  }, [
    page,
    categoryId,
    robotId,
    auth.sign,
    auth.email,
    auth.timestamp,
    auth.appID,
    searchWord,
  ]);

  const handleInfiniteOnLoad = useCallback(() => {
    if (loading) {
      return;
    }
    setPage((prevState) => prevState + 1);
  }, [loading]);

  return (
    <div className={css["questionList"]}>
      <InfiniteScroll
        initialLoad={false}
        pageStart={0}
        loadMore={handleInfiniteOnLoad}
        hasMore={paging.pageNum * paging.pageSize < paging.total}
        useWindow={false}
      >
        <List
          dataSource={data}
          renderItem={(item) => <RobotQuestionItem data={item} />}
        >
          {loading && (
            <div className={css.loadingContainer}>
              <Spin />
            </div>
          )}
        </List>
      </InfiniteScroll>
    </div>
  );
};

export default RobotQuestionList;
