import React, { useCallback, useContext, useMemo } from "react";
import { Popover, Tree } from "udesk-ui";
import { TreeProps } from "udesk-ui/es/tree";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import { Spin } from "udesk-ui/es";
import ScopeSwitch from "../../../components/repository-widget/scope-switch";
import { useTranslation } from "react-i18next";
import {
  hit,
  RepositoryWidgetContext,
  useCatalogMap,
  useExpandedKeys,
  useRequestData,
  useSafeScope,
} from "../../../components/repository-widget/utilities";
import DOMPurify from "dompurify";
import css from "../style.module.scss";
import classNames from "classnames";
import { ICatalog, IExpression } from "../types";
import SearchInput from "../../../components/repository-widget/search-input";
import { customFieldValueToText } from "../../../utils/custom-field";
import moment from "moment";
import { YYYY_MM_DD_HH_MM_SS } from "../../../utils";
import { TFunction } from "i18next";

interface Props {
  onSearch: (query: string) => void;
  query: string;
}

function replacePlaceholder(
  content: string,
  {
    customer,
    agent,
    ticket,
    customerCustomFields,
    ticketCustomFields,
    t,
  }: {
    customer?: Customer;
    agent?: IAgent;
    ticket?: ITicket;
    customerCustomFields?: CustomField[];
    ticketCustomFields?: CustomField[];
    t: TFunction;
  }
) {
  content = content.replace("{{agent.name}}", agent?.nick_name ?? ""); // 当前客服姓名
  content = content.replace("{{agent.nick_name}}", agent?.aliase ?? ""); // 当前客服昵称
  content = content.replace("{{ticket.field_num}}", ticket?.field_num ?? ""); // 工单编号
  content = content.replace("{{ticket.id}}", `${ticket?.id ?? ""}`); // 工单id
  content = content.replace("{{ticket.user}}", ticket?.customer_name ?? ""); // 客户
  content = content.replace(
    "{{ticket.created_at}}",
    ticket?.created_at
      ? moment(ticket.created_at).format(YYYY_MM_DD_HH_MM_SS)
      : ""
  ); // 创建时间
  content = content.replace("{{ticket.subject}}", ticket?.subject ?? ""); // 主题
  content = content.replace("{{ticket.content}}", ticket?.content ?? ""); // 描述
  content = content.replace("{{ticket.priority}}", ticket?.priority ?? ""); // 优先级
  content = content.replace("{{ticket.status}}", ticket?.status ?? ""); // 状态
  content = content.replace(
    "{{ticket.platform}}",
    ticket?.platform_name ? t(ticket.platform_name) : ""
  ); // 渠道
  content = content.replace(
    "{{ticket.tags}}",
    ticket?.all_tags
      ? ticket.all_tags
      : ticket?.tags
      ? ticket.tags.map((i) => i.name).join(",")
      : ""
  ); // 标签
  content = content.replace("{{ticket.assignee}}", ticket?.assignee_name ?? ""); // 受理客服
  content = content.replace(
    "{{ticket.user_group}}",
    ticket?.agent_group_name ?? ""
  ); // 受理客服组
  content = content.replace(
    "{{ticket.followers}}",
    ticket?.followers ? ticket.followers.map((i) => i.nick_name).join(",") : ""
  ); // 工单关注人
  content = content.replace("{{ticket.web_url}}", ticket?.web_url ?? ""); // 工单链接
  content = content.replace(
    "{{ticket.solved_deadline}}",
    ticket?.solved_deadline ?? ""
  ); // 到期时间
  content = content.replace("{{ticket.replier}}", ticket?.replied_by ?? ""); // 最后回复人
  content = content.replace(
    "{{ticket.replied_at}}",
    ticket?.replied_at
      ? moment(ticket?.replied_at).format(YYYY_MM_DD_HH_MM_SS)
      : ""
  ); // 最后回复时间
  content = content.replace("{{ticket.latest_comment_formatted}}", ""); // TODO 最新回复
  content = content.replace(
    "{{ticket.customer_replied_at}}",
    ticket?.customer_replied_at ?? ""
  ); // 客户最后回复时间
  content = content.replace("{{ticket.latest_public_comment_at}}", ""); // TODO 最新公开回复时间
  content = content.replace("{{ticket.latest_internal_comment}}", ""); // TODO 最新内部备注
  content = content.replace("{{ticket.latest_public_comment_formatted}}", ""); //TODO 最新公开回复
  content = content.replace(
    "{{ticket.agent_replied_at}}",
    ticket?.agent_replied_at ?? ""
  ); // 客服最后回复时间
  content = content.replace(
    "{{customer.nick_name}}",
    customer?.nick_name ?? ""
  ); // 姓名
  content = content.replace("{{customer.email}}", customer?.email ?? ""); // 邮箱
  content = content.replace(
    "{{customer.cellphone}}",
    customer?.cellphones
      ? customer.cellphones.map((i) => i.content).join(",")
      : ""
  ); // 手机号
  content = content.replace(
    "{{customer.description}}",
    customer?.description ?? ""
  ); // 描述
  content = content.replace("{{customer.owner}}", customer?.owner_name ?? ""); // 负责人
  content = content.replace(
    "{{customer.owner_group}}",
    customer?.owner_group_name ?? ""
  ); // 负责组
  content = content.replace(
    "{{customer.organization}}",
    customer?.organizations
      ? customer.organizations.map((i) => i.name).join(",")
      : ""
  ); // 公司
  ticketCustomFields?.forEach((i) => {
    content = content.replace(
      `{{ticket.${i.name}}}`,
      customFieldValueToText(i, ticket?.custom_fields?.[i.name]) + ""
    );
  });
  customerCustomFields?.forEach((i) => {
    content = content.replace(
      `{{customer.${i.name}}}`,
      customFieldValueToText(i, customer?.custom_fields?.[i.name]) + ""
    );
  });
  return content;
}

function buildTreeData(
  catalogs: ICatalog[],
  catalogMap: { [key: string]: IExpression[] },
  parentKey?: string
): TreeProps["treeData"] {
  return catalogs.map((i) => {
    const key = parentKey ? `${parentKey}-${i.id}` : i.id + "";
    return {
      key: key,
      title: i.name,
      selectable: false,
      children: (() => {
        if (i.children) {
          return buildTreeData(i.children, catalogMap, key);
        } else {
          return (catalogMap[i.id] || []).map((j) => {
            return {
              key: `${key}-content${j.id}`,
              className: classNames(css.noWrap, css.flexNode),
              originData: j,
              title: (
                <Popover
                  overlayClassName={css.expressionPopover}
                  placement="left"
                  content={<pre>{j.content}</pre>}
                >
                  <div>{j.content}</div>
                </Popover>
              ),
              icon: <FileTextTwoTone />,
            };
          });
        }
      })(),
    };
  });
}

const UsefulExpression: React.FC<Props> = ({ onSearch, query }) => {
  const {
    onUse,
    onExpand,
    agent,
    customer,
    ticket,
    ticketCustomFields,
    customerCustomFields,
    activeScope,
    onScopeChange,
  } = useContext(RepositoryWidgetContext);

  const _scope = useSafeScope(activeScope);
  const { t } = useTranslation();
  const params = useMemo(() => ({ scope: _scope, per_page: 1000, query }), [
    _scope,
    query,
  ]);
  let { loading, catalogs, data } = useRequestData("useful_expression", params);

  const catalogMap = useCatalogMap(data?.expressions ?? []);
  const _expandedKeys = useExpandedKeys(catalogMap);

  const treeData = useMemo(() => {
    if (!query) {
      return buildTreeData(catalogs, catalogMap);
    } else {
      if (!data || !data.expressions) {
        return [];
      }
      return data.expressions.map((i: IExpression) => {
        const safeContent = DOMPurify.sanitize(i.content);
        return {
          key: i.id,
          originData: i,
          className: classNames(css.noWrap, css.highlight, css.flexNode),
          title: (
            <Popover
              overlayClassName={css.expressionPopover}
              placement="left"
              content={
                <pre dangerouslySetInnerHTML={{ __html: safeContent }} />
              }
            >
              <div dangerouslySetInnerHTML={{ __html: safeContent }} />
            </Popover>
          ),
          icon: <FileTextTwoTone />,
        };
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [data, catalogMap, catalogs]);

  const onClick = useCallback(
    (e, treeNode) => {
      const originData = treeNode?.props?.originData;
      if (originData) {
        onUse?.("useful_expression", {
          ...originData,
          content: replacePlaceholder(originData.content, {
            customer,
            agent,
            ticket,
            customerCustomFields,
            ticketCustomFields,
            t,
          }),
        });
        hit(originData.id, "UsefulExpression");
      }
    },
    [
      agent,
      customer,
      customerCustomFields,
      onUse,
      t,
      ticket,
      ticketCustomFields,
    ]
  );

  const mainContent = useMemo(() => {
    if (loading) {
      return <Spin />;
    } else if (treeData && treeData.length) {
      return (
        <Tree
          blockNode
          showIcon
          expandedKeys={_expandedKeys}
          onExpand={onExpand}
          selectable={false}
          treeData={treeData}
          onClick={onClick}
        />
      );
    } else {
      return (
        <p>
          {t("page.im.km.no_result", { name: "$t(label.useful_expression)" })}
        </p>
      );
    }
  }, [_expandedKeys, loading, onClick, onExpand, t, treeData]);

  return (
    <>
      <ScopeSwitch value={_scope} onChange={onScopeChange} />
      <SearchInput onSearch={onSearch} defaultValue={query} />
      {mainContent}
    </>
  );
};

export default UsefulExpression;
