import React, { useCallback, useContext, useMemo } from "react";
import { Button, Tree } from "udesk-ui";
import { TreeProps } from "udesk-ui/es/tree";
import Typography from "udesk-ui/es/typography";
import LinkOutlined from "@ant-design/icons/LinkOutlined";
import { Spin } from "udesk-ui/es";
import ScopeSwitch from "../scope-switch";
import { useTranslation } from "react-i18next";
import {
  hit,
  RepositoryWidgetContext,
  useCatalogMap,
  useExpandedKeys,
  useRequestData,
  useSafeScope,
} from "../utilities";
import { TFunction } from "i18next";
import css from "../style.module.scss";
import { MiniProgram } from "../../icon/icon";
import classNames from "classnames";
import DOMPurify from "dompurify";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import { ILink } from "../types";
import SearchInput from "../../repository-widget/search-input";

interface Props {
  query: string;
  onSearch: (query: string) => void;
}

interface Catalog {
  id: number;
  name: string;
  parent_id: number;
  sort: number;
  subitem_count: number;
  children: Catalog[];
}

const { Text } = Typography;

function buildTreeData(
  catalogs: Catalog[],
  catalogMap: { [key: string]: ILink[] },
  t: TFunction,
  parentKey?: string
): TreeProps["treeData"] {
  return catalogs.map((i) => {
    const key = parentKey ? `${parentKey}-${i.id}` : i.id + "";
    return {
      key: key,
      title: i.name,
      selectable: false,
      className: css["treeNode"],
      children: (() => {
        if (i.children) {
          return buildTreeData(i.children, catalogMap, t, key);
        } else {
          return (catalogMap[i.id] || []).map((j) => {
            return {
              key: `${key}-content${j.id}`,
              className: css["treeNode"],
              originData: j,
              title: (
                <>
                  <Text>{j.name}</Text>
                  {j.mode === "mini" ? (
                    <div className={css.nodeOperations}>
                      <Button type="link" size="small">
                        {t("label.send")}
                      </Button>
                    </div>
                  ) : null}
                </>
              ),
              icon: j.mode === "mini" ? <MiniProgram /> : <LinkOutlined />,
            };
          });
        }
      })(),
    };
  });
}

const UsefulLink: React.FC<Props> = ({ query, onSearch }) => {
  const {
    onUse,
    onExpand,
    expandedKeys,
    activeScope,
    onScopeChange,
  } = useContext(RepositoryWidgetContext);
  const _scope = useSafeScope(activeScope);
  const { t } = useTranslation();
  const params = useMemo(() => ({ scope: _scope, per_page: 1000, query }), [
    _scope,
    query,
  ]);
  const { loading, catalogs, data } = useRequestData("useful_link", params);
  const catalogMap = useCatalogMap(data?.links ?? []);
  const _expandedKeys = useExpandedKeys(catalogMap);

  const treeData = useMemo(() => {
    if (!query) {
      return buildTreeData(catalogs, catalogMap, t);
    } else {
      if (!data || !data.links) {
        return [];
      }
      return data.links.map((i: ILink) => ({
        key: i.id,
        originData: i,
        className: classNames(css.noWrap, css.highlight),
        title: (
          <span
            dangerouslySetInnerHTML={{ __html: DOMPurify.sanitize(i.name) }}
          />
        ),
        icon: <FileTextTwoTone />,
      }));
    }
  }, [catalogMap, catalogs, data, query, t]);
  const onClick = useCallback(
    (e, treeNode) => {
      const originData = treeNode?.props?.originData;
      if (originData) {
        onUse?.("useful_link", originData);
        hit(originData.id, "UsefulLink");
      }
    },
    [onUse]
  );

  const mainContent = useMemo(() => {
    if (loading) {
      return <Spin />;
    } else if (treeData && treeData.length) {
      return (
        <Tree
          blockNode
          showIcon
          expandedKeys={_expandedKeys}
          onExpand={onExpand}
          selectable={false}
          treeData={treeData}
          onClick={onClick}
        />
      );
    } else {
      return (
        <p>{t("page.im.km.no_result", { name: "$t(label.useful_link)" })}</p>
      );
    }
  }, [expandedKeys, loading, onClick, onExpand, t, treeData]);
  return (
    <>
      <ScopeSwitch value={_scope} onChange={onScopeChange} />
      <SearchInput onSearch={onSearch} defaultValue={query} />
      {mainContent}
    </>
  );
};

export default UsefulLink;
