import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import { getJSON, post, postJSON, request } from "../../utils";
import { csApiGetJSON, csApiRequest, csApiPostJSON} from "../../utils/csApi";
import qs from "qs";
import { RobotQuestion } from "./question/robot-question/types";
import weixinDefaultPng from "./weixin_default.png";
import {
  IArticle,
  IAttachment,
  IExpression,
  ILink,
  IQuestion,
  RepositoryType,
  Scope,
} from "./types";
import useDebounce from "react-use/esm/useDebounce";
import { useSelector } from "../../store/store-hook";
import { RootState } from "../../store/root-reducer";

export const useRequestData = (
  type: RepositoryType,
  params: {
    query?: string;
    scope?: Scope;
    per_page?: number;
    page?: number;
    current_works?: boolean;
    catalog_id?: number | string;
    knowledge_catalog_id?: number;
  }
) => {
  const refreshKnowledge = useSelector(
    (state: RootState) => state.initData.modals.refreshKnowledge
  );
  const { scope } = params;
  const [catalogs, setCatalogs] = useState([]);
  const [loading, setLoading] = useState(true);
  const [data, setData] = useState<any>();

  const dataUrl = (() => {
    switch (type) {
      case "useful_expression":
      case "useful_link":
        return `/cs/ts/api/${type}s/search`;
      case "question":
        return `/cs/ts/api/knowledge_questions/search`;
      case "attachment":
        return "/knowledge_attachments";
    }
  })();
  const method = (() => {
    switch (type) {
      case "useful_expression":
      case "useful_link":
      case "question":
        return "post";
      case "attachment":
        return "get";
    }
  })();
  const csRequest = (() => {
    switch (type) {
        case 'useful_expression':
        case 'useful_link':
        case 'question':
            return true;
        case 'attachment':
            return false;
    }
})();

  const requestOptions: Parameters<typeof request>[0] = useMemo(() => {
    let res: typeof requestOptions = {
      url: dataUrl,
      method: method
    };
    if (method === "post") {
      res.headers = { "content-type": "application/x-www-form-urlencoded" };
      res.data = qs.stringify(params);
    } else {
      res.data = params;
    }

    return res;
  }, [dataUrl, method, params]);

  useDebounce(
    () => {
      setLoading(true);

      Promise.all([
        csRequest? csApiRequest(requestOptions) : request(requestOptions),
        csApiGetJSON("/cs/ts/api/knowledge_catalogs", {
          params: {
            knowledge_type_name: type,
            scope,
            query: params.query,
          }
        }).then(res=>res.data).then((res) => res.catalogs),
      ])
        .then(([data, catalogs]) => {
          setData(data);
          setCatalogs(catalogs);
        })
        .finally(() => setLoading(false));
    },
    50,
    [dataUrl, params, scope, type, method, requestOptions, refreshKnowledge]
  );
  return { loading, catalogs, data };
};

/**
 * 返回一个 object，key 为 catalog的 id，值为该 catalog下的内容
 */
type Catalog =
  | { knowledge_catalog_id: number }
  | { knowledge_attachment: { knowledge_catalog_id: number } };
export const useCatalogMap = (data: Catalog[]) => {
  return useMemo(() => {
    const _catalogMap: { [key: number]: any[] } = {};
    if (data) {
      data.forEach((i: Catalog) => {
        let knowledge_catalog_id;
        if ("knowledge_attachment" in i) {
          knowledge_catalog_id = i.knowledge_attachment.knowledge_catalog_id;
        } else {
          knowledge_catalog_id = i.knowledge_catalog_id;
        }
        if (!_catalogMap[knowledge_catalog_id]) {
          _catalogMap[knowledge_catalog_id] = [];
        }
        _catalogMap[knowledge_catalog_id].push(i);
      });
    }
    return _catalogMap;
  }, [data]);
};

export function hit(
  id: string | number,
  type:
    | "KnowledgeQuestion"
    | "UsefulExpression"
    | "UsefulLink"
    | "KnowledgeAttachment"
) {
  return csApiPostJSON("/cs/ts/api/knowledges/hit",  {id: id + "", type: type });
}

export const RepositoryWidgetContext = React.createContext<{
  allowSendQuestion?: boolean;
  onUse?: (
    type: RepositoryType | "km",
    data:
      | IExpression
      | ILink
      | IQuestion
      | RobotQuestion
      | IAttachment["knowledge_attachment"]
      | IArticle
  ) => void;
  showRobotQuestionTab?: boolean;
  onSend?: (
    type: "attachment" | "question" | "robotQuestion",
    data: IAttachment["knowledge_attachment"] | IQuestion | RobotQuestion
  ) => void;
  kmApiBase?: string;
  onExpand?: (keys: (number | string)[]) => void;
  expandedKeys?: (string | number)[];
  activeScope?: Scope;
  onScopeChange?: (scope: Scope) => void;
  onCatalogChange?: (catalog?: number | string) => void;
  onSpaceChange?: (space?: number | string) => void;
  activeCatalog?: number;
  activeSpace?: number | string;
  showPublicScope?: boolean;
  previewUrl: string;
  agent?: IAgent;
  customer?: Customer;
  ticket?: ITicket;
  ticketCustomFields?: CustomField[];
  customerCustomFields?: CustomField[];
}>({
  previewUrl: "",
});

export const replaceHtmlTag = function (text: string) {
  return text.replace(/<(.|\n)*?>/g, "");
};

export const makeQuestionData = function (
  item: {
    content: string;
    content_type: string;
    title: string;
    url?: string;
    id: number;
    isRobotQuestion?: boolean;
  },
  platform?: string,
  content?: string,
  url?: string
) {
  let data;
  let qContent = item.content;
  let content_type = item.content_type;

  if (content_type === "html") {
    let pic_url = weixinDefaultPng;
    let pic_title;

    if (qContent.indexOf("<img") > -1) {
      var reg = /<img[^>]+src=['"]([^'"]+)['"]+/gi;
      var reg2 = /<img[^>]+alt=['"]([^'"]+)['"]+/gi;
      pic_url = (reg.exec(qContent) || [])[1];
      pic_title = reg2.exec(qContent);
      if (pic_title && pic_title[1]) {
        pic_title = pic_title[1];
      }
    }
    data = {
      content: qContent,
      title: item.title,
      desc: replaceHtmlTag(qContent),
      pic_title: pic_title || "",
      pic_url: pic_url,
      url: item.isRobotQuestion ? item.url : url ? url + item.id : "",
      platform: platform,
      type: "html",
      isRobotQuestion: item.isRobotQuestion || false,
    };
  } else {
    let reg = /<[^>]+>/g;
    if (reg.test(qContent)) {
      qContent = replaceHtmlTag(qContent);
    }
    data = {
      content: qContent,
      title: item.title,
      type: "text",
    };
  }
  return data;
};

/**
 * 机器人问题转换为普通问题格式
 * @param {RobotQuestion} robotQuestion
 * @return Question
 */
export const robotToCommonQuestion = function (
  robotQuestion: RobotQuestion
): IQuestion {
  return {
    id: robotQuestion.id + "",
    // title: robotQuestion.content,
    // content: robotQuestion?.answer?.content ?? robotQuestion.content,
    content_text: robotQuestion?.answer?.content ?? robotQuestion.content,
    content_type: "html",
    updated_at: robotQuestion.updateTime,
    ...makeQuestionData({
      content_type: "html",
      ...robotQuestion,
      title: robotQuestion.content,
      content: robotQuestion.answer
        ? robotQuestion.answer.content
        : robotQuestion.content,
      url: robotQuestion.wechatUrl,
      isRobotQuestion: true,
    }),
  };
};

export const articleToCommonQuestion = function (article: IArticle): IQuestion {
  return {
    id: article.id + "",
    // title: article.title,
    // content: article.contentBody,
    content_text: article.contentBody,
    content_type: "html",
    updated_at: article.updateTime,
    ...makeQuestionData({
      content_type: "html",
      ...article,
      content: article.contentBody,
    }),
  };
};

interface TabSetting {
  activeTab?: RepositoryType | "km";
  activeScope?: Scope;
  activeSpace?: string;
  activeCatalog?: number;
}

export const useTabSetting = () => {
  const [tabSetting, setTabSetting] = useState<TabSetting>();
  const [loading, setLoading] = useState(true);
  const url = "/user_profiles/knowledge_tabs";
  const csUrl = '/cs/ts/api/user_profiles/knowledge_tabs';

  const saveTab = useCallback((key) => {
    if (key === "km") {
      setTabSetting((prevState) => {
        post(url, {
          "knowledge_tabs[im]": `${key}_${prevState?.activeSpace ?? ""}_${
            prevState?.activeCatalog ?? ""
          }`,
        }).then(() => null);
        return {
          activeSpace: prevState?.activeSpace,
          activeCatalog: prevState?.activeCatalog,
          activeTab: "km",
        };
      });
    } else {
      setTabSetting((prevState) => {
        post(url, {
          "knowledge_tabs[im]": `${key}_${prevState?.activeScope ?? ""}`,
        }).then(() => null);
        return {
          ...prevState,
          activeScope: prevState?.activeScope ?? "public",
          activeTab: key,
        };
      });
    }
  }, []);

  const saveScope = useCallback((scope) => {
    setTabSetting((prevState) => {
      post(url, {
        "knowledge_tabs[im]": `${
          prevState?.activeTab ?? "useful_expression"
        }_${scope}`,
      }).then(() => null);
      return {
        ...prevState,
        activeTab: prevState?.activeTab ?? "useful_expression",
        activeScope: scope,
      };
    });
  }, []);
  const saveSpace = useCallback((space) => {
    setTabSetting((prevState) => {
      if (typeof space === "string" && space.indexOf(",") === -1) {
        post(url, {
          "knowledge_tabs[im]": `km_${space}_${prevState?.activeCatalog ?? ""}`,
        }).then(() => null);
      }
      return {
        ...prevState,
        activeTab: "km",
        activeCatalog: prevState?.activeCatalog ?? -1,
        activeSpace: space + "",
      };
    });
  }, []);
  const saveCatalog = useCallback((catalog) => {
    setTabSetting((prevState) => {
      post(url, {
        "knowledge_tabs[im]": `km_${prevState?.activeSpace ?? ""}_${catalog}`,
      }).then(() => null);
      return {
        ...prevState,
        activeTab: "km",
        activeSpace: prevState?.activeSpace ?? "",
        activeCatalog: catalog,
      };
    });
  }, []);

  useEffect(() => {
    csApiGetJSON(csUrl)
      .then(res=>res.data)
      .then((res) => {
        if (res?.knowledge_tabs?.im) {
          if (res?.knowledge_tabs.im.startsWith("km")) {
            setTabSetting(
              (prevState) =>
                ({
                  activeTab: "km",
                  activeScope: prevState?.activeScope,
                  activeSpace: res.knowledge_tabs.im.split("_")[1],
                  activeCatalog:
                    parseInt(res.knowledge_tabs.im.split("_")[2]) || -1,
                } as TabSetting)
            );
          } else {
            setTabSetting((prevState) => {
              const matchResult = res?.knowledge_tabs.im.match(
                /^(useful_expression|useful_link|useful|link|question|attachment)_(public|robot|personal)?/
              );
              let result: Pick<TabSetting, "activeTab" | "activeScope"> = {};
              if (matchResult && matchResult.length > 1) {
                let key = matchResult[1];
                if (key === "useful") {
                  key = "useful_expression";
                } else if (key === "link") {
                  key = "useful_link";
                }
                result.activeTab = key;
              }
              if (matchResult && matchResult.length > 2) {
                result.activeScope = matchResult[2];
              }
              return result;
            });
          }
        }
      })
      .finally(() => setLoading(false));
  }, []);

  return { tabSetting, saveTab, loading, saveScope, saveSpace, saveCatalog };
};

/**
 * 除了 question 以外，其他的标签页如果 scope 是 robot，则返回 public
 * @param {Scope} scope
 * @returns {string | "public" | "personal"}
 */
export const useSafeScope = (scope?: Scope) => {
  const { showPublicScope } = useContext(RepositoryWidgetContext);

  return useMemo(() => {
    let result = scope;
    if (!result || result === "robot") {
      result = "public";
    }
    if (!showPublicScope) {
      result = "personal";
    }
    return result;
  }, [scope, showPublicScope]);
};

/**
 * 如果 context 里的 expandedKeys 为 undefined，返回所有的 catalog，即展开所有的节点
 * @param {object} catalogMap
 */
export const useExpandedKeys = (catalogMap: { [key: string]: any }) => {
  const { expandedKeys } = useContext(RepositoryWidgetContext);

  return useMemo(() => {
    if (!expandedKeys) {
      return Object.keys(catalogMap);
    } else {
      return expandedKeys;
    }
  }, [catalogMap, expandedKeys]);
};

interface TabSelectionCache {
  expandedKeys?: (number | string)[];
  scrollTop?: number;
}

const SELECTION_CACHE: {
  [key: string]: {
    activeTab?: RepositoryType | "km";
    query?: string;
  } & Partial<
    Record<
      RepositoryType | "km",
      Record<Scope, TabSelectionCache> & { activeScope?: Scope }
    >
  >;
} = {};
export const useSelectionCache = (
  key: number | string | undefined = "selection_key",
  {
    defaultTab,
    defaultScope,
  }: { defaultTab?: RepositoryType | "km"; defaultScope?: Scope }
) => {
  const keyCache = useMemo(() => {
    if (!SELECTION_CACHE[key]) {
      SELECTION_CACHE[key] = {};
    }
    return SELECTION_CACHE[key];
  }, [key]);
  const [activeTab, setActiveTab] = useState<RepositoryType | "km">(
    "useful_expression"
  );

  //默认 tab 更新以后
  useEffect(() => {
    if (defaultTab && !keyCache.activeTab) {
      keyCache.activeTab = defaultTab;
      setActiveTab(keyCache.activeTab || "useful_expression");
    }
  }, [defaultTab, keyCache]);

  useEffect(() => {
    setActiveTab(keyCache.activeTab || "useful_expression");
    setQuery(keyCache.query || "");
  }, [keyCache]);

  const tabCache = useMemo(() => {
    if (!keyCache[activeTab]) {
      keyCache[activeTab] = {
        public: {},
        personal: {},
        robot: {},
      };
    }
    return keyCache[activeTab];
  }, [keyCache, activeTab]);

  const [activeScope, setActiveScope] = useState<Scope>("public");
  useEffect(() => {
    if (defaultScope && !tabCache!.activeScope) {
      tabCache!.activeScope = defaultScope;
      setActiveScope(defaultScope);
    }
  }, [defaultScope, tabCache]);
  useEffect(() => {
    setActiveScope(tabCache!.activeScope || "public");
  }, [tabCache]);

  const scopeCache = useMemo(() => {
    if (!tabCache![activeScope]) {
      tabCache![activeScope] = {};
    }
    return tabCache![activeScope];
  }, [activeScope, tabCache]);

  const [query, setQuery] = useState("");
  const [expandedKeys, setExpandedKeys] = useState<(number | string)[]>();
  const [scrollTop, setScrollTop] = useState<number>();

  useEffect(() => {
    setExpandedKeys(scopeCache.expandedKeys);
    setScrollTop(scopeCache.scrollTop);
  }, [scopeCache]);

  return {
    activeTab,
    setActiveTab: (tab: RepositoryType | "km") => {
      keyCache!.activeTab = tab;
      setActiveTab(tab);
    },
    query,
    setQuery: (keyword: string) => {
      keyCache!.query = keyword;
      setQuery(keyword);
    },
    activeScope,
    setActiveScope: (scope: Scope) => {
      tabCache!.activeScope = scope;
      setActiveScope(scope);
    },
    expandedKeys,
    setExpandedKeys: (keys: (number | string)[]) => {
      scopeCache!.expandedKeys = keys;
      setExpandedKeys(keys);
    },
    scrollTop,
    setScrollTop: (scrollTop: number) => {
      scopeCache!.scrollTop = scrollTop;
      setScrollTop(scrollTop);
    },
  };
};
