import React, {
  useCallback,
  useEffect,
  useLayoutEffect,
  useRef,
  useState,
} from "react";
import style from "./tinymce.module.scss";
import { randomString, removeHTMLTag, uploadFile } from "../../utils/index";
import UploadModal from "../uploadModal/index";

interface editorProps {
  value?: string;
  onChange?: (value: string) => void;
  maxLength?: number;
  autoresize_min_height?: number;
  statusbar?: boolean;
  toolbar?: string;
  showImg?: boolean;
  suffix?: string;
  autoresize_max_height?: number;
  paste_data_images?: boolean;
}

const defaultStyle = {
  color: "rgba(0, 0, 0, 0.25)",
};
const error = {
  color: "rgba(251, 81, 39, 1)",
};
const FormTinymce: React.FC<editorProps> = (props) => {
  const { paste_data_images } = props;
  const toolBars = `upimg | bold italic underline | alignleft aligncenter alignright | bullist numlist outdent indent | formatselect fontsizeselect | link image |  fontselect  forecolor backcolor code `;
  const {
    value,
    onChange,
    autoresize_min_height,
    autoresize_max_height = 50,
    statusbar = false,
    toolbar = toolBars,
    showImg = true,
    suffix = "",
    maxLength,
  } = props;
  const [editorContent, setContent] = useState<string>(
    removeHTMLTag(value || "")
  ); //更新富文本内容
  const [showModal, setShowModal] = useState<boolean>(false);
  const [tinymceInstance, setTinymceInstance] = useState<any>(null);
  const tinymceElementId = useRef<string>(randomString(10));

  // 内容改变时    去html标签 计算输入的长度
  const handleEditorChange = useCallback(() => {
    const content = tinymceInstance?.getContent();
    setContent(removeHTMLTag(content));
    onChange?.(content);
  }, [onChange, tinymceInstance?.getContent]);

  const hideImageUploadModal = useCallback(() => setShowModal(false), []);

  //点击弹窗的确定  将选择的图片插入并 关闭弹窗
  const addImg = useCallback(
    (imgList: UploadImgList[]) => {
      if (!tinymceInstance) {
        return;
      }
      imgList.forEach((item) => {
        tinymceInstance.execCommand(
          "mceInsertContent",
          false,
          `<img src=${item.src} alt=""/>`
        );
      });
      setShowModal(false);
    },
    [tinymceInstance]
  );

  useLayoutEffect(() => {
    let editor: any;
    let win: any = window;
    win.tinymce.init({
      selector: `#${tinymceElementId.current}${suffix}`,
      autoresize_min_height: autoresize_min_height,
      autoresize_max_height: autoresize_max_height,
      autoresize_bottom_margin: 5,
      menubar: false,
      statusbar: statusbar,
      elementpath: false, //底部状态栏左侧不显示标签名
      branding: false, //底部状态栏右侧签名
      paste_data_images: paste_data_images,
      images_upload_handler: (blobInfo: any, success: any, failure: any) => {
        console.log(
          "images_upload_handler",
          blobInfo,
          new File([blobInfo.blob()], blobInfo.filename())
        );
        uploadFile(new File([blobInfo.blob()], blobInfo.filename()), {
          marking: "web.mg.rich_editor_copy",
        })
          .then((resp) => {
            success(resp.src);
          })
          .catch((e) => {
            failure("上传图片失败");
          });
      },
      plugins: [
        `link image autoresize lists charmap preview hr `,
        "visualchars insertdatetime nonbreaking",
        "table contextmenu textcolor",
        "paste code imagetools",
      ],
      toolbar,
      setup(instance: any) {
        editor = instance;
        instance.on("init", () => {
          setTinymceInstance(instance);
        });
        if (showImg) {
          instance.addButton("upimg", {
            text: "上传图片",
            onclick: function () {
              setShowModal(true);
            },
          });
        }
      },
    });
    return () => {
      editor?.destroy();
    };
  }, [
    autoresize_max_height,
    autoresize_min_height,
    paste_data_images,
    showImg,
    statusbar,
    suffix,
    toolbar,
  ]);

  useEffect(() => {
    if (tinymceInstance && handleEditorChange) {
      tinymceInstance.on("change", handleEditorChange);
      return () => tinymceInstance.off("change", handleEditorChange);
    }
  }, [handleEditorChange, tinymceInstance]);

  useLayoutEffect(() => {
    if (tinymceInstance && props.value !== tinymceInstance.getContent()) {
      tinymceInstance.setContent(props.value || "");
    }
  }, [props.value, tinymceInstance]);

  return (
    <div className={style["rich-text-wrapper"]}>
      <div id={`${tinymceElementId.current}${suffix}`} />
      {maxLength && (
        <div
          className={style["editor-num"]}
          style={editorContent.length > maxLength ? error : defaultStyle}
        >
          {editorContent.length}/{maxLength}
        </div>
      )}
      <UploadModal
        show={showModal}
        close={hideImageUploadModal}
        addImg={addImg}
      />
    </div>
  );
};

FormTinymce.defaultProps = {
  paste_data_images: false,
};
export default FormTinymce;
