import Tree from "udesk-ui/es/tree";
import React, { ReactNode, useCallback, useEffect, useMemo, useState } from "react";
import produce from "immer";
import uniq from "lodash-es/uniq";
import Node from "./node";
import FolderOutlined from "@ant-design/icons/FolderOutlined";
import css from "./style.module.scss";
import { deleteTreeNode, findItemCommon, findTreeNode, moveTreeNode } from "./utilities";
import get from 'lodash-es/get';
import set from "lodash-es/set";

interface TreeData<T> {
    key: string,
    title: ReactNode,
    children?: TreeData<T>[],
    origin: T
}

/**
 * 将业务数据转换为符合 ant-design tree 的数据
 * @param {Option[]} value - 业务数据
 * @param {onAddNode} onAdd - 新增子节点事件
 * @param {onClickNode} onClick - 点击节点事件
 * @param {onChangeNode} onChange - 节点文字修改事件
 * @param {onDeleteNode} onDelete - 节点删除事件
 * @param shouldShowNode
 * @param {string} parentKey - 父节点 key，用于节点生成唯一 key
 * @param nodeChildrenPath
 * @returns {TreeData[]}
 */
type MapSelectOptionOptions<T> =
    { shouldShowNode: (node: T) => boolean, parentKey?: string }
    & Omit<NodeProps<T>, 'value' | 'nodeKey'>

function mapSelectOptions<T>(value: T[], {shouldShowNode, parentKey, nodeChildrenPath, ...props}: MapSelectOptionOptions<T>): TreeData<T>[] {
    return value.filter(shouldShowNode).map((i, index) => {
        const key = parentKey ? `${parentKey}-${index}` : `${index}`;
        return {
            key,
            title: <Node value={i} nodeKey={key}
                         nodeChildrenPath={nodeChildrenPath}
                         {...props}/>,
            icon: <FolderOutlined/>,
            blockNode: true,
            className: css.node,
            origin: i,
            children: mapSelectOptions(get(i, nodeChildrenPath) || [], {
                shouldShowNode,
                nodeChildrenPath,
                parentKey: key,
                ...props
            })
        };
    });
}

type Props<T> =
    {
        onChange?: (treeData: T[], objectData?: { nodeKey: string, type: string, deleteItem?: any }) => void
        value?: T[],
        onSelect: (nodeKey: string) => void
        shouldShowNode?: (node: T) => boolean
        allowSelect?: (node: T) => boolean
        createNode: (parentNode: T) => T
    }
    & Pick<NodeProps<T>, 'allowAddChild' | 'allowEditTitle' | 'allowRemoveNode' | 'nodeChildrenPath' | 'nodeTitlePath' | 'nodeValuePath'>

function TreeEditor<T extends object>({onChange, value, onSelect, shouldShowNode = () => true, nodeChildrenPath = 'subs', nodeTitlePath = 'title', nodeValuePath = 'value', allowSelect = () => true, createNode, ...props}: Props<T>): ReturnType<React.FC> {
    const [expandedKeys, setExpandedKeys] = useState<string[]>([]);
    const [selectedKeys, setSelectedKeys] = useState<string[]>([]);
    const onAddChild = useCallback((option: T, parents: string) => {
        if (!onChange || !value) {
            return;
        }
        const nextValue = produce(value, (draft: typeof value) => {
            const parentNode = findTreeNode(draft, parents, nodeChildrenPath);
            if (!parentNode) {
                return;
            }
            if (!get(parentNode, nodeChildrenPath)) {
                set(parentNode, nodeChildrenPath, []);
            }
            get(parentNode, nodeChildrenPath).push(createNode(parentNode));
        });
        onChange(nextValue || []);
        setExpandedKeys(prevState => uniq([...prevState, parents]));
    }, [onChange, value, nodeChildrenPath, createNode]);
    const onClickNode = useCallback((nodeKey, node) => {
        if (allowSelect(node)) {
            setSelectedKeys([nodeKey]);
        }
    }, [allowSelect]);
    const onDeleteNode = useCallback((nodeKey) => {
        if (!onChange || !value) {
            return;
        }

        let nodeKeyLen = nodeKey.split('-')
        let father = nodeKeyLen.length > 1 ? findItemCommon(value, nodeKeyLen.slice(0, nodeKeyLen.length - 1), true) : value
        let deleteItem: any = nodeKeyLen.length > 1 ? father.subs[nodeKeyLen[nodeKeyLen.length - 1]] : father[nodeKey]

        onChange(produce(value, (draft: typeof value) => {
            deleteTreeNode(draft, nodeKey, nodeChildrenPath);
        }) || [], {nodeKey, type: 'delete', deleteItem});
    }, [onChange, value, nodeChildrenPath]);
    const onNodeTitleChange = useCallback((title, nodeKey) => {
        if (!onChange || !value) {
            return;
        }
        const newValue = produce(value, (draft: typeof value) => {
            const node = findTreeNode(draft, nodeKey, nodeChildrenPath);
            if (node) {
                set(node, nodeTitlePath, title)
            }
        });
        onChange(newValue || [], {nodeKey, type: 'change'});
    }, [nodeChildrenPath, nodeTitlePath, onChange, value]);
    const treeData = useMemo(() => {
        if (!value) {
            return [];
        }
        return mapSelectOptions(value, {
            onAdd: onAddChild,
            onClick: onClickNode,
            onChange: onNodeTitleChange,
            onDelete: onDeleteNode,
            nodeChildrenPath,
            shouldShowNode,
            nodeTitlePath,
            nodeValuePath,
            ...props
        });

    }, [value, onAddChild, onClickNode, onNodeTitleChange, onDeleteNode, nodeChildrenPath, shouldShowNode, nodeTitlePath, nodeValuePath, props]);

    useEffect(() => {
        if (selectedKeys) {
            onSelect(selectedKeys[0]);
        }
    }, [selectedKeys, onSelect]);
    const onDrop = useCallback(({node, dragNode}) => {
        if (!onChange || !value) {
            return;
        }
        if (node.dragOver) {
            onChange(produce(value, (draft: T[]) => moveTreeNode(draft, dragNode.key, node.key, nodeChildrenPath)));
        } else if (node.dragOverGapBottom || node.dragOverGapTop) {
            onChange(produce(value, (draft: T[]) => {
                let position = parseInt(node.key.substring(node.key.length - 1));
                if (node.dragOverGapBottom) {
                    position += 1;
                }
                moveTreeNode(draft, dragNode.key, node.key.substring(0, node.key.lastIndexOf('-')), nodeChildrenPath, position);
            }));
        }
    }, [nodeChildrenPath, onChange, value]);
    return <Tree selectable showIcon blockNode treeData={treeData}
                 expandedKeys={expandedKeys}
                 selectedKeys={selectedKeys}
                 draggable={true}
                 onDrop={onDrop}
                 onExpand={(keys) => setExpandedKeys(keys as string[])}/>
}

export default TreeEditor;
export * from './utilities';
