import React, { useCallback, useEffect, useRef, useState } from "react";
import { message, Upload } from "udesk-ui";
import { useTranslation } from "react-i18next";
import { getJSON, postJSON, r } from "../../utils/index";
import { UploadChangeParam, UploadProps } from "udesk-ui/es/upload";
import isEqual from "lodash-es/isEqual";
import unionBy from "lodash-es/unionBy";
interface OSSData {
  accessid: string;
  bucket: string;
  code: number;
  dir: string;
  expire: number;
  host: string;
  policy: string;
  signature: string;
  storage_policy: string;
  fields: object;
  file_path: string;
}

type FileObj = Parameters<NonNullable<UploadProps["onPreview"]>>[0];

type Props = Omit<
  UploadProps,
  | "withCredentials"
  | "beforeUpload"
  | "fileList"
  | "onChange"
  | "transformFile"
  | "action"
  | "data"
> & {
  value?: FileObj[];
  single?: number; //单个文件字节
  total?: number; //所有文件字节
  marking: string;
  accept?: string;
  onChange?: (fileList: FileObj[]) => void;
  noAttachment?: boolean; //上传结束后是否请求/attachments 接口，true 为是，false 为否
  onError?: (file: FileObj) => void;
};

const UploadBox: React.FC<Props> = ({
  showUploadList = true,
  multiple = true,
  value,
  accept = "",
  headers = {},
  ...props
}) => {
  const { t } = useTranslation(); //多语言
  const [fileList, setFileList] = useState<FileObj[]>([]); //上传的文件  必须usestate  不然onchange只触发一次
  const onChangeRef = useRef(props.onChange);

  useEffect(() => {
    if (!value) {
      return;
    }
    setFileList((v) => {
      let newList = v.concat(value);
      return unionBy(newList, "url");
      // const doneFiles = v.filter(i => i.status === 'done');
      // if (isEqual(value, doneFiles) || isEqual(value, v)) {
      //     return v;
      // }
      // const newFiles = differenceBy(value, doneFiles, 'uid');
      // const removedFiles = differenceBy(doneFiles, value, 'uid');
      // if (!newFiles.length && !removedFiles.length) { //既无新增也无删除，则不修改 fileList
      //     return v;
      // }

      // let result = v.filter(i => removedFiles.every(j => j.uid !== i.uid)); //根据 value 删掉 file，此情况出现在 value 里的文件少与 fileList 里的文件
      // result = [...result, ...newFiles]; //根据 value 新增 file，此情况出现在 value 里有新的文件

      // return result;
    });
  }, [value]);

  const [OSSData, setOSSData] = useState<OSSData>({} as OSSData); //鉴权信息

  // 上传之前 获取鉴权信息
  const beforeUpload = useCallback<
    (file: FileObj, beforeFileList: FileObj[]) => Promise<void>
  >(
    (file, beforeFileList) => {
      // 如果单个文件 大小 超过限制   提示并且 停止上传

      if (props.single && file.size! > props.single) {
        message.warning(
          r(
            t("page.im.ticket.warning.exceedsSize"),
            file.name,
            props.single / 1024 / 1024 + "M"
          )
        );
        return Promise.reject();
      }

      // 如果总大小超过限制   提示并停止上传
      let totalSize: number = 0;
      //计算已上传文件的总大小
      fileList
        .filter((item) => item.status === "done")
        .forEach((item) => (totalSize += item.size!));
      if (multiple) {
        // 多选   再加上 beforeFileList的文件总大小
        beforeFileList.forEach((item) => (totalSize += item.size!));
      } else {
        // 单选  加上 本次上传文件大小
        totalSize += file.size!;
      }

      if (props.total && totalSize > props.total) {
        message.warning(
          r(
            t("page.im.ticket.warning.exceedsTotalSize"),
            totalSize / 1024 / 1024 + "M",
            props.total
          )
        );
        return Promise.reject();
      }
      // 发起请求
      return new Promise((resolve, reject) => {
        getJSON(`/file_upload_policy?marking=${props.marking}&key=${file.name}`)
          .then((res) => {
            if (res.code === 1000) {
              // 存储鉴权信息
              setOSSData(res);
              resolve();
            } else {
              reject();
            }
          })
          .catch(() => reject());
      });
    },
    [fileList, multiple, props.marking, props.single, props.total, t]
  );

  //上传到服务器
  const fetchAttachment = useCallback(
    (file: FileObj) => {
      const url = `/attachments`;
      const params = {
        attachment: {
          qiniu_key: file.url,
          file_file_name: file.name,
          file_content_type: file.type,
          file_file_size: file.size,
          bucket: OSSData.bucket,
          storage_policy: OSSData.storage_policy,
        },
      };
      return postJSON(url, params).then((res) => {
        if (res.code === 1000) {
          return res;
        }
      });
    },
    [OSSData.bucket, OSSData.storage_policy]
  );

  // 上传状态变化触发   status:uploading  error done remove
  const onChange = useCallback(
    (info: UploadChangeParam) => {
      let fileList = info.fileList;

      if (["ali", "huawei_cloud_obs"].includes(OSSData.storage_policy)) {
        fileList.forEach((item) => {
          if (!props.noAttachment && item.url && !item.url.includes("http"))
            item.url = OSSData.host + "/" + item.url;
        });
      }
      if (info.file.status === "done") {
        if (props.noAttachment) {
          //需要等到请求结束后才 setFileList
          fetchAttachment(info.file).then((res) => {
            if (!res) {
              return;
            }
            setFileList((v) =>
              v.map((item) => {
                if (item.url === info.file.url) {
                  item.uid = res.attachment_id;
                  return { ...item, uid: res.attachment_id };
                }
                return item;
              })
            );
          });
          return;
        }
      } else if (info.file.status === "error") {
        props.onError?.(info.file);
      }
      setFileList(unionBy(fileList, "url"));
    },
    [
      OSSData.host,
      OSSData.storage_policy,
      fetchAttachment,
      props.noAttachment,
      props.onError,
    ]
  );

  // 在上传之前转换文件
  const transformFile = useCallback(
    (file) => {
      const suffix = file.name.slice(file.name.lastIndexOf("."));
      const filename = Date.now() + suffix;
      file.url = OSSData.dir + filename;
      return file;
    },
    [OSSData.dir]
  );

  // 处理上传接口  阿里/私有部署
  const handleAction = useCallback(() => {
    return ["ali", "huawei_cloud_obs"].includes(OSSData.storage_policy)
      ? OSSData.host
      : `${OSSData.host}/${OSSData.file_path}`;
  }, [OSSData.file_path, OSSData.host, OSSData.storage_policy]);

  useEffect(() => {
    // props?.onChange?.(fileList.filter(i => i.status === 'done'));
    if (!isEqual(fileList, value)) {
      onChangeRef?.current?.(fileList.filter((i) => i.status === "done"));
    }
  }, [fileList]);

  const handleData = useCallback(
    (file) => {
      if ("ali" === OSSData.storage_policy) {
        return {
          key: file.url,
          OSSAccessKeyId: OSSData.accessid,
          policy: OSSData.policy,
          Signature: OSSData.signature,
        };
      } else if ("huawei_cloud_obs" === OSSData.storage_policy) {
        return {
          key: file.url,
          AccessKeyId: OSSData.accessid,
          policy: OSSData.policy,
          Signature: OSSData.signature,
        };
      } else {
        let data: { [index: string]: any } = {};
        Object.entries(OSSData.fields || {}).forEach(([key, value]) => {
          data[key] = value;
        });
        return data;
      }
    },
    [
      OSSData.accessid,
      OSSData.fields,
      OSSData.policy,
      OSSData.signature,
      OSSData.storage_policy,
    ]
  );
  return (
    <Upload
      {...props}
      headers={{ "X-Requested-With": null, ...headers } as any}
      accept={accept}
      name={props.name || "file"}
      withCredentials={true}
      beforeUpload={beforeUpload}
      multiple={multiple}
      fileList={fileList}
      onChange={onChange}
      transformFile={transformFile}
      action={handleAction}
      showUploadList={showUploadList}
      data={handleData}
    >
      {props.children}
    </Upload>
  );
};

export default UploadBox;
