/** @jsx jsx */
import style from './style.js';
import {jsx} from '@emotion/react';
import {ChangeEvent, Component, ComponentType, ElementType, FormEvent, ReactElement} from 'react';
import List from './list';
import Categories from './categories';
import {AngleLeft, AngleRight, ArrowDownIcon, ArrowUpIcon, SearchIcon} from './icons';
import {Column, ColumnIdType, ValueType} from "./types";
import {DndProvider} from 'react-dnd';
import {HTML5Backend} from "react-dnd-html5-backend";
import DndList from './DndList';

type Props = {
    columns: Column[],
    value?: ValueType,
    setoDefaultValue?: ValueType,
    onChange?: (value: ValueType) => void,
    onCancel?: () => void,
    leftTitle?: string | null,
    rightTitle?: string | null,
    className?: string,
    t: (string) => string,
    classNameOfSearchInput?: string,
    onReset?: () => void,
    allowReset?: boolean,
    allowSort?: boolean,
    allowAutoSelectNext?: boolean,
    allowSetoDefault?: boolean,
    CancelButton: ComponentType<{ onClick: () => void, t: (string) => string }>,
    ResetButton: ComponentType<{ onClick: () => void, t: (string) => string }>,
    ConfirmButton: ComponentType<{ onClick: () => void, t: (string) => string }>
}

type State = {
    activeCategory: string,
    leftSelectedColumnIds: ColumnIdType[],
    rightSelectedColumnIds: ColumnIdType[],
    keyword: string,
    originValue: ValueType,
    value: ValueType
}

let _groupByCategoryCache: any[] = [];

function _groupByCategory(columns: Column[]): { string?: Column[] } {
    if (_groupByCategoryCache[0] === columns) {
        return _groupByCategoryCache[1];
    }
    _groupByCategoryCache[0] = columns;

    let categoryColumnMap = {};
    columns.forEach(column => {
        if (column.category) {
            if (!categoryColumnMap[column.category]) {
                categoryColumnMap[column.category] = [];
            }
            categoryColumnMap[column.category].push(column);
        }
    });

    _groupByCategoryCache[1] = categoryColumnMap;
    return categoryColumnMap;
}

function TitleNode({children, className = ''}): null | ReactElement<'h4'> {
    if (children) {
        return <h4 className={className}>{children}</h4>;
    }
    return null;
}

class Control extends Component<Props, State> {
    state = {
        activeCategory: '',
        leftSelectedColumnIds: [],
        rightSelectedColumnIds: [],
        keyword: '',
        value: this.props.value || [],
        originValue: this.props.value || []
    } as State;

    static getDerivedStateFromProps(props: Props, state: State): null | State {

        if (props.value !== state.originValue) {
            return {
                originValue: props.value || [],
                value: props.value || [],
                keyword: '',
                activeCategory: '',
                leftSelectedColumnIds: [],
                rightSelectedColumnIds: []
            };
        }

        return null;
    }

    _getCategories(): string[] {
        let {columns} = this.props;
        return Object.keys(_groupByCategory(columns));
    }

    _getLeftColumns(): Column[] {
        let {
            activeCategory,
            keyword,
            value
        } = this.state;
        let {
            columns,
            t
        } = this.props;

        let categoriesObj = _groupByCategory(columns);
        let categories = Object.keys(categoriesObj);
        if (categories.length && categories.indexOf(activeCategory) === -1) {
            activeCategory = categories[0];
        }

        let leftCandidate: Column[] = columns;
        if (activeCategory) {
            leftCandidate = categoriesObj[activeCategory];
        }

        value = Array.isArray(value) ? value : [];
        leftCandidate = leftCandidate.filter(i => {
            let name = i.name || t(i.id);
            if (keyword && typeof name === 'string' && name.indexOf(keyword) === -1) {
                return false;
            }

            return value.indexOf(i.id) === -1;
        });

        return leftCandidate;
    }

    _getRightColumns(): Column[] {
        let {value} = this.state;
        let {columns} = this.props;
        let rightColumns: Column[] = [];
        (value || []).forEach(i => {
            let column = columns.find(c => c.id === i);
            if (column) {
                rightColumns.push(column);
            }
        });
        return rightColumns;
    }

    _getLastValue(values): string {
        return values.length > 0 ? values[values.length - 1] : undefined;
    }

    _getNextId(type: string = 'left'): ValueType {
        const {allowAutoSelectNext} = this.props;
        if (!allowAutoSelectNext) return [];
        const {leftSelectedColumnIds, rightSelectedColumnIds} = this.state;
        const lastValue = this._getLastValue(type === 'left' ? leftSelectedColumnIds : rightSelectedColumnIds);
        const ids = (type === 'left' ? this._getLeftColumns() : this._getRightColumns()).map(column => column.id);
        const index = ids.findIndex(id => id === lastValue);
        return index < ids.length - 1 ? [ids[index + 1]] : [];
    }

    _moveToRight = () => {
        let {
            leftSelectedColumnIds,
            value
        } = this.state;
        if (leftSelectedColumnIds.length === 0) {
            return;
        }

        this.setState({
            value: [...value, ...leftSelectedColumnIds],
            leftSelectedColumnIds: this._getNextId('left')
        });
    };

    _moveToLeft = () => {
        let {
            rightSelectedColumnIds,
            value
        } = this.state;
        if (rightSelectedColumnIds.length === 0) {
            return;
        }

        this.setState({
            rightSelectedColumnIds: this._getNextId('right'),
            value: value.filter(i => rightSelectedColumnIds.indexOf(i) === -1)
        });
    };

    _up = () => {
        let {
            rightSelectedColumnIds,
            value
        } = this.state;

        if (!value || value.length === 0) {
            return;
        }

        let newAry = rightSelectedColumnIds.reduce((preValue, item) => {
            let index = preValue.indexOf(item);
            //如果已经在顶部了，则不需要排序
            if (index === 0) {
                return preValue;
            }
            //如果这个的上面一个是选中的，也不需要排序
            if (preValue[index - 1] && rightSelectedColumnIds.indexOf(preValue[index - 1]) > -1) {
                return preValue;
            }
            preValue.splice(index, 0, ...preValue.splice(index - 1, 1));
            return preValue;
        }, [...value]);

        this.setState({value: newAry});
    };

    _down = () => {
        let {
            rightSelectedColumnIds,
            value
        } = this.state;

        if (!value || value.length === 0) {
            return;
        }

        let newAry = [...rightSelectedColumnIds].reverse().reduce((preValue, item) => {
            let index = preValue.indexOf(item);
            //如果已经在最底部了，则不在排序
            if (index === preValue.length - 1) {
                return preValue;
            }
            //如果这个的下面那个是选中的，也不需要排序
            if (preValue[index + 1] && rightSelectedColumnIds.indexOf(preValue[index + 1]) > -1) {
                return preValue;
            }
            preValue.splice(index + 1, 0, ...preValue.splice(index, 1));
            return preValue;
        }, [...value]);

        this.setState({value: newAry});
    };

    _updateKeyword = (e: ChangeEvent<HTMLInputElement>) => {
        this.setState({keyword: e.target.value});
    };

    _onConfirmClick = () => {
        let {onChange} = this.props;
        let {value} = this.state;
        if (typeof onChange === 'function') {
            onChange(value);
        }
    };

    _onCancelClick = () => {
        let {onCancel} = this.props;
        if (typeof onCancel === 'function') {
            onCancel();
        }
    };

    _onSetoDefaultClick = () => {
        const {setoDefaultValue} = this.props;
        const {leftSelectedColumnIds, rightSelectedColumnIds} = this.state;
        if (Array.isArray(setoDefaultValue)) {
            this.setState({
                value: setoDefaultValue,
                leftSelectedColumnIds: leftSelectedColumnIds.filter(id => !setoDefaultValue.includes(id)),
                rightSelectedColumnIds: rightSelectedColumnIds.filter(id => setoDefaultValue.includes(id))
            });
        }
    };

    _onResetClick = () => {
        let {onReset} = this.props;
        if (typeof onReset === 'function') {
            onReset();
        }
    };

    _onDropEnd = (list) => {
        console.log('list', list);
        const newValue: Array<any> = [];
        list.forEach((item: Column)=> {
            newValue.push(item.id);
        });
        this.setState({
            value: newValue
        });
    };

    _onDndItemClick = (item) => {
        const { rightSelectedColumnIds } = this.state;
        if (item.id !== rightSelectedColumnIds[0]) {
            this.setState({
                rightSelectedColumnIds: [item.id]
            });
        }
    };

    render() {
        let {
            leftTitle,
            rightTitle,
            className,
            t = (v) => v,
            classNameOfSearchInput,
            allowReset,
            allowSort,
            allowSetoDefault,
            CancelButton,
            ResetButton,
            ConfirmButton
        } = this.props;
        let {
            activeCategory,
            leftSelectedColumnIds,
            rightSelectedColumnIds,
            keyword
        } = this.state;

        let categories = this._getCategories();
        let rightColumns = this._getRightColumns();
        let leftColumns = this._getLeftColumns();
        if (!activeCategory) {
            activeCategory = categories[0];
        }

        return (
            <div css={style} className={className || ''}>
                <div className="control-header">{t('label.chooseCloumn')}</div>
                <div className='control-content'>
                    <div className='left-panel'>
                        <div className='title'>
                            <TitleNode>{leftTitle}</TitleNode>
                        </div>
                        {(() => {
                            if (categories.length) {
                                return <Categories className='category-tab'
                                                   content={categories}
                                                   value={activeCategory}
                                                   activeClassName='active'
                                                   t={t}
                                                   onChange={(activeCategory) => this.setState({activeCategory})}/>;
                            }
                            return null;
                        })()}
                        <div className='select-wrapper'>
                            <div className='search-wrapper'>
                                <input 
                                    value={keyword} 
                                    className={classNameOfSearchInput || 'form-control'}
                                    onChange={this._updateKeyword} 
                                />
                                <SearchIcon className='search-icon'/>
                            </div>
                            <List 
                                content={leftColumns}
                                className='select custom-scroller'
                                onChange={(leftSelectedColumnIds) => this.setState({leftSelectedColumnIds})}
                                value={leftSelectedColumnIds}
                                t={t}
                            />
                        </div>
                    </div>
                    <div className='mid-panel'>
                        <button onClick={this._moveToLeft}><AngleLeft/></button>
                        <button onClick={this._moveToRight}><AngleRight/></button>
                    </div>
                    <div className='right-panel'>
                        <div className='title'>
                            <TitleNode>{rightTitle}</TitleNode>
                            {allowSetoDefault ? <a className='set-to-default'
                                                   onClick={this._onSetoDefaultClick}>{t('label.setoDefault')}</a> : null}
                        </div>
                        <div className='select-wrapper'>
                            {/* {allowSort ? <div className='sort-btn-wrapper'>
                                <button onClick={this._up}><ArrowUpIcon/></button>
                                <button onClick={this._down}><ArrowDownIcon/></button>
                            </div> : null} */}
                            <DndProvider backend={HTML5Backend}>
                                <DndList
                                    t={t}
                                    allowSort={allowSort}
                                    value={rightSelectedColumnIds}
                                    list={rightColumns}
                                    onDropEnd={this._onDropEnd}
                                    onClick={this._onDndItemClick}
                                ></DndList>
                            </DndProvider>
                            {/* <List 
                                content={rightColumns}
                                className='select'
                                onChange={(rightSelectedColumnIds) => this.setState({rightSelectedColumnIds})}
                                value={rightSelectedColumnIds}
                                scrollbarFollow={true}
                                t={t}
                            /> */}
                        </div>
                    </div>
                </div>
                <div className='bottom-bar'>
                    {allowReset && <ResetButton onClick={this._onResetClick} t={t}/>}
                    <CancelButton onClick={this._onCancelClick} t={t}/>
                    <ConfirmButton onClick={this._onConfirmClick} t={t}/>
                </div>
            </div>
        );
    }

    static defaultProps = {
        allowReset: true,
        allowSort: true,
        allowAutoSelectNext: false,
        allowSetoDefault: false
    };
}

export default Control;
