const https = require('https');
const md5 = require('blueimp-md5');
const url = require('url');
const fs = require('fs');
const prettier = require('prettier');
const { default: traverse } = require('@babel/traverse');
const log = require('./log');
// const MagicString = require('magic-string');

// const N = '\n';
// const prefix = 'ctxLocales';
const chineseFindReg = /\p{Unified_Ideograph}/u;

async function core(filePath, formatLocalesKey = (key) => key) {
    log.info('开始转换文件 ::', filePath);

    let code = fs.readFileSync(filePath, 'utf-8');
    let requests = [];
    // let count = 0; // 因为百度翻译每秒钟有并发限制，所以要降低请求的频率
    prettier.format(code, {
        parser(text, parsers) {
            let ast = parsers['babel-ts'](text);
            traverse(ast, {
                JSXText(path) {
                    if (chineseFindReg.test(path.node.value)) {
                        requests.push(requestBdTran(formatLocalesKey, path.node.value, path.node));
                        // count++;
                    }
                },
                StringLiteral(path) {
                    if (chineseFindReg.test(path.node.value)) {
                        requests.push(
                            requestBdTran(formatLocalesKey, path.node.value, path.node, path)
                        );
                        // count++;
                    }
                },
                TemplateElement(path) {
                    if (chineseFindReg.test(path.node.value.cooked)) {
                        requests.push(
                            requestBdTran(formatLocalesKey, path.node.value.cooked, path.node)
                        );
                        // count++;
                    } else if (chineseFindReg.test(path.node.value.raw)) {
                        requests.push(
                            requestBdTran(formatLocalesKey, path.node.value.raw, path.node)
                        );
                        // count++;
                    }
                },
            });
            return ast;
        },
    });

    const sleep = () => {
        return new Promise((res) => {
            setTimeout(() => {
                res();
            }, 1100);
        });
    };

    let attributes = [];
    const REQUEST_COUNT = 9;
    while (requests.length) {
        let queue = requests.splice(0, REQUEST_COUNT).map((i) => i());

        const part = await Promise.all([...queue, sleep()]);

        part.length = part.length - 1;
        attributes = [...attributes, ...part];
    }
    // const attributes = await Promise.all(requests);

    const attributesMap = attributes.reduce((prev, current) => {
        const key = `${formatKey(current.src)}_$#_${formatKey(current.dst)}`;
        if (Array.isArray(prev[key])) {
            prev[key].push(current);
        } else {
            prev[key] = [current];
        }
        return prev;
    }, {});

    return Object.values(attributesMap);
}

function bdTran(word, targetLanguage) {
    let options = getOptions(word, targetLanguage);
    return new Promise((resolve, rej) => {
        let request = https.get(
            'https://fanyi-api.baidu.com/api/trans/vip/translate?' +
                new url.URLSearchParams(options).toString(),
            cb
        );

        request.on('error', (e) => {
            log.error(`获取数据失败: ${e.message}`);
            rej(e.message);
        });

        function cb(res) {
            let body = '';
            res.on('data', (data) => {
                body += data;
            });

            res.on('end', () => {
                try {
                    JSON.parse(body).trans_result[0];
                } catch (e) {
                    log.error('???', body);
                }
                const ret = JSON.parse(body).trans_result[0];
                log.success(`✅  翻译成功: ${ret.src} ==> ${ret.dst}`);
                resolve(ret.dst);
            });
        }
    });
}

core.bdTran = bdTran;
core.chineseFindReg = chineseFindReg;

module.exports = core;

function formatKey(string) {
    return string.replace(/[^a-zA-Z0-9\u4e00-\u9fa5]/g, '').replace(/[0-9]+/, '');
}

function requestBdTran(formatLocalesKey, q, pathNode, path) {
    let options = getOptions(q);
    const { start, end, type } = pathNode;

    let trigger = () => {
        return new Promise((resolve, rej) => {
            let request = https.get(
                'https://fanyi-api.baidu.com/api/trans/vip/translate?' +
                    new url.URLSearchParams(options).toString(),
                cb
            );

            request.on('error', (e) => {
                console.log(`获取数据失败: ${e.message}`);
                rej(e.message);
            });

            function cb(res) {
                let body = '';
                res.on('data', (data) => {
                    body += data;
                });

                res.on('end', () => {
                    try {
                        JSON.parse(body).trans_result[0];
                    } catch (e) {
                        console.log('???', body);
                    }
                    const ret = JSON.parse(body).trans_result[0];
                    log.success(`✅  翻译成功: ${ret.src} ==> ${ret.dst}`);
                    resolve(
                        Object.assign(
                            {},
                            {
                                ...ret,
                                dst: formatKeyToCamelCase(ret.dst, formatLocalesKey),
                                start,
                                end,
                                type,
                            },
                            path ? { parentNodeType: path.parentPath.type } : {}
                        )
                    );
                });
            }
        });
    };

    return trigger;
}

function getOptions(q, to = 'en') {
    let options = {
        q,
        from: 'zh',
        to,
        appid: '20220112001053416',
        salt: '123a',
    };
    options.sign = md5(`${options.appid}${options.q}${options.salt}sKbkzc8L6UEsLhLK6ZTa`);
    return options;
}

function formatKeyToCamelCase(str, formatLocalesKey, firstLetterUpperCase = false) {
    const defaultFormatFunc = (key) => {
        if (key.length > 30) {
            const reservedWordCount = Math.min(Math.round(key.length / 3), 10);
            return `${key.slice(0, reservedWordCount)}${
                key.slice(reservedWordCount, -reservedWordCount).length
            }${key.slice(-reservedWordCount)}`;
        }
        return key;
    };
    return formatLocalesKey(
        formatKey(
            `${str
                .split(' ')
                .map((_, index) =>
                    _.replace(/^\w/, (item) =>
                        firstLetterUpperCase
                            ? item.toUpperCase()
                            : index === 0
                                ? item.toLowerCase()
                                : item.toUpperCase()
                    )
                )
                .join('')}`
        ),
        defaultFormatFunc
    );
}
