import React, { useCallback, useMemo } from 'react';
import { Button, notification, message, Modal, Dropdown } from 'udesk-ui';
import { ButtonProps } from 'udesk-ui/es/button';
import classNames from 'classnames';
import { usePrefixCls } from '../utils';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import { ArrowDownSOutlined, ExportOutlined } from '@udesk/icons';
import { ModalFuncProps } from 'udesk-ui/es/modal';
import BatchedActions from '../batched-actions';
import { DropDownProps } from 'udesk-ui/es/dropdown';

const { confirm } = Modal;

export interface ActionMenuItem {
  label: string;
  type: string;
  // onClick: (item: any) => void;
  // danger?: boolean;
  disabled?: boolean | ((item: any) => boolean);
  confirm?: Omit<ModalFuncProps, 'onOk'>;
}

export interface ExportButtonProps extends Omit<ButtonProps, 'onClick'> {
  exportDataCount?: number;
  onClick: (
    type?: string | React.MouseEvent<HTMLElement, MouseEvent>,
    e?: React.MouseEvent<HTMLElement, MouseEvent>,
  ) => Promise<any>;
  noSecondConfirmation?: boolean;
  goToDownloadCenter?: () => void;
  confirm?: Omit<ModalFuncProps, 'onOk'>;
  dropdown?: ActionMenuItem[];
  getPopupContainer?: DropDownProps['getPopupContainer'];
}

const ExportButton: React.FC<ExportButtonProps> = (props) => {
  const {
    prefixCls: customizePrefixCls,
    className,
    children,
    exportDataCount,
    goToDownloadCenter,
    confirm: confirmProps,
    dropdown,
    onClick,
    noSecondConfirmation = false,
    getPopupContainer,
    ...rest
  } = props;
  const [contextLocale] = useLocaleReceiver('ExportButton');

  const [buttonPrefixCls, prefixCls] = usePrefixCls('export-button', customizePrefixCls);

  const classes = classNames(buttonPrefixCls, className);

  const isExportAll = () => typeof exportDataCount !== 'number';

  const onOk = useCallback(
    (type) => {
      return props.onClick?.(type)?.then(() => {
        notification.success({
          key: 'exportNotification',
          message: contextLocale.lang.successfully,
          getContainer: getPopupContainer as any,
          description: (
            <div>
              {contextLocale.lang.pleaseJumpTo}
              <Button
                prefixCls={`${prefixCls}-btn`}
                type="link"
                size="small"
                onClick={goToDownloadCenter}
              >
                {contextLocale.lang.uploadAndDownloadCenter}
              </Button>
              {contextLocale.lang.check}
            </div>
          ),
        });
      });
    },
    [props.onClick, goToDownloadCenter, getPopupContainer],
  );
  const defaultConfirmProps = useMemo(() => {
    return {
      title: isExportAll() ? contextLocale.lang.exportAll : `${contextLocale.lang.total} ${exportDataCount || 0} ${contextLocale.lang.confirm}`,
      // icon: <ExclamationCircleOutlined />,
      maskClosable: true,
      okText: contextLocale.lang.export,
      ...confirmProps,
      onOk,
    };
  }, [exportDataCount, confirmProps, prefixCls, onOk]);
  const showConfirm = () => {
    if (exportDataCount === 0) {
      message.error(contextLocale.lang.error);
      return;
    }
    confirm(defaultConfirmProps);
  };

  const exportDropdown: ActionMenuItem[] | undefined = useMemo(() => {
    const res = dropdown?.map((item) => ({
      ...item,
      confirm: noSecondConfirmation
        ? undefined
        : {
            ...defaultConfirmProps,
            ...item.confirm,
          },
      onClick: onOk.bind(null, item.type),
    }));

    return res;
  }, [dropdown, defaultConfirmProps, onOk, noSecondConfirmation]);

  return (
    <React.Fragment>
      {dropdown ? (
        <BatchedActions menu={exportDropdown} getPopupContainer={getPopupContainer}>
          <Button
            className={classes}
            {...rest}
            // onClick={showConfirm}
            icon={<ExportOutlined style={{ marginRight: 4 }} />}
          >
            {children || contextLocale.lang.export}
            <ArrowDownSOutlined style={{ marginLeft: 4 }} />
          </Button>
        </BatchedActions>
      ) : (
        <Button
          className={classes}
          icon={<ExportOutlined style={{ marginRight: 4 }} />}
          {...rest}
          onClick={noSecondConfirmation ? props.onClick : showConfirm}
        >
          {children || contextLocale.lang.export}
        </Button>
      )}
    </React.Fragment>
  );
};

export default ExportButton;
