const path = require('path');
const fs = require('fs');
const MagicString = require('magic-string');
const prettier = require('prettier');
const { default: traverse } = require('@babel/traverse');

function locales(attributes) {
    return attributes.reduce((prev, current) => {
        if (!Array.isArray(current)) {
            return prev;
        }
        const attr = current[0];
        prev[attr.dst] = attr.src;
        return prev;
    }, {});
}

function code(
    attributes,
    format,
    filePath,
    keyPath,
    generatePathMapValue,
    autoImportOptions,
    sourceMap = false
) {
    const code = fs.readFileSync(filePath, 'utf-8');
    let s = new MagicString(code);

    attributes.forEach((attr) => {
        attr.forEach((c) => {
            let key = `${generatePathMapValue}.${keyPath.join('.')}${keyPath.length ? '.' : ''}${
                c.dst
            }`;
            s.overwrite(
                c.start,
                c.end,
                formatString(c.type, key, format, c.parentNodeType, c.src, Boolean(sourceMap))
            );
        });
    });

    return autoImport(s.toString(), autoImportOptions);
}

module.exports = {
    locales,
    code,
};

function autoImport(code, options) {
    if (!options) return code;
    let importedSource = false;
    let lastImportDeclarationPosition = 0;

    let s = new MagicString(code);
    prettier.format(code, {
        parser(text, parsers) {
            let ast = parsers['babel-ts'](text);
            traverse(ast, {
                ImportDeclaration(path) {
                    // console.log('path', path.node);
                    lastImportDeclarationPosition = path.node.end;
                    const source = path.node.source.value;
                    if (options.source === source) {
                        importedSource = true;
                        const specifiers = path.node.specifiers;
                        if (
                            !specifiers.some(
                                (s) =>
                                    s.imported.name === options.importDefaultSpecifier ||
                                    s.imported.name === options.importSpecifier
                            )
                        ) {
                            if (options.importDefaultSpecifier) {
                                s.overwrite(
                                    path.node.start,
                                    path.node.start + 7,
                                    `import ${options.importDefaultSpecifier}, `
                                );
                            } else if (options.importSpecifier) {
                                s.appendRight(
                                    specifiers[specifiers.length - 1].end,
                                    `, ${options.importSpecifier}`
                                );
                            }
                        }
                    }
                },
            });
            return ast;
        },
    });
    if (!importedSource) {
        const importDefaultSpecifierString = `
import ${options.importDefaultSpecifier} from '${options.source}';`;
        const importSpecifierString = `
import { ${options.importSpecifier} } from '${options.source}';`;
        s.appendRight(
            lastImportDeclarationPosition,
            options.importDefaultSpecifier
                ? importDefaultSpecifierString
                : options.importSpecifier
                ? importSpecifierString
                : ''
        );
    }
    return s.toString();
}

function formatString(type, key, format, parentNodeType, zhValue, sourceMap) {
    const formattedKey = typeof format === 'function' ? format(key) : key;
    const zhValueCommentInJs = sourceMap ? `/* ${zhValue} */` : '';
    const zhValueCommentInJsx = zhValueCommentInJs ? `{${zhValueCommentInJs}}` : '';
    switch (type) {
        case 'JSXText': {
            return `${zhValueCommentInJsx}{${formattedKey}}`;
        }
        case 'StringLiteral': {
            return parentNodeType === 'JSXAttribute'
                ? `{${zhValueCommentInJs}${formattedKey}}`
                : `${zhValueCommentInJs}${formattedKey}`;
        }
        case 'TemplateElement': {
            return `\${${zhValueCommentInJs}${formattedKey}}`;
        }
    }
}
