import React, { useCallback, useEffect, useState } from 'react';
import { message } from 'udesk-ui';
import classNames from 'classnames';
import { useTranslation } from 'react-i18next';
import { MaximizeIcon } from '../../components/icon/icon';
import { getJSON } from '../../utils';
import { useSelector } from '../../store/store-hook';
import { RootState } from '../../store/root-reducer';
import isEqual from 'lodash-es/isEqual';
import 'loaders.css/src/animations/ball-beat.scss';
import style from './index.module.scss';
type PluginTypes = 'call_in' | 'call_out' | 'im' | 'customer_detail' | 'videocall_in';
type Props = {
    type: PluginTypes;
    currentUser: IUser;
    /**
     * 所属客户，必传。如果customer在父组件中是异步获取的，确保在外层组件判断customer。只有customer有值时渲染本组件。
     *
     * @type {Customer}
     */
    customer?: Customer;
    pluginId?: string;
    pluginHeight?: number;
    link?: string;
    platform?: string;
    postMsgEnabled?: boolean;
    params?: any;
    /**
     *  仅在im模块中使用
     */
    chat?: IChat;
    /**
     *  仅在im模块中使用
     *
     * @type {Field[]}
     */
    customerFields?: Field[];
    /**
     *  仅在im模块中使用
     *
     * @type {Field[]}
     */
    noteFields?: Field[];
    ticket?: any;
    event: any;
    onRequestLinkError?: (response: any) => void;
    onRequestLinkReady?: (src: string) => void;
    onSrcEncoded?: (src: string) => void;
};

const IMCustomPlugin: React.FC<Props> = (props) => {
    const { t } = useTranslation();
    const initData = useSelector((state: RootState) => state.initData);
    const [iframeRef] = useState(React.createRef<HTMLIFrameElement>());
    const [iframeEl, setIframeEl] = useState<HTMLIFrameElement>();
    const [isDestroyed, setIsDestroyed] = useState(false);
    const [error, setError] = useState('');
    useEffect(() => {
        const iframeDom = iframeRef.current as HTMLIFrameElement;
        setIframeEl(iframeDom);
        return function cleanup() {
            setIsDestroyed(true);
        };
    }, [iframeRef]);

    const [pluginHeight] = useState(getPluginHeight(props.type, props.pluginHeight as number));
    const [requestLink, setRequestLink] = useState<any>();
    const [src, setSrc] = useState<any>();
    // 拼接src
    useEffect(() => {
        let url = requestLink;
        if (!url) {
            setSrc('');
            return;
        }

        const customer = props.customer;
        const platform = props.platform;
        if (!customer) {
            setSrc('');
            return;
        }

        url += (url.indexOf('?') > -1 ? '&' : '?') + 'agent_email=' + props.currentUser.email;
        url += '&agent_phone_number=' + (props.currentUser.cellphone || '');
        const type = props.type;
        const ticket_id = props.ticket?.id;

        const params = props.params;

        if (isEqual(initData.page, 'mpv') && props.chat?.roomId) {
            url += '&call_id=' + props.chat?.roomId;
        }

        if (type) {
            url += '&type=' + type;
        }
        if (platform) {
            url += '&platform=' + platform;
        }
        if (ticket_id) {
            url += '&ticket_id=' + ticket_id;
        }

        const urlObj = getCustomerPlatformId(props);
        url += urlObj.url;

        if (props.params) {
            for (let i in params) {
                url += '&' + i + '=' + params[i];
            }
        }
        const encodeUrl = encodeURI(url);
        if (props.onSrcEncoded) {
            props.onSrcEncoded(encodeUrl);
        }

        setSrc(encodeUrl);
    }, [props.currentUser.cellphone, props.currentUser.email, props, props.customer, props.ticket, requestLink]);

    // 生成并缓存iframe load事件
    const iframeLoadEvent = useCallback(
        function (e) {
            console.log('iframe初始化完成');
            if (props.type === 'im' && props.postMsgEnabled) {
                if (isDestroyed) {
                    return;
                }
                const customer = props.customer;
                if (!customer) {
                    return;
                }
                const customerCustomFieldsValue = customer.custom_fields || {};
                const chat = props.chat;
                // 访问信息
                const accessInfo = chat?.accessInfo || {};

                // 客户信息
                const cellphones = customer.cellphones?.map((phone: { content: any }) => phone.content) || [];
                const organizations = chat?.organizations?.map((org: { name: any }) => org.name) || [];
                const sysFields = {
                    email: customer.email || '',
                    cellphones: cellphones || [],
                    nick_name: customer.nick_name || '',
                    organizations: organizations || [],
                    description: customer.description || '',
                    tags: customer.tags || '',
                };

                const customerIdObj = getCustomerPlatformId(props);
                const customerCustomFields: any[] = [];
                props.customerFields?.forEach((item: any) => {
                    customerCustomFields.push({
                        [item.name]: customerCustomFieldsValue[item.name] || '',
                        title: item.title || item.name || '',
                        content_type: item.content_type || '',
                    });
                });
                const customerInfo = {
                    ...customerIdObj.idObj,
                    sysFields: sysFields,
                    customFields: customerCustomFields,
                };
                // 业务记录
                const noteInfo: any = {};
                const noteFields: any[] = [];
                loadNote(chat, isDestroyed, setError).then(function (data: any) {
                    const noteCustomFields = data.custom_fields;
                    const subject = data.content || '';
                    noteInfo.subject = subject;
                    props.noteFields?.forEach((item: any) => {
                        noteFields.push({
                            [item.name]: noteCustomFields[item.name] || '',
                            title: item.title,
                            content_type: item.content_type,
                        });
                    });
                    noteInfo.noteFields = noteFields;
                    // 获取工号
                    loadAgentWorkId(props, isDestroyed, setError).then(function (agent = {}) {
                        // 客服信息
                        let agentInfo = {
                            name: props.currentUser.nick_name,
                            aliase: props.currentUser.aliase || '',
                            id: props.currentUser.id,
                            work_id: agent.work_id || '',
                            email: props.currentUser.email || '',
                            cellphone: props.currentUser.cellphone || '',
                        };

                        // 拼数据
                        const postData = {
                            agentInfo: agentInfo,
                            accessInfo: accessInfo,
                            customerInfo: customerInfo,
                            noteInfo: noteInfo,
                        };
                        // 发送postMessage
                        iframeRef.current?.contentWindow?.postMessage(
                            JSON.stringify(postData, (k, v) => {
                                if (v === null) {
                                    return '';
                                } else {
                                    return v;
                                }
                            }),
                            new URL(src).origin
                        );
                    });
                });
            }
        },
        [iframeRef, isDestroyed, props, src]
    );

    // 监听参数变化，重新获取requestLink
    useEffect(() => {
        if (props.customer?.id || props.ticket?.id) {
            const customerId = props.customer?.id;
            const ticketId = props.ticket?.id;
            const pluginId = props.pluginId;

            // 重置src
            setRequestLink('');

            getJSON(props.link as string, {
                customer_id: customerId,
                ticket_id: ticketId,
                id: pluginId,
            })
                .then(function (res) {
                    setError('');
                    if (!res || !res.url) {
                        if (props.onRequestLinkError) {
                            props.onRequestLinkError(res);
                        }
                        return;
                    }
                    if (isDestroyed) {
                        return;
                    }
                    setRequestLink(res.url);
                    if (props.onRequestLinkReady) {
                        props.onRequestLinkReady(src);
                    }
                })
                .catch((reason) => {
                    setError(reason.code_message);
                });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [props.pluginId, props.customer?.id, props.ticket?.id]);

    useEffect(() => {
        const chat = props.chat as IMineChat;
        let sendTime: any = '';
        const handleMsg = function (e: any) {
            let data: any = {};
            try {
                data = JSON.parse(e.data);
            } catch (ex) {
                console.log(ex);
            }
            let { type, method, content } = data;
            if (method !== 'sendMsg') return;

            if (chat.customer_driving_close) {
                return message.warning('会话已经被客户关闭,不能继续发送消息,请关闭会话后操作!');
            }

            if (sendTime && +new Date() - (sendTime || 0) < 1000) {
                sendTime = new Date().getTime();
                message.warning('请不要太快发送消息');
            } else {
                sendTime = new Date().getTime();
                props.event.sendMsg &&
                    props.event.sendMsg({
                        msg: {
                            type: data.type === 'text' ? 'message' : data.type,
                            data: {
                                content: data.content,
                            },
                            content: data.content,
                        },
                        chat: props.chat as IMineChat,
                    });
            }
        };

        window.addEventListener('message', handleMsg);
        return function () {
            window.removeEventListener('message', handleMsg);
        };
    }, []);
    return (
        <div className={style['udesk-component-custom-plugin']}>
            <a
                href={src}
                className="open-link"
                target="_blank"
                rel="noopener noreferrer"
                title={t('page.im.assist.openInNewWindow')}
            >
                {/* className="btn-plugin-extend"  */}
                <MaximizeIcon />
            </a>
            {error && <div className={style.error}>{error}</div>}
            {src && (
                <iframe
                    className={classNames(style['plugin-iframe'])}
                    style={{ height: pluginHeight }}
                    src={src}
                    title=" "
                    ref={iframeRef}
                    onLoad={iframeLoadEvent}
                ></iframe>
            )}
        </div>
    );
};

IMCustomPlugin.defaultProps = {
    pluginHeight: 380,
    link: '/user_defined_plugins/request_link',
};

function loadNote(chat: any, isDestroyed: boolean, setError: React.Dispatch<any>) {
    return new Promise((resolve) => {
        const noteCustomFields = chat.note_custom_field;
        if (noteCustomFields) {
            resolve(noteCustomFields);
        } else {
            getJSON('notes/detail', {
                type: 'im',
                ref_id: chat.im_sub_session_id,
            })
                .then(function (res) {
                    setError('');
                    if (isDestroyed) {
                        return;
                    }
                    resolve(res.data);
                })
                .catch((reason) => {
                    setError(reason.code_message);
                });
        }
    });
}

function loadAgentWorkId(props: Props, isDestroyed: boolean, setError: React.Dispatch<any>): Promise<any> {
    const id = props.currentUser.id;
    return new Promise((resolve) => {
        getJSON('agents/' + id)
            .then(function (res) {
                setError('');
                if (isDestroyed) {
                    return;
                }
                resolve(res.agent);
            })
            .catch((reason) => {
                setError(reason.code_message);
            });
    });
}

function getCustomerPlatformId(props: Props) {
    let customer = props.customer as Customer;
    let {
        weixin_openid,
        weixins,
        unionid,
        weibo_openid,
        weibos,
        open_api_token,
        id: customer_id,
        weixin_minis,
        sdk_token,
        web_token,
        customer_token,
    } = customer;
    let weixin_mini_openid;
    weixin_openid = (weixins && weixins.length > 0 && weixins[0].openid) || weixin_openid;
    unionid = (weixins && weixins.length > 0 && weixins[0].unionid) || unionid;
    weibo_openid = (weibos && weibos.length > 0 && weibos[0].openid) || weibo_openid;
    unionid = (weixins && weixins.length > 0 && weixins[0].unionid) || unionid;
    weixin_mini_openid = weixin_minis && weixin_minis.length > 0 && weixin_minis[0].openid;
    let url = '';
    let idObj: any = {};
    if (weixin_openid) {
        url += '&weixin_openid=' + weixin_openid;
        unionid && (url += '&unionid=' + unionid);
        idObj.weixin_open_id = weixin_openid;
    }
    if (weibo_openid) {
        url += '&weibo_openid=' + weibo_openid;
        idObj.weibo_id = weibo_openid;
    }
    if (open_api_token) {
        url += '&open_api_token=' + open_api_token;
        idObj.open_api_token = open_api_token;
    }
    if (customer_id) {
        url += '&customer_id=' + customer_id;
        idObj.id = customer_id;
    }
    if (weixin_mini_openid) {
        url += '&weixin_mini_openid=' + weixin_mini_openid;
        idObj.weixin_mini_openid = weixin_mini_openid;
    }
    if (sdk_token) {
        idObj.sdk_token = sdk_token;
    }
    if (web_token) {
        idObj.web_token = web_token;
    }
    if (customer_token) {
        idObj.customer_token = customer_token;
    }
    return {
        url: url,
        idObj: idObj,
    };
}

function getPluginHeight(type: PluginTypes, pluginHeight: number): string {
    if (pluginHeight < 150) {
        pluginHeight = 150;
    } else if (pluginHeight > 600) {
        pluginHeight = 600;
    }

    if (type === 'customer_detail' || type === 'im' || type === 'call_out' || type === 'call_in') {
        return '100%';
    } else {
        return `${pluginHeight}px`;
    }
}

export default IMCustomPlugin;
