import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { getJSON } from '../../utils';
import ImSubSession from './imSubSession';
import Ticket from './ticket';
import CcConversation from './ccConversation';
import style from './style.module.scss';
import { useTranslation } from 'react-i18next';
import { message, Select, Timeline } from 'udesk-ui';
import Loader from 'react-loaders';
import 'loaders.css/src/animations/ball-beat.scss';
import cloneDeep from 'lodash-es/cloneDeep';
import pick from 'lodash-es/pick';
import useDebounce from 'react-use/esm/useDebounce';
import { TICKET_STATUS } from '../../common/enums/ticket';

const { Option } = Select;

interface Props {
    createdTicket: any;
    roleCan: IRoleCan;
    chat: Pick<IMineChat, 'customer_id' | 'contact_agent_id'>;
    agents: IAgent[];
    event: any;
}

interface Count {
    All: number;
    Ticket: number;
    UnsolveTicket: number;
    CcConversation: number;
    ImSubSession: number;
    Video: number;
}

interface FeedsItem {
    agent: string;
    duration: number;
    id: number;
    result: string;
    source: string;
    time: string;
    type: string;
}

const ImContact: React.FC<Props> = ({ createdTicket, ...props }) => {
    const { roleCan, chat, agents = [], event } = props;
    const { customer_id: customerId } = chat;
    const { t } = useTranslation();
    const _agents = agents.map((agent: IAgent) => {
        return {
            id: agent.id + '',
            nick_name: agent.nick_name,
        };
    });
    const allAgents = [{ id: 'all', nick_name: t('page.im.imContact.allAgent') }].concat(_agents);
    const [page, setPage] = useState(1);
    const [, setMeta] = useState<any>({});
    const [imContactAgentId, setImContactAgentId] = useState<any>(chat.contact_agent_id || 'all');
    const [feeds, setFeeds] = useState<any[]>([]);
    const [loading, setLoading] = useState(true);
    const [loaded, setLoaded] = useState(true);
    const [selectFilterKey, setSelectFilterKey] = useState('all');
    const [loadMore, setLoadMore] = useState(false);
    const [hasMore, setHasMore] = useState(false);
    const [allCount, setAllCount] = useState<Count>({
        All: 0,
        Ticket: 0,
        UnsolveTicket: 0,
        CcConversation: 0,
        ImSubSession: 0,
        Video: 0,
    });
    const scrollRef = useRef<HTMLDivElement | null>(null);

    useEffect(() => {
        if (createdTicket) {
            setFeeds((v) => [
                {
                    ...pick(createdTicket, [
                        'id',
                        'priority_id',
                        'priority',
                        'status_id',
                        'status',
                        'subject',
                        'content',
                        'created_at',
                    ]),
                    type: 'Ticket',
                    agent: pick(createdTicket, [
                        'assignee_id',
                        'assignee_name',
                        'agent_group_id',
                        'agent_group_name',
                    ]),
                },
                ...v,
            ]);
            setAllCount((v) => {
                const isCreatedTicketResolved = createdTicket.status_id === TICKET_STATUS.CLOSED;
                return {
                    ...v,
                    Ticket: v.Ticket + 1,
                    UnsolveTicket: isCreatedTicketResolved ? v.UnsolveTicket : v.UnsolveTicket + 1,
                };
            });
        }
    }, [createdTicket]);

    const params = useMemo(() => {
        return {
            agent_id: imContactAgentId === 'all' ? '' : imContactAgentId,
            feed_type: selectFilterKey === 'all' ? '' : selectFilterKey,
            page: page,
            customer_id: customerId,
        };
    }, [customerId, imContactAgentId, page, selectFilterKey]);

    useEffect(() => {
        const customer_id = params.customer_id;
        const agent_id = params.agent_id;
        if (!customer_id) return;

        let sizeParams: any = {
            customer_id: customer_id,
        };

        if (agent_id) {
            sizeParams.agent_id = agent_id;
        }

        getJSON('/im/customer_feeds_size', sizeParams)
            .then((res: any) => {
                if (res && res.code === 1000) {
                    setAllCount({
                        All: res.All,
                        Ticket: res.Ticket,
                        UnsolveTicket: res.UnsolveTicket,
                        CcConversation: res.CcConversation,
                        ImSubSession: res.ImSubSession,
                        Video: res.VideoRecord || 0,
                    });
                }
            })
            .catch((err) => {
                console.log(err);
            });
    }, [params.agent_id, params.customer_id]);

    useDebounce(
        () => {
            if (!params.customer_id) return;

            let _params: any = cloneDeep(params);

            if (!_params.agent_id) {
                delete _params.agent_id;
            }

            if (!_params.customer_id) {
                delete _params.customer_id;
            }

            let _isAdd = _params.page > 1;

            getJSON('/im/customer_feeds', _params)
                .then((res: any = {}) => {
                    if (res.code === 1000) {
                        res.feeds.forEach((item: FeedsItem) => (item.type =item.type?item.type: _params.feed_type));
                        setMeta({
                            ...res.meta,
                        });
                        if (_isAdd) {
                            setFeeds((prevFeeds) => {
                                return [...prevFeeds, ...res.feeds];
                            });
                        } else {
                            setFeeds(res.feeds);
                        }

                        setLoading(false);
                        setLoaded(true);
                        setLoadMore(false);
                        setHasMore(res.meta.total_pages > res.meta.current_page);
                    }
                })
                .catch(() => {
                    message.error('加载失败');
                });
        },
        500,
        [params]
    );

    const handleChangeAgent = useCallback(
        (value: string) => {
            if (value !== imContactAgentId) {
                if (scrollRef.current) {
                    scrollRef.current.scrollTop = 0;
                }
                setLoading(true);
                setFeeds([]);
                setPage(1);
                setImContactAgentId(value);

                event?.updateChat({
                    chat: chat,
                    key: 'contact_agent_id',
                    value: value,
                });
            }
        },
        [chat, imContactAgentId, event?.updateChat]
    );

    const handleSwitchType = useCallback(
        (type: string) => {
            if (type !== selectFilterKey) {
                if (scrollRef.current) {
                    scrollRef.current.scrollTop = 0;
                }
                setLoading(true);
                setFeeds([]);
                setPage(1);
                setSelectFilterKey(type);
            }
        },
        [selectFilterKey]
    );

    const showTotal = useCallback(
        (type: string): string => {
            let text: string = '';
            switch (type) {
                case 'All':
                    text = (allCount.All || 0) + '';
                    break;
                case 'ImSubSession':
                    text = allCount.ImSubSession + '';
                    break;
                case 'CcConversation':
                    text = allCount.CcConversation + '';
                    break;
                case 'Video':
                    text = allCount.Video + '';
                    break;
                case 'Ticket':
                    text = allCount.UnsolveTicket + '/' + allCount.Ticket + '';
                    break;
            }
            return `(${text})`;
        },
        [
            allCount.All,
            allCount.CcConversation,
            allCount.ImSubSession,
            allCount.Ticket,
            allCount.UnsolveTicket,
            allCount.Video,
        ]
    );

    const handleScroll = useCallback(() => {
        if (scrollRef.current && hasMore) {
            let $El = scrollRef.current;
            let allHeight = $El.scrollHeight;
            let scrollTop = $El.scrollTop;
            let contentHeight = $El.clientHeight || $El.offsetHeight;

            // 滚动到底部时, 且没有在加载数据时
            if (scrollTop + contentHeight + 50 >= allHeight && loaded) {
                // 判断是否有更多页
                setLoadMore(true);
                setLoaded(false);
                setPage((prevPage) => {
                    return prevPage + 1;
                });
            }
        }
    }, [hasMore, loaded]);

    return (
        <div
            className={`${style['im-contact']} ${
                roleCan.only_show_self_customer_feed ? style.noSearch : ''
            }`}
        >
            {roleCan.only_show_self_customer_feed ? null : (
                <div className={style.searchDiv}>
                    <label>{t('agent')}</label>
                    <div className={style.searchMenu}>
                        <Select
                            showSearch
                            size={'small'}
                            style={{ width: 120 }}
                            placeholder='请选择'
                            value={imContactAgentId}
                            optionFilterProp='children'
                            onChange={handleChangeAgent}
                            dropdownClassName={style['drop-box']}
                            maxTagCount={10}
                            filterOption={(input, option: any) =>
                                option.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                            }
                        >
                            {allAgents.map((agent) => {
                                return (
                                    <Option value={agent.id} key={agent.id}>
                                        {agent.nick_name}
                                    </Option>
                                );
                            })}
                        </Select>
                    </div>
                </div>
            )}
            <div className={style['feed-filter-box']}>
                <div>
                    <span
                        className={`${style['filter-span-text']} ${
                            selectFilterKey === 'all' ? style['active'] : ''
                        }`}
                        onClick={() => handleSwitchType('all')}
                    >
                        {t('label.all')}
                        {showTotal('All')}
                    </span>
                    <span className={style['filter-span-separator']}>|</span>
                </div>
                <div>
                    <span
                        className={`${style['filter-span-text']} ${
                            selectFilterKey === 'Ticket' ? style['active'] : ''
                        }`}
                        onClick={() => handleSwitchType('Ticket')}
                    >
                        {t('ticket')}
                        {showTotal('Ticket')}
                    </span>
                    <span className={style['filter-span-separator']}>|</span>
                </div>
                <div>
                    <span
                        className={`${style['filter-span-text']} ${
                            selectFilterKey === 'CcConversation' ? style['active'] : ''
                        }`}
                        onClick={() => handleSwitchType('CcConversation')}
                    >
                        {t('call')}
                        {showTotal('CcConversation')}
                    </span>
                    <span className={style['filter-span-separator']}>|</span>
                </div>
                <div>
                    <span
                        className={`${style['filter-span-text']} ${
                            selectFilterKey === 'ImSubSession' ? style['active'] : ''
                        }`}
                        onClick={() => handleSwitchType('ImSubSession')}
                    >
                        {t('im')}
                        {showTotal('ImSubSession')}
                    </span>
                    <span className={style['filter-span-separator']}>|</span>
                </div>
                {/* 等视频有了之后在放开 */}
                <div>
                    <span
                        className={`${style['filter-span-text']} ${
                            selectFilterKey === 'Video' ? style['active'] : ''
                        }`}
                        onClick={() => handleSwitchType('Video')}
                    >
                        视频{showTotal('Video')}
                    </span>
                    <span className={style['filter-span-separator']}>|</span>
                </div>
            </div>
            {loading ? (
                <div className={style.pl20}>
                    <Loader type='ball-beat' active />
                </div>
            ) : (
                <div
                    className={`${style['scroll-contact']} scrollbar`}
                    ref={scrollRef}
                    onScroll={handleScroll}
                >
                    {feeds.length > 0 ? (
                        <Timeline>
                            {feeds.map((item, index) => {
                                let Element;
                                switch (item.type) {
                                    case 'ImSubSession':
                                        Element = ImSubSession;
                                        break;
                                    case 'Ticket':
                                        Element = Ticket;
                                        break;
                                    case 'CcConversation':
                                        Element = CcConversation;
                                        break;
                                    case 'Video':
                                        Element = CcConversation;
                                        break;
                                    default:
                                        Element = CcConversation;
                                }
                                return (
                                    <Timeline.Item
                                        dot={<span className={style['timeline-dot']}></span>}
                                        key={item.id}
                                        className={`${style['contact-timeline']}`}
                                    >
                                        <div
                                            className={`${style['contact-record']} ${
                                                style['contact-' + item.type]
                                            }`}
                                            key={item.index}
                                        >
                                            {Element && (
                                                <Element feed={item} {...props} key={item.id} />
                                            )}
                                        </div>
                                    </Timeline.Item>
                                );
                            })}
                        </Timeline>
                    ) : (
                        <div>{t('page.im.imContact.noFeed')}</div>
                    )}
                    {loadMore && <div className={style['text-center']}>loading...</div>}
                </div>
            )}
        </div>
    );
};

export default ImContact;
