import React, {ComponentProps, useCallback, useMemo, useRef} from 'react';
import style from '../index.module.scss';
import { Button, Form, message } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import isEqual from 'lodash-es/isEqual';
import * as FieldComponent from '../../ticket/components/index';
import { Store } from 'antd/lib/form/interface';
import { formatCustomField, getFieldsDefaultValue, postJSON } from '../../../utils/index';
import { csApiPostJSON } from '../../../utils/csApi';
import CustomField from '../../../components/custom-field-container';
import Qs from 'qs';
import { useSelector } from '../../../store/store-hook';
import { RootState } from '../../../store/root-reducer';

interface CreateOrgFormdata {
    name: string;
    level: string;
    description: string;
    org_email_suffix: string;
    domains: string;

    [propName: string]: any;
}

interface SubmitParams {
    organization: {
        name: string;
        description: string;
        domains: string;
        level: string;
        custom_fields: { [index: string]: any };
    };
}

function CreateOrg(props: any) {
    const formRef: any = useRef(null);
    const { t } = useTranslation();
    const organizationFieldsV2 = useSelector((state: RootState) => state.fields.organizationFieldsv2);
    const initData = useSelector((state: RootState) => state.initData);
    const _organizationFields = useMemo(() => {
        return organizationFieldsV2
            .map((i: any) => ({
                ...i,
                is_required: i.agent_permission === 1,
            }))
            .filter((item: any) => {
                if (isEqual(initData.page, 'im')) {
                    return item.agent_permission && item.im_show;
                } else if (isEqual(initData.page, 'mpv')) {
                    return item.agent_permission && item.mpv_show;
                } else {
                    return item.agent_permission && item.cc_show;
                }
            });
    }, [organizationFieldsV2]);
    const orgFields = formatCustomField({
        fields: _organizationFields,
    });

    // 提交时触发
    const handleSubmit:ComponentProps<typeof Form>['onFinish'] = useCallback(
        (values) => {
            const custom_fields: any = {};
            const sys_fields: any = {};
            for (const k in values) {
                if (values[k]) {
                    if (k.indexOf('TextField') > -1 || k.indexOf('SelectField') > -1) {
                        custom_fields[k] = values[k];
                    } else {
                        sys_fields[k] = values[k];
                    }
                }
            }

            const params = {
                organization: {
                    ...sys_fields,
                    custom_fields: {
                        ...custom_fields,
                    },
                },
            };

            // 开始提交请求
            csApiPostJSON('/cs/ts/api/organizations', params, {
                transformRequest: [
                        function (data) {
                            return Qs.stringify(data, { arrayFormat: 'brackets' });
                        },
                    ],
            }).then(res=>res.data)
                .then((res) => {
                    // 创建成功后将创建的用户更新   给新增工单页面回显用
                    if (res.code === 1000) {
                        props.onSave({
                            label: res.organization.name,
                            value: res.organization.id,
                        });
                    } else {
                        message.error(t('tip.error.saveTry'));
                    }
                })
                .catch((err) => {
                    message.error(t('tip.error.saveTry'));
                });
        },
        [props, t]
    );

    // 表单数据改变触发  开发时方便查看  提测注释掉
    const handleChangeValue = useCallback((changeValue: any, allValue: any) => {
        // console.log(allValue)
        // setFormData(allValue);
    }, []);

    const handleInitialValue = useMemo(() => {
        const initValue = getFieldsDefaultValue({
            fields: _organizationFields,
            valuse: {},
        });

        return Object.assign(
            {
                name: '',
                level: '',
                description: '',
                org_email_suffix: '',
                domains: '',
            },
            initValue
        );
    }, [_organizationFields]);
    return (
        <React.Fragment>
            <div className={style['ticket-form']}>
                <Form
                    ref={formRef}
                    initialValues={handleInitialValue}
                    onFinish={handleSubmit}
                    layout="vertical"
                    onValuesChange={handleChangeValue}
                >
                    {/* 公司名称 */}
                    <Form.Item
                        name="name"
                        label={t('page.im.customerInfo.organization.name')}
                        required={true}
                        rules={[
                            {
                                required: true,
                                message: t('page.im.customerInfo.organization.validateFail'),
                            },
                        ]}
                    >
                        <FieldComponent.text trigger="onChange" isRequired={true} id="name" type="text" />
                    </Form.Item>
                    {/* 等级 */}
                    <Form.Item name="level" label={t('page.im.customerInfo.customer.level')}>
                        <FieldComponent.droplist
                            id="level"
                            type="droplist"
                            trigger="onChange"
                            options={[
                                {
                                    value: 'normal',
                                    title: t('page.im.customerInfo.customer.enum.level.normal'),
                                },
                                {
                                    value: 'vip',
                                    title: t('page.im.customerInfo.customer.enum.level.vip'),
                                },
                            ]}
                        />
                    </Form.Item>
                    {/* '描述',*/}
                    <Form.Item name="description" label={t('page.im.customerInfo.customer.description')}>
                        <FieldComponent.textarea
                            trigger="onChange"
                            placeholder={t('page.im.ticket.descTip')}
                            id="description"
                            type="textarea"
                        />
                    </Form.Item>
                    {/* 公司邮箱后缀 */}
                    <Form.Item name="org_email_suffix" label={t('page.im.customerInfo.organization.org_email_suffix')}>
                        <FieldComponent.textHasPoint remark={t('page.im.ticket.emailSuffixHelpText')} />
                    </Form.Item>
                    {/* 公司域名 */}
                    <Form.Item name="domains" label={t('page.im.customerInfo.organization.domains')}>
                        <FieldComponent.textHasPoint remark={t('page.im.ticket.helpBlock')} />
                    </Form.Item>
                    {orgFields.length > 1 && (
                        <div className={style['org-custom-list']}>
                            <CustomField fields={orgFields} />
                        </div>
                    )}
                </Form>
            </div>
            <div style={{ textAlign: 'center', marginTop: '15px' }}>
                <Button style={{ marginRight: '10px' }} onClick={props.onCancel}>
                    {t('cancel')}
                </Button>
                <Button type="primary" onClick={() => formRef.current.submit()}>
                    {t('confirm')}
                </Button>
            </div>
        </React.Fragment>
    );
}

export default CreateOrg;
