import React, { useCallback, useEffect, useMemo, useState } from 'react';
import axios from 'axios';
import { buildHookRequestParamsForCustomer } from '../../../utils/field-hook';
import merge from 'lodash-es/merge';
import { Form, message } from 'udesk-ui';
import { formatCustomField, getFieldsDefaultValue, isSysField, putJSON } from '../../../utils/index';
import { csApiPutJSON } from '../../../utils/csApi';
import { Store } from 'udesk-ui/es/form';
import FromList from './component/form-list';
import PlatformList from './component/platform-type';
import Qs from 'qs';
import style from './customer-form.module.scss';
import { useTranslation } from 'react-i18next';
import isEqual from 'lodash-es/isEqual';
import cloneDeep from 'lodash-es/cloneDeep';
import { RootState } from '../../../store/root-reducer';
import { useSelector } from '../../../store/store-hook';
interface CustomerProps {
    customer: Customer;
    organizations: Organization[];
    customerFieldsv2?: any; //其实是数组
    organizationFields?: any;
    sessionKey?: string;
    imSubSessionId: number;
    customerId: number;
    chat: any;
    event: any;
    roleCan: any;
    company: Company;
    companyCategoryConfig: any;
    switchFlag?: boolean;
    currentUser: any;
    smsapp?: any;
    fieldHooks?: any;
    isAdmin?: any;
    user?: any;
    agents?: IAgent[];
    customerFieldsAuthority?: any;
}

interface IEmberProps {
    chat: any;
    updateProps: any;
}

function judgeValueChanged(changedValue: any, initialValue: any) {
    let key = Object.keys(changedValue)[0];
    if (isEqual(initialValue[key], changedValue[key])) {
        return false;
    }
    return true;
}

function getParam(changedValues: any) {
    let param = {};
    let cellResult: any = [];
    for (const changedKey in changedValues) {
        const _isSysField = isSysField(changedKey);
        if (_isSysField) {
            switch (changedKey) {
                case 'nick_name':
                case 'email':
                case 'description':
                    merge(param, {
                        customer: { [changedKey]: changedValues[changedKey] },
                    });
                    break;
                case 'cellphones':
                    cellResult = changedValues[changedKey]
                        .filter((item: any) => {
                            return item.id || item.content;
                        })
                        .map((cell: any) => {
                            if (cell.id) {
                                if (cell.content) {
                                    return cell;
                                } else {
                                    return Object.assign({}, cell, {
                                        content: cell.content,
                                        id: cell.id,
                                        _destroy: 1,
                                    });
                                }
                            } else {
                                return cell;
                            }
                        });

                    merge(param, {
                        customer: {
                            cellphones_attributes: cellResult,
                        },
                    });
                    break;
                case 'organizations':
                    merge(param, {
                        customer: {
                            organizations:
                                changedValues[changedKey] && changedValues[changedKey].length > 0
                                    ? changedValues[changedKey].map((item: any) => item.value).filter((t: any) => t)
                                    : '',
                        },
                    });
                    break;
                case 'tags':
                    merge(param, {
                        tags: changedValues[changedKey].map((item: any) => item.label).join(','),
                    });
                    break;
            }
        } else {
            merge(param, {
                customer: {
                    custom_fields: { [changedKey]: changedValues[changedKey] },
                },
            });
        }
    }

    return param;
}

const CustomerInfoForm: React.FC<CustomerProps> = ({
    currentUser,
    smsapp,
    chat = {},
    company,
    roleCan,
    customerFieldsv2,
    organizationFields,
    event,
    companyCategoryConfig,
    switchFlag,
    customerFieldsAuthority,
    fieldHooks,
    isAdmin,
    user,
    agents
}) => {
    const [form] = Form.useForm();
    const [initialValue, setInitialValue] = useState<any>({});
    const [lastValues, setLastValues] = useState<any>();
    const { t } = useTranslation();
    const initData = useSelector((state: RootState) => state.initData);
    const { customer, organizations, im_sub_session_id: imSubSessionId, session_key: sessionKey } = chat;
    let _customerFieldsv2: any = useMemo(() => {
        let customerFieldsv2_bk = cloneDeep(customerFieldsv2);
        if (customerFieldsv2_bk) {
            return customerFieldsv2_bk.filter((field: any) => {
                if (isEqual(initData.page, 'im')) {
                    return field.im_show;
                } else if (isEqual(initData.page, 'mpv')) {
                    return field.mpv_show;
                } else {
                    return field.cc_show;
                }
            });
        } else {
            return [];
        }
    }, [customerFieldsv2]);

    const customerFields = useMemo(
        () =>
            formatCustomField({
                fields: _customerFieldsv2,
                trigger: 'onBlur',
                fieldsAuthority: customerFieldsAuthority,
                fieldType: 'customer',
                editType: 'edit',
            }),
        [_customerFieldsv2, customerFieldsAuthority]
    );

    const getValues = useCallback(
        (_customer, organizations) => {
            let initValues =
                getFieldsDefaultValue({
                    fields: _customerFieldsv2,
                    values: _customer.custom_fields || {},
                }) || {};

            let _organizations = (organizations || []).map((org: any) => {
                return {
                    label: org.name,
                    value: org.id,
                };
            });
            initValues = Object.assign({}, initValues, _customer.custom_fields, {
                nick_name: _customer.nick_name,
                cellphones: _customer.cellphones || [{ content: '' }],
                email: _customer.email,
                organizations: _organizations,
                tags: _customer.rich_tags,
                description: _customer.description,
            });

            return initValues;
        },
        [_customerFieldsv2]
    );

    useEffect(() => {
        let values = getValues(customer, organizations);
        setInitialValue(values);
        form.setFieldsValue(values);
        setLastValues(values);
    }, [customer, form, getValues, organizations, sessionKey]);

    const availableFieldHooks: IFieldHook[] = useMemo(() => {
        if (fieldHooks) {
            return fieldHooks.filter(
                (i: IFieldHook) => i.active && i.trigger_type === 'edit' && i.field_owner === 'Customer'
            );
        } else {
            return [];
        }
    }, [fieldHooks]);

    const submitFn = useCallback(
        (changedValues) => {
            const param: any = getParam(changedValues);
            return csApiPutJSON(`/cs/ts/api/customers/${customer.id}`, param, {
                transformRequest: [
                    function (data) {
                        return Qs.stringify(data, { arrayFormat: 'brackets' });
                    },
                ],
            }).then(res=>res.data)
                .then((res: any) => {
                    if (res.code === 1000) {
                        let values = getValues(res.customer || {}, res.organizations);
                        if (values.cellphones && values.cellphones.length) {
                            values.cellphones = [
                                {
                                    content: '',
                                },
                            ];
                        }
                        setInitialValue(values);
                        message.success(t('tip.success.save'));
                        let custom_fields = Object.assign(res.customer.custom_fields, param?.customer?.custom_fields);
                        res.customer.custom_fields = custom_fields;
                        event.updateChat(chat, {
                            organizations: res.organizations,
                            customer: res.customer,
                            nick_name: res.customer.nick_name,
                            rich_tags: res.customer.rich_tags || [],
                        });
                    } else {
                        message.error(
                            res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry')
                        );
                    }
                })
                .catch((err) => {
                    console.error(err);
                });
        },
        [chat, customer.id, event, getValues, t]
    );

    const formSubmit = useCallback((changedValues: Store, values: Store) => {
        const changed = judgeValueChanged(changedValues, initialValue);
        if (!changed) {
            return;
        }
        const fieldHook = availableFieldHooks.find((i) => {
            let triggerField = i.trigger_field;
            triggerField === 'name' && (triggerField = 'nick_name');
            return triggerField && triggerField in changedValues;
        });
        if (fieldHook) {
            return axios
                .post(
                    fieldHook.url,
                    buildHookRequestParamsForCustomer(fieldHook, merge({}, customer, values), {
                        customerCustomFields: customerFieldsv2,
                        otherParams: { im_sub_session_id: imSubSessionId },
                    })
                )
                .then((res) => {
                    if (res.data.error) {
                        message.error(res.data.error);
                    }
                    if (res.data.code === 1000) {
                        // form.setFieldsValue(res.data.customer);
                        return form
                            .validateFields(Object.keys(res.data.customer))
                            .then(() => {
                                submitFn(res.data.customer).finally(() => setLastValues(values));
                            })
                            .catch(() => {
                                form.setFieldsValue(lastValues);
                            });
                    } else if (!res.data.error) {
                        message.error('请求数据错误，请重试!');
                        throw new Error('请求数据错误，请重试!');
                    } else {
                        throw new Error(res.data.error);
                    }
                });
        } else {
            submitFn(changedValues).finally(() => setLastValues(values));
        }
    }, []);

    const onValueChange = (changedValues: Store, values: Store) => {
        form.validateFields(Object.keys(changedValues))
            .then(() => {
                formSubmit(changedValues, values);
            })
            .catch((err: Store) => {
                console.log(err);
                let key = Object.keys(changedValues)[0];
                let canSubmit = true;
                err.errorFields.forEach((item: any) => {
                    if (item.name[0] === key) {
                        canSubmit = false;
                    }
                });
                if (canSubmit) {
                    formSubmit(changedValues, values);
                }
            });
    };

    return (
        <div className={style['customer-form']} style={{ marginTop: '10px' }}>
            {/* 公司信息form表单 系统字段 */}
            <Form layout="vertical" name="customer-info-form" onValuesChange={onValueChange} form={form}>
                <PlatformList customer={customer} />
                <FromList
                    switchFlag={switchFlag}
                    customerFields={customerFields}
                    organizationFields={organizationFields}
                    company={company}
                    customer={customer}
                    imSubSessionId={imSubSessionId}
                    event={event}
                    chat={chat}
                    companyCategoryConfig={companyCategoryConfig}
                    organizations={organizations}
                    roleCan={roleCan}
                    currentUser={currentUser}
                    smsapp={smsapp}
                    isAdmin={isAdmin}
                    user={user}
                    agents={agents}
                />
            </Form>
        </div>
    );
};
export default CustomerInfoForm;
