import React, { ElementRef, useCallback, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import { Tabs } from 'udesk-ui';
import classnames from 'classnames';
import { useTranslation } from 'react-i18next';
import unionBy from 'lodash-es/unionBy';
import isFunction from 'lodash-es/isFunction';
import includes from 'lodash-es/includes';
import {
    IM_CONTACT_TAB_NAME,
    IM_CUSTOMER_TAB_NAME,
    IM_HISTORY_MESSAGE_TAB_NAME,
    IM_NOTE_TAB_NAME,
    IM_REPOSITORY_TAB_NAME,
    IM_RETURN_VISIT_TAB_NAME,
} from '../utils';
import { csApiGetJSON } from '../utils/csApi';
import {
    ImContact,
    ImCustomer,
    ImCustomPlugin,
    ImKnowledge,
    ImMsgRecord,
    ImNote,
    ImTicket,
    ImVisit,
} from './component';
import TabSettingModal from './parallel/tab-setting-modal/tab-setting-modal';
import TabParallel from './parallel/tab-parallel';
import { Setting } from '../components/icon/icon';
import ErrorBoundary from '../components/simple-error-boundary';
import { RootState } from '../store/root-reducer';
import { useSelector } from '../store/store-hook';
import style from './index.module.scss';
import isEqual from 'lodash-es/isEqual';
import concat from 'lodash-es/concat';

const { TabPane } = Tabs;

function getTabKey(tabName: string, customer_id: number) {
    return `${tabName}-${customer_id}`;
}

const ImPanel: React.FC<any> = React.forwardRef(({ width, onWidthChange, cssMaxWidth, ...props }, ref) => {
    const { t } = useTranslation();
    const { chat = {}, agentConfig, currentUser, noteFields, event } = props;
    const [tabPlugins, setTabPlugins] = useState<any[]>([]);
    const { customer_id, plugTab = {} } = chat;
    const containerRef = useRef<ElementRef<'div'>>(null);
    const initData = useSelector((state: RootState) => state.initData);
    const agentTabConfig = useMemo(() => {
        return agentConfig?.agent_tab_config || {};
    }, [agentConfig?.agent_tab_config]);
    const minWidth = useMemo(() => {
        if (agentTabConfig.parallel) {
            return 600;
        } else {
            return 400;
        }
    }, [agentTabConfig.parallel]);

    // const tabsWidth = 420;
    const [currentTab, setCurrentTab] = useState<Tab>();
    const [showModal, setShowModal] = useState(false);
    const [showTicket, setShowTicket] = useState<any>({
        show: false,
        ticketCover: '',
        zIndex: 0,
    });
    const [showVisit, setShowVisit] = useState<any>({
        show: false,
        visitCover: '',
        zIndex: 0,
    });
    const [showRecord, setShowRecord] = useState<any>({
        show: false,
        recordCover: '',
        zIndex: 0,
    });
    const [lastCreatedTicket, setLastCreatedTicket] = useState<any>();

    const tabsHash = useMemo(() => {
        if (tabPlugins && tabPlugins.length > 0) {
            const _parallel = agentTabConfig.parallel;
            let order = agentTabConfig.order;
            let updatePlugins: any[] = [];
            let parallelPlugins: any = {};
            const list: any = [];
            if (order) {
                if (isEqual(initData.page, 'mpv')) {
                    order = concat(
                        order,
                        (initData?.pluginFilter || [])
                            .filter((item) => !order.includes(`${item}`))
                            .map((ite) => `${ite}`)
                    );
                }
                order.forEach((index: string) => {
                    const plugin = tabPlugins.find((plugin: any) => {
                        return plugin.name === index;
                    });

                    if (_parallel) {
                        if (_parallel === index) {
                            parallelPlugins = {
                                ...plugin,
                            };
                        } else {
                            updatePlugins.push({
                                ...plugin,
                            });
                        }
                    } else {
                        updatePlugins.push({
                            ...plugin,
                        });
                    }
                    list.push({
                        ...plugin,
                        checked: _parallel === index,
                    });
                });
                updatePlugins = unionBy(updatePlugins, tabPlugins, 'name');
                return {
                    updatePlugins,
                    parallelPlugins,
                    list,
                };
            } else {
                tabPlugins.forEach((plugin: any) => {
                    if (_parallel) {
                        if (_parallel === plugin.name) {
                            parallelPlugins = {
                                ...plugin,
                            };
                        } else {
                            updatePlugins.push({
                                ...plugin,
                            });
                        }
                    } else {
                        updatePlugins.push({
                            ...plugin,
                        });
                    }

                    list.push({
                        ...plugin,
                        checked: _parallel === plugin.name,
                    });
                });
                return {
                    updatePlugins,
                    parallelPlugins,
                    list,
                };
            }
        } else {
            return {
                updatePlugins: [],
                parallelPlugins: {},
                list: [],
            };
        }
    }, [tabPlugins, agentTabConfig, initData.pluginFilter]);

    const tabs: Tab[] = useMemo(() => tabsHash['updatePlugins'], [tabsHash]);
    const parallel = useMemo(() => tabsHash['parallelPlugins'], [tabsHash]);
    const settingList = useMemo(() => tabsHash['list'], [tabsHash]);

    const handleChange = useCallback(
        (activeKey: string) => {
            let plugTab = tabs.find((tab) => getTabKey(tab.name, customer_id) === activeKey);
            if (plugTab) {
                setCurrentTab(plugTab);
            }
            if (isFunction(event.updateChat)) {
                event.updateChat({
                    chat: chat,
                    updateProps: {
                        plugTab,
                    },
                });
            }
        },
        [chat, customer_id, event, tabs]
    );

    useImperativeHandle(
        ref,
        () => ({
            showTicket: function (config: any) {
                const ticketCover = agentTabConfig.ticketCover;
                const _parallel = agentTabConfig.parallel;
                setShowTicket({
                    show: config.show,
                    ticketCover: _parallel ? ticketCover : 'all',
                    zIndex: config.zIndex++,
                });
            },
            showVisit: function (config: any) {
                const visitCover = agentTabConfig.visitCover;
                const _parallel = agentTabConfig.parallel;
                setShowVisit({
                    show: config.show,
                    visitCover: _parallel ? visitCover : 'all',
                    zIndex: config.zIndex++,
                });
            },
            showRecord: function (config: any) {
                const recordCover = agentTabConfig.recordCover;
                const _parallel = agentTabConfig.parallel;
                setShowRecord({
                    show: config.show,
                    recordCover: _parallel ? recordCover : 'all',
                    zIndex: config.zIndex++,
                });
            },
            changeImPanelTab: function (id: string, customerId: number) {
                handleChange(`${id}-${customerId}`);
            },
        }),
        [
            agentTabConfig.ticketCover,
            agentTabConfig.parallel,
            agentTabConfig.visitCover,
            agentTabConfig.recordCover,
            handleChange,
        ]
    );

    // 获取tabs数据
    // useEffect(() => {
    //   const plugins = (initData.tabPlugins || [])
    //     .filter((plugin: any) => plugin.enabled)
    //     .map((i: any) => {
    //       return {
    //         name: i.id + "",
    //         text: i.id < 1 ? t("page.im.tabs.tagName." + i.id) : i.name,
    //         active: i.is_default,
    //         postMsgEnabled: i.post_message_enabled,
    //       };
    //     });
    //   setTabPlugins(plugins);
    // }, [t]);

    useEffect(() => {
        csApiGetJSON(`/cs/ts/api/user_defined_plugins?show_page=${initData.page}`)
            .then(res=>res.data)
            .then((res: any = {}) => {
                if (res.code === 1000) {
                    const plugins = res.plugins
                        .filter(
                            (plugin: any) =>
                                plugin.enabled &&
                                (isEqual(initData.page, 'mpv') ? includes(initData?.pluginFilter, plugin.id) : true)
                        )
                        .map((i: any) => {
                            return {
                                name: i.id + '',
                                text: i.id < 1 ? t('page.im.tabs.tagName.' + i.id) : i.name,
                                active: i.is_default,
                                postMsgEnabled: i.post_message_enabled,
                            };
                        });
                    setTabPlugins(plugins);
                }
            })
            .catch((e: any) => {
                console.log(e);
            });
    }, [t, initData.pluginFilter]);
    useEffect(() => {
        if(currentTab?.text=='客户信息' && isFunction(event.reloadModel)){
            event.reloadModel();
        }
        
    }, [currentTab]);

    const tabsWidth = useMemo(() => {
        const panelWidth = window.localStorage.getItem('stretchable-box-right-panel-width') || '370';
        let _panelWidth = parseInt(panelWidth, 10);

        if (agentTabConfig.parallel) {
            if (_panelWidth < minWidth) {
                _panelWidth = minWidth;
                window.localStorage.setItem('stretchable-box-right-panel-width', minWidth + '');
            }
        } else {
            if (_panelWidth > minWidth) {
                _panelWidth = minWidth;
                window.localStorage.setItem('stretchable-box-right-panel-width', minWidth + '');
            }
        }
        return _panelWidth;
    }, [agentTabConfig.parallel]);

    useEffect(() => {
        if (tabs && tabs.length > 0) {
            let activeTab: Tab | null | undefined = null;
            const normalTabs = tabs.filter((t) => t.name !== parallel.name);
            const pluginName = plugTab?.name;

            // 客户身上的配置
            if (pluginName) {
                activeTab = normalTabs.find((t) => t.name === pluginName);
            }
            // 系统配置
            if (activeTab == null) {
                activeTab = normalTabs.find((t) => t.active);
            }
            // 第一个
            if (activeTab == null) {
                activeTab = normalTabs[0];
            }
            if (activeTab) {
                setCurrentTab(activeTab);
            }
        }
    }, [parallel.name, tabs, tabsWidth, plugTab.name]);

    const getComponent = useCallback(
        (key: string) => {
            if (key) {
                const plugin = tabPlugins.find((p) => p.name === key);
                switch (key) {
                    case IM_CUSTOMER_TAB_NAME:
                        return (
                            <ImCustomer
                                customer={props.customer}
                                currentUser={props.currentUser}
                                event={props.event}
                                organizations={props.organization}
                                chat={props.chat}
                                company={props.company}
                                imSubSessionId={props.imSubSessionId}
                                customerId={props.customerId}
                                companyCategoryConfig={props.companyCategoryConfig}
                                roleCan={props.roleCan}
                                imNewPlatforms={props.imNewPlatforms}
                                organizationFieldsv2={props.organizationFieldsv2}
                                showCount={props.showCount}
                                customerFieldsAuthority={props.customerFieldsAuthority}
                                customerFieldsv2={props.customerFieldsv2}
                                fieldHooks={props.fieldHooks}
                                isAdmin={props.isAdmin}
                                organizationFields={props.organizationFields}
                                sessionKey={props.sessionKey}
                                smsapp={props.smsapp}
                                switchFlag={props.switchFlag}
                                user={props.user}
                                agents={props.agents}
                            />
                        );
                    case IM_NOTE_TAB_NAME:
                        return (
                            <ImNote
                                chat={props.chat}
                                company={props.company}
                                noteTemplates={props.noteTemplates}
                                event={props.event}
                            />
                        );
                    case IM_CONTACT_TAB_NAME:
                        return (
                            <ImContact
                                createdTicket={lastCreatedTicket}
                                chat={props.chat}
                                roleCan={props.roleCan}
                                event={props.event}
                                agents={props.agents}
                            />
                        );
                    case IM_REPOSITORY_TAB_NAME:
                        return (
                            <ImKnowledge
                                event={props.event}
                                roleCan={props.roleCan}
                                chat={props.chat}
                                company={props.company}
                                user={props.user}
                                kmrobot_url={props.kmrobot_url}
                                previewUrl={props.previewUrl}
                            />
                        );
                    case IM_HISTORY_MESSAGE_TAB_NAME:
                        return (
                            <ImMsgRecord chat={props.chat} event={props.event} setShowRecord={props.setShowRecord} />
                        );
                    case IM_RETURN_VISIT_TAB_NAME:
                        return (
                            <ImVisit
                                content=""
                                showCancel={false}
                                event={event}
                                chat={chat}
                                setShowVisit={setShowVisit}
                            />
                        );
                    default: {
                        return (
                            <ImCustomPlugin
                                type={isEqual(initData.page, 'mpv') ? 'videocall_in' : 'im'}
                                currentUser={currentUser}
                                customer={chat.customer}
                                pluginId={plugin?.name}
                                postMsgEnabled={plugin?.postMsgEnabled}
                                platform={chat.platform}
                                chat={chat}
                                noteFields={noteFields}
                                event={event}
                            />
                        );
                    }
                }
            } else {
                return '';
            }
        },
        [
            tabPlugins,
            props.customer,
            props.currentUser,
            props.event,
            props.organization,
            props.chat,
            props.company,
            props.imSubSessionId,
            props.customerId,
            props.companyCategoryConfig,
            props.roleCan,
            props.imNewPlatforms,
            props.organizationFieldsv2,
            props.showCount,
            props.customerFieldsAuthority,
            props.customerFieldsv2,
            props.fieldHooks,
            props.isAdmin,
            props.organizationFields,
            props.sessionKey,
            props.smsapp,
            props.switchFlag,
            props.user,
            props.noteTemplates,
            props.agents,
            props.kmrobot_url,
            props.previewUrl,
            props.setShowRecord,
            lastCreatedTicket,
            event,
            chat,
            currentUser,
            noteFields,
        ]
    );

    const parallelComponent = useMemo(() => getComponent(parallel.name), [getComponent, parallel.name]);

    const handlerShowTabSettingModel = useCallback(() => setShowModal(true), []);
    const cancelTabs = useCallback(() => setShowModal(false), []);

    const onTicketCreateSuccess = useCallback((ticket: ITicket) => {
        setLastCreatedTicket(ticket);
    }, []);

    const getShowNewTab = useCallback(
        (coverType: string) => {
            return (
                <React.Fragment>
                    {showTicket.ticketCover === coverType && showTicket.show && (
                        <div className={style['new-tab']} style={{ zIndex: showTicket.zIndex }}>
                            <ErrorBoundary>
                                <ImTicket
                                    setShowTicket={setShowTicket}
                                    onCreateSuccess={onTicketCreateSuccess}
                                    {...props}
                                />
                            </ErrorBoundary>
                        </div>
                    )}
                    {showVisit.visitCover === coverType && showVisit.show && (
                        <div className={style['new-tab']} style={{ zIndex: showVisit.zIndex }}>
                            <ErrorBoundary>
                                <ImVisit
                                    content=""
                                    showCancel={false}
                                    chat={chat}
                                    event={event}
                                    setShowVisit={setShowVisit}
                                />
                            </ErrorBoundary>
                        </div>
                    )}
                    {showRecord.recordCover === coverType && showRecord.show && (
                        <div className={style['new-tab']} style={{ zIndex: showRecord.zIndex }}>
                            <ErrorBoundary>
                                <ImMsgRecord setShowRecord={setShowRecord} {...props} />
                            </ErrorBoundary>
                        </div>
                    )}
                </React.Fragment>
            );
        },
        [
            chat,
            event,
            onTicketCreateSuccess,
            props,
            showRecord.recordCover,
            showRecord.show,
            showRecord.zIndex,
            showTicket.show,
            showTicket.ticketCover,
            showTicket.zIndex,
            showVisit.show,
            showVisit.visitCover,
            showVisit.zIndex,
        ]
    );

    const handleSwitchParallel = useCallback(() => console.log('x'), []);
    const handleHideParallel = useCallback(() => console.log('x'), []);
    const [_width, _setWidth] = useState<number>();

    useEffect(() => {
        let startX = 0;
        let isMove = false;
        const maskElement = document.createElement('div');
        let containerWidth = 0;
        maskElement.style.cssText =
            'position:fixed;top:0;left:0;right:0;bottom:0;z-index:99999;background:transparent;cursor:col-resize';

        const fc = {
            mousedown: function (e: any) {
                startX = e.clientX;
                isMove = true;
                document.body.appendChild(maskElement);
                containerWidth = containerRef?.current?.offsetWidth ?? 0;
            },
            mousemove: (e: MouseEvent) => {
                if (isMove) {
                    _setWidth(containerWidth + (startX - e.clientX));
                    e.preventDefault();
                }
            },
            mouseup: function (e: any) {
                if (isMove) {
                    isMove = false;
                }
                maskElement.remove();
            },
        };
        document.getElementById('col-drag-line')?.addEventListener('mousedown', fc.mousedown);
        document.addEventListener('mousemove', fc.mousemove);
        document.addEventListener('mouseup', fc.mouseup);
        return function () {
            document.getElementById('col-drag-line')?.removeEventListener('mousedown', fc.mousedown);
            document.removeEventListener('mousemove', fc.mousemove);
            document.removeEventListener('mouseup', fc.mouseup);
        };
    }, []);

    useEffect(() => {
        if (_width) {
            return onWidthChange(Math.max(minWidth, _width));
        }
    }, [_width, minWidth, onWidthChange]);

    const containerStyle = useMemo(() => ({ width: width + 'px', maxWidth: cssMaxWidth }), [cssMaxWidth, width]);
    const tabBarExtraContent = useMemo(() => {
        if (!isEqual(initData.page, 'mpv')) {
            return (
                <div className={style['tab-extra']} onClick={handlerShowTabSettingModel}>
                    <Setting />
                </div>
            );
        }
    }, []);
    const tabChildren = useMemo(
        () =>
            tabs.map((tab) => {
                let component = getComponent(tab.name);
                return (
                    <TabPane tab={tab.text} key={getTabKey(tab.name, customer_id)}>
                        <ErrorBoundary>{component}</ErrorBoundary>
                    </TabPane>
                );
            }),
        [customer_id, getComponent, tabs]
    );
    return (
        <div className={style['im-tab-panel']} style={containerStyle} ref={containerRef}>
            {/* 左右拖拽区域 */}
            <div className={style['drag-line']} id="col-drag-line" />
            <div
                className={classnames(style['panel-container'], {
                    [style['has-parallel']]: !!agentTabConfig.parallel,
                    [style['view-reverse']]: !!agentTabConfig.sortDesc,
                })}
            >
                {/* 左侧tabs区域 */}
                {chat.belong === 'leave' && (
                    <div style={{ padding: '20px 0 ', width: '100%' }}>
                        <ImCustomer {...props} />
                    </div>
                )}
                {chat.belong !== 'leave' && tabs.length > 0 && (
                    <div className={style['panle-main']}>
                        <Tabs
                            activeKey={getTabKey(currentTab?.name || '', customer_id)}
                            tabBarGutter={0}
                            tabBarExtraContent={tabBarExtraContent}
                            onChange={handleChange}
                            className={style.tabs}
                            destroyInactiveTabPane={true}
                            animated={false}
                        >
                            {tabChildren}
                        </Tabs>
                        <TabSettingModal
                            tabs={settingList}
                            agentTabConfig={agentTabConfig}
                            showModal={showModal}
                            hideTabSetingModal={cancelTabs}
                            event={event}
                        />
                        {/* 覆盖平行视界普通区域 */}
                        {getShowNewTab('parallelNormal')}
                    </div>
                )}

                {/* 中间分割线 */}
                {!isEqual(initData.page, 'mpv') && agentTabConfig.parallel && <div className={style['panel-line']} />}
                {/* 右侧平行视界区域 <如果有> */}
                {!isEqual(initData.page, 'mpv') && agentTabConfig.parallel && (
                    <div className={style['panel-parallel']}>
                        <TabParallel
                            {...parallel}
                            component={parallelComponent}
                            switchParallel={handleSwitchParallel}
                            hideParallel={handleHideParallel}
                            agentTabConfig={agentTabConfig}
                            event={event}
                        />
                        {/* 覆盖平行视界 */}
                        {getShowNewTab('parallel')}
                    </div>
                )}
                {/* 覆盖整个信息区 */}
                {getShowNewTab('all')}
            </div>
        </div>
    );
});

const ImPanelMemo = React.memo(ImPanel);
export default ImPanelMemo;
