import React, { useCallback, useEffect, useState } from 'react';
import { Button, Col, Form, message, Modal, Radio, Row } from 'udesk-ui';
import { csApiPostJSON } from '../../../utils/csApi';
import { DndProvider } from 'react-dnd';
import { HTML5Backend } from 'react-dnd-html5-backend';
import style from './style.module.scss';
import update from 'immutability-helper';
import { DndContainer } from './dnd-container';
import { RadioChangeEvent } from 'udesk-ui/es/radio';
import isFunction from 'lodash-es/isFunction';
import cloneDeep from 'lodash-es/cloneDeep';
import { useTranslation } from 'react-i18next';
import { RootState } from '../../../store/root-reducer';
import { useSelector } from '../../../store/store-hook';
import { isEqual } from 'lodash-es';
interface TabSettingModalProps {
    tabs: any;
    agentTabConfig: AgentTabConfig;
    showModal: boolean;
    hideTabSetingModal: () => void;
    event: any;
}

const TabSettingModal: React.FC<TabSettingModalProps> = ({
    tabs,
    agentTabConfig = {},
    showModal,
    hideTabSetingModal,
    event,
}) => {
    const { t } = useTranslation();
    const type: string = 'TabSettingDrap';
    const [loading, setLoading] = useState(false);
    const [cover, setCover] = useState<any>({
        ticketCover: agentTabConfig.ticketCover || 'all',
        visitCover: agentTabConfig.visitCover || 'all',
        recordCover: agentTabConfig.ticketCover || 'all',
    });
    const [list, setList] = useState<any>(() => {
        return (tabs || [])
            .filter((item: any) => item.name)
            .map((tab: Tab) => {
                return {
                    id: tab.name,
                    key: tab.name,
                    text: tab.text,
                    checked: tab.checked,
                };
            });
    });
    const initData = useSelector((state: RootState) => state.initData);

    useEffect(() => {
        setList(resetList());
    }, [showModal, tabs]);

    const resetList: () => any = useCallback(() => {
        return (tabs || [])
            .filter((item: any) => item.name)
            .map((tab: Tab) => {
                return {
                    id: tab.name,
                    key: tab.name,
                    text: tab.text,
                    checked: tab.checked,
                };
            });
    }, [tabs]);

    // 拖拽
    const moveItem = (id: string, atIndex: number) => {
        const { item, index } = findItem(id);
        setList(
            update(list, {
                $splice: [
                    [index, 1],
                    [atIndex, 0, item],
                ],
            })
        );
    };
    // 查找对应的tab
    const findItem = (id: string) => {
        const item = list.filter((c: any) => `${c.id}` === id)[0];
        return {
            item,
            index: list.indexOf(item),
        };
    };
    // 选中平行视界
    const checkedItem = (id: string) => {
        setList((prevList: any) => {
            let _prevList = cloneDeep(prevList);
            _prevList.map((item: any) => {
                if (item.id === id) {
                    item.checked = !item.checked;
                } else {
                    item.checked = false;
                }
                return item;
            });
            return _prevList;
            // return produce(prevList, (draft: any) => {
            //     draft.map((item: any) => {
            //         if (item.id === id) {
            //             item.checked = !item.checked
            //         } else {
            //             item.checked = false
            //         }
            //         return item;
            //     })
            // })
        });
    };

    // 保存
    const handleSaveModalSetting = () => {
        setLoading(true);
        const _parallel = (list.find((item: any) => item.checked) || {}).id || '';
        const _order = list.map((item: any) => item.id);
        const _data: any = {
            parallel: _parallel,
            order: _order,
            sortDesc: agentTabConfig.sortDesc,
            ...cover,
        };
        csApiPostJSON(`/cs/im/agent/im/agent_config`, {
            data: {
                agent_tab_config: _data,
            },
        }).then(res=>res.data)
            .then(() => {
                if (isFunction(event.updateAgentConfig)) {
                    event.updateAgentConfig({
                        ...agentTabConfig,
                        parallel: _parallel,
                        order: _order,
                        sortDesc: agentTabConfig.sortDesc,
                        ...cover,
                    });
                }
            })
            .catch((err) => {
                message.error('保存失败,请重试');
            })
            .finally(() => {
                setLoading(false);
                hideTabSetingModal();
            });
    };

    // 取消
    const handleCancel = () => {
        const baseList = resetList();
        setList(baseList);
        setCover({
            ticketCover: agentTabConfig.ticketCover,
            visitCover: agentTabConfig.visitCover,
            recordCover: agentTabConfig.recordCover,
        });
        hideTabSetingModal();
    };

    const headerStyle: React.CSSProperties = {
        padding: '8px 12px',
        borderBottom: '1px solid #f0f0f0',
        backgroundColor: '#fafafa',
    };

    const formItemLayout = {
        labelCol: {
            span: 4,
        },
        wrapperCol: {
            span: 20,
        },
    };

    const handleShowType = (e: RadioChangeEvent) => {
        const { name, value } = e.target;
        // 调用agent_config接口更新数据
        setCover({
            ticketCover: name === 'ticket' ? value : cover.ticketCover,
            visitCover: name === 'visit' ? value : cover.visitCover,
            recordCover: name === 'record' ? value : cover.recordCover,
        });
    };

    return (
        <Modal
            visible={showModal}
            title={t('label.preferences')}
            closable={false}
            destroyOnClose={true}
            onOk={handleSaveModalSetting}
            onCancel={handleCancel}
            wrapClassName={style['tab-setting-modal']}
            footer={[
                <Button key="back" onClick={handleCancel}>
                    {t('cancel')}
                </Button>,
                <Button key="submit" type="primary" loading={loading} onClick={handleSaveModalSetting}>
                    {t('confirm')}
                </Button>,
            ]}
        >
            <div style={{ maxHeight: '350px', overflowY: 'auto' }}>
                <DndProvider backend={HTML5Backend}>
                    <div style={headerStyle}>
                        <Row>
                            <Col span={4}>{t('label.order')}</Col>
                            <Col span={12}>{t('label.component')}</Col>
                            <Col span={4}>{t('label.parallelHorizon')}</Col>
                        </Row>
                    </div>
                    <DndContainer
                        list={list}
                        type={type}
                        moveItem={moveItem}
                        findItem={findItem}
                        checkedItem={checkedItem}
                    />
                </DndProvider>
                <br style={{ marginBottom: '15px' }} />
                {isEqual(initData.page, 'im') && (
                    <div>
                        <Form name="validate_other" {...formItemLayout}>
                            <Form.Item label={t('page.im.ticket.createTicket')}>
                                <Radio.Group name="ticket" value={cover.ticketCover} onChange={handleShowType}>
                                    <Radio value={'all'}>{t('label.overallCoverage')}</Radio>
                                    <Radio value={'parallel'}>{t('label.coveringParallelHorizon')}</Radio>
                                    <Radio value={'parallelNormal'}>{t('label.coverCommonArea')}</Radio>
                                </Radio.Group>
                            </Form.Item>

                            <Form.Item label={t('page.im.visit.title')}>
                                <Radio.Group name="visit" value={cover.visitCover} onChange={handleShowType}>
                                    <Radio value={'all'}>{t('label.overallCoverage')}</Radio>
                                    <Radio value={'parallel'}>{t('label.coveringParallelHorizon')}</Radio>
                                    <Radio value={'parallelNormal'}>{t('label.coverCommonArea')}</Radio>
                                </Radio.Group>
                            </Form.Item>

                            <Form.Item label={t('page.im.record.imLogRecords')}>
                                <Radio.Group name="record" value={cover.recordCover} onChange={handleShowType}>
                                    <Radio value={'all'}>{t('label.overallCoverage')}</Radio>
                                    <Radio value={'parallel'}>{t('label.coveringParallelHorizon')}</Radio>
                                    <Radio value={'parallelNormal'}>{t('label.coverCommonArea')}</Radio>
                                </Radio.Group>
                            </Form.Item>
                        </Form>
                    </div>
                )}
            </div>
        </Modal>
    );
};

export default TabSettingModal;
