import React, { Component } from 'react';
import filter from 'lodash-es/filter';
import forEach from 'lodash-es/forEach';
import isEmpty from 'lodash-es/isEmpty';
import map from 'lodash-es/map';
import PropTypes from 'prop-types';

const labelStyle = {
    display: 'inline-block',
    padding: '0 15px',
    width: '165px',
    whiteSpace: 'nowrap',
    textOverflow: 'ellipsis',
    overflow: 'hidden'
};

/**
 * @property {array<object>} columns - 将要显示的内容
 * @property {number} cols - 内容显示为几列
 */
export default class ColumnControl extends Component {
    constructor(props) {
        super(props);
        this.state = {
            keyword: ''
        };
    }

    /**
     * 恢复默认
     * @returns {null|*}
     */
    renderRecoverDefault() {
        const { showRecoverDefault, classNameOfRecoverDefaultWrapper, styleOfRecoverDefaultWrapper, onRecoverDefaultClick, t } = this.props;
        if (showRecoverDefault) {
            return <div className={classNameOfRecoverDefaultWrapper} style={styleOfRecoverDefaultWrapper}>
                <a href='javascript:' className='link' onClick={onRecoverDefaultClick}>
                    {t('label.recover_default')}
                </a>
            </div>;
        }
        return null;
    };

    /**
     * 搜索框
     * @returns {null|*}
     */
    renderSearch() {
        const { keyword } = this.state;
        const {
            showSearch,
            classNameOfSearchInputWrapper,
            styleOfSearchInputWrapper,
            classNameOfSearchInput,
            styleOfSearchInput,
            searchPlaceholder,
            classNameOfSearchInputIcon,
            styleOfSearchInputIcon
        } = this.props;
        if (showSearch) {
            return <div className={classNameOfSearchInputWrapper} style={styleOfSearchInputWrapper}>
                <input className={classNameOfSearchInput}
                       style={styleOfSearchInput}
                       value={keyword}
                       onChange={(e) => this.setState({ keyword: e.target.value })}
                       placeholder={searchPlaceholder}
                />
                <span className={classNameOfSearchInputIcon} style={styleOfSearchInputIcon}/>
            </div>;
        }
        return null;
    };

    render() {
        let categories = this.determineCategories();
        let {
            columns, cols = 1, style, className, t = v => v,
            classNameOfColumnsWrapper,
            styleOfColumnsWrapper
        } = this.props;

        if (!style) {
            style = {};
        }

        if (!style.boxSizing) {
            style.boxSizing = 'content-box';
        }
        if (!style.width) {
            style.width = `${cols * 165}px`;
        }

        return <div style={style} className={className}>
            {this.renderSearch()}
            {this.renderRecoverDefault()}
            <div className={classNameOfColumnsWrapper} style={styleOfColumnsWrapper}>
                {
                    isEmpty(categories) ?
                    this.renderColumns(columns, cols) :
                    map(categories, category => {
                        return <div key={category}>
                            <h5>{t(category)}</h5>
                            {this.renderColumns(filter(columns, ['category', category]), cols)}
                        </div>;
                    })
                }
            </div>
        </div>;
    }

    /**
     * @param {array} columns
     * @param {number} cols
     */
    renderColumns(columns, cols) {
        let { t = (v) => v } = this.props;
        let { keyword } = this.state;
        return <div>
            {
                map(columns, (item) => {
                    let title;
                    if (!item.name) {
                        title = t(item.id);
                    } else if (typeof item.name === 'string') {
                        title = item.name;
                    }

                    if (keyword && title.indexOf(keyword) === -1) {
                        return null;
                    }

                    return <label key={item.id} style={labelStyle} title={title}>
                        <input type="checkbox" checked={item.display} style={{ marginRight: '5px' }}
                               onChange={(e) => this._onChange(item, e)}/>
                        {item.name || t(item.id)}
                    </label>;
                })
            }
        </div>;
    }

    determineCategories() {
        let categories = [];
        forEach(this.props.columns, item => {
            if (item.category && categories.indexOf(item.category) === -1) {
                categories.push(item.category);
            }
        });
        return categories;
    }

    _onChange(column, e) {
        this.props.onChange(column, e.target.checked);
    }
}

//@formatter:off
ColumnControl.propTypes = {
    /**
     * 哪些列将受到控制
     * category属性请使用国际化key值
     * @example
     * [
     *  {id:'name', display:true, category:'label.person'},
     *  {id:'age',  display:true, category:'label.person'}
     * ]
     */
    columns                             : PropTypes.arrayOf(
                                              PropTypes.shape({
                                                  id       : PropTypes.string.isRequired,
                                                  display  : PropTypes.bool,
                                                  category : PropTypes.string,
                                                  name     : PropTypes.oneOfType([PropTypes.string,PropTypes.node])
                                              })
                                          ).isRequired,
    //显示为几列
    cols                                : PropTypes.number,
    style                               : PropTypes.object,
    className                           : PropTypes.string,
    onChange                            : PropTypes.func,
    t                                   : PropTypes.func,
    showSearch                          : PropTypes.bool,
    classNameOfSearchInput              : PropTypes.string,
    classNameOfSearchInputWrapper       : PropTypes.string,
    classNameOfSearchInputIcon          : PropTypes.string,
    classNameOfRecoverDefaultWrapper    : PropTypes.string,
    classNameOfColumnsWrapper           : PropTypes.string,
    showRecoverDefault                  : PropTypes.bool,
    onRecoverDefaultClick               : PropTypes.func,
    styleOfSearchInput                  : PropTypes.object,
    styleOfSearchInputWrapper           : PropTypes.object,
    styleOfSearchInputIcon              : PropTypes.object,
    styleOfRecoverDefaultWrapper        : PropTypes.object,
    styleOfColumnsWrapper               : PropTypes.object,
    searchPlaceholder                   : PropTypes.string
};
//@formatter:on

ColumnControl.defaultProps = {
    classNameOfSearchInput: 'form-control input-sm',
    classNameOfSearchInputWrapper: 'form-group has-feedback',
    classNameOfSearchInputIcon: 'icon icon-search form-control-feedback text-muted',
    classNameOfRecoverDefaultWrapper: 'recover-default-wrapper',
    classNameOfColumnsWrapper: 'columns-wrapper',
    className: '',
    styleOfSearchInput: {},
    styleOfSearchInputWrapper: {},
    styleOfSearchInputIcon: {},
    styleOfRecoverDefaultWrapper: { padding: '0 15px 10px' },
    styleOfColumnsWrapper: { height: '270px', overflowY: 'auto' },
    searchPlaceholder: null
};
