import { dirname, join } from 'path';
import { readFileSync } from 'fs';
import { IApi, utils, webpack } from 'umi';
import { ConfigProviderProps } from 'antd/es/config-provider';

const { Mustache } = utils;

interface IAntdOpts {
    dark?: boolean;
    compact?: boolean;
    config?: ConfigProviderProps;
    globalCss?: boolean;
}

export default (api: IApi) => {
    api.describe({
        config: {
            schema(joi) {
                return joi.object({
                    dark: joi.boolean(),
                    compact: joi.boolean(),
                    config: joi.object(),
                    globalCss: joi.boolean(),
                });
            },
        },
    });
    api.addRuntimePluginKey(() => ['udeskUI1']);
    api.skipPlugins(['@umijs/plguin-antd']);

    const opts: IAntdOpts = api.userConfig.udeskUI || {};

    api.modifyBabelPresetOpts(opts => {
        return {
            ...opts,
            import: (opts.import || []).concat([
                { libraryName: 'udesk-ui', libraryDirectory: 'es', style: false },
                // { libraryName: 'antd-mobile', libraryDirectory: 'es', style: true },
            ]),
        };
    });

    api.modifyBundleConfig((value, { env, type, bundler }) => {
        if (!value.resolve) {
            value.resolve = {};
        }
        if (!value.resolve.alias) {
            value.resolve.alias = {};
        }
        // 添加 antd 和 @ant-design/icons 的alias，强制引用udesk-ui下面的版本
        try {
            value.resolve.alias['antd'] = dirname(
                require.resolve('udesk-ui/node_modules/antd/package.json')
            );
        } catch (err) {
            value.resolve.alias['antd'] = dirname(require.resolve('antd/package.json'));
        }
        try {
            value.resolve.alias['@ant-design/icons'] = dirname(
                require.resolve('udesk-ui/node_modules/@ant-design/icons/package.json')
            );
        } catch (err) {
            value.resolve.alias['@ant-design/icons'] = dirname(
                require.resolve('@ant-design/icons/package.json')
            );
        }

        if (opts.globalCss) {
            if (!value.plugins) {
                // 添加 NormalModuleReplacementPlugin，去掉全局样式
                value.plugins = [];
            }
            value.plugins.push(
                new webpack.NormalModuleReplacementPlugin(
                    /\/style\/index.less$/,
                    (opt: { rawRequest: any; context: string; request: string }) => {
                        if (!opt.rawRequest && /antd\/es\/(\w|-)*\/?style/.test(opt.context)) {
                            const request = opt.request.split('!');
                            if (request.length > 0) {
                                const last = request[request.length - 1];
                                if (/antd\/es\/(\w|-)+\/style/.test(last)) {
                                    return;
                                }
                            }
                            request.pop();
                            const newPath = 'udesk-ui/es/style/reset.less';
                            request.push(newPath);
                            opt.request = request.join('!');
                        }
                    }
                )
            );
        }
        return value;
    });

    if (opts?.dark || opts?.compact) {
        // support dark mode, user use antd 4 by default
        const { getThemeVariables } = require('antd/dist/theme');
        api.modifyDefaultConfig(config => {
            config.theme = {
                ...getThemeVariables(opts),
                ...config.theme,
            };
            return config;
        });
    }

    if (opts?.config) {
        api.onGenerateFiles({
            fn() {
                // runtime.tsx
                const runtimeTpl = readFileSync(join(__dirname, 'runtime.tpl'), 'utf-8');
                api.writeTmpFile({
                    path: 'umi-plugin-udesk-ui/runtime.tsx',
                    content: Mustache.render(runtimeTpl, {
                        config: JSON.stringify(opts?.config),
                    }),
                });
            },
        });
        // Runtime Plugin
        api.addRuntimePlugin(() => [
            join(api.paths.absTmpPath!, 'umi-plugin-udesk-ui/runtime.tsx'),
        ]);
    }
};
