import React, {cloneElement, ReactElement, ReactNode, useCallback, useMemo, useRef} from 'react';
import TreeSelect, {TreeSelectProps} from "antd/es/tree-select";
import Tag from 'antd/es/tag';
import "antd/es/tree-select/style/index.less";
import "antd/es/select/style/index.less";
import 'antd/es/tag/style/index.less';
import 'antd/es/empty/style/index.less';
import get from 'lodash-es/get';

interface SelectOptions {
    value: string,
    title: string,
    default?: boolean,
    subs?: SelectOptions[],
    node_type?: 'branch' | 'leaf',
    icon_url?: string,
    color?: string
}

type ObjectPropertyPath = string | string[];

type Options = {
    nodeChildrenPath: ObjectPropertyPath, nodeValuePath: ObjectPropertyPath
}

declare type ContentType =
    "link"
    | "radio"
    | "datetime"
    | "text"
    | "date"
    | "time"
    | "checkbox"
    | "radio_droplist"
    | "checkbox_droplist"
    | "numeric"
    | "calculate"
    | "chained_droplist"
    | "area_text"
    | "number"
    | "tag_tree"
    | "location"

declare type Field = {
    select_options?: SelectOptions[]
    parent_checkable?: boolean
    input_tip?: string
    default_expand_all?: boolean;
}

// 自定义组件接受的参数
declare type FieldProps = {
    value?: string | string[],//formitem 时  当前组件对应值
    onChange?: (value: string | string[]) => void,//formitem 时  更新form值用
    formData: Field,
    warningText?: string,//警告信息
    disabled?: boolean | (() => boolean),//disabled
    changeErrMessage: (value: string) => void,//更新警告信息
    size?: 'small'|'middle'|'large',
    prefixCls?: TreeSelectProps<any>['prefixCls']
}

export function forEachTree<T>(treeData: T[] = [], callback: (node: T) => void, options: Pick<Options, 'nodeChildrenPath'>) {
    const {nodeChildrenPath = 'subs'} = options || {};
    treeData?.forEach(i => {
        callback(i);
        forEachTree(get(i, nodeChildrenPath), callback, options);
    });
}

function buildTreeData(selectOptions: SelectOptions[] | undefined | null, parentCheckable: boolean | undefined, parentKey?: string): TreeSelectProps<any>['treeData'] {
    if (!selectOptions) {
        return [];
    }
    return selectOptions.map(i => {
        let title: ReactNode = i.title;
        let isLeaf: boolean = false;
        if (i.node_type === 'leaf') {
            isLeaf = true;
        } else if (!i.node_type && (!i.subs || i.subs.length === 0)) {
            isLeaf = true;
        }
        if (isLeaf) {
            title = <Tag color={i.color || '#00B38B'}>
                {i.icon_url ?
                    <img src={i.icon_url} alt='' style={{width: "1em", height: "1em", marginRight: '3px'}}/> : null}
                {i.title}
            </Tag>
        }
        return {
            title: title,
            value: i.value,
            checkable: parentCheckable ? true : isLeaf,
            disableCheckbox: parentCheckable ? false : !isLeaf,
            children: buildTreeData(i.subs, parentCheckable),
            titleText: i.title
        }
    });
}

const tagRender: TreeSelectProps<String[]>['tagRender'] = (props) => {
    const {label, closable, onClose} = props;

    return cloneElement(label as ReactElement, {closable, onClose})
}

const TagTree: React.FC<FieldProps> = ({value, formData, onChange, size, disabled}) => {
    // const treeNodes = useMemo(() => renderTreeNode(formData.select_options, formData.parent_checkable), [formData.parent_checkable, formData.select_options]);
    const treeData = useMemo(() => buildTreeData(formData.select_options, formData.parent_checkable), [formData.parent_checkable, formData.select_options]);
    const ref = useRef<HTMLDivElement>(null);
    const getPopupContainer = useCallback(() => ref.current || document.body, []);
    const disableStatus = typeof disabled === 'function' ? disabled() : disabled
    const valueAry = useMemo(() => {
        if (typeof value === 'string') {
            return value.split(',');
        } else if (value) {
            return value;
        } else {
            return [];
        }
    }, [value]);
    const safeValueAry = useMemo(() => { //options 可以随意删除，这会导致 value 的内容与 option 匹配不上，需要将匹配不上的 value 剔除
        if (valueAry.length === 0) {
            return [];
        }
        const result: string[] = [];
        forEachTree<SelectOptions>(formData.select_options, (i) => valueAry?.includes(i.value as string) && result.push(i.value as string), {nodeChildrenPath: 'subs'});
        return result;
    }, [formData.select_options, valueAry]);

    return (
        <div ref={ref} style={{position: 'relative'}}>
            <TreeSelect<string[]> showSearch
                                     style={{width: '100%'}}
                                     value={safeValueAry}
                                     dropdownStyle={{maxHeight: 400, overflow: 'auto'}}
                                     placeholder={formData.input_tip}
                                     allowClear
                                     treeCheckable={true}
                                     onChange={onChange}
                                     getPopupContainer={getPopupContainer}
                                     transitionName=''
                                     treeDefaultExpandAll={formData.default_expand_all}
                                     multiple={true}
                                     tagRender={tagRender}
                                     treeData={treeData}
                                     size={size}
                                     disabled={disableStatus}
                                     treeNodeFilterProp='titleText'
            />
        </div>
    );
}

export default TagTree;
