/** @jsx jsx */
import {jsx} from '@emotion/react';
import {Component, ComponentProps, createRef, CSSProperties, ElementRef} from 'react';
import {Manager, Popper, Reference} from 'react-popper';
import Control from './control';
import assign from 'lodash-es/assign';
import {popper} from './style';
import './control.css';

export type RenderFunction = () => React.ReactNode;

type Props =
    {
        className: string
        columnButtonClassName: string
        columnButtonStyle: CSSProperties
        buttonComponent?: RenderFunction
        onClose: () => void
        onOpen: () => void
    }
    & Pick<ComponentProps<typeof Control>, 'columns' | 't' | 'classNameOfSearchInput' | 'setoDefaultValue'
    | 'value'
    | 'leftTitle'
    | 'rightTitle'
    | 'onChange'
    | 'onReset'
    | 'allowReset' | 'allowSort' | 'allowAutoSelectNext' | 'allowSetoDefault' | 'CancelButton' | 'ResetButton' | 'ConfirmButton'>

interface State {
    isOpen: boolean
}

const CancelButton = ({onClick, t}) => {
    return <button className='btn btn-default' onClick={onClick}>{t('label.cancel')}</button>;
};

const ResetButton = ({onClick, t}) => {
    return <button className='btn btn-default' onClick={onClick}>{t('label.restoreDefault')}</button>;
};

const ConfirmButton = ({onClick, t}) => {
    return <button className='btn btn-primary' onClick={onClick}>{t('label.confirm')}</button>;
};

class ColumnControlPopover extends Component<Props, State> {
    private _ref = createRef<ElementRef<'div'>>();
    private _doNotClosePopover = false;

    constructor(props) {
        super(props);
        this.state = {isOpen: false};
    }

    render() {
        let managerClassNames = ['dropdown'];
        if (this.state.isOpen) {
            managerClassNames.push('open');
        }
        let {
            columns, t, className = '',
            classNameOfSearchInput,
            leftTitle,
            rightTitle,
            value,
            setoDefaultValue,
            allowReset,
            allowSort,
            allowAutoSelectNext,
            allowSetoDefault,
            columnButtonClassName,
            columnButtonStyle,
            buttonComponent,

            CancelButton,
            ResetButton,
            ConfirmButton
        } = this.props;

        const defaultBtnStyle =  {};  

        return (
            <div className={managerClassNames.join(' ') + ' ' + className} ref={this._ref}>
                <Manager>
                    <Reference>
                        {({ref}) => {
                            //@formatter:off
                            if(buttonComponent) {
                                return <div onClick= {() => this._toggle()} ref = {ref} className='table-column-control-btn'>{getRenderPropValue(buttonComponent)}</div>
                            } else {
                                return <button type="button"
                                        className={`${columnButtonClassName} table-column-control-btn`}
                                        style={Object.assign(defaultBtnStyle, columnButtonStyle)}
                                        ref = {ref}
                                        onClick= {() => this._toggle()}>
                                    {t('label.column')} <i className="icon icon-caret-down"/>
                                </button>
                            }
                            //@formatter:on
                        }}
                    </Reference>
                    {this.state.isOpen && <Popper 
                        placement="bottom-end"
                        positionFixed={true}
                        modifiers={{
                            preventOverflow: {
                                boundariesElement: 'viewport',
                            }
                        }}>
                        {({ref, style, placement}) => (
                            <div ref={ref} style={assign({zIndex: 1000000}, style)}
                                 css={popper}
                                 data-placement={placement}>
                                {/*@formatter:off*/}
                                <Control
                                    columns                                  = {columns}
                                    onChange                                 = {this._onChange}
                                    onCancel                                 = {this._onCancel}
                                    onReset                                  = {this._onReset}
                                    t                                        = {t}
                                    classNameOfSearchInput                   = {classNameOfSearchInput}
                                    value                                    = {value}
                                    setoDefaultValue                         = {setoDefaultValue}
                                    leftTitle                                = {leftTitle}
                                    rightTitle                               = {rightTitle}
                                    className                                = {className}
                                    allowReset                               = {allowReset}
                                    allowSort                                = {allowSort}
                                    allowAutoSelectNext                      = {allowAutoSelectNext}
                                    allowSetoDefault                         = {allowSetoDefault}
                                    CancelButton                             = {CancelButton}
                                    ResetButton                              = {ResetButton}
                                    ConfirmButton                            = {ConfirmButton}
                                />
                                {/*@formatter:on*/}
                            </div>
                        )}
                    </Popper>}
                </Manager>
            </div>
        );
    }

    private _bodyEvent = () => {
        if (this._doNotClosePopover) {
            this._doNotClosePopover = false;
            return;
        }
        this._close();
    }

    componentDidMount() {
        if (this._ref.current) {
            this._ref.current.addEventListener('click', () => {
                this._doNotClosePopover = true;
            });
            document.addEventListener('click', this._bodyEvent);
        }
    }

    componentWillUnmount() {
        document.removeEventListener('click', this._bodyEvent);
    }

    _close() {
        if (this.state.isOpen) {
            let {onClose} = this.props;
            onClose && onClose();
            this.setState({isOpen: false});
        }
    }

    _open() {
        let {onOpen} = this.props;
        onOpen && onOpen();
        this.setState({isOpen: true});
    }

    _toggle() {
        if (this.state.isOpen) {
            this._close();
        } else {
            this._open();
        }
    }

    _onChange = (value) => {
        this._close();
        let {onChange} = this.props;
        if (typeof onChange === 'function') {
            onChange(value);
        }
    };

    _onReset = () => {
        let {onReset} = this.props;
        if (typeof onReset === 'function') {
            onReset();
        }
    };

    _onCancel = () => {
        this._close();
    };

    static defaultProps = {
        allowSort: true,
        allowReset: true,
        columnButtonClassName: 'btn btn-default',
        CancelButton: CancelButton,
        ResetButton: ResetButton,
        ConfirmButton: ConfirmButton
    };
}

export default ColumnControlPopover;

export const getRenderPropValue = (
  propValue?: React.ReactNode | RenderFunction,
): React.ReactNode => {
  if (!propValue) {
    return null;
  }

  if (typeof propValue === 'function') {
    return propValue();
  }

  return propValue;
};
