import React, {useCallback, useEffect, useImperativeHandle, useLayoutEffect, useRef, useState} from 'react';
import css from './index.module.less';
import {removeHTMLTag} from './utils/common';
import {uploadFile} from './utils/upload';
import UploadModal from './uploadModal/index';
import Translate from './translate/index';
import { I18nextProvider } from 'react-i18next';
import newI18n, { useTranslation} from './i18n';
import tinymce, {Editor} from 'tinymce';
import {message} from 'udesk-ui';

type UploadImgList = {
    src: string,
    isSelect: boolean
}

interface EditorProps {
    value?: string;
    maxLength?: number,
    autoresize_min_height?: number,
    statusbar?: boolean,
    toolbar?: string
    showImg?: boolean
    autoresize_max_height?: number
    paste_data_images?: boolean
    readonly?: boolean
    marking?: string
    backgroundColor?: string
    translate?: {onTranslate:Function, persistenceKey?:string}

    onChange?: (value: string) => void;
    postRender?: (e: any) => void;
}

const defaultStyle = {
    color: 'rgba(0, 0, 0, 0.25)'
};
const error = {
    color: 'rgba(251, 81, 39, 1)'
};

// 项目语言码转Tiny语言码
const Locale2TinyLangCodeMap = {
    'en-US': 'en_GB',
    'zh-CN': 'zh_CN',
    'zh-TW': 'zh_TW',
    'zh-HK': 'zh_HK',
    'ja-JP': 'ja', 
    'ar-EG': 'ar', 
    'ru-RU': 'ru', 
    'th-TH': 'th_TH', 
    'tr-TR': 'tr_TR', 
    'vi-VN': 'vi_VN',
    'de': 'de', 
    'fr': 'fr_FR',
    'pt-PT': 'pt_PT',
    'es-ES': 'es',
    'el': 'el',
    'it': 'it',
    'ko': 'ko_KR',
    'id': 'id',
};

const FormTinymce = React.forwardRef<{insertContent:(content:string)=>void}, EditorProps>(({paste_data_images, readonly, marking, ...props}, ref) => {
    const { showImg = true, translate = null, postRender } = props;
    const toolBars = `${showImg ? "upimg | " : ""} bold italic underline | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | formatselect fontsizeselect | link image |  fontselect  forecolor backcolor code ${translate ? " | translate" : ""}`;

    const {
        value,
        onChange,
        autoresize_min_height,
        autoresize_max_height = 50,
        statusbar = false,
        toolbar = toolBars,
        maxLength,
        backgroundColor,
    } = props;
    //多语言配置
    const {t, i18n} = useTranslation();
    const [editorContent, setContent] = useState<string>(removeHTMLTag(value || ''));//更新富文本内容
    const [showModal, setShowModal] = useState<boolean>(false);
    const [showOriginal , setShowOriginal] = useState<boolean>(false);
    const tinymceInstance = useRef<Editor | null>(null);
    const targetRef = useRef<HTMLTextAreaElement>(null);
    const hasManualChange = useRef<boolean>(false); // 是否手动输入了文本
    const idRef = useRef(`${Math.floor(Math.random() * 10000)}${new Date().getTime()}`);

    const { language } = i18n;

    // 内容改变时    去html标签 计算输入的长度
    const handleEditorChange = useCallback(() => {
        const content = tinymceInstance?.current?.getContent();
        setContent(removeHTMLTag(content || ''));
        onChange?.(content || '');
    }, [onChange]);

    useImperativeHandle(ref, () => ({
        insertContent: (content: string) => {
            if (!tinymceInstance) {
                return;
            }
            if (tinymceInstance?.current && tinymceInstance?.current?.selection?.getSel()) {
                tinymceInstance?.current.execCommand("mceInsertContent", false, content);
            } else {
                const curContent = tinymceInstance?.current?.getContent() || '';
                setContent(curContent + content);
            }
        }
    }));

    const hideImageUploadModal = useCallback(() => setShowModal(false), []);

    //点击弹窗的确定  将选择的图片插入并 关闭弹窗
    const addImg = useCallback((imgList: UploadImgList[]) => {
        if (!tinymceInstance) {
            return;
        }
        imgList.forEach(item => {
            tinymceInstance?.current?.execCommand('mceInsertContent', false, `<img src=${item.src} alt=""/>`);
        });
        setShowModal(false);
        hasManualChange.current = true;
    }, []);

    useLayoutEffect(() => {
        if (!targetRef.current) {
            return;
        }
        var tinymceLang = Locale2TinyLangCodeMap[language] || 'en_GB';
        var ed = new tinymce.Editor(idRef.current, {
            autoresize_min_height: autoresize_min_height,
            autoresize_max_height: autoresize_max_height,
            autoresize_bottom_margin: 5,
            menubar: false,
            language : tinymceLang,
            statusbar: statusbar,
            elementpath: false,//底部状态栏左侧不显示标签名
            branding: false,//底部状态栏右侧签名
            paste_data_images: paste_data_images,
            convert_urls: false,
            relative_urls: false,
            remove_script_host: false,
            document_base_url: '',
            extended_valid_elements: '-table[id|border=2|cellspacing|cellpadding|width=300|frame|rules|height=100|align|summary|bgcolor|background|bordercolor|style=border-color:black]',
            images_upload_handler: (blobInfo: any, success: any, failure: any) => {
                const curContent = tinymceInstance.current?.getContent() || '';
                const loadingImg = (location.origin || '') + "/entry/images/loading.svg?" + blobInfo.name();
                // const imgSrc = "data:image/png;base64," + blobInfo.base64();
                const newContent = curContent.replace(/data:image\/(png|gif|jpg|jpeg|tif|bmp);base64,[^"]*/, loadingImg);
                tinymceInstance.current?.setContent(newContent || '');
                message.info(t('uploader.uploadImgIng'));
                uploadFile(new File([blobInfo.blob()], blobInfo.filename()), {marking: 'web.mg.rich_editor_copy'}).then(resp => {
                    // success(resp.src);
                    let imgArr = tinymceInstance.current?.dom.select('img');
                    imgArr?.forEach((item: any) => {
                        if (item.currentSrc === loadingImg) {
                            tinymceInstance.current?.selection.select(item);
                            tinymceInstance.current?.dom.remove(item);
                            tinymceInstance.current?.insertContent(
                                `<img src="${resp.src}" data-mce-src="${resp.src}">`
                            );
                        }
                    });
                    hasManualChange.current = true;
                }).catch(e => {
                    failure(t('uploader.uploadImgFail'));
                });
            },
            plugins: [
                `link ${showImg ? "image" : ''} autoresize lists charmap preview hr `,
                'visualchars insertdatetime nonbreaking',
                'table contextmenu textcolor',
                'paste code imagetools'
            ],
            toolbar,
            // body_class: 'panel-body',
            content_style: backgroundColor ? `.mce-content-body{background-color: transparent}` : '',
            setup(instance: any) {
                tinymceInstance.current = instance;
                if (showImg) {
                    instance.addButton('upimg', {
                        text: t('uploader.uploadImg'),
                        onclick: function () {
                            setShowModal(true);
                        }
                    });
                }
                if (translate) {
                    instance.addButton('translate', {
                        tooltip: t('statusbar.translate'),
                        icon: 'translate',
                        onclick: function () {
                            setShowOriginal((v)=>!v);
                        }
                    });
                }
                instance.on('PostRender', (e)=>{
                    postRender?.(e);
                })
            }
        }, tinymce.EditorManager);

        ed.render();
        tinymceInstance.current = ed;
        return () => {
            ed?.destroy();
        };
    }, []);

    useEffect(() => {
        tinymceInstance.current?.setMode(readonly ? 'readonly' : 'design');
    }, [readonly]);

    useEffect(() => {
        if (tinymceInstance.current && handleEditorChange) {
            tinymceInstance.current.on('change', handleEditorChange);
            return () => {
                tinymceInstance.current!.off('change', handleEditorChange);
            };
        }
    }, [handleEditorChange]);

    useLayoutEffect(() => {
        if (tinymceInstance && props.value !== tinymceInstance.current?.getContent()) {
            tinymceInstance.current?.setContent(props.value || '');
        }
    }, [props.value]);

    useEffect(() => {
        if (tinymceInstance.current) {
            const handleEditorKeyup = () => {
                const curContent = tinymceInstance?.current?.getContent() || '';
                hasManualChange.current = !!curContent; // 有内容表示手动修改过  如果清空 重置为没有手动修改过
            };
            tinymceInstance.current.on('keyup', handleEditorKeyup);
            return () => {
                tinymceInstance.current!.off('keyup', handleEditorKeyup);
            };
        }
    }, []);

    const onTranslate = useCallback((params)=>{
        if (translate && translate.onTranslate) {
            return translate.onTranslate(params).then((res: string) => {
                if (!res) return true;
                const cont = formatTranslateContent(res);
                if (!hasManualChange.current) { // 如果没有手动修改过 直接用译文替换
                    tinymceInstance.current?.setContent(cont);
                } else { // 如果手动修改过  后面插入
                    if (tinymceInstance?.current && tinymceInstance?.current?.selection?.getSel()) {
                        tinymceInstance?.current.execCommand('mceInsertContent', false, cont);
                    } else {
                        const curContent = tinymceInstance?.current?.getContent() || '';
                        setContent(curContent + cont);
                    }
                }
                handleEditorChange();
                return true;
            });
        }
        return Promise.resolve(true);
    }, [translate]);

    return (
        <I18nextProvider i18n={newI18n}>
            {showOriginal && <Translate onTranslate={onTranslate} persistenceKey={translate?.persistenceKey}/>}
            <div className={css['rich-text-wrapper']} style={{ backgroundColor: backgroundColor }}>
                <textarea ref={targetRef} value={props.value || ''} id={idRef.current}/>
                {maxLength && <div className={'editor-num'}
                                   style={editorContent.length > maxLength ? error : defaultStyle}>{editorContent.length}/{maxLength}</div>}
                <UploadModal show={showModal} close={hideImageUploadModal} addImg={addImg} marking={marking}/>
            </div>
        </I18nextProvider>
    );
});

// 保留原文框格式
function formatTranslateContent(res:string) {
    if (/\r\n|\n/.test(res)) {
        const resArr = res.split(/\r\n|\n/);
        let c = '';
        resArr.forEach(r => {
            c += `<p>${r}</p>`;
        });
        return c;
    }
    return res;
}

FormTinymce.defaultProps = {
    paste_data_images: false
};
export default FormTinymce;
