import React, { useState, useEffect, useImperativeHandle, ReactNode } from 'react';
import { List } from 'udesk-ui';
import classNames from 'classnames';
import { ListProps } from 'udesk-ui/es/list';
import { usePrefixCls } from '../utils';
import ListHeader from './components/list-header';
import ListItem from './components/list-item';
import './style/index';

interface ListCheckboxProps<T> extends ListProps<T> {
  prefixCls?: string;
  className?: string;
  extraButton?: ReactNode;
  value?: any[];
  onChange?: (checkedList: any[]) => void;
  itemDisable?: (val: any) => boolean;
}

const ListCheckbox: React.ForwardRefRenderFunction<any, ListCheckboxProps<any>> = (props, ref) => {
  const {
    prefixCls: customizePrefixCls,
    className = '',
    extraButton = [],
    dataSource = [],
    value,
    renderItem,
    onChange,
    itemDisable,
    ...listRest
  } = props;

  const [checkedList, setCheckedList] = useState<any[]>([]);
  const [listCheckboxPrefixCls] = usePrefixCls('list-checkbox', customizePrefixCls || '');

  const classes = classNames(listCheckboxPrefixCls, className);

  useImperativeHandle(
    ref,
    () => ({
      resetSelect: handleCancelSelect,
    }),
    [],
  );

  const handleCheckAllChange = (e: any) => {
    let currCheckedList: any[] = [...checkedList];
    if (e.target.checked) {
      let currCheckedIds = currCheckedList.map((el: any) => el.id);
      dataSource.forEach((el: any) => {
        if (!currCheckedIds.includes(el.id)) {
          if (typeof itemDisable === 'function') {
            !itemDisable?.(el) && currCheckedList.push({ ...el });
          } else {
            currCheckedList.push({ ...el });
          }
        }
      });
    } else {
      let currCheckedListIds = dataSource.map((el: any) => el.id);
      currCheckedList = currCheckedList.filter((el) => {
        if (!currCheckedListIds.includes(el.id)) {
          return el;
        }
        return false;
      });
    }
    setCheckedList(currCheckedList);
    onChange?.(currCheckedList);
  };

  const handleSingleCheck = (item: any) => {
    let currCheckedList: any[] = [...checkedList];
    if (currCheckedList?.some((el: any) => el.id === item.id)) {
      currCheckedList = currCheckedList.filter((el: any) => el.id !== item.id);
    } else {
      currCheckedList.push({ ...item });
    }
    setCheckedList(currCheckedList);
    onChange?.(currCheckedList);
  };

  const handleCancelSelect = () => {
    setCheckedList([]);
    onChange?.([]);
  };

  useEffect(() => {
    if (Array.isArray(value) && value.length) {
      setCheckedList(value);
    }
  }, [value]);

  return (
    <List
      className={classes}
      dataSource={dataSource}
      {...listRest}
      header={
        <ListHeader
          extraButton={extraButton}
          dataSource={dataSource}
          checkedList={checkedList}
          handleCheckAllChange={handleCheckAllChange}
          handleCancelSelect={handleCancelSelect}
          itemDisable={itemDisable}
        ></ListHeader>
      }
      renderItem={(item: any, index: number) => (
        <ListItem
          item={item}
          checkedList={checkedList}
          handleSingleCheck={handleSingleCheck}
          itemDisable={itemDisable}
          key={item.key}
        >
          {renderItem?.(item, index)}
        </ListItem>
      )}
    />
  );
};

const FancyListCheckbox = React.forwardRef(ListCheckbox);

export default FancyListCheckbox;
