import React, { useRef, forwardRef, useImperativeHandle, useMemo } from 'react';
import { Table } from 'udesk-ui';
import type { TableProps as AntTableProps } from 'udesk-ui/lib/table';
import styled from 'styled-components';
import { usePrefixCls } from '../utils';
import {
  useMemoScrollPosition,
  useScroll as useTableScrollProps,
  useRowSelection,
  useColumnsFormat,
} from './utils';
import { UseRowSelectionOptionsType } from './utils/hooks/useRowSelection';
import './style/index';

interface TableWrapProps {
  prefixCls: string;
  columnsEllipsis: any[];
}

const TableWrap = styled.div<TableWrapProps>`
  width: 100%;
  height: 100%;
  ${(props) => `.${props.prefixCls}-table-wrapper`} {
    height: 100%;
  }
  ${(props) => `.${props.prefixCls}-pagination`} {
    min-height: 32px;
  }
  ${(props) =>
    props.columnsEllipsis.map(
      (column) => `.gm-ui-table-col-max-width-${column.key} {
     max-width: ${column.width}px;
  }`,
    )}
`;

interface RowSelectionEnhance<T> extends UseRowSelectionOptionsType<T> {
  type?: 'checkbox' | 'radio';
}

export interface TableProps<T extends object = any> extends AntTableProps<T> {
  cacheId?: string;
  autoSize?: boolean;
  rowSelectionEnhance?: RowSelectionEnhance<T>;
  onTableScroll?: () => void;
}

export interface TableInstance {
  clearScrollPosition: () => void;
  scrollTo: (x: number, y: number) => void;
  resetToEmpty: () => void;
}

const MemoTable = forwardRef<TableInstance, TableProps>((props, ref) => {
  const {
    scroll: propsScroll,
    columns: propsColumns,
    cacheId = '',
    prefixCls: customizePrefixCls,
    autoSize,
    rowSelectionEnhance,
    onTableScroll,
    ...antdProps
  } = props;
  //   const [columns, setColumns] = useState<TableProps['columns']>([]);
  const [tableBodyPrefixCls, prefixCls] = usePrefixCls('table-body', customizePrefixCls);
  const [tableWrapPrefixCls] = usePrefixCls('memo-table', customizePrefixCls);
  const wrapRef = useRef<HTMLDivElement>(null);
  const scroll = useTableScrollProps(
    { wrapRef, propsScroll, prefixCls, dataSource: antdProps.dataSource },
    [propsScroll],
  );
  const { tryToSavePrevScrollPosition, tryToResetPrevScrollPosition, storage } =
    useMemoScrollPosition({
      cacheId,
      tableBodyPrefixCls,
      wrapRef,
      dataSource: antdProps.dataSource,
      columns: propsColumns,
    });
  const { rowSelection, instance: rowSelectionInstance } = useRowSelection(
    typeof rowSelectionEnhance === 'object' ? rowSelectionEnhance?.type ?? 'checkbox' : undefined,
    rowSelectionEnhance,
  );

  const columns = useColumnsFormat(propsColumns);

  useImperativeHandle(ref, () => {
    return {
      clearScrollPosition: () => storage.delete(),
      scrollTo: tryToResetPrevScrollPosition,
      resetToEmpty: rowSelectionInstance.resetToEmpty,
    };
  });

  const columnsEllipsis = useMemo(() => {
    return columns?.filter((i) => i.ellipsis) ?? [];
  }, [columns]);

  return (
    <TableWrap
      columnsEllipsis={columnsEllipsis}
      onScrollCapture={() => {
        onTableScroll && requestAnimationFrame(onTableScroll);
        requestAnimationFrame(tryToSavePrevScrollPosition);
      }}
      className={tableWrapPrefixCls}
      ref={wrapRef}
      prefixCls={prefixCls}
    >
      <Table
        rowSelection={rowSelection}
        {...antdProps}
        scroll={autoSize ? scroll : propsScroll}
        columns={columns}
        rowKey={rowSelectionEnhance?.rowKeysFieldName ?? 'id'}
      />
    </TableWrap>
  );
});

MemoTable.defaultProps = {
  autoSize: true,
  cacheId: '',
};

export default React.memo(MemoTable);
