import { useEffect, useState } from 'react';
import { TableRowSelection } from 'udesk-ui/lib/table';

type RowSelectionType = 'checkbox' | 'radio' | undefined;
export interface UseRowSelectionOptionsType<T>
  extends Omit<
    TableRowSelection<T>,
    | 'onChange'
    | 'onSelect'
    | 'onSelectMultiple'
    | 'onSelectAll'
    | 'onSelectInvert'
    | 'onSelectNone'
    | 'preserveSelectedRowKeys'
    | 'type'
    | 'selectedRowKeys'
    | 'defaultSelectedRowKeys'
  > {
  defaultSelectedRowKeys?: string[] | number[];
  selectedRowKeys?: string[] | number[];
  rowKeysFieldName?: string;
  onSelectedRowKeysChange?: (keys: string[], items: any[]) => void;
}

interface UseRowSelectionReturnType<T> {
  rowSelection: TableRowSelection<T> | undefined;
  instance: {
    resetToEmpty: () => void;
  };
}

const useRowSelection = <T>(
  rowSelectionType: RowSelectionType,
  options?: UseRowSelectionOptionsType<T>,
): UseRowSelectionReturnType<T> => {
  const { rowKeysFieldName = 'id', selectedRowKeys, onSelectedRowKeysChange } = options ?? {};
  const [selectedItems, setSelectedItems] = useState<any[]>(options?.defaultSelectedRowKeys || []);
  const resetToEmpty = () => {
    setSelectedItems([]);
    onSelectedRowKeysChange?.([], []);
  };
  if (!rowSelectionType)
    return {
      rowSelection: undefined,
      instance: {
        resetToEmpty,
      },
    };

  const triggerOnChange = (newSelectedItems: any[]) => {
    onSelectedRowKeysChange?.(
      newSelectedItems.map((i) => i[rowKeysFieldName]),
      newSelectedItems,
    );
  };
  const onCheckChange = (record: any, selected: boolean, selectedRows: any[], nativeEvent: any) => {
    let newSelectedItems = [...selectedItems];
    if (selected) {
      newSelectedItems.push(record);
    } else {
      newSelectedItems = newSelectedItems.filter(
        (item) => item[rowKeysFieldName] !== record[rowKeysFieldName],
      );
    }
    setSelectedItems(newSelectedItems);
    triggerOnChange(newSelectedItems);
  };
  const onSelectAll = (selected: boolean, selectedRows: any[], changeRows: any) => {
    let newSelectedItems = [...selectedItems];
    if (selected) {
      newSelectedItems = [...newSelectedItems, ...changeRows];
    } else {
      newSelectedItems = newSelectedItems.filter(
        (item) => !changeRows.find((r: any) => r[rowKeysFieldName] === item[rowKeysFieldName]),
      );
    }
    setSelectedItems(newSelectedItems);
    triggerOnChange(newSelectedItems);
  };

  useEffect(() => {
    if (!selectedRowKeys) return;
    setSelectedItems(selectedRowKeys.map((i) => ({ [rowKeysFieldName]: i })));
  }, [selectedRowKeys]);

  return {
    rowSelection: {
      ...options,
      type: rowSelectionType,
      onSelect: onCheckChange,
      onSelectAll: onSelectAll,
      selectedRowKeys: selectedItems.map((i) => i[rowKeysFieldName]),
    },
    instance: {
      resetToEmpty,
    },
  };
};

export default useRowSelection;
