import React, { useState, useEffect, useImperativeHandle } from 'react';
import classNames from 'classnames';
import { usePrefixCls } from '../utils';
import { Form, Input, Radio, Popover, Icon } from 'udesk-ui';
import PopContent from './components/pop-content';
import {
  getJudgmentLogicTypes,
  getConfigRuleTypes,
  getFieldOperators,
  getManyAnswerChannelTypes,
} from './constant';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import CustomLogicInput from './components/custom-logic-input';
import CustomLogicFilter from './components/custom-logic-filter';
// import { If } from 'tsx-control-statements/components';
import { validateValues, formatData, formatToComponentData } from './utils';
import './style/index';

interface RulesConfigProps<T> {
  prefixCls?: string;
  className?: string;
  value?: any;
  onChange?: (changedValues: any) => void;
  onFinish: (values: any) => void;
  formKeys?: {
    nameKey?: string;
    tagValueListsKey?: any;
    judgmentLogicKey?: string;
    tagKey?: string;
  };
}
const formLayout = {
  labelCol: { span: 4 },
  wrapperCol: { span: 16 },
};
const tailLayout = {
  wrapperCol: { offset: 4, span: 16 },
};
const defauleFormKeys = {
  nameKey: 'name',
  tagValueListsKey: 'tagValueLists',
  judgmentLogicKey: 'judgmentLogic',
  tagKey: 'tag',
};

const RulesConfig: React.ForwardRefRenderFunction<any, RulesConfigProps<any>> = (props, ref) => {
  const {
    prefixCls: customizePrefixCls,
    className = '',
    value = {},
    onChange,
    formKeys = defauleFormKeys,
    onFinish,
  } = props;

  const [contextLocale] = useLocaleReceiver('RulesConfig');
  const [form] = Form.useForm();
  const judgmentLogicTypes = getJudgmentLogicTypes(contextLocale);
  const configRuleTypes = getConfigRuleTypes(contextLocale);
  const fieldOperators = getFieldOperators(contextLocale);
  const manyAnswerChannelTypes = getManyAnswerChannelTypes(contextLocale);
  const [judgmentLogic, setJudgmentLogic] = useState(judgmentLogicTypes.meetAll.id);

  const [rulesConfigPrefixCls] = usePrefixCls('rules-config', customizePrefixCls || '');

  const classes = classNames(rulesConfigPrefixCls, className);
  const MANY_ANSWER_EXTRA_LIST = [configRuleTypes.emotionCategory.id];

  useImperativeHandle(
    ref,
    () => ({
      reset: handleReset,
      submit: handleSubmit,
    }),
    [],
  );

  const handleSubmit = () => {
    form?.submit();
  };

  const handleFinish = (values: any) => {
    const val = {
      ...values,
      [formKeys.tagValueListsKey as string]: formatToComponentData(
        value?.[formKeys.tagValueListsKey as string]?.map((el: any) => ({ ...el })) || [],
        configRuleTypes,
        manyAnswerChannelTypes,
      ),
    };
    const flag = validateValues(
      val,
      configRuleTypes,
      fieldOperators,
      judgmentLogicTypes,
      contextLocale,
    );
    if (flag) {
      onFinish?.(formatData(val, formKeys.tagValueListsKey, judgmentLogicTypes, configRuleTypes));
    }
  };

  const handleReset = () => {
    setJudgmentLogic(judgmentLogicTypes.meetAll.id);
    form?.setFieldsValue({
      [formKeys.nameKey as string]: '',
      [formKeys.tagValueListsKey as string]: {
        field: configRuleTypes.platform.id,
        operator: fieldOperators.is.id,
        value: null,
      },
      [formKeys.judgmentLogicKey as string]: judgmentLogicTypes.meetAll.id,
      [formKeys.tagKey as string]: '',
    });
  };

  const onJudgmentLogicChange = (e: any) => {
    let value = e.target.value;
    setJudgmentLogic(value);
  };

  const handleValuesChange = (changedValues: any, allValues: any) => {
    onChange?.({ ...value, ...allValues });
  };

  const handleLogicChange = (val: any) => {
    value[formKeys.tagValueListsKey as string] = val;
    onChange?.({ ...value, [formKeys.tagValueListsKey as string]: val });
  };

  useEffect(() => {
    setJudgmentLogic(value?.[formKeys.judgmentLogicKey as string] || judgmentLogicTypes.meetAll.id);
    form?.setFieldsValue({
      [formKeys.nameKey as string]: value?.[formKeys.nameKey as string] || '',
      [formKeys.judgmentLogicKey as string]:
        value?.[formKeys.judgmentLogicKey as string] || judgmentLogicTypes.meetAll.id,
      //   [formKeys.tagValueListsKey as string]: formatToComponentData(
      //     value?.[formKeys.tagValueListsKey as string]?.length
      //       ? value?.[formKeys.tagValueListsKey as string]
      //       : [
      //           {
      //             field: configRuleTypes.platform.id,
      //             operator: fieldOperators.is.id,
      //             value: null,
      //           },
      //         ],
      //     configRuleTypes,
      //     manyAnswerChannelTypes,
      //   ),
      [formKeys.tagKey as string]: value?.[formKeys.tagKey as string] || '',
    });
  }, [value]);

  return (
    <div className={classes}>
      <Form {...formLayout} form={form} onValuesChange={handleValuesChange} onFinish={handleFinish}>
        <Form.Item
          label={contextLocale.lang.name}
          name={formKeys.nameKey}
          rules={[
            {
              required: true,
              message: contextLocale.lang.inputRuleName,
            },
          ]}
        >
          <Input placeholder={contextLocale.lang.inputRuleName} />
        </Form.Item>
        <Form.Item
          {...tailLayout}
          //   name={formKeys.tagValueListsKey}
        >
          <CustomLogicFilter
            value={formatToComponentData(
              value?.[formKeys.tagValueListsKey as string] || [],
              configRuleTypes,
              manyAnswerChannelTypes,
            )}
            onChange={handleLogicChange}
            blackList={[]}
            extraList={MANY_ANSWER_EXTRA_LIST}
          />
        </Form.Item>
        <Form.Item
          label={contextLocale.lang.judgmentLogic}
          rules={[
            {
              required: true,
              message: contextLocale.lang.inputRuleName,
            },
          ]}
        >
          <Form.Item name={formKeys.judgmentLogicKey} noStyle>
            <Radio.Group onChange={onJudgmentLogicChange}>
              {judgmentLogicTypes.map((item: any) => (
                <Radio.Button value={item.id}>{item.name}</Radio.Button>
              ))}
            </Radio.Group>
          </Form.Item>
          {/* If 不要去掉 */}
          {/* @ts-ignored */}
          <If condition={judgmentLogic === judgmentLogicTypes.custom.id}>
            {judgmentLogic === judgmentLogicTypes.custom.id && (
              <Popover content={<PopContent />}>
                <Icon
                  type="QuestionCircleOutlined"
                  antdIcon={true}
                  style={{ marginLeft: '12px' }}
                />
              </Popover>
            )}
            {/* @ts-ignored */}
          </If>
        </Form.Item>
        {/* @ts-ignored */}
        <If condition={judgmentLogic === judgmentLogicTypes.custom.id}>
          {judgmentLogic === judgmentLogicTypes.custom.id && (
            <Form.Item
              {...tailLayout}
              name={formKeys.tagKey}
              rules={[
                {
                  required: true,
                  message: contextLocale.lang.inputJudgmentLogicCustom,
                },
              ]}
            >
              <CustomLogicInput />
            </Form.Item>
          )}
          {/* @ts-ignored */}
        </If>
      </Form>
    </div>
  );
};

const FancyListCheckbox = React.forwardRef(RulesConfig);

export default FancyListCheckbox;
