import React, {
  forwardRef,
  ReactNode,
  useEffect,
  useContext,
  useImperativeHandle,
  useMemo,
  useState,
  useRef,
} from 'react';
import { StandardizeTableInstance, StandardizeTableProps } from './types';
import StandardizeTableView from './view';
import { TreeMenuProps } from '../tree-menu/types';
import { Button, Modal, Space, Switch } from 'udesk-ui';
import { TableRowSelection } from 'udesk-ui/lib/table';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';
import BatchedActions from '../batched-actions';
import { MoreOutlined } from '@udesk/icons';

// import './style/index';

const StandardizeTable = React.forwardRef<StandardizeTableInstance, StandardizeTableProps>(
  (props, ref) => {
    const { prefixCls, className } = props;
    const {
      table,
      filter,
      classification,
      columnsControl,
      exportBtn,
      batchedActions,
      // autoTriggerOnSearchOnMount,
    } = props;
    const [contextLocale] = useLocaleReceiver('BatchedActions');
    const [rowSelectionItems, setRowSelectionItems] = useState<any[]>([]);
    const [showColumns, setShowColumns] = useState<string[]>([]);
    const [filterValues, setFilterValues] = useState<any>(filter?.initialValues ?? {});
    const [classificationId, setClassificationId] = useState<any>(null);
    const [tableProps, setTableProps] = useState<StandardizeTableProps['table']>(
      formatTableProps(table, showColumns, setRowSelectionItems, rowSelectionItems, contextLocale),
    );
    const instance = useRef<StandardizeTableInstance>(null);

    useImperativeHandle(ref, () => {
      return instance.current as StandardizeTableInstance;
    });

    useEffect(() => {
      setTableProps(
        formatTableProps(
          table,
          showColumns,
          setRowSelectionItems,
          rowSelectionItems,
          contextLocale,
        ),
      );
    }, [table, showColumns, rowSelectionItems]);

    useEffect(() => {
      if (columnsControl?.showColumnKeys) {
        setShowColumns(columnsControl?.showColumnKeys);
      } else {
        setShowColumns(table.columns?.map((c) => c?.key as string) ?? []);
      }
    }, [columnsControl?.showColumnKeys, table.columns]);

    const columnsControlProps = useMemo(() => {
      if (!columnsControl) return undefined;
      return {
        columns: table.columns?.filter(Boolean),
        showColumnKeys: showColumns,
        onChange: (showKeys: string[]) => {
          setShowColumns(showKeys);
          columnsControl?.onChange?.(showKeys);
        },
      };
    }, [columnsControl, showColumns, table.columns]);
    const filterProps = useMemo(() => {
      if (!filter) return undefined;
      return {
        ...filter,
        onSearch: (v: any) => {
          filter?.onSearch?.(v, {
            pagination: Object.assign({}, tableProps.pagination, { current: 1 }),
            classificationId,
          });
          setFilterValues(v);
          instance.current?.tableInstance.resetToEmpty();
        },
      };
    }, [filter, tableProps, classification]);
    const exportBtnProps = useMemo(() => {
      if (!exportBtn) return undefined;
      return {
        ...exportBtn,
        exportDataCount: (table?.pagination as any)?.total ?? 0,
      };
    }, [exportBtn, table?.pagination]);
    const classificationProps: TreeMenuProps | undefined = useMemo(() => {
      if (!classification) return undefined;
      const onSelect: TreeMenuProps['onSelect'] = (selectedKeys) => {
        const selectedKeysCopy = [...selectedKeys];
        const selectedKey = selectedKeysCopy.pop();
        setClassificationId(selectedKey);

        classification.onSelect?.(selectedKey, {
          filter: filterValues,
          pagination: Object.assign({}, tableProps.pagination, { current: 1 }),
        });
        instance.current?.tableInstance.resetToEmpty();
      };
      return {
        indentUnitWidth: 16,
        expandAction: false,
        ...classification,
        onSelect,
      };
    }, [classification, filterValues]);
    useEffect(() => {
      if (Array.isArray(classification?.selectedKeys) && classification?.selectedKeys.length) {
        setClassificationId(classification.selectedKeys[0]);
      }
    }, [classification]);
    const batchedActionsProps = useMemo(() => {
      if (!batchedActions) return undefined;
      let items = rowSelectionItems;
      const { selectedRowKeys, rowKeysFieldName, onSelectedRowKeysChange } =
        tableProps?.rowSelectionEnhance || {};
      if (tableProps?.rowSelectionEnhance && selectedRowKeys && rowKeysFieldName) {
        items = rowSelectionItems.filter((item) =>
          (selectedRowKeys as any[]).includes(item[rowKeysFieldName]),
        );
      }
      return batchedActions.map((i) => ({
        ...i,
        onClick: () => {
          return i.onClick(items)?.then?.((res) => {
            onSelectedRowKeysChange?.([], []);
            return res;
          });
        },
      }));
    }, [batchedActions, rowSelectionItems, tableProps]);

    // useEffect(() => {
    //   if (autoTriggerOnSearchOnMount) {
    //     filter?.onSearch?.(filter.initialValues, {
    //       pagination: Object.assign({}, tableProps.pagination, { current: 1 }),
    //       classificationId,
    //     });
    //   }
    // }, []);

    return (
      <StandardizeTableView
        {...props}
        table={tableProps}
        columnsControl={columnsControlProps}
        prefixCls={prefixCls}
        className={className}
        filter={filterProps}
        rowSelectionCount={tableProps.rowSelectionEnhance?.selectedRowKeys?.length || 0}
        exportBtn={exportBtnProps}
        classification={classificationProps}
        batchedActions={batchedActionsProps}
        ref={instance}
      />
    );
  },
);

export default React.memo(StandardizeTable);

function formatTableProps(
  table: StandardizeTableProps['table'],
  showColumns: string[],
  setRowSelectionItems: React.Dispatch<React.SetStateAction<any[]>>,
  rowSelectionItems: any[],
  contextLocale: any,
): StandardizeTableProps['table'] {
  const rowSelectionEnhance = table.rowSelectionEnhance
    ? { ...table.rowSelectionEnhance }
    : undefined;
  if (rowSelectionEnhance) {
    rowSelectionEnhance.onSelectedRowKeysChange = (keys: string[], items: any[]) => {
      setRowSelectionItems(items);
      table.rowSelectionEnhance!.onSelectedRowKeysChange?.(keys, items);
    };
    const key = rowSelectionEnhance.rowKeysFieldName ?? 'id';
    if (!rowSelectionEnhance.selectedRowKeys) {
      rowSelectionEnhance.selectedRowKeys = rowSelectionItems?.map((i) => i[key]) ?? [];
    }
  }
  const columns =
    table.columns
      ?.filter((i = {}) => showColumns.includes(i.key as string))
      .map((i = {}) => {
        if (i.renderType === 'actions') {
          return {
            render: (text: any, record: any, index: any) => {
              const btnCount = i.buttons.length;
              let showBtns = [...i.buttons];
              let hiddenBtns: typeof i.buttons = [];
              if (btnCount > 3) {
                const SHOW_BTNS_COUNT = 2;
                showBtns.length = SHOW_BTNS_COUNT;
                hiddenBtns = i.buttons.slice(SHOW_BTNS_COUNT).map((i) => ({
                  ...i,
                  onClick: i.onClick.bind(null, record),
                  disabled: typeof i.disabled === 'function' ? i.disabled?.(record) : i.disabled,
                }));
              }
              return (
                <Space style={{ marginLeft: '-8px' }}>
                  {showBtns.map((b) => {
                    const { label, disabled, onClick, confirm, ...btnProps } = b;
                    return (
                      <Button
                        {...btnProps}
                        onClick={
                          btnProps.danger || confirm
                            ? () => {
                                Modal.confirm({
                                  // ...defaultConfirm,
                                  ...contextLocale.lang.confirm,
                                  ...(confirm || {}),
                                  onOk: onClick?.bind(null, record),
                                });
                              }
                            : onClick?.bind(null, record)
                        }
                        disabled={typeof disabled === 'function' ? disabled?.(record) : disabled}
                        type="link"
                        size="small"
                      >
                        {label}
                      </Button>
                    );
                  })}
                  {hiddenBtns.length > 0 && (
                    <BatchedActions menu={hiddenBtns} trigger={['click']}>
                      <Button type="link" icon={<MoreOutlined />}></Button>
                    </BatchedActions>
                  )}
                </Space>
              );
            },
            fixed: 'right' as TableRowSelection<any>['fixed'],
            ...i,
          };
        }
        if (i.renderType === 'switch') {
          return {
            render: (text: any, record: any, index: any) => {
              if (i.key || (i as any).dataIndex)
                return (
                  <Switch
                    onChange={(checked) => {
                      i.onChange?.(checked, record, index);
                    }}
                    checked={record[i.key || (i as any).dataIndex]}
                  />
                );
            },
            ...i,
          };
        }
        return i;
      }) ?? [];
  return {
    ...table,
    columns,
    rowSelectionEnhance,
  };
}
