import React, {
  forwardRef,
  ReactNode,
  useEffect,
  useContext,
  useImperativeHandle,
  useMemo,
  useState,
  useRef,
} from 'react';
import { Button, FormBuilder, Icon, Modal, Space } from 'udesk-ui';
import { ExpandWithDefaultProps, usePrefixCls } from '../utils';
import CommonFilter, { CommonFilterInstance } from '../common-filter';
import MemoTable, { TableInstance } from '../memo-table';
import screenfull from 'screenfull';
import TreeMenu from '../tree-menu';
import classNames from 'classnames';
import ColumnsControl from './components/columns-control-btn';
import ExportButton, { ExportButtonProps } from '../export-button';
import {
  StandardizeTableInstance,
  StandardizeTableViewProps,
  TableWrapProps,
  ToolbarProps,
} from './types';
import {
  ClassificationWrap,
  FilterWrap,
  Layout,
  RowSelectionInfo,
  TableWrap,
  Toolbar,
} from './components/ui-components';
import { ArrowDownSOutlined, ImportOutlined } from '@udesk/icons';
import BatchedActions from '../batched-actions';
import { ButtonProps } from 'udesk-ui/es/button';
import { TreeMenuProps } from '../tree-menu/types';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';

// import './style/index';

const StandardizeTable = forwardRef<
  StandardizeTableInstance,
  StandardizeTableViewProps & { rowSelectionCount: number }
>((props, ref) => {
  const { prefixCls: customizePrefixCls, className } = props;
  const {
    table,
    filter,
    classification: classificationWithTitle,
    toolBarRightButtons,
    columnsControl,
    exportBtn: exportProps,
    rowSelectionCount,
    importBtn,
    batchedActions,
    showFullscreen,
  } = props;
  const classificationTitle = classificationWithTitle?.title;
  const [contextLocale] = useLocaleReceiver('StandardizeTable');

  const pageInstance = useRef<HTMLDivElement>(null);
  const tableInstance = useRef<TableInstance>(null);
  const filterInstance = useRef<CommonFilterInstance>(null);

  const [isFullscreen, setIsFullscreen] = useState(screenfull.isFullscreen);

  useImperativeHandle(ref, () => {
    return {
      pageInstance: pageInstance.current!,
      tableInstance: tableInstance.current!,
      filterInstance: filterInstance.current!,
    };
  });

  const [standardizeTableCls, prefixCls] = usePrefixCls('standardize-table', customizePrefixCls);

  const classes = useMemo(
    () => classNames(standardizeTableCls, className),
    [standardizeTableCls, className],
  );

  const tablePagination = useMemo(() => {
    return {
      showSizeChanger: true,
      showQuickJumper: true,
      showTotal: (total: number, range: number[]) =>
        `${contextLocale.lang.num} ${range[0]}-${range[1]} ${contextLocale.lang.total} ${total} ${contextLocale.lang.strip}`,
      ...(table?.pagination ?? {}),
    };
  }, [table?.pagination]);

  const buttonsCount = useMemo(() => {
    let buttonsCount = 0;
    // if (rowSelectionCount) buttonsCount++;
    if (exportProps) buttonsCount++;
    if (columnsControl) buttonsCount++;
    if (importBtn) buttonsCount++;
    if (toolBarRightButtons) buttonsCount++;
    if (batchedActions) buttonsCount++;

    if (buttonsCount > 5 && process.env.NODE_ENV === 'development') {
      throw new Error(
        `UI规范规定，工具栏除1个主按钮外，最多只能同事存在5个副按钮，当前存在${buttonsCount}个副按钮，请与产品沟通保留哪些`,
      );
    }
    const mainBtnCount = toolBarRightButtons?.filter((i) => i.mainBtn).length ?? 0;
    if (mainBtnCount > 1 && process.env.NODE_ENV === 'development') {
      throw new Error(
        `UI规范规定，工具栏至多只能存在1个主按钮，当前存在${mainBtnCount}个主按钮，请与产品沟通保留哪个`,
      );
    }

    return buttonsCount;
  }, [
    // rowSelectionCount,
    toolBarRightButtons,
    exportProps,
    columnsControl,
    importBtn,
    batchedActions,
  ]);

  const toolBarRightButtonsFormatted = useMemo(() => {
    if (!toolBarRightButtons) return null;
    const mainBtn = toolBarRightButtons.find((i) => i.mainBtn) as ButtonProps & {
      label: string;
      mainBtn: true;
    };
    if (mainBtn) {
      mainBtn.type = 'primary';
    }
    const others = toolBarRightButtons.filter((i) => !i.mainBtn);
    const subButtonsCount = buttonsCount - 1 + others.length;
    const displaySubButtons = others.slice(0, 5 - buttonsCount + 1);
    const hiddenSubButtons = others.slice(5 - buttonsCount + 1);
    if (hiddenSubButtons.length && displaySubButtons.length) {
      const buttonNeedHidden = displaySubButtons.pop();
      hiddenSubButtons.unshift(buttonNeedHidden!);
    }
    const { mainBtn: _, ...mainBtnProps } = mainBtn || {};

    return (
      <React.Fragment>
        {displaySubButtons.map((item: any) => {
          return (
            <Button
              {...item}
              onClick={
                item.confirm
                  ? () => {
                      Modal.confirm({
                        ...(item.confirm || {}),
                        onOk: item.onClick,
                        getContainer: (triggerNode: any) => pageInstance.current || document.body,
                      });
                    }
                  : item.onClick
              }
            >
              {item?.label}
            </Button>
          );
        })}
        {Boolean(hiddenSubButtons.length) && (
          <BatchedActions
            menu={hiddenSubButtons}
            getPopupContainer={(triggerNode: any) => pageInstance.current || document.body}
          >
            <Button>
              {contextLocale.lang.more}
              <ArrowDownSOutlined style={{ marginLeft: 4 }} />
            </Button>
          </BatchedActions>
        )}
        {mainBtn && <Button {...mainBtnProps}>{mainBtn?.label}</Button>}
      </React.Fragment>
    );
  }, [toolBarRightButtons, buttonsCount]);

  useEffect(() => {
    const callback = () => {
      setIsFullscreen(screenfull.isFullscreen);
    };
    screenfull.on('change', callback);
    return () => {
      screenfull.off('change', callback);
    };
  }, []);

  const closeFilter = () => {
    filterInstance.current?.closeFilter?.();
  };

  return (
    <Layout ref={pageInstance} className={classes}>
      {filter && (
        <FilterWrap>
          <CommonFilter {...filter} ref={filterInstance} />
        </FilterWrap>
      )}
      <TableWrap prefixCls={prefixCls} padding={classificationWithTitle ? '16px 16px 16px 0' : 16}>
        {classificationWithTitle && (
          <ClassificationWrap>
            {classificationTitle && <div style={{ marginBottom: 8 }}>{classificationTitle}</div>}
            <div style={{ flex: 1 }}>
              <TreeMenu
                {...classificationWithTitle}
                getPopupContainer={(triggerNode: any) => pageInstance.current || document.body}
              />
            </div>
          </ClassificationWrap>
        )}
        <div style={{ flex: 1, display: 'flex', flexDirection: 'column', overflow: 'hidden' }}>
          <Toolbar prefixCls={prefixCls} visible={Boolean(buttonsCount) || Boolean(showFullscreen)}>
            <Space style={{ flex: 1 }}>
              {Boolean(rowSelectionCount) && (
                <React.Fragment>
                  <RowSelectionInfo>
                    {contextLocale.lang.chosen} <span>{rowSelectionCount}</span> 条
                  </RowSelectionInfo>
                  <Button
                    type="link"
                    size="small"
                    onClick={() => tableInstance.current?.resetToEmpty()}
                  >
                    {contextLocale.lang.cancel}
                  </Button>
                </React.Fragment>
              )}
            </Space>
            <Space>
              {showFullscreen && (
                <Button
                  icon={
                    <Icon
                      type={isFullscreen ? 'FullscreenExitOutlined' : 'FullscreenOutlined'}
                      antdIcon={true}
                    />
                  }
                  tooltip={isFullscreen ? '退出全屏' : '全屏'}
                  onClick={() =>
                    pageInstance.current &&
                    screenfull.isEnabled &&
                    (isFullscreen ? screenfull.exit() : screenfull.request(pageInstance.current))
                  }
                ></Button>
              )}

              {batchedActions && (
                <BatchedActions
                  menu={batchedActions}
                  // getPopupContainer={(triggerNode: any) => pageInstance.current || document.body}
                  disabled={!rowSelectionCount}
                >
                  <Button disabled={!rowSelectionCount}>
                    {contextLocale.lang.bulkOperations}
                    <ArrowDownSOutlined style={{ marginLeft: 4 }} />
                  </Button>
                </BatchedActions>
              )}
              {importBtn && (
                <ExportButton
                  {...importBtn}
                  noSecondConfirmation={true}
                  icon={<ImportOutlined style={{ marginRight: 4 }} />}
                  // getPopupContainer={(triggerNode: any) => pageInstance.current || document.body}
                >
                  {contextLocale.lang.import}
                </ExportButton>
              )}
              {exportProps && (
                <ExportButton
                  {...exportProps}
                  // getPopupContainer={(triggerNode: any) => pageInstance.current || document.body}
                />
              )}
              {columnsControl && <ColumnsControl {...columnsControl} />}
              {toolBarRightButtonsFormatted}
            </Space>
          </Toolbar>
          <div style={{ flex: 1, overflow: 'hidden' }}>
            <MemoTable
              {...table}
              onTableScroll={closeFilter}
              pagination={tablePagination}
              ref={tableInstance}
            />
          </div>
        </div>
      </TableWrap>
    </Layout>
  );
});

export default React.memo(StandardizeTable);
