import React, { useMemo, useRef, useCallback, Key, useEffect, useState } from 'react';
import { SorterResult } from 'udesk-ui/lib/table';
import StandardizeTable from './model';
import { StandardizeTableInstance, StandardizeTableProps } from './types';

type OnRequestParams = {
  filter: { [key: string]: any };
  classificationId: number | string | null;
  pagination: any;
  sorter: SorterResult<any> | SorterResult<any>[];
};

export interface RequestTableProps extends StandardizeTableProps {
  onRequest: (params: OnRequestParams) => void;
  autoRequestOnMount?: boolean;
}

const noop = () => {};

const RequestTable: React.FC<RequestTableProps> = (props) => {
  const { onRequest = noop, autoRequestOnMount, ...others } = props;
  const {
    filter: originFilter,
    classification: originClassification,
    table: originTable,
    batchedActions: originBatchedActions,
  } = props;

  const paginationValues = useRef(originTable.pagination ?? {});
  const filterValues = useRef(originFilter?.initialValues ?? {});
  const sorterValues = useRef({});
  const classificationId = useRef<string | number | null>(
    (originClassification?.defaultSelectedKeys?.[0] || originClassification?.selectedKeys?.[0]) ??
      null,
  );

  const instance = useRef(null);

  const [initial, setInitial] = useState<boolean>(true);

  useEffect(() => {
    if (originClassification && !classificationId.current) return;
    if (!autoRequestOnMount) return;
    if (!initial) return;

    setInitial(false);
    classificationId.current =
      (originClassification?.defaultSelectedKeys?.[0] || originClassification?.selectedKeys?.[0]) ??
      null;
    onRequest({
      filter: filterValues.current,
      classificationId: classificationId.current,
      pagination: paginationValues.current,
      sorter: sorterValues.current,
    });
  }, [classificationId, originClassification]);

  const triggerOnRequest = useCallback(() => {
    onRequest({
      filter: filterValues.current,
      classificationId: classificationId.current,
      pagination: paginationValues.current,
      sorter: sorterValues.current,
    });
  }, [onRequest]);

  const filter: RequestTableProps['filter'] = useMemo(() => {
    if (!originFilter) return undefined;
    const { onSearch } = originFilter;

    return {
      ...originFilter,
      onSearch: (...args) => {
        const [formValues, others] = args;
        const { classificationId: cID, pagination } = others;
        filterValues.current = formValues;
        classificationId.current = cID;
        paginationValues.current = pagination;
        triggerOnRequest();
        onSearch?.(...args);
      },
    };
  }, [originFilter, triggerOnRequest]);

  const batchedActions: RequestTableProps['batchedActions'] = useMemo(() => {
    if (!originBatchedActions) return undefined;

    return originBatchedActions.map((item) => {
      return {
        ...item,
        onClick: (...args) => {
          return item.onClick?.(...args)?.then((res) => {
            triggerOnRequest();
          });
        },
      };
    });
  }, [originBatchedActions, triggerOnRequest]);

  const classification = useMemo(() => {
    if (!originClassification) return undefined;

    // 初始化classificationId数据
    if (classificationId.current === null) {
      if (isArrayAndNotEmpty(originClassification.defaultSelectedKeys)) {
        classificationId.current = originClassification.defaultSelectedKeys![0];
      }
      if (isArrayAndNotEmpty(originClassification.selectedKeys)) {
        classificationId.current = originClassification.selectedKeys![0];
      }
    }

    return {
      ...originClassification,
      onSelect: (selectedKey: Key | undefined, params: any) => {
        originClassification.onSelect(selectedKey, params);
        classificationId.current = selectedKey ?? null;
        paginationValues.current = {
          ...paginationValues.current,
          current: 1,
        };
        triggerOnRequest();
      },
      // treeData: originClassification.treeData ?? [],
    };
  }, [originClassification, triggerOnRequest]);

  const table = useMemo(() => {
    // if (!originTable) return undefined;

    const { pagination: originPagination, onChange: originTableOnChange, columns: originColumns = [] } = originTable;

    // if (!originPagination) return originTable;

    // const { onChange: originOnChange, onShowSizeChange: originOnShowSizeChange } = originPagination;
    // const pagination = { ...originPagination };

    // pagination.onChange = (current, pageSize) => {
    //   originOnChange?.(current, pageSize);
    //   paginationValues.current = {
    //     ...paginationValues.current,
    //     current,
    //     pageSize,
    //   };
    //   triggerOnRequest();
    // };
    // pagination.onShowSizeChange = (current, pageSize) => {
    //   originOnShowSizeChange?.(current, pageSize);
    //   paginationValues.current = {
    //     ...paginationValues.current,
    //     current,
    //     pageSize,
    //   };
    //   triggerOnRequest();
    // };
    const columns = originColumns.map(c => {
      if(c.renderType === 'actions'){
        return {
          ...c,
          buttons: c.buttons.map(b => ({
            ...b,
            onClick: (item: any) => {
              b.onClick(item)?.then(() => {
                triggerOnRequest();
              })
            }
          })),
        }
      }
      return c;
    })
    const onChange: typeof originTableOnChange = (...args) => {
      originTableOnChange?.(...args);
      const [pagination, filters, sorter, extra] = args;
      paginationValues.current = {
        ...paginationValues.current,
        ...pagination,
      };
      sorterValues.current = sorter;
      triggerOnRequest();
    };
    return {
      ...originTable,
      onChange,
      columns,
      // pagination,
    };
  }, [originTable, triggerOnRequest]);
  
  return (
    <StandardizeTable
      {...others}
      batchedActions={batchedActions}
      table={table}
      classification={classification}
      filter={filter}
      ref={instance}
    />
  );
};

RequestTable.defaultProps = {
  autoRequestOnMount: true,
}

export default React.memo(RequestTable);

function isArrayAndNotEmpty(data: any) {
  if (!Array.isArray(data)) return false;
  if (!data.length) return false;
  return true;
}
