import React, { useEffect, useRef, useState } from 'react';
import { Select, Input, Button, message } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import { SwapOutlined } from '@ant-design/icons';

import dayjs from 'dayjs';
import { originalOptions, translateOptions, originalMap } from './enums';

import css from './index.module.less';

type TranslateProps = {
    onTranslate: Function,
    persistenceKey?: string
}

const { TextArea } = Input; 
const TRANSLATE_CACHE = 'translateConfig';
// 原文组件
const Translate = (props: TranslateProps) => {
    const { onTranslate, persistenceKey='' } = props;
    const {t} = useTranslation();
    const [originalValue, setOriginalValue] = useState(originalMap.ZhCn); // 原文语言
    const [translateValue, setTranslateValue] = useState(originalMap.EnUs); // 译文语言
    const [originalText, setOriginalText] = useState(''); // 原文
    const [loading, setLoading] = useState(false);

    useEffect(() => {  // 翻译配置
        const translateConfig = JSON.parse(window.localStorage.getItem(TRANSLATE_CACHE)|| '{}');
        if (translateConfig.from) {
            setOriginalValue(translateConfig.from);
        }
        if (translateConfig.to) {
            setTranslateValue(translateConfig.to);
        }
    }, []);

    useEffect(() => {  // 原文localStorage缓存  每个工单缓存不同，故使用入口传入的key
        if (persistenceKey) {
            let originalCache = JSON.parse(window.localStorage.getItem(persistenceKey)|| '{}');
            // 缓存有效期1天 超出清空
            if (originalCache.cacheTime && dayjs(originalCache.cacheTime).diff(dayjs(), 'day') >= 1) {
                window.localStorage.removeItem(persistenceKey);
                originalCache = {};
            }
            if (originalCache.text) {
                setOriginalText(originalCache.text);
            }
        }
    }, []);

    // 译文下拉change
    const handleTranslateChange = (v) => {
        setTranslateValue(v);
        // 设置缓存
        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: originalValue,
            to: v
        }));
    };

    // 原文变化
    const handleOriginChange = (e: React.ChangeEvent<HTMLTextAreaElement>) => {
        const v = e.target.value;
        setOriginalText(v);
        // 设置缓存
        if (persistenceKey){
            let data = {
                cacheTime: new Date().getTime(),
                text: v
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }
    };

    // 点击翻译
    const onHandleTranslate = () => {
        if (loading) return;
        if (!originalText) {
            return message.error(t('originalPlaceholder'));
        }

        let o = originalValue;
        if (/[\u4e00-\u9fa5]/.test(originalText)) { // 有一个中文就算中文
            if (originalValue !== originalMap.ZhCn) {
                o = originalMap.ZhCn;
                setOriginalValue(o);
            }
        } else { // auto
            if (originalValue !== originalMap.Auto) {
                o = originalMap.Auto;
                setOriginalValue(o);
            }
        }

        if (onTranslate) {
            setLoading(true);
            onTranslate({
                from: o,
                to: translateValue,
                original: originalText
            }).then(()=>{
                setLoading(false);
            });
        }

        // 设置缓存
        if (persistenceKey){
            let data = {
                cacheTime: new Date().getTime(),
                text: originalText
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }

        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: o,
            to: translateValue
        }));
    };

    return (
        <div className={css['original-wrap']}>
            <div className={css['original-toolbar-wrap']}>
                <Select
                    disabled={true}
                    style={{ width: 120 }}
                    value={originalValue}
                    options={originalOptions}
                    onChange={(v) => setOriginalValue(v)}
                />
                <SwapOutlined className={css['original-toolbar-icon']}/>
                <Select
                    style={{ width: 120 }}
                    value={translateValue}
                    options={translateOptions}
                    onChange={handleTranslateChange}
                    className={css['original-toolbar-translate']}
                />
            </div>
            <div className={css['original-content-wrap']}>
                <TextArea
                    value={originalText}
                    onChange={handleOriginChange}
                    bordered={false}
                    autoSize={{minRows:6}}
                    placeholder={t('originalPlaceholder')}
                />
            </div>
            
            <Button type='primary' onClick={onHandleTranslate} loading={loading}>{t('statusbar.translate')}</Button>
        </div>
    );
};

export default React.memo(Translate);
