import React, { useCallback, useEffect, useState } from 'react';
import { Empty, Spin, Tree } from 'udesk-ui';
import classNames from 'classnames';
import { TreeNode } from './components/TreeNode';
import { DataNode } from 'rc-tree/lib/interface';
import { usePrefixCls } from '../utils';
import { TreeMenuWrap } from './components/TreeMenuWrap';
import { filterTreeDataByKeyword, formatTreeDataFromArrayToTree } from './utils';
import { TreeMenuProps } from './types';
import './style/index';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';

const { DirectoryTree } = Tree;
const defaultIconRender = (props: any) => null;

const TreeMenu: React.FC<TreeMenuProps> = (props) => {
  const {
    prefixCls: customizePrefixCls,
    className,
    treeData: treeDataSafe,
    treeNodeActionMenu,
    icon = defaultIconRender,
    switcherIconPosition = 'left',
    indentUnitWidth = 24,
    titleRender,
    treeNodeActionMenuTrigger,
    highlightKeyword,
    loading = false,
    getPopupContainer,
    ...rest
  } = props;

  const [treeMenuWrapPrefixCls, prefixCls] = usePrefixCls('tree-menu', customizePrefixCls);

  const classes = classNames(treeMenuWrapPrefixCls, className);

  const [treeDataWithHighlight, setTreeDataWithHighlight] = useState(treeDataSafe);

  const getNode = useCallback(
    (nodeData: DataNode) => {
      return (
        <TreeNode
          itemKey={nodeData.key}
          {...nodeData}
          menu={treeNodeActionMenu}
          switcherIconPosition={switcherIconPosition}
          switcherIcon={rest.switcherIcon}
          fieldNames={props.fieldNames}
          titleRender={titleRender}
          treeNodeActionMenuTrigger={treeNodeActionMenuTrigger}
          highlightKeyword={highlightKeyword}
          getPopupContainer={getPopupContainer}
        />
      );
    },
    [
      treeNodeActionMenu,
      switcherIconPosition,
      rest.switcherIcon,
      props.fieldNames,
      titleRender,
      highlightKeyword,
      getPopupContainer,
    ],
  );
  useEffect(() => {
    if (!highlightKeyword) {
      setTreeDataWithHighlight(treeDataSafe);
      return;
    }
    setTreeDataWithHighlight(
      filterTreeDataByKeyword(treeDataSafe, highlightKeyword, props.fieldNames),
    );
  }, [treeDataSafe, highlightKeyword, props.fieldNames]);

  return (
    <TreeMenuWrap
      className={classes}
      prefixCls={prefixCls}
      switcherIconPosition={switcherIconPosition}
      indentUnitWidth={indentUnitWidth}
    >
      <Spin spinning={loading}>
        {hasTreeData(treeDataWithHighlight) ? (
          <DirectoryTree
            treeData={treeDataWithHighlight}
            titleRender={getNode}
            blockNode={true}
            icon={(props) => (typeof icon === 'function' ? icon(props) : icon)}
            {...rest}
          />
        ) : (
          <Empty image={Empty.PRESENTED_IMAGE_SIMPLE} />
        )}
      </Spin>
    </TreeMenuWrap>
  );
};

type InternalTreeMenuType = typeof TreeMenu;

interface TreeMenuType extends InternalTreeMenuType {
  formatTreeDataFromArrayToTree: typeof formatTreeDataFromArrayToTree;
}

const RefTreeMenu: TreeMenuType = TreeMenu as TreeMenuType;

RefTreeMenu.formatTreeDataFromArrayToTree = formatTreeDataFromArrayToTree;

export default RefTreeMenu;

function hasTreeData(treeData: TreeMenuProps['treeData']): boolean {
  return Array.isArray(treeData) && treeData.length > 0;
}
