import {
    ArgumentNullError,
    InvalidArgumentError
} from '../../error';

const DEFAULT_VERSION_CHECK_INTERVAL_SECONDS = 60;

export default class LoadVersionIntervalClass {
    constructor() {
        this._pool = new Map();
    }

    add({ versionPath, callback, versionCheckIntervalSeconds, instance }) {
        if (versionPath == null || versionPath === "") {
            throw new ArgumentNullError(`versionPath`);
        }
        if (typeof versionPath !== "string") {
            throw new InvalidArgumentError(`versionPath`);
        }
        if (callback == null || callback === "") {
            throw new ArgumentNullError(`callback`);
        }
        if (typeof callback !== "function") {
            throw new InvalidArgumentError(`callback`);
        }

        if (instance == null || instance === "") {
            throw new ArgumentNullError(`instance`);
        }
        if (typeof instance !== "object") {
            throw new InvalidArgumentError(`instance`);
        }
        if (versionCheckIntervalSeconds != null && typeof versionCheckIntervalSeconds !== "number") {
            throw new InvalidArgumentError(`versionCheckIntervalSeconds`);
        }


        if (this._pool.has(versionPath)) {
            let callbackPollInstance = {
                versionCheckIntervalSeconds: versionCheckIntervalSeconds || DEFAULT_VERSION_CHECK_INTERVAL_SECONDS,
                instance: instance,
                callback: callback
            }
            let targetVersionInterval = this._pool.get(versionPath);
            targetVersionInterval.callbackPool.push(callbackPollInstance);
            if (versionCheckIntervalSeconds > targetVersionInterval.currentVersionCheckIntervalSeconds) {
                targetVersionInterval.intervalId.clear();
                targetVersionInterval.intervalId = getTimeoutInstance({
                    versionPath: versionPath,
                    getCallbackPool: function () {
                        return targetVersionInterval.callbackPool;
                    },
                    versionCheckIntervalSeconds: versionCheckIntervalSeconds
                });
            }
            this._pool.set(versionPath, targetVersionInterval);
        } else {
            this._pool.set(versionPath, buildNewVersionInterval(...arguments));
        }
    }

    remove(versionPath, instance) {
        if (versionPath == null || versionPath === "") {
            throw new ArgumentNullError(`versionPath`);
        }
        if (typeof versionPath !== "string") {
            throw new InvalidArgumentError(`versionPath`);
        }
        if (instance == null || instance === "") {
            throw new ArgumentNullError(`instance`);
        }
        if (typeof instance !== "object") {
            throw new InvalidArgumentError(`instance`);
        }
        if (this._pool.has(versionPath)) {
            let targetVersionInterval = this._pool.get(versionPath);
            targetVersionInterval.callbackPool = targetVersionInterval.callbackPool.filter((item) => item.instance !== instance);
            if (targetVersionInterval.callbackPool.length === 0) {
                targetVersionInterval.intervalId.clear();
                this._pool.delete(versionPath);
            } else {
                let minVersionCheckIntervalSeconds = null;
                targetVersionInterval.callbackPool.forEach((item) => {
                    if (minVersionCheckIntervalSeconds == null) {
                        minVersionCheckIntervalSeconds = item.versionCheckIntervalSeconds;
                    } else {
                        if (minVersionCheckIntervalSeconds > item.versionCheckIntervalSeconds) {
                            minVersionCheckIntervalSeconds = item.versionCheckIntervalSeconds;
                        }
                    }
                });
                if (minVersionCheckIntervalSeconds !== targetVersionInterval.currentVersionCheckIntervalSeconds) {
                    targetVersionInterval.currentVersionCheckIntervalSeconds = minVersionCheckIntervalSeconds;
                    targetVersionInterval.intervalId.clear();
                    targetVersionInterval.intervalId = getTimeoutInstance({
                        versionPath: versionPath,
                        getCallbackPool: function () {
                            return targetVersionInterval.callbackPool;
                        },
                        versionCheckIntervalSeconds: minVersionCheckIntervalSeconds
                    });
                }
                this._pool.set(versionPath, targetVersionInterval);
            }
        }
    }

    clear() {
        this._pool.forEach((item) => {
            clearTimeout(item.intervalId);
        });
        this._pool.clear();
    }
}



class TimeoutInstanceClass {
    constructor() {
        this.timeoutInstance = null;
    }
    setInterval(callback, seconds) {
        if (typeof callback !== "function") {
            throw new InvalidArgumentError(`callback`);
        }
        if (seconds == null || typeof seconds !== "number" || seconds <= 0) {
            throw new InvalidArgumentError(`seconds`);
        }
        let context = this;
        this.timeoutInstance = setTimeout(function () {
            callback();
            context.setInterval(callback, seconds);
        }, seconds * 1000);
    }
    clear() {
        clearTimeout(this.timeoutInstance);
    }
}

function buildNewVersionInterval(options) {
    if (options == null) {
        options = {};
    }
    let versionCheckIntervalSeconds = options.versionCheckIntervalSeconds || DEFAULT_VERSION_CHECK_INTERVAL_SECONDS;
    let newVersionInterval = {
        currentVersionCheckIntervalSeconds: versionCheckIntervalSeconds,
        callbackPool: [{
            versionCheckIntervalSeconds: versionCheckIntervalSeconds,
            instance: options.instance,
            callback: options.callback
        }]
    };
    newVersionInterval.intervalId = getTimeoutInstance({
        versionPath: options.versionPath,
        getCallbackPool: function () {
            return newVersionInterval.callbackPool;
        },
        versionCheckIntervalSeconds: versionCheckIntervalSeconds
    });
    return newVersionInterval;
}

function getTimeoutInstance({ versionPath, getCallbackPool, versionCheckIntervalSeconds }) {
    let timeoutInstance = new TimeoutInstanceClass();
    timeoutInstance.setInterval(function () {
        let resolveVersion = function (versionText) {
            let ver = typeof resp === "string" ? versionText : null;
            if (!ver || ver.indexOf("<") > -1) {
                ver = new Date().getTime();
            }
            let callbackPool = getCallbackPool();
            for (let i = 0; i < callbackPool.length; i++) {
                callbackPool[i].callback(ver);
            }
        };
        let rejectVersion = function () {
            let callbackPool = getCallbackPool();
            for (let i = 0; i < callbackPool.length; i++) {
                callbackPool[i].callback(null);
            }
        };
        if (window.$ && window.$.get) {
            window.$.get(`${versionPath}?v=${new Date().getTime()}`).then(function (resp) {
                resolveVersion(resp);
            }, function (reason) {
                rejectVersion();
            });
        }
        else if (typeof window.fetch === "function") {
            fetch(`${versionPath}?v=${new Date().getTime()}`)
                .then(function (response) {
                    return response.text();
                }, function () {
                    rejectVersion();
                })
                .then(function (versionText) {
                    resolveVersion(versionText);
                });
        }
    }, versionCheckIntervalSeconds);
    return timeoutInstance;
}

