// import common from '../common';
import enums from './enums';

const DEFAULT_HEAD_CHECK_INTERVAL_TIME = 50000;
const DEFAULT_SERVER_OPTIONS = {
    headCheckIntervalTime: DEFAULT_HEAD_CHECK_INTERVAL_TIME,
    hasHeadCheck: true,
    onConnected: function () {

    },
    onMessage: function (data) {

    },
    onClosed: function () {

    },

};

export default class UdeskCtiJsSdkWebsocketServer {
    constructor(options, connectMode) {
        this._websocketInstance = null;
        if (options == null || typeof options !== "object") {
            options = {};
        }
        this.options = Object.assign({}, DEFAULT_SERVER_OPTIONS, options);
        this._headCheckInterval = null;
    }

    get isConnected() {
        if (this._websocketInstance != null) {
            return this._websocketInstance.readyState === enums.websocketStates.open.id;
        }
        return false;
    }

    connect(url) {
        if (url == null || url === "") {
            throw new Error(`WebSocket connection error in websocket-server.js. url is not valid.`);
        }
        this._websocketInstance = new WebSocket(url);
        this._websocketInstance.onopen = () => {
            if (this.isConnected) {
                if (this.options.hasHeadCheck) {
                    this._startHeadCheck();
                }
                if (typeof this.options.onConnected === "function") {
                    this.options.onConnected(...arguments);
                }
            } else {
                if (typeof this.options.onClosed === "function") {
                    this.options.onClosed(...arguments);
                }
            }
        };
        this._websocketInstance.onmessage = (resp) => {
            let data = resp.data;
            //重置发送心跳的时间
            this._resetHeadCheck();
            if (data && data !== "pong" && data !== "") {
                try {
                    data = JSON.parse(data);
                } catch (error) {
                    window.console.error("JSON parse error", error);
                    // throw new Error(`Error in websocket-server.js. JSON.parse error.`);
                }
            }
            if (typeof this.options.onMessage === "function") {
                this.options.onMessage(data);
            }
        };
        this._websocketInstance.onerror = () => {
            if (this._websocketInstance.readyState === enums.websocketStates.closed) {
                if (this.options.hasHeadCheck) {
                    this._stopHeadCheck();
                }
                if (typeof this.options.onClosed === "function") {
                    this.options.onClosed(...arguments);
                }
            }
        };
    }

    close() {
        if (this._websocketInstance != null) {
            if (this.options.hasHeadCheck) {
                this._stopHeadCheck();
            }
            this._websocketInstance.close();
        }
    }

    send(data) {
        if (!this.isConnected) {
            throw new Error(`Error in websocket-server.js. websocket is not connect.`);
        }
        if (this._websocketInstance != null) {
            this._websocketInstance.send(data);
        }
    }

    _startHeadCheck() {
        let headCheckIntervalTime = this.options.headCheckIntervalTime > 0 ? this.options.headCheckIntervalTime : DEFAULT_HEAD_CHECK_INTERVAL_TIME;
        clearInterval(this._headCheckInterval);
        this._headCheckInterval = setInterval(() => {
            this._websocketInstance.send("ping");
        }, headCheckIntervalTime);
    }

    _stopHeadCheck() {
        clearInterval(this._headCheckInterval);
    }

    _resetHeadCheck() {
        this._startHeadCheck();
    }
}