//@flow
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import filter from 'lodash-es/filter';
import difference from 'lodash-es/difference';
import includes from 'lodash-es/includes';
import map from 'lodash-es/map';
import reduce from 'lodash-es/reduce';
import uniqWith from 'lodash-es/uniq';
import isEqual from 'lodash-es/isEqual';
import differenceWith from 'lodash-es/differenceWith';
import Thead from './thead';

type Props = {
    columns: Column[],
    data: Object[],
    formatter?: Object,
    className?: string,
    t?: (string) => string,
    onRowClick?: (any, number) => void,
    checkbox?: boolean, //是否显示checkbox
    onCheckedDataChange?: (Object[])=> void,
    dnd?: boolean,
    onColumnPlaceChange?: (Column, number, number) => void,
    onSort?: (Column, 'asc' | 'desc', number) => void,
    sortingColumns?: Object,
}

type State = {
    checkedData: Object[],
    originData: Object[]
}

/**
 * columns示例
 * ```
 * [
 * {id:'col1', display: true,  name: '列1'}
 * {id:'col2', display: true,  name: '列2'}
 * {id:'col3', display: false, name: '列3'}
 * ]
 * ```
 * formatter示例
 * ```
 * {
 *     col1: function(value){
 *         return format(value)
 *     }
 * }
 * ```
 *
 * @property {array<string>} columns - 列，字符串数组
 * @property {string} translatePrefix - 多语言前缀，该前缀与columns里的每一项组成key去查找对应的翻译内容
 * @property {array} data
 * @property {object} formatter - 格式化列的数据，是个object，key为columns的项
 */
class UdeskTable extends Component<Props, State> {
    defaultFormatter: Object;

    constructor(props: Props) {
        super(props);
        this.defaultFormatter = {
            cellphones(value) {
                if (Array.isArray(value)) {
                    return map(value, i => i.content).join(',');
                }
                return value;
            },
            level(value) {
                if (typeof props.t === 'function') {
                    return props.t('customer.level.' + value);
                } else {
                    return value;
                }
            }
        };
        let isControlledCheckedData = false;
        if ('checkedData' in props) {
            isControlledCheckedData = true;
        } 
        this.state = {
            checkedData: [],
            originData: this.props.data || [],
            isControlledCheckedData,
        }
    }

    static getDerivedStateFromProps(props: Props, state: State) {
        let { isControlledCheckedData } = state;
        let newState = {};
        if (props.data !== state.originData) {
            if (typeof props.onCheckedDataChange === 'function' && !isControlledCheckedData) {
                props.onCheckedDataChange([]);
            }
            newState.originData = props.data || [];
            !isControlledCheckedData && (newState.checkedData = []);
        }
        if (isControlledCheckedData && props.checkedData !== state.checkedData) {
            newState.checkedData = props.checkedData || [];
        }
        if (Object.keys(newState).length > 0) {
            return newState;
        }
        return null;
    }

    _isAllChecked = () => {
        let { checkedData } = this.state;
        let { data = [], isRowChecked } = this.props;
        return data && data.length && data.every(item => isRowChecked(item, checkedData));
    };

    _onRowCheckboxChange = (e: SyntheticInputEvent<HTMLInputElement>, rowData: Object) => {
        let { checkedData, isControlledCheckedData } = this.state;
        let { onCheckedDataChange } = this.props;
        let newCheckedData = [];
        if (e.target.checked) {
            newCheckedData = [...checkedData, rowData];
        } else {
            newCheckedData = filter(checkedData, i => !isEqual(i, rowData));
        }
        //受控模式下不更改state.checkedData,有props同步
        !isControlledCheckedData && this.setState({ checkedData: newCheckedData });
        if (typeof onCheckedDataChange === 'function') {
            onCheckedDataChange(newCheckedData);
        }
    };
    _onHeadCheckboxChange = (e: SyntheticInputEvent<HTMLInputElement>) => {
        let newCheckedData = [];
        let { data = [], onCheckedDataChange } = this.props;
        let { checkedData, isControlledCheckedData } = this.state;
        if (e.target.checked) {
            newCheckedData = uniqWith([...checkedData, ...(data || [])], isEqual);
        } else {
            newCheckedData = differenceWith(checkedData, data || [], isEqual);

        }
        //受控模式下不更改state.checkedData,有props同步
        !isControlledCheckedData && this.setState({ checkedData: newCheckedData });
        if (typeof onCheckedDataChange === 'function') {
            onCheckedDataChange(newCheckedData);
        }
    };

    render() {
        let {
            t = (v) => v, className, data = [], columns = [], formatter: globalFormatter = this.defaultFormatter, onRowClick,
            checkbox,
            dnd,
            onColumnPlaceChange,
            onSort,
            sortingColumns,
            isRowChecked,
        } = this.props;
        let { checkedData } = this.state;
        return (
            <table className={className}>
                <Thead 
                    dnd={dnd} 
                    columns={columns} 
                    onColumnPlaceChange={onColumnPlaceChange} 
                    checkbox={checkbox}
                    checkboxChecked={this._isAllChecked()}
                    onCheckboxChange={this._onHeadCheckboxChange}
                    onSort={onSort}
                    sortingColumns={sortingColumns}
                    t={t} />
                <tbody>
                    {
                        map(data, (dataItem, idx) => {
                            let shouldFireOnRowClick = true;
                            return <tr key={idx} onClick={() => {
                                if (!shouldFireOnRowClick) {
                                    shouldFireOnRowClick = true;
                                    return;
                                }
                                onRowClick && onRowClick(dataItem, idx);
                            }}>
                                {checkbox && <td onClick={() => shouldFireOnRowClick = false}>
                                    <input type='checkbox' checked={isRowChecked(dataItem, checkedData)}
                                        onChange={(e) => this._onRowCheckboxChange(e, dataItem)} />
                                </td>}
                                {
                                    map(columns, (column) => {
                                        let { id: col, formatter: colFormatter } = column;
                                        let result;
                                        let keyPath = col.split('.');
                                        let value = reduce(keyPath, (preVal, key) => {
                                            return preVal && preVal[key];
                                        }, dataItem);

                                        if (globalFormatter[col]) {
                                            result = globalFormatter[col](value, dataItem);
                                        } else if (colFormatter) {
                                            result = colFormatter(value, dataItem);
                                        } else {
                                            result = value;
                                        }
                                        if (column.cellComponent) {
                                            return <column.cellComponent key={col}
                                                className={column.className}
                                                rowData={dataItem}
                                                colData={column}>
                                                {result}
                                            </column.cellComponent>;
                                        }
                                        return <td key={col} className={column.className}>{result}</td>;
                                    })
                                }
                            </tr>;
                        })
                    }
                </tbody>
            </table>
        );
    }

    static defaultProps = {
        checkbox: false,
        allChecked: false,
        onCheckedDataChange: () => {
        },
        isRowChecked: (rowData, checkedData) => {
            return includes(checkedData, rowData);
        },
    };
}

UdeskTable.propTypes = {
    columns: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        name: PropTypes.string,
        formatter: PropTypes.func
    })).isRequired,
    data: PropTypes.arrayOf(PropTypes.object),
    checkedData: PropTypes.arrayOf(PropTypes.object),
    formatter: PropTypes.object,
    className: PropTypes.string,
    t: PropTypes.func,
    onRowClick: PropTypes.func,
    checkbox: PropTypes.bool, //是否显示checkbox
    onCheckedDataChange: PropTypes.func,
    isRowChecked: PropTypes.func,
};

export default UdeskTable;
