import pick from "lodash-es/pick";
import get from "lodash-es/get";
import merge from "lodash-es/merge";
import axios from "axios";
import difference from "lodash-es/difference";
import differenceWith from "lodash-es/differenceWith";
import isArray from "lodash-es/isArray";
import { saveCustomer } from "./customer-utils";
import { ValuesType } from "utility-types";
import { message } from "udesk-ui";

interface ITicketFieldHookResponse {
  code?: number;
  error?: string;
  ticket?: {
    subject?: string; //	字符串	工单主题
    content?: string; //	富文本	工单描述
    customer_id?: number; //	数值	工单客户ID
    priority_id?: number; //	数值	优先级ID
    status_id?: number; //	数值	状态ID
    tags?: string; //	字符串	逗号分割的标签名称
    agent_group_id?: number; //	数值	负责组ID
    assignee_id?: number; //	数值	负责人ID，负责者和负责人需要关联
    [customFieldName: string]: any;
  };
}

interface ICustomerFieldHookResponse {
  code?: number;
  error?: string;
  customer?: {
    nick_name?: string; //客户名称
    cellphones_attributes?: string[]; //	['159xxxx','159xxxx']	电话数组，更改电话触发
    email?: string; //	'udesk@udesk.cn'	更改主邮箱或副邮箱触发
    other_emails?: string[]; //	['udesk@udesk.cn','udesk2@udesk.cn']	更改主邮箱或副邮箱触发
    description?: string; //	字符串	描述
    organization_id?: string; //	字符串	公司id
    tags?: string; //	'标签,标签2'	逗号分割的标签名称
    lang?: string; //	'zh-cn'	客户语言码
    owner_group_id?: number; //	数值	负责组ID
    owner_id?: number; //	数值	负责人ID
    level?: "vip" | "normal";
    [customFieldName: string]: any;
  };
}

export function isSelectFieldByFieldName(fieldName: string) {
  return /^SelectField_\d+$/.test(fieldName);
}

//防止同样的客户推送多次
let pushingCustomersId: number[] = [];

export async function executeImFieldHook(
  fieldHook: IFieldHook,
  chat: IMineChat,
  customer: Customer,
  options: { customerCustomFields: CustomField[] } & Parameters<
    typeof saveCustomer
  >[1]
) {
  if (pushingCustomersId.includes(customer.id!)) {
    return;
  }
  pushingCustomersId.push(customer.id!);

  let otherParams: { [key: string]: any } = {
    im_sub_session_id: chat.im_sub_session_id,
    customer_id: chat.customer_id,
  };
  if (chat.platform === "web") {
    otherParams.web_token = customer.web_token;
  } else if (chat.platform === "wechat") {
    otherParams.weixins = (customer.weixins || []).map((i) => i.openid);
  } else if (chat.platform === "weibo") {
    otherParams.weibos = (customer.weibos || []).map((i) => i.openid);
  } else if (["android", "ios"].includes(chat.platform)) {
    otherParams.sdk_token = customer.sdk_token;
  } else if (chat.platform === "weixin_mini") {
    otherParams.weixin_minis = (customer.weixin_minis || []).map(
      (i) => i.openid
    );
  }
  let hookRes;
  try {
    hookRes = await postData(
      fieldHook.url,
      buildHookRequestParamsForCustomer(fieldHook, customer, {
        ...options,
        otherParams,
      })
    );
    pushingCustomersId = pushingCustomersId.filter((i) => i !== customer.id);
  } catch (e) {
    pushingCustomersId = pushingCustomersId.filter((i) => i !== customer.id);
    throw e;
  }
  if (hookRes.error) {
    message.error(hookRes.error);
  }
  if (!hookRes.customer) {
    return;
  }

  ignoreDuplicateEmail(hookRes, customer);
  ignoreDuplicatePhone(hookRes, customer);

  let changedCustomerData = convertFieldHookResponse(hookRes.customer);
  if (!changedCustomerData) {
    return;
  }
  changedCustomerData.id = customer.id;

  return saveCustomer(changedCustomerData, {
    agent: options.agent,
    role_can: options.role_can,
    company: options.company,
  });
}

async function postData(url: IFieldHook["url"], params: any) {
  const resp = await axios.post(url, params);
  if (resp.data.code === 1000) {
    return resp.data;
  } else {
    throw new Error("请求数据错误，请重试!");
  }
}

/**
 * 创建请求field hook的参数
 * @param {object}  fieldHook
 * @param {string}  fieldHook.url - field hook的url
 * @param {array}   fieldHook.target_fields - 要传给field hook的字段
 * @param {key}     fieldHook.key - 用于field hook验证
 * @param {object}  targetObject - 客户或者工单
 * @param {object}  otherParams - 其他的参数，呼叫中心弹屏时是电话号码，im进线是微信号、微博号等等
 * @param {string}  otherParams.phoneNumber - 电话号码，field hook的type是cc时有效
 * @param {string}  otherParams.web_token
 * @param {string}  otherParams.weixin_openid
 * @param {string}  otherParams.weibo_openid
 * @param {string}  otherParams.sdk_token
 * @param {string}  otherParams.im_sub_session_id
 * @return {object}
 */
export function buildHookRequestParamsForCustomer(
  fieldHook: IFieldHook,
  targetObject: Customer,
  options: {
    customerCustomFields: CustomField[];
    otherParams: { [key: string]: any };
  }
) {
  const { customerCustomFields, otherParams = {} } = options;
  let data = fieldHook.target_fields.reduce<{ [key: string]: any }>(
    (preVal, i) => {
      if (i === "cellphone") {
        if ("cellphones" in targetObject) {
          preVal[i] = targetObject["cellphones"];
        } else if ("cellphones_attributes" in targetObject) {
          preVal[i] = targetObject["cellphones_attributes"];
        }
        if (Array.isArray(preVal[i])) {
          preVal[i] = preVal[i].map((i: Customer["cellphones"]) =>
            pick(i, "id", "content", "_destroy")
          );
        }
      } else if (i === "organization_id") {
        //为兼容老数据，字段推送时依然会推organization_id
        preVal[i] = get(targetObject, "organizations[0].id");
        if (Array.isArray(targetObject.organizations)) {
          preVal["organizations"] = targetObject.organizations.map((item) => {
            if ("object" === typeof item) {
              return item.id + "";
            } else {
              return item + "";
            }
          });
        }
      } else if (i === "unionids") {
        preVal[i] = (targetObject.weixins || []).map((i) => i.openid);
      } else if (i === "userids") {
        preVal[i] = (targetObject.qywxes || []).map((i) => i.qywx_userid);
      } else if (isSelectFieldByFieldName(i) && targetObject.custom_fields) {
        let value = targetObject.custom_fields[i];
        if (!Array.isArray(value) && typeof value !== "string") {
          return preVal;
        }
        if (typeof value === "string") {
          value = value.split(",");
        }
        let customField = customerCustomFields.find((j) => j.name === i);
        if (!customField) {
          return preVal;
        }
        let fieldType = customField.content_type;
        let options = customField.select_options;
        let getOption = function (
          valueItem: { [key: string]: any } | string,
          opts: SelectField["select_options"]
        ) {
          if ("object" === typeof valueItem) {
            return opts.find((i) => i.value === valueItem.value + "");
          } else {
            return opts.find((i) => i.value === valueItem);
          }
        };

        preVal[i] = [];
        if (fieldType === "chained_droplist") {
          value.reduce(
            (
              preOpts: Parameters<typeof getOption>[1],
              v: Parameters<typeof getOption>[0]
            ) => {
              let selectedOption = getOption(v, preOpts);
              if (!selectedOption) {
                return preOpts;
              }
              preVal[i].push({
                value: selectedOption.value,
                text: selectedOption.title,
              });
              return selectedOption.subs;
            },
            options
          );
        } else {
          value.forEach((v: Parameters<typeof getOption>[0]) => {
            let option = getOption(v, options);
            if (option) {
              preVal[i].push({ value: option.value, text: option.title });
            }
          });
        }
      } else {
        preVal[i] =
          targetObject[i as keyof Customer] ||
          (targetObject.custom_fields && targetObject.custom_fields[i]);
      }

      return preVal;
    },
    {}
  );

  return merge({ [fieldHook.field_owner.toLowerCase()]: data }, otherParams);
}

// export function buildHookRequestParams(fieldHook: IFieldHook, targetObject: Customer | Ticket, otherParams = {}, options: { customerCustomFields: any, ticketCustomFields: any }) {
//     let data = fieldHook.target_fields.reduce<{ [key: string]: any }>((preVal, i) => {
//         if (i === 'cellphone') {
//             if ('cellphones' in targetObject) {
//                 preVal[i] = targetObject['cellphones'];
//             } else if ('cellphones_attributes' in targetObject) {
//                 preVal[i] = targetObject['cellphones_attributes'];
//             }
//             if (Array.isArray(preVal[i])) {
//                 preVal[i] = preVal[i].map((i: Customer['cellphones']) => pick(i, 'id', 'content', '_destroy'));
//             }
//         } else if (i === 'customer' && ('customer' in targetObject)) {
//             let customerData: { [key: string]: any } = pick(targetObject[i], 'open_api_token', 'weixin_openid', 'weibo_openid', 'web_token', 'sdk_token', 'email', 'cellphones');
//             if ('weixins' in targetObject[i]) {
//                 let weixins = targetObject[i].weixins;
//                 if (Array.isArray(weixins) && weixins.length > 0) {
//                     customerData.weixin_openid = weixins[0].openid;
//                 }
//             }
//
//             customerData.cellphone = customerData.cellphones;
//             delete customerData.cellphones;
//
//             preVal[i] = customerData;
//         } else if (i === 'organization_id') {
//             //为兼容老数据，字段推送时依然会推organization_id
//             preVal[i] = Ember.get(targetObject, 'organizations.firstObject.id');
//             if (Ember.isArray(Ember.get(targetObject, 'organizations'))) {
//                 preVal['organizations'] = Ember.get(targetObject, 'organizations').map(item => {
//                     if (['instance', 'object'].contains(Ember.typeOf(item))) {
//                         return Ember.get(item, 'id') + '';
//                     } else {
//                         return item + '';
//                     }
//                 });
//             }
//         } else if (isSelectFieldByFieldName(i) && targetObject.custom_fields) {
//             let value = targetObject.custom_fields[i];
//             if (!Ember.isArray(value) && (typeof value !== 'string')) {
//                 return preVal;
//             }
//             if (typeof value === 'string') {
//                 value = value.split(',');
//             }
//             let customField = customerCustomFields.findBy('name', i) || ticketCustomFields.findBy('name', i);
//             let fieldType = Ember.get(customField, 'content_type');
//             let options = Ember.get(customField, 'select_options');
//             let getOption = function (valueItem, opts) {
//                 if (['object', 'instance'].contains(typeof valueItem)) {
//                     return opts.findBy('value', Ember.get(valueItem, 'value') + '');
//                 } else {
//                     return opts.findBy('value', valueItem);
//                 }
//             };
//
//             preVal[i] = [];
//             if (fieldType === CONTENT_TYPE_CHAINED_DROPLIST) {
//                 value.reduce((preOpts, v) => {
//                     let selectedOption = getOption(v, preOpts);
//                     if (!selectedOption) {
//                         return preOpts;
//                     }
//                     preVal[i].push({
//                         value: Ember.get(selectedOption, 'value'),
//                         text: Ember.get(selectedOption, 'title')
//                     });
//                     return Ember.get(selectedOption, 'subs');
//                 }, options);
//             } else {
//                 value.forEach(v => {
//                     let option = getOption(v, options);
//                     if (option) {
//                         preVal[i].push({value: Ember.get(option, 'value'), text: Ember.get(option, 'title')});
//                     }
//                 });
//             }
//         } else {
//             preVal[i] = targetObject[i] || (targetObject.custom_fields && targetObject.custom_fields[i]);
//         }
//
//         return preVal;
//     }, {});
//
//     if (Ember.get(targetObject, 'field_num') && fieldHook.field_owner.toLowerCase() === 'ticket') {
//         data['field_num'] = Ember.get(targetObject, 'field_num');
//     }
//     return Ember.merge({[fieldHook.field_owner.toLowerCase()]: data}, otherParams);
// }

/**
 * 忽略返回数据里已重复的电话号码
 * @param {object} fieldHookResp - 字段推送返回的数据
 * @param {object} customer - 客户
 * @return {object} - 删除了重复项的field hook response
 */
function ignoreDuplicatePhone(
  fieldHookResp: ICustomerFieldHookResponse,
  customer: Pick<Customer, "cellphones" | "cellphones_attributes">
) {
  let respCustomer = fieldHookResp.customer;
  if (!respCustomer) {
    return fieldHookResp;
  }

  let originalPhones = customer.cellphones || customer.cellphones_attributes;
  respCustomer.cellphones_attributes = difference<string>(
    respCustomer.cellphones_attributes || [],
    originalPhones?.map((i: any) => i?.content || i) ?? []
  );

  return fieldHookResp;
}

/**
 * 忽略返回数据里已重复的邮箱
 * @param {object} fieldHookResp - 字段推送返回的数据
 * @param {object} customer - 客户
 * @return {object} - 删除了重复项的field hook response
 */
function ignoreDuplicateEmail(
  fieldHookResp: ICustomerFieldHookResponse,
  customer: Pick<Customer, "email" | "other_emails">
) {
  let respCustomer = fieldHookResp.customer;
  if (!respCustomer) {
    return fieldHookResp;
  }

  let originalEmail = customer.email || "";
  if (originalEmail === respCustomer.email) {
    delete respCustomer.email;
  }

  respCustomer.other_emails = differenceWith<
    string,
    ValuesType<NonNullable<Customer["other_emails"]>>
  >(
    respCustomer.other_emails || [],
    customer.other_emails || [],
    (a, b) => a === b![1]
  );

  return fieldHookResp;
}

/**
 * field hook返回的数据无法直接传到后端修改,需要做一步转换
 * 1、如果没有custom_fields字段，则加一个空的custom_fields字段
 * 2、把cellphones_attributes的元素用content包装一下
 * 3、把自定义字段全部移动到custom_fields属性里
 * @param {object} changedData - field hook返回的客户参数
 */
export function convertFieldHookResponse(
  changedData:
    | ICustomerFieldHookResponse["customer"]
    | ITicketFieldHookResponse["ticket"]
) {
  const res: any = {};
  for (let attr in changedData) {
    let val = changedData[attr];
    if (attr === "cellphones_attributes" && isArray(val)) {
      res[attr] = val.map((i) => ({ content: i }));
    } else if (attr === "other_emails" && isArray(val)) {
      res[attr] = val.map((i) => [null, i]);
    } else if (/^[a-zA-Z]+Field_\d+$/.test(attr)) {
      if (!res.custom_fields) {
        res.custom_fields = {};
      }
      res.custom_fields[attr] = val;
    } else {
      res[attr] = val;
    }
  }
  return res as Customer;
}
