import { getJSON, postJSON } from './ajax'
import axios from 'axios';
import { PromiseType } from "utility-types";

type StoragePolicy = 'qiniu' | 'ali' | 'huawei_cloud_obs' | 'minio';

interface AliParams {  //上传到阿里 oss 的扩展参数
    'Cache-Control'?: string
}

export function randomString(wordCount: number) {
    if (!wordCount) {
        wordCount = Math.ceil(Math.random() * 10);
    }
    const words = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_';
    let res = '';
    for (let i = 0; i < wordCount; i++) {
        res = res + words.charAt(Math.floor(Math.random() * words.length));
    }
    return res;
}

function getFileName(file: File) {
    const [fileName, fileExt] = file.name.split('.');
    let threeRandom = Math.random().toString(32).slice(-5);
    return fileName + "_" + (new Date().getTime()) + '_' + threeRandom + '.' + (fileExt || 'jpg');
}

type Marking = 'web.mg.rich_editor_copy';

export async function uploadFile(file: File, options: Parameters<typeof fetchFileUploadPolicy>[1] & { aliParams?: AliParams, saveAttachment?: boolean }): Promise<{ src: string, attachment_id?: number, attachment_mark?: string }> {
    const {saveAttachment, ...opts} = options;
    const fileUploadPolicyResp = await fetchFileUploadPolicy(file, opts);
    await uploadToOss(file, {...fileUploadPolicyResp, aliParams: opts.aliParams});
    const fileUrl = fileUploadPolicyResp.host + '/' + fileUploadPolicyResp.key;
    if (!saveAttachment) {
        return {src: fileUrl};
    }
    return await fetchAttachment(file, {
        storage_policy: fileUploadPolicyResp.storage_policy,
        bucket: fileUploadPolicyResp.bucket,
        key: fileUploadPolicyResp.key
    });
}

export async function fetchFileUploadPolicy(file: File, option: { marking: Marking, key?: string, access_token?: string, user_id?: number }) {
    let {key = getFileName(file)} = option; //如果 key 是传进来的，就使用传进来的 key，否则获取随机后的文件名
    const policyResp: {
        accessid: string,
        bucket: string,
        dir: string,
        expire: number,
        host: string,
        policy: string,
        signature: string,
        storage_policy: StoragePolicy,
        key?: string,
        signed_url?: string,
        file_path?: string,
        'x-oss-object-acl': string
    } = await getJSON('/file_upload_policy', {
        ...option,
        key
    });
    const storage_policy = policyResp.storage_policy;
    switch (storage_policy) {
        case 'qiniu':
            key = policyResp.key || key;
            break;
        case 'ali':
        case 'huawei_cloud_obs':
            key = policyResp.key || (policyResp.dir + key);
            break;
        case 'minio':
            key = policyResp.file_path!;
            break;
    }

    return {...policyResp, key}
}

async function uploadToOss(file: File, options: PromiseType<ReturnType<typeof fetchFileUploadPolicy>> & { aliParams?: AliParams }) {
    const {host, ...fileUploadPolicy} = options;
    const formData = new FormData();

    formData.append('policy', fileUploadPolicy.policy);
    formData.append('OSSAccessKeyId', fileUploadPolicy.accessid);
    formData.append('Signature', fileUploadPolicy.signature);
    if (fileUploadPolicy['x-oss-object-acl']) {
        formData.append('x-oss-object-acl', fileUploadPolicy['x-oss-object-acl']);
    }
    formData.append('key', fileUploadPolicy.key);
    if (options.aliParams?.["Cache-Control"]) {
        formData.append('Cache-Control', options.aliParams?.["Cache-Control"]);
    }
    formData.append('file', file);
    return await axios.post(host, formData, {headers: {'Content-Type': 'multipart/form-data', 'accept':'application/json'}});
}

// function checkSize(fileObject: fileObj) {
//     return new Promise<fileObj>(resolve => {
//         const {file} = fileObject;
//         if (file.size > imageSize) {
//             fileObject.error = {
//                 msg: r(fileObject.t('ticket.warning.exceedsTotalSize'), file.name, imageSizeLabel)
//             };
//         }
//         resolve(fileObject);
//     }).then(res => res.error ? res : fetchToken(res));
// }

// function fetchToken(fileObject: fileObj) {
//     return new Promise<fileObj>(resolve => {
//         const {file} = fileObject;
//         // const url = `${apiPath}/file_upload_policy`;
//         // const params = {
//         //     marking: this.tokenKey,
//         //     key: file.name
//         // };
//         getJSON('/file_upload_policy?marking=web.ts.ticket_reply&key=' + file.name).then(res => {
//             if (!res) fileObject.error = {
//                 msg: r(fileObject.t('ticket.warning.token'), file.name)
//             };
//             if (res.code === 1000) fileObject.policy_config = res;
//             else {
//                 const message = res.exception ? res.exception.message : '';
//                 fileObject.error = {
//                     msg: r(fileObject.t('ticket.warning.token'), file.name) + (message ? ': ' + message : '')
//                 };
//             }
//             resolve(fileObject);
//         });
//     }).then(res => res.error ? res : upload(res));
// }

// function upload(fileObject: fileObj) {
//     return new Promise<fileObj>(resolve => {
//         const {file} = fileObject;
//         const {url, params, src} = buildUploadParams(fileObject);
//         fileObject.uploadSrc = src
//         postJSON(url, params).catch(res => {
//             res = JSON.parse(res.message)
//             if (res && res.status !== 204) {
//                 fileObject.error = {
//                     msg: r(fileObject.t('ticket.warning.import'), file.name),
//                     detial: res.responseText
//                 };
//             }
//             resolve(fileObject);
//         });
//     }).then(res => {
//         if (res.noAttachment) {
//             return res
//         } else {
//             return res.error ? res : fetchAttachment(res)
//         }
//     });
// }


async function fetchAttachment(file: File, options: { key: string, bucket: string, storage_policy: StoragePolicy }) {
    const url = `/attachments`;
    const params = {
        attachment: {
            qiniu_key: options.key,
            file_file_name: file.name,
            file_content_type: file.type,
            file_file_size: file.size,
            bucket: options.bucket,
            storage_policy: options.storage_policy
        }
    };
    return await postJSON(url, params);
}
