import { PromiseType } from "utility-types";
import axios from "axios";
import { getJSON, postJSON } from "./ajax";

type StoragePolicy = "qiniu" | "ali" | "huawei_cloud_obs" | "minio";
type Marking = "web.mg.rich_editor_copy";
interface AliParams {
    //上传到阿里 oss 的扩展参数
    "Cache-Control"?: string;
}

export async function uploadFile(
    file: File,
    options: Parameters<typeof fetchFileUploadPolicy>[1] & {
        aliParams?: AliParams;
        saveAttachment?: boolean;
    }
): Promise<{ src: string; attachment_id?: number; attachment_mark?: string }> {
    const { saveAttachment, ...opts } = options;
    const fileUploadPolicyResp = await fetchFileUploadPolicy(file, opts);
    switch (fileUploadPolicyResp?.storage_policy) {
        case "ali":
            await uploadToOss(file, { ...fileUploadPolicyResp, aliParams: opts.aliParams });
            break;
        case "minio":
            await uploadToMinio(file, fileUploadPolicyResp);
            break;
        case "huawei_cloud_obs":
            // 暂时不考虑华为云
            await uploadToOss(file, { ...fileUploadPolicyResp, aliParams: opts.aliParams });
            break;
    }
    const fileUrl =
        fileUploadPolicyResp?.signed_url ||
        fileUploadPolicyResp.host + "/" + fileUploadPolicyResp.key;
    if (!saveAttachment) {
        return { src: fileUrl };
    }
    return await fetchAttachment(file, {
        storage_policy: fileUploadPolicyResp.storage_policy,
        bucket: fileUploadPolicyResp.bucket,
        key: fileUploadPolicyResp.key,
    });
}

function getFileName(file: File) {
    const [fileName, fileExt] = file.name.split(".");
    let threeRandom = Math.random().toString(32).slice(-5);
    return fileName + "_" + new Date().getTime() + "_" + threeRandom + "." + (fileExt || "jpg");
}

export async function fetchFileUploadPolicy(
    file: File,
    option: { marking: Marking; key?: string; access_token?: string; user_id?: number }
) {
    let { key = getFileName(file) } = option; //如果 key 是传进来的，就使用传进来的 key，否则获取随机后的文件名
    const policyResp: {
        accessid: string;
        bucket: string;
        dir: string;
        expire: number;
        host: string;
        policy: string;
        signature: string;
        storage_policy: StoragePolicy;
        key?: string;
        signed_url?: string;
        file_path?: string;
        "x-oss-object-acl": string;
    } = await getJSON("/file_upload_policy", {
        ...option,
        key,
    });
    const storage_policy = policyResp.storage_policy;
    switch (storage_policy) {
        case "qiniu":
            key = policyResp.key || key;
            break;
        case "ali":
        case "huawei_cloud_obs":
            key = policyResp.key || policyResp.dir + key;
            break;
        case "minio":
            key = policyResp.file_path!;
            break;
    }

    return { ...policyResp, key };
}

async function uploadToOss(
    file: File,
    options: PromiseType<ReturnType<typeof fetchFileUploadPolicy>> & { aliParams?: AliParams }
) {
    const { host, ...fileUploadPolicy } = options;
    const formData = new FormData();

    formData.append("policy", fileUploadPolicy.policy);
    formData.append("OSSAccessKeyId", fileUploadPolicy.accessid);
    formData.append("Signature", fileUploadPolicy.signature);
    if (fileUploadPolicy["x-oss-object-acl"]) {
        formData.append("x-oss-object-acl", fileUploadPolicy["x-oss-object-acl"]);
    }
    formData.append("key", fileUploadPolicy.key);
    if (options.aliParams?.["Cache-Control"]) {
        formData.append("Cache-Control", options.aliParams?.["Cache-Control"]);
    }
    formData.append("file", file);
    return await axios.post(host, formData, {
        headers: { "Content-Type": "multipart/form-data", accept: "application/json" },
    });
}
async function uploadToMinio(
    file: File,
    options: PromiseType<ReturnType<typeof fetchFileUploadPolicy>> & {
        aliParams?: AliParams;
        fields?: { [key: string]: any };
    }
) {
    const { host, ...fileUploadPolicy } = options;
    const formData = new FormData();
    const { fields } = fileUploadPolicy;
    if (fields) {
        Object.keys(fields).forEach((key) => {
            formData.append(key, fields[key]);
        });
    }
    formData.append("file", file);

    return await axios.post(host, formData, {
        headers: { "Content-Type": "multipart/form-data", accept: "application/json" },
    });
}

async function fetchAttachment(
    file: File,
    options: { key: string; bucket: string; storage_policy: StoragePolicy }
) {
    const url = "/attachments";
    const params = {
        attachment: {
            qiniu_key: options.key,
            file_file_name: file.name,
            file_content_type: file.type,
            file_file_size: file.size,
            bucket: options.bucket,
            storage_policy: options.storage_policy,
        },
    };
    return await postJSON(url, params);
}
